package pwc.taxtech.atms.common;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.CodeSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.entity.InteractionInteractiveLog;
import pwc.taxtech.atms.entity.InteractionInternationLog;
import pwc.taxtech.atms.entity.InteractionTaxManageLog;
import pwc.taxtech.atms.entity.PwcTaxLog;
import pwc.taxtech.atms.service.impl.DistributedIdService;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Component
@Aspect
public class AopLogger {
    private static Logger logger = LoggerFactory.getLogger(AopLogger.class);

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private PwcTaxLogMapper pwcTaxLogMapper;

    @Autowired
    protected DistributedIdService idService;

    @Autowired
    private InteractionInternationLogMapper interactionInternationLogMapper;

    @Autowired
    private InteractionInteractiveLogMapper interactionInteractiveLogMapper;

    @Autowired
    private InteractionTaxManageLogMapper interactionTaxManageLogMapper;

    @Pointcut("execution(public * pwc.taxtech.atms.service.*.*(..))")
    public void pointCut() {
        // 定义Pointcut 此方法不能有返回值，该方法只是一个标示
    }

    //这个事税享页面
    @Pointcut("execution(* pwc.taxtech.atms.service.impl.TaxEamilServiceImpl.getTaxEailDetail(..))")
    public void pointCutInteractionShare() {
        // 定义Pointcut 单独切入展示功能

    }
    //这个是国际机制
    @Pointcut("execution(* pwc.taxtech.atms.service.impl.InteractionInternationSystemServiceImpl.getUserShareDataById(..))")
    public void pointCutInteractionSystem() {
        // 定义Pointcut 单独切入展示功能

    }

    //这个是互动
    @Pointcut("execution(* pwc.taxtech.atms.service.impl.InteractionInteractiveServiceImpl.showDetail(..)) || execution(* pwc.taxtech.atms.service.impl.InteractionInteractiveServiceImpl.preLoadTaskshowDetail(..))")
    public void pointCutInteractivieSystem() {
        // 定义Pointcut 单独切入展示功能

    }



    //这个是互动
    @Pointcut("execution(* pwc.taxtech.atms.service.impl.InteractionTaxManageServiceImpl.getTaxManageDetail(..))")
    public void pointCutTaxManageSystem() {
        // 定义Pointcut 单独切入展示功能

    }


    @Before("pointCut()")
    public void before(JoinPoint joinPoint) {
        if (logger.isDebugEnabled()) {
            logger.debug("aop before:{}", joinPoint);
        }
    }

    @After("pointCut()")
    public void after(JoinPoint joinPoint) {
        if (logger.isDebugEnabled()) {
            logger.debug("aop after:{}", joinPoint);
        }
    }

    // 后置通知  这里是二期 税享模块浏览日志,统计浏览人数
    @AfterReturning("pointCutInteractionShare()")
    public void afterInteractionShare(JoinPoint joinPoint) {
        Map<String, Object> params = getNameAndValue(joinPoint);
        String emailId = (String) params.get("id");
        String userId = authUserHelper.getCurrentUserId();
        PwcTaxLog pwcTaxLog=new PwcTaxLog();
        pwcTaxLog.setId(String.valueOf(idService.nextId()));
        pwcTaxLog.setDelFlag(0);
        pwcTaxLog.setType(0);
        pwcTaxLog.setEmailId(emailId);
        pwcTaxLog.setUserId(userId);
        pwcTaxLog.setCreatetime(new Date());
        pwcTaxLogMapper.insert(pwcTaxLog);

    }

    // 后置通知  这里是二期 国际税模块浏览日志,统计浏览人数
    @AfterReturning("pointCutInteractionSystem()")
    public void afterInteractionSystem(JoinPoint joinPoint) {
        Map<String, Object> params = getNameAndValue(joinPoint);
        String internationId = (String) params.get("id");
        String userId = authUserHelper.getCurrentUserId();
        InteractionInternationLog interactionInternationLog=new InteractionInternationLog();
        interactionInternationLog.setId(String.valueOf(idService.nextId()));
        interactionInternationLog.setDelFlag(0);
        interactionInternationLog.setType(0);
        interactionInternationLog.setInternationId(internationId);
        interactionInternationLog.setUserId(userId);
        interactionInternationLog.setCreateTime(new Date());
        interactionInternationLogMapper.insert(interactionInternationLog);

    }

    // 后置通知  这里是二期  互动模块浏览日志,统计浏览人数
    @AfterReturning("pointCutInteractivieSystem()")
    public void afterInteractiveSystem(JoinPoint joinPoint) {
        Map<String, Object> params = getNameAndValue(joinPoint);
        String internationId = (String) params.get("id");
        String userId = authUserHelper.getCurrentUserId();

        InteractionInteractiveLog interactionInteractiveLog=new InteractionInteractiveLog();
        interactionInteractiveLog.setId(String.valueOf(idService.nextId()));
        interactionInteractiveLog.setDelFlag(0);
        interactionInteractiveLog.setType(0);
        interactionInteractiveLog.setInternationId(internationId);
        interactionInteractiveLog.setUserId(userId);
        interactionInteractiveLog.setCreateTime(new Date());
        interactionInteractiveLogMapper.insert(interactionInteractiveLog);

    }




    // 后置通知  这里是二期 滴滴税管浏览日志,统计浏览人数
    @AfterReturning("pointCutTaxManageSystem()")
    public void afterTaxManageSystem(JoinPoint joinPoint) {
        Map<String, Object> params = getNameAndValue(joinPoint);
        String internationId = (String) params.get("id");
        String userId = authUserHelper.getCurrentUserId();

        InteractionTaxManageLog interactionTaxManageLog=new InteractionTaxManageLog();
        interactionTaxManageLog.setId(String.valueOf(idService.nextId()));
        interactionTaxManageLog.setDelFlag(0);
        interactionTaxManageLog.setType(0);
        interactionTaxManageLog.setInternationId(internationId);
        interactionTaxManageLog.setUserId(userId);
        interactionTaxManageLog.setCreateTime(new Date());
        interactionTaxManageLogMapper.insert(interactionTaxManageLog);

    }

    @AfterThrowing(pointcut = "pointCut()", throwing = "error")
    public void afterThrowing(JoinPoint joinPoint, Throwable error) {
        if (logger.isWarnEnabled()) {
            logger.warn("aop afterThrowing:{}, exception class:{}, exception message:{} ", joinPoint,
                    error.getClass().toString(), error.getMessage());
        }
    }

    // @After("pointCut()")
    // public void doAfter(JoinPoint joinPoint) {
    // System.out.println("AOP After Advice...");
    // }
    //
    // @AfterReturning(pointcut = "pointCut()", returning = "returnVal")
    // public void afterReturn(JoinPoint joinPoint, Object returnVal) {
    // System.out.println("AOP AfterReturning Advice:" + returnVal);
    // }
    //
    //
    // @Around("pointCut()")
    // public void around(ProceedingJoinPoint pjp) {
    // System.out.println("AOP Aronud before...");
    // try {
    // pjp.proceed();
    // } catch (Throwable e) {
    // e.printStackTrace();
    // }
    // System.out.println("AOP Aronud after...");
    // }


    /**
     * 获取参数Map集合
     *
     * @param joinPoint
     * @return
     */
    Map<String, Object> getNameAndValue(JoinPoint joinPoint) {
        Map<String, Object> param = new HashMap<>();

        Object[] paramValues = joinPoint.getArgs();
        String[] paramNames = ((CodeSignature) joinPoint.getSignature()).getParameterNames();

        for (int i = 0; i < paramNames.length; i++) {
            param.put(paramNames[i], paramValues[i]);
        }

        return param;
    }

}