package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dpo.InteractionInteractiveSystemDto;
import pwc.taxtech.atms.dpo.PwcTicketDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.InteractionInteractiveComment;
import pwc.taxtech.atms.entity.InteractionInteractiveSystem;
import pwc.taxtech.atms.service.impl.InteractionInteractiveServiceImpl;

import javax.jws.Oneway;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


/**
 * @author: sam
 **/
@RestController
@RequestMapping("/api/v1/Interactive")
public class InteractionInteractiveController extends BaseController {


    @RequestMapping(value = "{interactionId}/vote/up", method = RequestMethod.POST)
    public OperationResultDto voteUp(@PathVariable String interactionId) {
        interactionInteractiveService.voteUp(interactionId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "{interactionId}/vote/cancel", method = RequestMethod.POST)
    public OperationResultDto voteCancel(@PathVariable String interactionId) {
        interactionInteractiveService.voteCancel(interactionId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/comment/up", method = RequestMethod.POST)
    public OperationResultDto commentUp(@RequestBody InteractionInteractiveComment interactionInteractiveComment) {
        interactionInteractiveService.commentUp(interactionInteractiveComment);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/{interactionId}/comment/cancel", method = RequestMethod.POST)
    public OperationResultDto commentCancel(@PathVariable String interactionId) {
        interactionInteractiveService.commentCancel(interactionId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "{interactionId}/commentList", method = RequestMethod.GET)
    public OperationResultDto showComment(@PathVariable String interactionId) {
        OperationResultDto opt  = new OperationResultDto();
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = null;
        try {
            interactionInteractiveSystemDto = interactionInteractiveService.showComment(interactionId);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        return OperationResultDto.success(interactionInteractiveSystemDto);
    }

    @Autowired
    private InteractionInteractiveServiceImpl interactionInteractiveService;


    /*获取互动信息*/
    @RequestMapping("listData")
    public OperationResultDto listData(InteractionInteractiveSystemDto interactionInteractiveSystemDto) {
        OperationResultDto opt  = new OperationResultDto();
        PageInfo<InteractionInteractiveSystemDto> pageInfo = null;
        try {
            pageInfo = interactionInteractiveService.listData(interactionInteractiveSystemDto, -1);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setData(pageInfo);
        opt.setResultMsg("success");
        return opt;
    }

    //根据类型获取互动数据
    @RequestMapping("getListByType")
    public OperationResultDto getListByType(int type, InteractionInteractiveSystemDto interactionInteractiveSystemDto) {
        PageInfo<InteractionInteractiveSystemDto> pageInfo = null;
        OperationResultDto opt  = new OperationResultDto();
        try {
            pageInfo = interactionInteractiveService.listData(interactionInteractiveSystemDto, type);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setData(pageInfo);
        opt.setResultMsg("success");
        return opt;
    }

    //发布留言
    @RequestMapping("postMessageSubmit")
    public OperationResultDto postMessageSubmit(@RequestBody InteractionInteractiveSystem interactionInteractiveSystem) {
        //interactionInteractiveSystem.setCreateBy();
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.postMessageSubmit(interactionInteractiveSystem);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //发布提问
    @RequestMapping("postQuestion")
    public OperationResultDto postQuestion(@RequestBody InteractionInteractiveSystem interactionInteractiveSystem) {
        //interactionInteractiveSystem.setCreateBy();
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.postQuestion(interactionInteractiveSystem);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //发布投票
    @RequestMapping("postVoteSubmit")
    public OperationResultDto postVoteSubmit(@RequestBody InteractionInteractiveSystem interactionInteractiveSystem) {
        //interactionInteractiveSystem.setCreateBy();
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.postVoteSubmit(interactionInteractiveSystem);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }


    /**
     * 查看详情
     */
    @RequestMapping("showDetail")
    public OperationResultDto showDetail(String id) {
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = null;
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveSystemDto = interactionInteractiveService.showDetail(id);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(interactionInteractiveSystemDto);
        return opt;
    }

    /*进行投票*/
    @RequestMapping("subitTicket")
    public OperationResultDto subitTicket(@RequestBody PwcTicketDto pwcTicketDto) {
        PwcTicketDto pwcTicketDto1 = null;
        OperationResultDto opt = new OperationResultDto();
        try {
            pwcTicketDto1 = interactionInteractiveService.subitTicket(pwcTicketDto, opt);
        } catch (Exception e) {
            e.printStackTrace();
            return opt;
        }
        if (opt.getResultMsg() != null) {
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(pwcTicketDto1);
        return opt;
    }

    //SSE服务推送投票信息 因为不支持请求头添加tocken，无法跨越权限请求到后台
    @RequestMapping(value = "pushSseVote", produces = "text/event-stream;charset=UTF-8")
    public OperationResultDto pushSseVote(String id) {
        OperationResultDto opt  = new OperationResultDto();
        List<PwcTicketDto> arr = new ArrayList<>();
        try {
            arr = interactionInteractiveService.pushSseVote(id);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(arr);
        return opt;
    }

    @RequestMapping("setIntervalVote")
    public OperationResultDto setIntervalVote(String id) {
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = null;
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveSystemDto = interactionInteractiveService.setIntervalVote(id);
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(interactionInteractiveSystemDto);
        return opt;
    }

    //预加载任务列表界面
    @RequestMapping("preLoadTask")
    public OperationResultDto preLoadTask(String id) {
        OperationResultDto opt  = new OperationResultDto();
        HashMap map = null;
        try {
            map = interactionInteractiveService.preLoadTaskshowDetail(id);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(map);
        return opt;
    }

    //添加任务
    @RequestMapping("taskSubmit")
    public OperationResultDto taskSubmit(@RequestBody InteractionInteractiveSystemDto interactionInteractiveSystem) {
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = null;
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveSystemDto = interactionInteractiveService.taskSubmit(interactionInteractiveSystem);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
        }
        opt.setData(interactionInteractiveSystemDto);
        opt.setResultMsg("success");
        return opt;
    }

    //更新数据库，告诉后台当前任务正在修改
    @RequestMapping("duringUpdate")
    public OperationResultDto duringUpdate(String id) {
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.duringUpdate(id);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //提交协同人
    @RequestMapping("collaboratorSumbit")
    public OperationResultDto collaboratorSumbit(@RequestBody InteractionInteractiveSystem interactionInteractiveSystem) {
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.collaboratorSumbit(interactionInteractiveSystem.getAttachUrl(), interactionInteractiveSystem.getId());
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //互动后台展示列表
    @RequestMapping(value = "getInteractiveAdminLists", method = RequestMethod.GET)
    public OperationResultDto getInteractiveAdminLists(InteractionInteractiveSystemDto interactionInteractiveSystemDto) {
        List<InteractionInteractiveSystemDto> list = null;
        OperationResultDto opt  = new OperationResultDto();
        try {
            list = interactionInteractiveService.getInteractiveAdminLists(interactionInteractiveSystemDto);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
        }
        opt.setResultMsg("success");
        opt.setData(list);
        return opt;
    }

    //删除互动信息
    @RequestMapping("deleteInteractiveAdmin")
    public OperationResultDto deleteInteractiveAdmin(String id) {
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.deleteInteractiveAdmin(id);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //设置为推荐
    @RequestMapping("doRecommended")
    public OperationResultDto doRecommended(String id, Integer type) {
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.doRecommended(id, type);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //修改互动信息
    @RequestMapping("postEdit")
    public OperationResultDto postEdit(@RequestBody InteractionInteractiveSystem interactionInteractiveSystem, Integer type) {
        OperationResultDto opt  = new OperationResultDto();
        try {
            interactionInteractiveService.postEdit(interactionInteractiveSystem, type);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        return opt;
    }

    //修改之前的查询
    @RequestMapping("selectById")
    public OperationResultDto selectById(String id) {
        OperationResultDto opt  = new OperationResultDto();
        InteractionInteractiveSystem interactionInteractiveSystem = null;
        try {
            interactionInteractiveSystem = interactionInteractiveService.selectById(id);
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setData(interactionInteractiveSystem);
        opt.setResultMsg("success");
        return opt;
    }

    //获取首页税享信息数据
    @RequestMapping("getHomeInteractiveData")
    public OperationResultDto getHomeInteractiveData() {
        OperationResultDto opt  = new OperationResultDto();
        List<InteractionInteractiveSystem> list = null;
        try {
            list = interactionInteractiveService.getHomeInteractiveData();
        } catch (Exception ex) {
            ex.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setData(list);
        opt.setResultMsg("success");
        return opt;
    }

}
