package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.dao.PwcTagMapper;
import pwc.taxtech.atms.dpo.InteractionInternationSystemDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.entity.InteractionInternationComment;
import pwc.taxtech.atms.entity.InteractionInternationSystem;
import pwc.taxtech.atms.entity.PwcTag;
import pwc.taxtech.atms.entity.PwcTagExample;
import pwc.taxtech.atms.service.impl.InteractionInternationSystemServiceImpl;
import pwc.taxtech.atms.service.impl.TaxDocumentServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @version 1.0
 * @program: atms
 * @description: 国际税制
 * @author: Kevin
 * @create: 2019-05-07 22:22
 **/
@RestController
@RequestMapping("/api/v1/InternationSystem")
public class InteractionInternationSystemController extends  BaseController {

    @Autowired
    private InteractionInternationSystemServiceImpl interactionInternationSystemService;

    @Autowired
    private PwcTagMapper pwcTagMapper;

    @RequestMapping(value = "getListData", method = RequestMethod.GET)
    @ResponseBody
    public List<InteractionInternationSystemDto> getListData(InteractionInternationSystemDto interactionInternationSystemDto) {
        return interactionInternationSystemService.listAdminTaxEmail(interactionInternationSystemDto);
    }

    @RequestMapping(value = "list", method = RequestMethod.GET)
    @ResponseBody
    public PageInfo<InteractionInternationSystemDto> getUserListData(InteractionInternationSystemDto interactionInternationSystemDto) {
        return interactionInternationSystemService.listInternationSystem(interactionInternationSystemDto);
    }


    @RequestMapping(value = "/editInternationSystem", method = RequestMethod.POST)
    public OperationResultDto editInternationSystem(@RequestBody InteractionInternationSystem taxEmail) {
        int add = 0;
        try {
            add = interactionInternationSystemService.editTaxEmail(taxEmail);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("编辑失败");
        }
        if (add == 0) {
            return OperationResultDto.error("编辑失败");
        }
        return OperationResultDto.success("编辑成功");
    }

    @RequestMapping(value = "/addInternationSystem", method = RequestMethod.POST)
    public OperationResultDto addInternationSystem(@RequestBody InteractionInternationSystem taxEmail) throws Exception {
        int add = 0;
        try {
            add = interactionInternationSystemService.addTaxEmail(taxEmail);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("添加失败");
        }
        if (add == 0) {
            return OperationResultDto.error("添加失败");
        }
        return OperationResultDto.success("添加成功");
    }

    @RequestMapping(value = "/getInternationTaxById", method = RequestMethod.GET)
    public OperationResultDto getInternationTaxById(String id) throws Exception {
        InteractionInternationSystem taxEmail = null;
        try {
            taxEmail = interactionInternationSystemService.getShareDataById(id);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("获取国际税制失败");
        }
        return OperationResultDto.success(taxEmail);
    }

    //国际税查询详情
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public OperationResultDto getUserInternationTaxDetailById(@PathVariable String id) throws Exception {
        InteractionInternationSystemDto interactionInternationSystemDto = null;
        try {
            interactionInternationSystemDto = interactionInternationSystemService.getUserShareDataById(id);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("获取国际税制详情失败");
        }
        return OperationResultDto.success(interactionInternationSystemDto);
    }


    //国际税回复
    @RequestMapping(value = "/{id}/commend", method = RequestMethod.GET)
    public OperationResultDto getCommendById(@PathVariable String id) throws Exception {
        InteractionInternationSystemDto interactionInternationSystemDto = null;
        try {
            interactionInternationSystemDto = interactionInternationSystemService.getCommendById(id);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("获取国际税制详情失败");
        }
        return OperationResultDto.success(interactionInternationSystemDto);
    }

    @RequestMapping("pdfUpload")
    @ResponseBody
    public OperationResultDto pdfUpload(@RequestParam MultipartFile file,InteractionInternationSystem interactionInternationSystem){
        try{
            interactionInternationSystemService.pdfUpload(file,interactionInternationSystem);
        }catch (Exception e){
            return  OperationResultDto.error("上传失败");
        }
        return  OperationResultDto.success();
    }

    @RequestMapping("pdfUploadEdit")
    @ResponseBody
    public OperationResultDto pdfUploadEdit(@RequestParam(required=false) MultipartFile file,InteractionInternationSystem interactionInternationSystem){
        try{
            interactionInternationSystemService.pdfUploadEdit(file,interactionInternationSystem);
        }catch (Exception e){
            return  OperationResultDto.error("上传失败");
        }
        return  OperationResultDto.success();
    }


    /**
     * 获取远程pdf 地址
     */
    @RequestMapping("getRemotePdfUrl")
    @ResponseBody
    public String getRemotePdfUrl(HttpServletResponse response, String id){
        try {
            return  interactionInternationSystemService.pdfView(response,id);
        } catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Autowired
    private TaxDocumentServiceImpl taxDocumentService;

    @RequestMapping("previewPDF")
    @ResponseBody
    public String previewPDF(HttpServletResponse response ,@RequestBody TaxDocumentDto taxDocumentDto) {
        return taxDocumentService.previewPDF2(response,taxDocumentDto.getPath());
    }
    @RequestMapping(value = "/{internationId}/delete", method = RequestMethod.POST)
    public OperationResultDto delete(@PathVariable String internationId) {
        interactionInternationSystemService.delete(internationId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/{internationId}/vote/up", method = RequestMethod.POST)
    public OperationResultDto voteUp(@PathVariable String internationId) {
        interactionInternationSystemService.voteUp(internationId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/{internationId}/vote/cancel", method = RequestMethod.POST)
    public OperationResultDto voteCancel(@PathVariable String internationId) {
        interactionInternationSystemService.voteCancel(internationId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/comment/up", method = RequestMethod.POST)
    public OperationResultDto commentUp(@RequestBody InteractionInternationComment interactionInternationComment) {
        interactionInternationSystemService.commentUp(interactionInternationComment);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/{internationId}/comment/cancel", method = RequestMethod.POST)
    public OperationResultDto commentCancel(@PathVariable String internationId) {
        interactionInternationSystemService.commentCancel(internationId);
        return OperationResultDto.success();
    }


    //加载tags数据
    @RequestMapping("getTagList")
    @ResponseBody
    public OperationResultDto getTagList(int type) {
        PwcTagExample example = new PwcTagExample();
        PwcTagExample.Criteria criteria = example.createCriteria();
        criteria.andTypeEqualTo(type);
        List<PwcTag> pwcTags = pwcTagMapper.selectByExample(example);
        OperationResultDto operationResultDto = new OperationResultDto();
        operationResultDto.setData(pwcTags);
        return operationResultDto;
    }

}
