package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dao.PwcTagMapper;
import pwc.taxtech.atms.dpo.InteractionTaxManageDto;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.InteractionTaxManage;
import pwc.taxtech.atms.entity.InteractionTaxManageComment;
import pwc.taxtech.atms.entity.PwcTag;
import pwc.taxtech.atms.entity.PwcTagExample;
import pwc.taxtech.atms.service.impl.InteractionTaxManageServiceImpl;

import java.util.List;


/**
 * 滴滴税管
 * 2019-05-23
 * @author: sam
 **/
@RestController
@RequestMapping("/api/v1/InteractionTaxManage")
public class InteractionTaxManageController {

    @Autowired
    InteractionTaxManageServiceImpl interactionTaxManageServiceImpl;

    @Autowired
    private PwcTagMapper pwcTagMapper;

    /**
     * admin审核部分的查询
     *
     * @param InteractionTaxManageDto
     * @return
     */
    @RequestMapping(value = "listAdmin", method = RequestMethod.GET)
    @ResponseBody
    public List<InteractionTaxManageDto> selectTaxAdminEmailList(InteractionTaxManageDto InteractionTaxManageDto) {
        return interactionTaxManageServiceImpl.listAdminInteractionTaxManage(InteractionTaxManageDto);
    }


    /**
     * 用户列表部分的查询
     *
     * @param InteractionTaxManageDto
     * @return
     */
    @RequestMapping(value = "list", method = RequestMethod.GET)
    @ResponseBody
    public PageInfo<InteractionTaxManageDto> selectInteractionTaxManageList(InteractionTaxManageDto InteractionTaxManageDto) {
        return interactionTaxManageServiceImpl.listInteractionTaxManage(InteractionTaxManageDto);
    }



    //加载tags数据
    @RequestMapping("getTagList")
    @ResponseBody
    public OperationResultDto getTagList(int type) {
        PwcTagExample example = new PwcTagExample();
        PwcTagExample.Criteria criteria = example.createCriteria();
        criteria.andTypeEqualTo(type);
        List<PwcTag> pwcTags = pwcTagMapper.selectByExample(example);
        OperationResultDto operationResultDto = new OperationResultDto();
        operationResultDto.setData(pwcTags);
        return operationResultDto;
    }

    /**
     * 税享详情查询
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public ApiResultDto getInteractionTaxManageDetail(@PathVariable String id) {
        InteractionTaxManageDto interactionTaxManage = interactionTaxManageServiceImpl.getTaxManageDetail(id);
        return ApiResultDto.success(interactionTaxManage);
    }

    /**
     * 税享详情查询
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/{id}/commend", method = RequestMethod.GET)
    public ApiResultDto getCommend(@PathVariable String id) {
        InteractionTaxManageDto interactionTaxManage = interactionTaxManageServiceImpl.getCommend(id);
        return ApiResultDto.success(interactionTaxManage);
    }


    /**
     * 编辑税享
     *
     * @param interactionTaxManage
     * @return
     */
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    public OperationResultDto editInteractionTaxManage(@RequestBody InteractionTaxManage interactionTaxManage) {
        int add = 0;
        try {
            add = interactionTaxManageServiceImpl.editTaxManage(interactionTaxManage);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("编辑失败");
        }
        if (add == 0) {
            return OperationResultDto.error("编辑失败");
        }
        return OperationResultDto.success("编辑成功");
    }

    //手动新增税享
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public OperationResultDto addInteractionTaxManage(@RequestBody InteractionTaxManage interactionTaxManage) throws Exception {
        int add = 0;
        try {
            add = interactionTaxManageServiceImpl.addTaxManage(interactionTaxManage);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("添加失败");
        }
        if (add == 0) {
            return OperationResultDto.error("添加失败");
        }
        return OperationResultDto.success("添加成功");
    }

    //详情
    @RequestMapping(value = "/getShareDataById", method = RequestMethod.GET)
    public OperationResultDto getShareDataById(String id) throws Exception {
        InteractionTaxManage interactionTaxManage = null;
        try {
            interactionTaxManage = interactionTaxManageServiceImpl.getTaxManageDetail(id);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("获取税享失败");
        }
        return OperationResultDto.success(interactionTaxManage);
    }

    //点赞
    @RequestMapping(value = "{internationId}/vote/up", method = RequestMethod.POST)
    public OperationResultDto voteUp(@PathVariable String internationId) {
        interactionTaxManageServiceImpl.voteUp(internationId);
        return OperationResultDto.success();
    }

    //取消点赞
    @RequestMapping(value = "{internationId}/vote/cancel", method = RequestMethod.POST)
    public OperationResultDto voteCancel(@PathVariable String internationId) {
        interactionTaxManageServiceImpl.voteCancel(internationId);
        return OperationResultDto.success();
    }

    //回复
    @RequestMapping(value = "/comment/up", method = RequestMethod.POST)
    public OperationResultDto commentUp(@RequestBody InteractionTaxManageComment interactionTaxManageComment) {
        interactionTaxManageServiceImpl.commentUp(interactionTaxManageComment);
        return OperationResultDto.success();
    }

    //删除回复
    @RequestMapping(value = "/{internationId}/comment/cancel", method = RequestMethod.POST)
    public OperationResultDto commentCancel(@PathVariable String internationId) {
        interactionTaxManageServiceImpl.commentCancel(internationId);
        return OperationResultDto.success();
    }

    //添加标签
    @RequestMapping("addTagSubmit")
    @ResponseBody
    public OperationResultDto addTagSubmit(String tag, int type) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            PwcTag pwcTag = interactionTaxManageServiceImpl.addTagSubmit(tag, type);
            if (pwcTag != null ) {
                operationResultDto.setResultMsg("success");
                operationResultDto.setData(pwcTag);
                return operationResultDto;
            } else {
                operationResultDto.setResultMsg("fail");
                return operationResultDto;
            }
        } catch (Exception e) {
            operationResultDto.setResultMsg("error");
            return operationResultDto;
        }
    }

    @RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
    @ResponseBody
    public ApiResultDto deleteLink(@PathVariable String id) {
        interactionTaxManageServiceImpl.delete(id);
        return ApiResultDto.success();
    }




}
