package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.common.PageResultVo;
import pwc.taxtech.atms.common.pwc.Exception.PwcException;
import pwc.taxtech.atms.dao.PwcTagMapper;
import pwc.taxtech.atms.dao.PwcTaxLogMapper;
import pwc.taxtech.atms.dpo.TaxEmailDto;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.service.impl.TaxEamilServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * 税享,第一版 命名taxemail 其他的是Interaction 开头的
 *
 * @author: sam
 **/
@RestController
@RequestMapping("/api/v1/taxEmail")
public class TaxEmailController extends BaseController {

    @Autowired
    TaxEamilServiceImpl taxEamilServiceImpl;

    /**
     * admin审核部分的查询
     *
     * @param taxEmailDto
     * @return
     */
    @RequestMapping(value = "listAdmin", method = RequestMethod.GET)
    @ResponseBody
    public List<TaxEmailDto> selectTaxAdminEmailList(TaxEmailDto taxEmailDto) {
        return taxEamilServiceImpl.listAdminTaxEmail(taxEmailDto);
    }


    /**
     * 用户列表部分的查询
     *
     * @param taxEmailDto
     * @return
     */
    @RequestMapping(value = "list", method = RequestMethod.GET)
    @ResponseBody
    public PageInfo<TaxEmailDto> selectTaxEmailList(TaxEmailDto taxEmailDto) {
        return taxEamilServiceImpl.listTaxEmail(taxEmailDto);
    }

    @Autowired
    private PwcTagMapper pwcTagMapper;

    //加载tags数据
    @RequestMapping("getTagList")
    @ResponseBody
    public OperationResultDto getTagList(int type) {
        PwcTagExample example = new PwcTagExample();
        PwcTagExample.Criteria criteria = example.createCriteria();
        criteria.andTypeEqualTo(type);
        List<PwcTag> pwcTags = pwcTagMapper.selectByExample(example);
        OperationResultDto operationResultDto = new OperationResultDto();
        operationResultDto.setData(pwcTags);
        return operationResultDto;
    }

    /**
     * 税享详情查询
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public ApiResultDto selectTaxEmailDetail(@PathVariable String id) {
        TaxEmailDto taxEmail = taxEamilServiceImpl.getTaxEailDetail(id);
        return ApiResultDto.success(taxEmail);
    }

    /**
     * 税享详情查询
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/{id}/loadCommends", method = RequestMethod.GET)
    public ApiResultDto loadCommends(@PathVariable String id) {
        TaxEmailDto taxEmail = taxEamilServiceImpl.loadCommends(id);
        return ApiResultDto.success(taxEmail);
    }

    @RequestMapping(value = "/{id}/delete", method = RequestMethod.POST)
    public OperationResultDto delete(@PathVariable String id) {
         taxEamilServiceImpl.delete(id);
        return OperationResultDto.success();
    }

    /**
     * 编辑税享
     *
     * @param taxEmail
     * @return
     */
    @RequestMapping(value = "/editTaxEmail", method = RequestMethod.POST)
    public OperationResultDto editTaxEmail(@RequestBody TaxEmail taxEmail) {
        int add = 0;
        try {
            add = taxEamilServiceImpl.editTaxEmail(taxEmail);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("编辑失败");
        }
        if (add == 0) {
            return OperationResultDto.error("编辑失败");
        }
        return OperationResultDto.success("编辑成功");
    }

    //手动新增税享
    @RequestMapping(value = "/addTaxEmail", method = RequestMethod.POST)
    public OperationResultDto addTaxEmail(@RequestBody TaxEmail taxEmail) throws Exception {
        int add = 0;
        try {
            add = taxEamilServiceImpl.addTaxEmail(taxEmail);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("添加失败");
        }
        if (add == 0) {
            return OperationResultDto.error("添加失败");
        }
        return OperationResultDto.success("添加成功");
    }

    @RequestMapping(value = "/getShareDataById", method = RequestMethod.GET)
    public OperationResultDto getShareDataById(String id) throws Exception {
        TaxEmail taxEmail = null;
        try {
            taxEmail = taxEamilServiceImpl.getShareDataById(id);
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error("获取税享失败");
        }
        return OperationResultDto.success(taxEmail);
    }


    @RequestMapping(value = "{emailId}/vote/up", method = RequestMethod.POST)
    public OperationResultDto voteUp(@PathVariable String emailId) {
        taxEamilServiceImpl.voteUp(emailId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "{emailId}/vote/cancel", method = RequestMethod.POST)
    public OperationResultDto voteCancel(@PathVariable String emailId) {
        taxEamilServiceImpl.voteCancel(emailId);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/comment/up", method = RequestMethod.POST)
    public OperationResultDto commentUp(@RequestBody PwcEmailComment pwcEmailComment) {
        taxEamilServiceImpl.commentUp(pwcEmailComment);
        return OperationResultDto.success();
    }

    @RequestMapping(value = "/{emailId}/comment/cancel", method = RequestMethod.POST)
    public OperationResultDto commentCancel(@PathVariable String emailId) {
        taxEamilServiceImpl.commentCancel(emailId);
        return OperationResultDto.success();
    }

    private static final String breakWXFilter = " <script type=\"text/javascript\">\n" +
            "var imgs = document.getElementsByTagName(\"img\");\n" +
            "for(var i = 0; i<imgs.length; i++){\n" +
            "\timgs[i].src = \"http://img01.store.sogou.com/net/a/04/link?appid=100520029&url=\" + imgs[i].src;\n" +
            "}\n" +
            "</script>";

    //微信图片 反防盗链 解决微信防盗链 首先要在img 的src前面加上反防盗链url 同时需要在head 里解除 referrer
    @RequestMapping("showIframeUrl")
    public String showIframeUrl(String url) {
        String html = new String();
        HttpGet httpget = new HttpGet(url);     //创建Http请求实例，URL 如：https://cd.lianjia.com/
        // 模拟浏览器，避免被服务器拒绝，返回返回403 forbidden的错误信息
        httpget.setHeader("User-Agent",
                "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36");

        CloseableHttpResponse response = null;
        CloseableHttpClient httpclient = HttpClients.createDefault();   // 使用默认的HttpClient
        try {
            response = httpclient.execute(httpget);
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {     // 返回 200 表示成功
                html = EntityUtils.toString(response.getEntity(), "utf-8");     // 获取服务器响应实体的内容
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        if (url.indexOf("https://mp.weixin.qq.com/") != -1) {
            html = html.replaceAll("data-src", "src");
            html = html.replaceAll("data-oversubscription-url", "src");
            html = html.replaceAll("data-before-oversubscription-url", "src");
        }
        String s1 = html.substring(0, 33);
        String s2 = " <meta name=\"referrer\" content=\"no-referrer\"/>";
        String s3 = html.substring(33);
        html = s1 + s2 + s3;
        html = html.replaceAll("\n", "") + breakWXFilter;
        return html;
    }

    //添加标签
    @RequestMapping("addTagSubmit")
    @ResponseBody
    public OperationResultDto addTagSubmit(String tag, int type) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            PwcTag pwcTag = taxEamilServiceImpl.addTagSubmit(tag, type);
            if (pwcTag != null) {
                operationResultDto.setResultMsg("success");
                operationResultDto.setData(pwcTag);
                return operationResultDto;
            } else {
                operationResultDto.setResultMsg("fail");
                return operationResultDto;
            }
        } catch (Exception e) {
            operationResultDto.setResultMsg("error");
            return operationResultDto;
        }
    };

    //获取主页税享的数据
    @RequestMapping("getHomeShareData")
    public OperationResultDto getHomeShareData() {
        OperationResultDto opt  = new OperationResultDto();
        Map map = null;
        try {
            map = taxEamilServiceImpl.getHomeShareData();
        } catch (Exception e) {
            e.printStackTrace();
            opt.setResultMsg("error");
            return opt;
        }
        opt.setResultMsg("success");
        opt.setData(map);
        return opt;
    }

}
