package pwc.taxtech.atms.pwc.websocket;

import pwc.taxtech.atms.common.util.BaseThreadFactory;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.dao.InteractionInteractiveSystemMapper;
import pwc.taxtech.atms.entity.InteractionInteractiveSystem;
import pwc.taxtech.atms.entity.InteractionInteractiveSystemExample;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @version 1.0
 * @program: atms
 * @description:
 * @author: Kevin
 * @create: 2019-05-24 14:12
 **/
@ServerEndpoint(value = "/websocket/{username}/{type}/{id}", encoders = {
        WebSocketEncoder.class })
public class WebSocket {
    private String interactivId;
    private String type;
    //用来存放每个客户端对应的MyWebSocket对象。
    private  static CopyOnWriteArraySet<WebSocket> webSocketSet = new CopyOnWriteArraySet<WebSocket>();
    private  static  javax.websocket.Session session = null;

    private  static  int corePoolSize = 15;

    private static  int maxPoolSize = 15;

    private  static  int qSize = 5000;

    private static  BaseThreadFactory factory = new BaseThreadFactory("websocket");
    private InteractionInteractiveSystemMapper interactionInteractiveSystemMapper = SpringContextUtil.getBean(InteractionInteractiveSystemMapper.class);
    private static  ExecutorService executorService = new ThreadPoolExecutor(corePoolSize, maxPoolSize,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(qSize), factory, new ThreadPoolExecutor.AbortPolicy());
    private Map ResponMap = new HashMap();//用来回传返回值
    public WebSocket() {
    }
    public WebSocket(String interactivId, String type) {
        this.interactivId = interactivId;
        this.type = type;
    }
    public WebSocket(String interactivId, String type,Map map) {
        this.interactivId = interactivId;
        this.type = type;
        this.ResponMap = map;
    }

    /*线程*/
    MyThread thread1 = new MyThread();
    Thread thread = new Thread(thread1);
    TaskInfomationThread thread2 = new TaskInfomationThread();
    Thread _thread2 = new Thread(thread2);
    /*对应业务code*/
    private static final String checkStatus = "checkStatus";
    private static final String pushTaskInfo = "pushTaskInfo";


    /**
     * @ClassName: onOpen
     * @Description: 开启连接的操作
     */
    @OnOpen
    public void onOpen(@PathParam("username") String username, @PathParam("id") String id, Session session) throws IOException {
        this.session = session;
        this.interactivId = id;
        webSocketSet.add(this);
        //设置线程属性
        thread1.setInterActiveId(interactivId);
        thread2.setInterActiveId(interactivId);
        //开启线程对数据库中的数据进行轮询
        executorService.submit(thread);
        executorService.submit(_thread2);
     /*   thread.start();
        _thread2.start();*/
    }

    /**
     * @ClassName: onClose
     * @Description: 连接关闭的操作
     */
    @OnClose
    public void onClose() {
        thread1.stopMe();
        thread2.stopMe();
        webSocketSet.remove(this);
        endStatus();
    }

    private void endStatus (){
        InteractionInteractiveSystem interactionInteractiveSystem = new InteractionInteractiveSystem();
        interactionInteractiveSystem.setId(this.interactivId);
        interactionInteractiveSystem.setDuring(0);
        interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem);
    }
    /**
     * @ClassName: onMessage
     * @Description: 给服务器发送消息告知数据库发生变化
     */
    @OnMessage
    public void onMessage(int status) {
        Map map = new HashMap<>();
        try {
            if (type.equals(checkStatus)) {
                map.put("checkStatus", status);
                sendMessage(map);
            } else if (type.equals(pushTaskInfo) && status == 1) {
                map.put("pushTaskInfo", ResponMap.get("pushTaskInfo"));
                sendMessage(map);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @ClassName: OnError
     * @Description: 出错的操作
     */
    @OnError
    public void onError(Throwable error) {
        System.out.println(error);
        error.printStackTrace();
    }
    /**
     * 这个方法与上面几个方法不一样。没有用注解，是根据自己需要添加的方法。
     *
     * @throws IOException 发送自定义信号，“1”表示告诉前台，数据库发生改变了，需要刷新
     */
    public void sendMessage(Object obj) throws IOException {
        for (WebSocket item : webSocketSet) {
            item.session.getAsyncRemote().sendObject(obj);
        }

    }
}
