package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.analysis.dao.*;
import pwc.taxtech.atms.analysis.entity.*;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.TemplateHeaderCheck;
import pwc.taxtech.atms.constant.enums.EnumAnalysisExpTempPath;
import pwc.taxtech.atms.constant.enums.EnumAnalysisImportType;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.analysis.*;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Auther: Gary J Li
 * @Date: 14/03/2019 11:31
 * @Description:
 */
@Service
public class AnalysisServiceImpl extends BaseService {

    @Resource
    private AnalysisDriverNumMapper analysisDriverNumMapper;
    @Resource
    private AnalysisEmployeeNumMapper analysisEmployeeNumMapper;
    @Resource
    private AnalysisExpectedTaxReturnMapper analysisExpectedTaxReturnMapper;
    @Resource
    private AnalysisActualTaxReturnMapper analysisActualTaxReturnMapper;
    @Resource
    private AnalysisFeeMapper analysisFeeMapper;
    @Resource
    private AnalysisFileManagementMapper analysisFileManagementMapper;
    @Resource
    private AnalysisGmvSubsidyMapper analysisGmvSubsidyMapper;
    @Resource
    private AnalysisInternationalBusinessDataMapper analysisInternationalBusinessDataMapper;
    @Resource
    private AnalysisInternationalTaxDataMapper analysisInternationalTaxDataMapper;
    @Resource
    private AnalysisMasterMapper analysisMasterMapper;
    @Resource
    private AnalysisSalesMapper analysisSalesMapper;
    @Resource
    private AnalysisTaxMapper analysisTaxMapper;
    @Resource
    private OrganizationMapper organizationMapper;

    public List<Object> displayAnalysisImportData(AnalysisDomesticlParam param) {
        List<Object> objects = new ArrayList<>();
        Integer period = DateUtils.strToPeriod(param.getPeriod());
        switch (param.getType()) {
            case 0:
                AnalysisTaxExample analysisTaxExample = new AnalysisTaxExample();
                analysisTaxExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisTax> list = analysisTaxMapper.selectByExample(analysisTaxExample);
                List<AnalysisTaxDto> dtos = Lists.newArrayList();
                Map<String, List<AnalysisTax>> actrs =
                        list.stream().collect(Collectors.groupingBy(AnalysisTax::getCompanyName));
                actrs.forEach((k, v) -> processAts(k, v, dtos));
                objects.addAll(dtos);
                break;
            case 1:
                AnalysisActualTaxReturnExample analysisActualTaxReturnExample = new AnalysisActualTaxReturnExample();
                analysisActualTaxReturnExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisActualTaxReturn> list2 = analysisActualTaxReturnMapper.selectByExample(analysisActualTaxReturnExample);
                List<AnalysisActualTaxReturnDto> dtos2 = Lists.newArrayList();
                Map<String, List<AnalysisActualTaxReturn>> actrs2 =
                        list2.stream().collect(Collectors.groupingBy(AnalysisActualTaxReturn::getCompanyName));
                actrs2.forEach((k, v) -> processActrs(k, v, dtos2));
                objects.addAll(dtos2);
                return objects;
            case 2:
                AnalysisGmvSubsidyExample analysisGmvSubsidyExample = new AnalysisGmvSubsidyExample();
                analysisGmvSubsidyExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisGMVSubsidyDto> dtos3 = generalDtos(analysisGmvSubsidyMapper.selectByExample(analysisGmvSubsidyExample));
                objects.addAll(dtos3);
                break;
            case 3:
                AnalysisEmployeeNumExample analysisEmployeeNumExample = new AnalysisEmployeeNumExample();
                analysisEmployeeNumExample.createCriteria().andPeriodEqualTo(period);
                objects.addAll(analysisEmployeeNumMapper.selectByExample(analysisEmployeeNumExample));
                break;
            case 4:
                AnalysisDriverNumExample analysisDriverNumExample = new AnalysisDriverNumExample();
                analysisDriverNumExample.createCriteria().andPeriodEqualTo(period);
                List<AnalysisDriverNumDto> dtos4 = processDns(analysisDriverNumMapper.selectByExample(analysisDriverNumExample));
                objects.addAll(dtos4);
                break;
            default:
                break;
        }
        return objects;
    }

    private List<AnalysisGMVSubsidyDto> generalDtos(List<AnalysisGmvSubsidy> list) {
        List<AnalysisGMVSubsidyDto> dtos = Lists.newArrayList();
        NumberFormat nt = NumberFormat.getPercentInstance();
        list.forEach(l -> {
            AnalysisGMVSubsidyDto dto = new AnalysisGMVSubsidyDto();
            dto.setBusinessLine(l.getBusinessLine());
            dto.setSegment1(nt.format(l.getOrderChainRatio().doubleValue()));
            dto.setSegment2(nt.format(l.getGmvChainRatio().doubleValue()));
            dto.setSegment3(nt.format(l.getbSubsidyRate().doubleValue()));
            dto.setSegment4(nt.format(l.getbEndLinkRatio().doubleValue()));
            dto.setSegment5(nt.format(l.getcSubsidyRate().doubleValue()));
            dto.setSegment6(nt.format(l.getcEndLinkRatio().doubleValue()));
            dtos.add(dto);
        });
        return dtos;
    }

    private void processAts(String k, List<AnalysisTax> v, List<AnalysisTaxDto> dtos) {
        AnalysisTaxDto dto = new AnalysisTaxDto();
        dto.setCompanyName(k);
        dto.setSegment1(getAmountByGroup(v, "城建税"));
        dto.setSegment2(getAmountByGroup(v, "教育费附加"));
        dto.setSegment3(getAmountByGroup(v, "地方教育费附加"));
        dto.setSegment4(getAmountByGroup(v, "员工个税"));
        dto.setSegment5(getAmountByGroup(v, "司机个税"));
        dto.setSegment6(getAmountByGroup(v, "印花税"));
        dtos.add(dto);
    }

    private void processActrs(String k, List<AnalysisActualTaxReturn> v, List<AnalysisActualTaxReturnDto> dtos) {
        AnalysisActualTaxReturnDto dto = new AnalysisActualTaxReturnDto();
        dto.setCompanyName(k);
        dto.setSegment1(getTaxAmountByGroup(v, "增值税返还"));
        dto.setSegment2(getTaxAmountByGroup(v, "城建税返还"));
        dto.setSegment3(getTaxAmountByGroup(v, "教育费附加返还"));
        dto.setSegment4(getTaxAmountByGroup(v, "地方教育费附加返还"));
        dto.setSegment5(getTaxAmountByGroup(v, "个人所得税返还"));
        dtos.add(dto);
    }

    private List<AnalysisDriverNumDto> processDns(List<AnalysisDriverNum> list) {
        List<AnalysisDriverNumDto> dtos = Lists.newArrayList();
        list.forEach(l -> {
            AnalysisDriverNumDto dto1 = new AnalysisDriverNumDto();
            dto1.setDriverType("加盟");
            dto1.setDriverNum(l.getJoinInAmount());
            AnalysisDriverNumDto dto2 = new AnalysisDriverNumDto();
            dto2.setDriverType("自营");
            dto2.setDriverNum(l.getSelfSupportAmount());
            AnalysisDriverNumDto dto3 = new AnalysisDriverNumDto();
            dto3.setDriverType("直营");
            dto3.setDriverNum(l.getDirectSaleAmount());
            AnalysisDriverNumDto dto4 = new AnalysisDriverNumDto();
            dto4.setDriverType("对公");
            dto4.setDriverNum(l.getRightPublicAmount());
            dtos.add(dto1);
            dtos.add(dto2);
            dtos.add(dto3);
            dtos.add(dto4);
        });
        return dtos;
    }

    private BigDecimal getTaxAmountByGroup(List<AnalysisActualTaxReturn> v, String group) {
        List<AnalysisActualTaxReturn> filterRes = v.stream().filter(o -> group.equals(o.getTaxReturnGroup())).collect(Collectors.toList());
        BigDecimal res = filterRes.size() > 0 ? filterRes.get(0).getTaxReturnAmount() : BigDecimal.ZERO;
        return res;
    }

    private BigDecimal getAmountByGroup(List<AnalysisTax> v, String group) {
        List<AnalysisTax> filterRes = v.stream().filter(o -> group.equals(o.getTaxGroup())).collect(Collectors.toList());
        BigDecimal res = filterRes.size() > 0 ? filterRes.get(0).getTaxAmount() : BigDecimal.ZERO;
        return res;
    }

    public OperationResultDto importDomesitcExcelFile(MultipartFile file, String periodDate, Integer type) {
        StringBuilder notOrgMatch = new StringBuilder();
        switch (type) {
            case 0:
                notOrgMatch = importAnalysisTaxExcelFile(file, periodDate);
                break;
            case 1:
                notOrgMatch = importAnalysisReturnTaxExcelFile(file, periodDate);
                break;
            case 2:
                importAnalysisGMVSubsidyExcelFile(file, periodDate);
                break;
            case 3:
                notOrgMatch = importAnalysisEmployeeNumExcelFile(file, periodDate);
                break;
            case 4:
                importAnalysisDriverNumExcelFile(file, periodDate);
                break;
            default:
                break;
        }
        return OperationResultDto.success(notOrgMatch);
    }

    public OperationResultDto importInterNationalExcelFile(MultipartFile file, String periodDate, Integer type,
                                                           String companyName, String country) {
        switch (type) {
            case 100:
                importAnalysisInterBuDataExcelFile(file, periodDate, companyName, country);
                break;
            case 101:
                importAnalysisInterTaxDataExcelFile(file, periodDate, companyName, country);
                break;
            default:
                break;
        }
        return OperationResultDto.success();
    }

    private String[] headerArr = null;


    private StringBuilder importAnalysisTaxExcelFile(MultipartFile file, String periodDate) {
        StringBuilder notMatchOrg = new StringBuilder();
        List orgNotList = Lists.newArrayList();
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            //验证表头
            headerArr = TemplateHeaderCheck.gszse;
            ExcelUtil.checkHeader(workbook.getSheetAt(0), headerArr, Exceptions.IMPORT_TEMPLATE_HEADER_ERROR);
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisTax> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    String companyName = getCellStringValue(cell1);
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(companyName);
                    List<Organization> orgs = organizationMapper.selectByExample(example);
                    if (orgs.isEmpty()) {
                        orgNotList.add(j + 1);
                        continue;
                    }
                    for (int k = 1; k < sheet.getRow(0).getLastCellNum(); k++) {
                        AnalysisTax model = getAnalysisTax(selectedPer, sheet, k, j, orgs.get(0));
                        lists.add(model);
                    }
                }
                AnalysisTaxExample example = new AnalysisTaxExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisTaxMapper.deleteByExample(example);
                if (lists.size() == 0)
                    throw new Exception("数据校验失败");
                lists.forEach(l -> {
                    analysisTaxMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            throw new ServiceException(e);
        }
        return notMatchOrg.append(orgNotList + "行, 机构匹配失败");
    }

    private AnalysisTax getAnalysisTax(Integer period, Sheet sheet, int k, int j, Organization org) {
        AnalysisTax model = new AnalysisTax();
        model.setId(idService.nextId());
        model.setOrganizationId(org.getId());
        model.setSeqNo(getSeqNoByPeriod(org.getId(), period));
        model.setPeriod(period);
        model.setCompanyName(org.getName());
        model.setTaxGroup(getCellStringValue(sheet.getRow(0).getCell(k)));
        model.setTaxAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(k)));
        return model;
    }

    public String getSeqNoByPeriod(String orgId, Integer period) {
        AnalysisMasterExample analysisMasterExample = new AnalysisMasterExample();
        analysisMasterExample.createCriteria().andPeriodEqualTo(period).andOrganizationIdEqualTo(orgId);
        List<AnalysisMaster> analysisMasters = analysisMasterMapper.selectByExample(analysisMasterExample);
        if (analysisMasters.size() != 0)
            return analysisMasters.get(0).getSeqNo();
        return null;
    }

    public Organization getOrgById(String organizationId) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andIdEqualTo(organizationId);
        List<Organization> organizations = organizationMapper.selectByExample(organizationExample);
        if (organizations.size() != 0)
            return organizations.get(0);
        return null;
    }



    public Organization getOrgByCompanyName(String companyName) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andNameEqualTo(companyName);
        List<Organization> organizations = organizationMapper.selectByExample(organizationExample);
        if (organizations.size() != 0)
            return organizations.get(0);
        return null;
    }


    private StringBuilder importAnalysisReturnTaxExcelFile(MultipartFile file, String periodDate) {
        StringBuilder notMatchOrg = new StringBuilder();
        List orgNotList = Lists.newArrayList();
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            //验证表头
            headerArr = TemplateHeaderCheck.sjfhse;
            ExcelUtil.checkHeader(workbook.getSheetAt(0), headerArr, Exceptions.IMPORT_TEMPLATE_HEADER_ERROR);
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisActualTaxReturn> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    String companyName = getCellStringValue(cell1);
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(companyName);
                    List<Organization> orgs = organizationMapper.selectByExample(example);
                    if (orgs.isEmpty()) {
                        orgNotList.add(j + 1);
                        continue;
                    }
                    for (int k = 1; k < sheet.getRow(0).getLastCellNum(); k++) {
                        AnalysisActualTaxReturn model = getAnalysisActualTaxReturn(selectedPer, sheet, k, j, orgs.get(0));
                        lists.add(model);
                    }
                }
                AnalysisActualTaxReturnExample example = new AnalysisActualTaxReturnExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisActualTaxReturnMapper.deleteByExample(example);
                if (lists.size() == 0)
                    throw new Exception("数据校验失败");
                lists.forEach(l -> {

                    analysisActualTaxReturnMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            throw new ServiceException(e);
        }
        return notMatchOrg.append(orgNotList + "行匹配失败");
    }

    private AnalysisActualTaxReturn getAnalysisActualTaxReturn(Integer period, Sheet sheet, int k, int j, Organization org) {
        AnalysisActualTaxReturn model = new AnalysisActualTaxReturn();
        model.setId(idService.nextId());
        model.setOrganizationId(org.getId());
        model.setSeqNo(getSeqNoByPeriod(org.getId(), period));
        model.setPeriod(period);
        model.setCompanyName(org.getName());
        model.setTaxReturnGroup(getCellStringValue(sheet.getRow(0).getCell(k)));
        model.setTaxReturnAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(k)));
        return model;
    }

    private StringBuilder importAnalysisEmployeeNumExcelFile(MultipartFile file, String periodDate) {
        StringBuilder notMatchOrg = new StringBuilder();
        List orgNotList = Lists.newArrayList();
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            ExcelUtil.checkHeader(workbook.getSheetAt(0), TemplateHeaderCheck.zgrs, Exceptions.IMPORT_TEMPLATE_HEADER_ERROR);
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisEmployeeNum> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    AnalysisEmployeeNum model = new AnalysisEmployeeNum();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);

                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }

                    //进行机构验证
                    String companyName = getCellStringValue(sheet.getRow(j).getCell(0));
                    OrganizationExample example1 = new OrganizationExample();
                    example1.createCriteria().andNameEqualTo(companyName);
                    List<Organization> orgs = organizationMapper.selectByExample(example1);
                    if (orgs.isEmpty()) {
                        orgNotList.add(j + 1);
                        continue;
                    }
                    model.setCompanyName(companyName);
                    try {
                        model.setSeqNo(getSeqNoByPeriod(getOrgByCompanyName(getCellStringValue(sheet.getRow(j).getCell(0))).getId(), selectedPer));
                    } catch (Exception e) {
                        logger.warn(getCellStringValue(sheet.getRow(j).getCell(0)) + "公司名称匹配不成功");
                        continue;
                    }
                    model.setFullTimeAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setInternAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setVendorAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setTotalAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    //通过公司名称查询orgId
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(getCellStringValue(sheet.getRow(j).getCell(0)));
                /*    if(organizationMapper.selectByExample(example).size() == 0)
                        continue;*/
                    try {
                        model.setSeqNo(getSeqNoByPeriod(organizationMapper.selectByExample(example).get(0).getId(), DateUtils.strToPeriod(periodDate)));
                    } catch (Exception e) {
                        System.out.println(getCellStringValue(sheet.getRow(j).getCell(0)));
                    }
                    lists.add(model);
                }
                AnalysisEmployeeNumExample example = new AnalysisEmployeeNumExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisEmployeeNumMapper.deleteByExample(example);
                if (lists.size() == 0)
                    throw new Exception("数据格式不正确");
                lists.forEach(l -> {
                    analysisEmployeeNumMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(e);
        }
        return notMatchOrg.append(orgNotList + "行匹配失败");
    }

    private void importAnalysisGMVSubsidyExcelFile(MultipartFile file, String periodDate) {

        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            //验证表头
            headerArr = TemplateHeaderCheck.ywxgmv;
            ExcelUtil.checkHeader(workbook.getSheetAt(0), headerArr, Exceptions.IMPORT_TEMPLATE_HEADER_ERROR);
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisGmvSubsidy> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    AnalysisGmvSubsidy model = new AnalysisGmvSubsidy();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    model.setBusinessLine(getCellStringValue(cell1));
                    model.setOrderChainRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setGmvChainRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setbSubsidyRate(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setbEndLinkRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    model.setcSubsidyRate(getCellBigDecimalValue(sheet.getRow(j).getCell(5)));
                    model.setcEndLinkRatio(getCellBigDecimalValue(sheet.getRow(j).getCell(6)));
                    lists.add(model);
                }
                AnalysisGmvSubsidyExample example = new AnalysisGmvSubsidyExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisGmvSubsidyMapper.deleteByExample(example);
                lists.forEach(l -> {
                    analysisGmvSubsidyMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            throw new ServiceException(e);
        }
    }

    private void importAnalysisDriverNumExcelFile(MultipartFile file, String periodDate) {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            ExcelUtil.checkHeader(workbook.getSheetAt(0), TemplateHeaderCheck.sjrs, Exceptions.IMPORT_TEMPLATE_HEADER_ERROR);
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                AnalysisDriverNum model = new AnalysisDriverNum();
                model.setId(idService.nextId());
                model.setPeriod(selectedPer);
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    Cell cell2 = sheet.getRow(j).getCell(1);
                    if ("加盟".equals(getCellStringValue(cell1))) {
                        model.setJoinInAmount(getCellBigDecimalValue(cell2));
                    } else if ("自营".equals(getCellStringValue(cell1))) {
                        model.setSelfSupportAmount(getCellBigDecimalValue(cell2));
                    } else if ("直营".equals(getCellStringValue(cell1))) {
                        model.setDirectSaleAmount(getCellBigDecimalValue(cell2));
                    } else if ("对公".equals(getCellStringValue(cell1))) {
                        model.setRightPublicAmount(getCellBigDecimalValue(cell2));
                    }
                }
                AnalysisMasterExample example2 = new AnalysisMasterExample();
                //todo 这里先写死滴滴出行科技有限公司
                example2.createCriteria().andNameEqualTo("滴滴出行科技有限公司");
                List<AnalysisMaster> size = analysisMasterMapper.selectByExample(example2);
                if (size.size() != 0)
                    model.setSeqNo(size.get(0).getSeqNo());
                AnalysisDriverNumExample example = new AnalysisDriverNumExample();
                example.createCriteria().andPeriodEqualTo(selectedPer);
                analysisDriverNumMapper.deleteByExample(example);
                analysisDriverNumMapper.insertSelective(model);
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            throw new ServiceException(e);
        }
    }

    private void importAnalysisInterTaxDataExcelFile(MultipartFile file, String periodDate,
                                                     String organizationId, String country) {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisInternationalTaxData> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    AnalysisInternationalTaxData model = new AnalysisInternationalTaxData();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    model.setOrganizationId(organizationId);
                    model.setCompanyName(getOrgById(organizationId).getName());
                    model.setCountry(country);

                    try {
                        model.setSeqNo(getSeqNoByPeriod(organizationId, DateUtils.strToPeriod(periodDate)));
                    } catch (Exception e) {
                        continue;
                    }
                    model.setTaxCategory(getCellStringValue(sheet.getRow(j).getCell(0)));
                    model.setTaxType(getCellStringValue(sheet.getRow(j).getCell(1)));
                    model.setTaxAmount(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    lists.add(model);
                }
                AnalysisInternationalTaxDataExample example = new AnalysisInternationalTaxDataExample();
                example.createCriteria().andPeriodEqualTo(selectedPer).andCountryEqualTo(country).andOrganizationIdEqualTo(organizationId);
                analysisInternationalTaxDataMapper.deleteByExample(example);
                lists.forEach(l -> {
                    analysisInternationalTaxDataMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            String errMsg = "分析模块-导入国际税务数据异常";
            logger.error(errMsg, e);
            throw new ServiceException(errMsg);
        }
    }

    private void importAnalysisInterBuDataExcelFile(MultipartFile file, String periodDate,
                                                    String organizationId, String country) {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            // 文件上的期间
            Integer selectedPer = DateUtils.strToPeriod(periodDate);
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AnalysisInternationalBusinessData> lists = Lists.newArrayList();
                for (int j = 1; j <= sheet.getLastRowNum(); j++) {
                    if (j > 1)
                        break;
                    AnalysisInternationalBusinessData model = new AnalysisInternationalBusinessData();
                    model.setId(idService.nextId());
                    model.setPeriod(selectedPer);
                    model.setOrganizationId(organizationId);
                    model.setCountry(country);
                    Cell cell1 = sheet.getRow(j).getCell(0);
                    try {
                        model.setSeqNo(getSeqNoByPeriod(organizationId, DateUtils.strToPeriod(periodDate)));
                    } catch (Exception e) {
                        continue;
                    }
                    model.setGmv(getCellBigDecimalValue(cell1));
                    model.setCompanyName(getOrgById(organizationId).getName());
                    model.setTrips(getCellBigDecimalValue(sheet.getRow(j).getCell(1)));
                    model.setSubsidyB(getCellBigDecimalValue(sheet.getRow(j).getCell(2)));
                    model.setSubsidyC(getCellBigDecimalValue(sheet.getRow(j).getCell(3)));
                    model.setRevenue(getCellBigDecimalValue(sheet.getRow(j).getCell(4)));
                    model.setProfit(getCellBigDecimalValue(sheet.getRow(j).getCell(5)));
                    model.setRef1(getCellBigDecimalValue(sheet.getRow(j).getCell(6)));
                    model.setRef2(getCellBigDecimalValue(sheet.getRow(j).getCell(7)));
                    model.setRef3(getCellBigDecimalValue(sheet.getRow(j).getCell(8)));
                    model.setRef4(getCellBigDecimalValue(sheet.getRow(j).getCell(9)));
                    model.setRef5(getCellBigDecimalValue(sheet.getRow(j).getCell(10)));
                    model.setRef6(getCellBigDecimalValue(sheet.getRow(j).getCell(11)));
                    lists.add(model);
                }
                AnalysisInternationalBusinessDataExample example = new AnalysisInternationalBusinessDataExample();
                example.createCriteria().andPeriodEqualTo(selectedPer).andCountryEqualTo(country).andOrganizationIdEqualTo(organizationId);
                analysisInternationalBusinessDataMapper.deleteByExample(example);
                lists.forEach(l -> {
                    analysisInternationalBusinessDataMapper.insertSelective(l);
                });
            }
        } catch (ServiceException se) {
            throw se;
        } catch (Exception e) {
            String errMsg = "分析模块-导入国际业务数据异常";
            logger.error(errMsg, e);
            throw new ServiceException(errMsg);
        }
    }

    public List<Object> displayAnalysisInternationalImportData(AnalysisInternationlParam param) {
        List<Object> objects = new ArrayList<>();
        Integer period = DateUtils.strToPeriod(param.getPeriod());
        String country = param.getCountry();
        String organizationId = param.getCompanyName();
        switch (param.getType()) {
            case 100:
                AnalysisInternationalBusinessDataExample analysisInternationalBusinessDataExample = new AnalysisInternationalBusinessDataExample();
                analysisInternationalBusinessDataExample.createCriteria().andPeriodEqualTo(period).andOrganizationIdEqualTo(organizationId).andCountryEqualTo(country);
                List<AnalysisInternationalBusinessData> list = analysisInternationalBusinessDataMapper.selectByExample(analysisInternationalBusinessDataExample);

                /*List<AnalysisInternationalBUDataDto> dtos = Lists.newArrayList();
                list.forEach(l->{
                    AnalysisInternationalBUDataDto dto1 = new AnalysisInternationalBUDataDto();
                    dto1.setCategory("GMV");
                    dto1.setName("GMV");
                    dto1.setAmount(l.getGmv());
                    dtos.add(dto1);
                    AnalysisInternationalBUDataDto dto2 = new AnalysisInternationalBUDataDto();
                    dto2.setCategory("Revenue");
                    dto2.setName("Revenue");
                    dto2.setAmount(l.getRevenue());
                    dtos.add(dto2);
                    AnalysisInternationalBUDataDto dto3 = new AnalysisInternationalBUDataDto();
                    dto3.setCategory("Profit");
                    dto3.setName("Profit");
                    dto3.setAmount(l.getProfit());
                    dtos.add(dto3);
                    AnalysisInternationalBUDataDto dto4 = new AnalysisInternationalBUDataDto();
                    dto4.setCategory("Trips");
                    dto4.setName("Trips");
                    dto4.setAmount(l.getTrips());
                    dtos.add(dto4);
                    AnalysisInternationalBUDataDto dto5 = new AnalysisInternationalBUDataDto();
                    dto5.setCategory("Subsidy");
                    dto5.setName("Subsidy B");
                    dto5.setAmount(l.getSubsidyB());
                    dtos.add(dto5);
                    AnalysisInternationalBUDataDto dto6 = new AnalysisInternationalBUDataDto();
                    dto6.setCategory("Subsidy");
                    dto6.setName("Subsidy C");
                    dto6.setAmount(l.getSubsidyC());
                    dtos.add(dto6);
                });*/
                objects.addAll(list);
                return objects;
            case 101:
                AnalysisInternationalTaxDataExample analysisInternationalTaxDataExample = new AnalysisInternationalTaxDataExample();
                analysisInternationalTaxDataExample.createCriteria().andPeriodEqualTo(period).andOrganizationIdEqualTo(organizationId).andCountryEqualTo(country);
                List<AnalysisInternationalTaxData> list2 = analysisInternationalTaxDataMapper.selectByExample(analysisInternationalTaxDataExample);
                List<AnalysisInternationalTaxDataDto> dtos2 = Lists.newArrayList();
                list2.forEach(l2 -> {
                    dtos2.add(beanUtil.copyProperties(l2, new AnalysisInternationalTaxDataDto()));
                });
                objects.addAll(dtos2);
                return objects;
            default:
                break;
        }
        return objects;
    }

    public HttpServletResponse downloadDomesticFile(HttpServletResponse response, AnalysisDomesticlParam param, String fileName) {
        String excelTemplatePathInClassPath = EnumAnalysisExpTempPath.getPath(param.getType());
        List<Object> datas = displayAnalysisImportData(param);
        try {
            if (datas.size() < 1) {
                throw new Exception("无可导出的数据");
            }
            OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(datas, excelTemplatePathInClassPath);
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出国内税数据异常:%s", e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse downloadInternationalFile(HttpServletResponse response, AnalysisInternationlParam param, String fileName) {
        String excelTemplatePathInClassPath = EnumAnalysisExpTempPath.getPath(param.getType());
        List<Object> datas = Lists.newArrayList();
        if (param.getType().equals(EnumAnalysisImportType.InternationalBuData.getCode())) {
            Integer period = DateUtils.strToPeriod(param.getPeriod());
            String country = param.getCountry();
            String companyName = param.getCompanyName();
            AnalysisInternationalBusinessDataExample analysisInternationalBusinessDataExample = new AnalysisInternationalBusinessDataExample();
            analysisInternationalBusinessDataExample.createCriteria().andPeriodEqualTo(period).andCompanyNameEqualTo(companyName).andCountryEqualTo(country);
            List<AnalysisInternationalBusinessData> list = analysisInternationalBusinessDataMapper.selectByExample(analysisInternationalBusinessDataExample);
            datas.addAll(list);
        } else {
            datas = displayAnalysisInternationalImportData(param);
        }
        if (datas.size() < 1) {
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(datas, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出国际税数据异常:%s", e.getMessage()));
        }
        return null;
    }

    public List<String> getAnalysisInternationalCompanyList(Integer type, String periodStr) {
        List<String> companyList = Lists.newArrayList();
        Integer period = DateUtils.strToPeriod(periodStr);
        switch (type) {
            case 100:
                companyList = analysisInternationalBusinessDataMapper.selectCompanyList(period);
                break;
            case 101:
                companyList = analysisInternationalTaxDataMapper.selectCompanyList(period);
                break;
            default:
                break;
        }
        return companyList;
    }

    public List<String> getAnalysisInternationalCountryList(Integer type, String periodStr) {
        List<String> countryList = Lists.newArrayList();
        Integer period = DateUtils.strToPeriod(periodStr);
        switch (type) {
            case 100:
                countryList = analysisInternationalBusinessDataMapper.selectCountryList(period);
                break;
            case 101:
                countryList = analysisInternationalTaxDataMapper.selectCountryList(period);
                break;
            default:
                break;
        }
        return countryList;
    }

    private String getCellStringValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return cell.getStringCellValue();
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            // 取整
            return String.valueOf((int) cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Boolean getCellBolleanValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            switch (cell.getStringCellValue()) {
                case "TRUE":
                    return Boolean.TRUE;
                case "FALSE":
                    return Boolean.FALSE;
                case "是":
                    return Boolean.TRUE;
                case "否":
                    return Boolean.FALSE;
                default:
                    return null;
            }
        } else if (cell.getCellTypeEnum().equals(CellType.BOOLEAN)) {
            return cell.getBooleanCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Integer getCellIntegerValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Integer.valueOf(cell.getStringCellValue());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return (int) cell.getNumericCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private BigDecimal getCellBigDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            String replace = cell.getStringCellValue().replace("%", "");
            return new BigDecimal(Double.parseDouble(replace) / 100);
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new BigDecimal(cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Long getCellLongDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Long.valueOf(cell.getStringCellValue());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new Double(cell.getNumericCellValue()).longValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (sheet.getLastRowNum() > 0 &&
                (null == sheet.getRow(0).getCell(0) || "Version".equals(sheet.getRow(0).getCell(0).getStringCellValue())) &&
                null == sheet.getRow(0).getCell(2)) {
            return true;
        }
        return false;
    }

    /*分析模块数据初始化*/
    private static List<Integer> SALE_LIST = Lists.newArrayList(10, 11, 12, 13, 14, 16, 18, 19, 32);
    private static List<Integer> TAX_LIST = Lists.newArrayList(38, 39, 40, 41, 42, 46, 47, 75, 76, 77, 78, 80);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    //分析模块数据初始化, 使用java命令直接运行该类的class文件
    public OperationResultDto analysisDataInit(Integer type) {
        OperationResultDto operationResultDto = new OperationResultDto();
        Long now = System.currentTimeMillis();
        Integer periodNow = DateUtils.getPeriodNow();
        OrganizationExample e = new OrganizationExample();

        if (type == 0) {
            String sql = "truncate  table analysis_sales; truncate  table analysis_tax";
            jdbcTemplate.update(sql);
        }

        //e.createCriteria().andIsActiveEqualTo(true);
        List<Organization> orgs = organizationMapper.selectByExample(e);
        //首先要先初始化master表数据，所有的数据要依赖master表
        analysisJobService.analysisMaster(orgs, periodNow, EnumTbImportType.Init.getCode());
        logger.info("初始化analysisMaster完成 ");

        analysisJobService.analysisTax(orgs, periodNow, EnumTbImportType.Init.getCode());
        logger.info("初始化analysisTax完成 ");

        analysisJobService.analysisFileManagement(orgs, periodNow, EnumTbImportType.Init.getCode());
        logger.info("初始化analysisFileManagement完成 ");

        analysisJobService.analysisSales(orgs, periodNow, EnumTbImportType.Init.getCode());
        logger.info("初始化analysisSales完成 ");
        analysisJobService.analysisFee(orgs, periodNow, EnumTbImportType.Init.getCode());
        logger.info("初始化analysisFee完成 ");

        //这里要手动提交事务
        TransactionAspectSupport.currentTransactionStatus().flush();

        InputStream resourceAsStream = AnalysisServiceImpl.class.getClassLoader().getResourceAsStream("analysis_excel_init/analysis_init_data.xlsx");
        if (resourceAsStream == null) {
            logger.warn("获取分析数据初始化文件失败");
            operationResultDto.error();
            return operationResultDto;
        }
        Workbook workbook = null;
        try {
            workbook = WorkbookFactory.create(resourceAsStream);
        } catch (Exception error) {
            error.printStackTrace();
        }
        AnalysisMaster analysisMaster = null;
        AnalysisSales sales = null;
        AnalysisTax tax = null;
        Sheet sheetAt = workbook.getSheetAt(0);
        List<AnalysisTax> taxList = new ArrayList<>();
        List<AnalysisSales> salesList = new ArrayList<>();
        int breakFlag = 0;
        try {
            for (int i = 5; i < sheetAt.getLastRowNum(); i++) {
                if (sheetAt.getRow(i).getCell(1) == null) {
                    break;//解决excel无限行的问题
                }
                AnalysisMasterExample example = new AnalysisMasterExample();
                System.out.println(getCellValue(sheetAt.getRow(i).getCell(1), false).toString());
                Integer period = Integer.parseInt(getCellValue(sheetAt.getRow(i).getCell(1), false).toString().substring(0, 6));
                example.createCriteria()
                        .andCompanyNameEqualTo(getCellValue(sheetAt.getRow(i).getCell(3), false).toString())
                        .andPeriodEqualTo(period);
                List<AnalysisMaster> analysisMasters = analysisMasterMapper.selectByExample(example);
                if (breakFlag > 20) {
                    logger.info("大于三条数据不匹配,直接停止进程");
                    break;
                }
                if (analysisMasters.size() == 0) {
                    logger.warn("公司名称不匹配");
                    breakFlag++;
                    continue;
                }
                analysisMaster = analysisMasters.get(0);
                sales = new AnalysisSales();
                for (int j = 0; j < sheetAt.getRow(0).getLastCellNum(); j++) {
                    Cell cell = sheetAt.getRow(i).getCell(j);
                    if (SALE_LIST.contains(j)) {
                        insertAnalysisSaleData(period, sales, cell, j, analysisMaster);
                    } else if (TAX_LIST.contains(j)) {
                        tax = new AnalysisTax();
                        taxData(period,
                                j,
                                tax,
                                analysisMaster,
                                cell,
                                taxList);
                    }
                }
                if (sales.getSeqNo() != null) {
                    sales.setPeriod(period);
                    salesList.add(sales);
                }
            }
            int salesInt = 0;
            if (salesList.size() != 0) {
                salesInt = analysisSalesMapper.insertBatch(salesList);
            }
            int taxInt = 0;
            if (taxList.size() != 0) {
                taxInt = analysisTaxMapper.insertBatch(taxList);
            }
            logger.info("analysis_salesInt 更新数据条数" + salesInt);
            logger.info("analysis_tax 更新数据条数" + taxInt);
            System.out.println("数据初始化消耗: " + (System.currentTimeMillis() - now));
            operationResultDto.setResultMsg(String.valueOf(System.currentTimeMillis() - now));
        } catch (Exception exx) {
            exx.printStackTrace();
            logger.warn("插入失败");
            operationResultDto.setResult(false);
            operationResultDto.setResultMsg("插入失败");

        }
        return operationResultDto;
    }

    @Autowired
    protected DistributedIdService idService;

    public void taxData(Integer period, Integer j, AnalysisTax tax, AnalysisMaster analysisMaster, Cell cell, List<AnalysisTax> taxList) {
        seqNo = analysisMaster.getSeqNo();
        conpanyName = analysisMaster.getCompanyName();
        organizationId = analysisMaster.getOrganizationId();
        tax.setSeqNo(seqNo);
        tax.setTaxAmount((BigDecimal) getCellValue(cell, true));
        tax.setCompanyName(conpanyName);
        tax.setOrganizationId(organizationId);
        tax.setCreateTime(new Date());
        tax.setUpdateTime(new Date());
        tax.setId(idService.nextId());
        tax.setPeriod(period);
        switch (j) {
            case 38:
                tax.setTaxGroup("企业所得税");
                break;
            case 39:
                tax.setTaxGroup("增值税");
                break;
            case 40:
                tax.setTaxGroup("城建税");
                break;
            case 41:
                tax.setTaxGroup("教育费附加");
                break;
            case 42:
                tax.setTaxGroup("地方教育费附加");
                break;

            case 46:
                tax.setTaxGroup("员工个税");
                break;
            case 47:
                tax.setTaxGroup("司机个税");
                break;
            case 75:
                tax.setTaxGroup("返还后增值税");
                break;

            case 76:
                tax.setTaxGroup("返还后城建税");
                break;


            case 77:
                tax.setTaxGroup("返还后教育费附加");
                break;

            case 78:
                tax.setTaxGroup("返还后地方教育费附加");
                break;
            case 80:
                tax.setTaxGroup("返还后个人所得税");
                break;
        }
        taxList.add(tax);
        tax = null;
        seqNo = null;
        conpanyName = null;
        organizationId = null;
    }

    public void saleData(AnalysisSales sales, String seqNo, String conpanyName, String organizationId, Cell cell) {
        sales.setSeqNo(seqNo);
        sales.setCompanyName(conpanyName);
        sales.setOrganizationId(organizationId);
        sales.setCreateTime(new Date());
        sales.setUpdateTime(new Date());
        sales.setId(idService.nextId());
    }

    private String seqNo = null;
    private String conpanyName = null;
    private String organizationId = null;

    public void insertAnalysisSaleData(Integer period, AnalysisSales sale, Cell cell, Integer col, AnalysisMaster analysisMaster) {
        seqNo = analysisMaster.getSeqNo();
        conpanyName = analysisMaster.getCompanyName();
        organizationId = analysisMaster.getOrganizationId();
        switch (col) {
            //10, 11, 12, 13, 14, 16, 18, 19, 32
            case 10:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment1((BigDecimal) getCellValue(cell, true));
                break;
            case 11:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment2((BigDecimal) getCellValue(cell, true));
                break;
            case 12:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment3((BigDecimal) getCellValue(cell, true));
                break;
            case 13:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment5((BigDecimal) getCellValue(cell, true));
            case 14:

                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment6((BigDecimal) getCellValue(cell, true));
                break;
            case 16:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment9((BigDecimal) getCellValue(cell, true));
                break;
            case 18:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment15((BigDecimal) getCellValue(cell, true));
                break;
            case 19:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment16((BigDecimal) getCellValue(cell, true));
                break;
            case 32:
                saleData(sale, seqNo, conpanyName, organizationId, cell);
                sale.setSegment17((BigDecimal) getCellValue(cell, true));
                break;
        }
    }

    /**
     * @param cell 获取值的单元格
     * @param zero 是否将空 或者null转换成  zero
     * @return
     */
    public static Object getCellValue(Cell cell, Boolean zero) {
        String cellValue = null;
        if (cell == null)
            return BigDecimal.ZERO;
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_STRING://字符串类型
                cellValue = cell.getStringCellValue();
                if (cellValue.trim().equals("") || cellValue.trim().length() <= 0)
                    cellValue = "";
                break;
            case Cell.CELL_TYPE_NUMERIC: //数值类型
                cellValue = String.valueOf(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_FORMULA: //公式
                cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
                cellValue = String.valueOf(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_BLANK:
                cellValue = "";
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                break;
            case Cell.CELL_TYPE_ERROR:
                break;
            default:
                break;
        }
        if ("".equals(cellValue) && zero)
            return BigDecimal.ZERO;
        if (zero) {
            try {
                return new BigDecimal(cellValue);
            } catch (Exception e) {
                System.out.println("获取Cell，在值转换成数字的地方出错");
                return BigDecimal.ZERO;
            }
        }
        return cellValue;
    }

    @Autowired
    private AnalysisJobServiceImpl analysisJobService;

    public OperationResultDto handleAnalysisData(Integer type) {

        OperationResultDto operationResultDto = new OperationResultDto();
        Long now = System.currentTimeMillis();
        try {
            if (type == 0) {
                String sql = "TRUNCATE  TABLE analysis_tax_return_end;\n" +
                        "TRUNCATE  TABLE analysis_employee_num;\n" +
                        "TRUNCATE  TABLE analysis_actual_tax_return;\n" +
                        "TRUNCATE  TABLE analysis_fee;\n" +
                        "TRUNCATE  TABLE analysis_expected_tax_return;\n" +
                        "TRUNCATE  TABLE analysis_file_management;\n" +
                        "TRUNCATE  TABLE analysis_international_tax_data;\n" +
                        "TRUNCATE  TABLE analysis_sales;\n" +
                        "TRUNCATE  TABLE analysis_tax;\n" +
                        "TRUNCATE  TABLE analysis_master;";
                int deleteCount = jdbcTemplate.update(sql);
                logger.info("成功删除数据，删除数量：" + deleteCount);
            }

            Integer period = DateUtils.getPeriodNow();
            OrganizationExample e = new OrganizationExample();
            //e.createCriteria().andIsActiveEqualTo(true);
            List<Organization> orgs = organizationMapper.selectByExample(e);

            logger.info(String.format("开始分析%s机构数据", period));
            analysisJobService.analysisMaster(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s预期返还税数据", period));
            analysisJobService.analysisExpectedTax(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s费用数据", period));
            analysisJobService.analysisFee(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s档案管理数据", period));
            analysisJobService.analysisFileManagement(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s申报表数据", period));
            analysisJobService.analysisSales(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s返还后税数据", period));
            analysisJobService.analysisTaxReturnEnd(orgs, period, EnumTbImportType.CoverImport.getCode());

            logger.info(String.format("开始分析%s税种税金计算数据", period));
            analysisJobService.analysisTax(orgs, period, EnumTbImportType.CoverImport.getCode());
            logger.info(String.format("开始分析%s税金计算表数据",period));
            analysisJobService.analysisTaxCalculation(orgs,type,EnumTbImportType.CoverImport.getCode());


        } catch (Exception ex) {
            ex.printStackTrace();
            operationResultDto.setResult(false);
        }

        operationResultDto.setResultMsg(String.valueOf((System.currentTimeMillis() - now) / 100));
        operationResultDto.setResult(true);
        return operationResultDto;
    }
}
