package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.nutz.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.BaseThreadFactory;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.HttpUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.EbsExtractTypeConstant;
import pwc.taxtech.atms.constant.enums.EnumApiCodeMsg;
import pwc.taxtech.atms.constant.enums.EnumEbsExtractType;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dpo.OrgSelectDto;
import pwc.taxtech.atms.dto.DataExtractParam;
import pwc.taxtech.atms.dto.DataImportLogDto;
import pwc.taxtech.atms.dto.DataValidateLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.dataimport.DataImportAllParam;
import pwc.taxtech.atms.dto.dataimport.DataImportParam;
import pwc.taxtech.atms.dto.dataimport.DataProcessParam;
import pwc.taxtech.atms.dto.ebsdto.EbsCallResp;
import pwc.taxtech.atms.dto.vatdto.DataImportErrorDto;
import pwc.taxtech.atms.dto.vatdto.DataImportResponseDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceParam;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.security.dd.DtsTokenService;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Service
public class DataImportService extends BaseService {

    @Resource
    private GlBalanceMapper glBalanceMapper;
    @Resource
    private OrganizationMapper organizationMapper;
    @Resource
    private StandardAccountMapper standardAccountMapper;
    @Resource
    private EnterpriseAccountSetOrgDao enterpriseAccountSetOrgDao;
    @Resource
    private AccountMappingDao accountMappingDao;
    @Resource
    private DataImportLogMapper dataImportLogMapper;
    @Resource
    private DataValidateLogMapper dataValidateLogMapper;
    @Resource
    private BalanceSheetPrcManualMapper balanceSheetPrcManualMapper;
    @Resource
    private ProfitLossStatementPrcManualMapper profitLossStatementPrcManualMapper;
    @Resource
    private BalanceSheetPrcFinalMapper balanceSheetPrcFinalMapper;
    @Resource
    private ProfitLossStatementPrcFinalMapper profitLossStatementPrcFinalMapper;
    @Resource
    private RedLetterInfoTableMapper redLetterInfoTableMapper;
    @Resource
    private AdjustmentTableMapper adjustmentTableMapper;
    @Resource
    private CashFlowManualMapper cashFlowManualMapper;
    @Resource
    private CashFlowFinalMapper cashFlowFinalMapper;
    @Resource
    private CoupaPurchasingReportMapper coupaPurchasingReportMapper;
    @Resource
    private InvoiceDataMapper invoiceDataMapper;
    @Resource
    private CertifiedInvoicesListMapper certifiedInvoicesListMapper;
    @Resource
    private InvoiceRecordMapper invoiceRecordMapper;
    @Resource
    private ProjectMapper projectMapper;
    @Autowired
    private OrganizationServiceImpl organizationService;
    @Autowired
    private OrganizationAccountingRateMapper organizationAccountingRateMapper;
    @Resource
    private UserMapper userMapper;

    private static String ebsCallUrl;

    @Value("${ebs_call_url}")
    public void setEbsCallUrl(String url) {
        this.ebsCallUrl = url;
    }

    private static int corePoolSize = 15;

    private static int maxPoolSize = 15;

    private static int qSize = 5000;

    private static BaseThreadFactory factory = new BaseThreadFactory("callEbs");

    private static ExecutorService executorService = new ThreadPoolExecutor(corePoolSize, maxPoolSize,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(qSize), factory, new ThreadPoolExecutor.AbortPolicy());

    /**
     * TB表查询
     *
     * @param param TrialBalanceParam
     * @return List<TrialBalanceDto>
     */
    public List<TrialBalanceDto> getBalanceDataForDisplay(TrialBalanceParam param) {
        if (!ObjectUtils.allNotNull(param, param.getPeriodStart(), param.getPeriodEnd()) || StringUtils.isBlank(param.getOrgId())) {
            return Collections.emptyList();
        }
        Organization organization = organizationMapper.selectByPrimaryKey(param.getOrgId());
        if (null == organization) {
            return Collections.emptyList();
        }
        Optional<EnterpriseAccountSetOrg> accountSetOrg = enterpriseAccountSetOrgDao.getByOrganization(organization.getId());
        if (!accountSetOrg.isPresent()) {
            return Collections.emptyList();
        }
        // mapping
        List<AccountMapping> mappingList = accountMappingDao.getAccountMapping(accountSetOrg.get().getEnterpriseAccountSetId(),
                organization.getId());
        List<String> etCodeList = mappingList.stream().map(AccountMapping::getEnterpriseAccountCode).distinct().collect(Collectors.toList());
        Map<String, List<AccountMapping>> mappingMap = mappingList.stream().collect(Collectors.groupingBy(AccountMapping::getStandardAccountCode));
        //standard account
        StandardAccountExample stdExample = new StandardAccountExample();
        stdExample.createCriteria().andIsLeafEqualTo(Boolean.TRUE);
        List<StandardAccount> stdAccountList = standardAccountMapper.selectByExample(stdExample);
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat dateFormat = new SimpleDateFormat(Constant.DateFormat.YEAR_MONTH);
        Map<String, TrialBalanceDto> resultStdMap = new HashMap<>();
        for (int i = param.getPeriodStart(); i <= param.getPeriodEnd(); i++) {
            calendar.set(Calendar.MONTH, i - 1);
            GlBalanceExample example = new GlBalanceExample();
            //科目段
            example.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime())).andSegment3In(etCodeList);
            List<GlBalance> sg3List = glBalanceMapper.selectByExample(example);

            GlBalanceExample example2 = new GlBalanceExample();
            //明细段
            example2.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime())).andSegment4In(etCodeList);
            List<GlBalance> sg4List = glBalanceMapper.selectByExample(example2);

            for (StandardAccount standardAccount : stdAccountList) {
                if (!resultStdMap.containsKey(standardAccount.getCode())) {
                    resultStdMap.put(standardAccount.getCode(), new TrialBalanceDto());
                }
//                if (!mappingMap.containsKey(standardAccount.getCode())) {
//                    continue;
//                }
                TrialBalanceDto dto = resultStdMap.get(standardAccount.getCode());
                dto.setChildren(Collections.emptyList());
                dto.setAcctCode(standardAccount.getCode());
                dto.setAccountName(standardAccount.getName());
                List<AccountMapping> tmp = mappingMap.get(standardAccount.getCode());
                if (CollectionUtils.isEmpty(tmp)) {
                    continue;
                }
                tmp.forEach(o -> {
                    dto.setCreditBal(add(dto.getCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdCr()));
                    dto.setBegCreditBal(add(dto.getBegCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginCrBalance()));
                    dto.setEndCreditBal(add(dto.getEndCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndCrBalance()));
                    dto.setYearCreditBal(add(dto.getYearCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdCr()));
                    dto.setDebitBal(add(dto.getDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdDr()));
                    dto.setBegDebitBal(add(dto.getBegDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginDrBalance()));
                    dto.setEndDebitBal(add(dto.getEndDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndDrBalance()));
                    dto.setYearDebitBal(add(dto.getYearDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdDr()));
                });
            }
        }

        return new ArrayList<>(resultStdMap.values()).stream().sorted(Comparator.comparing(TrialBalanceDto::getAcctCode))
                .collect(Collectors.toList());
    }

    private BigDecimal add(BigDecimal v1, BigDecimal v2) {
        v1 = null == v1 ? BigDecimal.ZERO : v1;
        v2 = null == v2 ? BigDecimal.ZERO : v2;
        return v1.add(v2);
    }

    private GlBalance getGlBalance(String code, List<GlBalance> seg3, List<GlBalance> seg4) {
        for (GlBalance balance : seg3) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }

        for (GlBalance balance : seg4) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }
        return new GlBalance();
    }

    /**
     * 计算总和
     *
     * @param list List<TrialBalanceDto>
     * @return TrialBalanceDto
     */
    public TrialBalanceDto calculateData(List<TrialBalanceDto> list) {
        TrialBalanceDto dto = new TrialBalanceDto();
        list.forEach(o -> {
            dto.setCreditBal(add(dto.getCreditBal(), o.getCreditBal()));
            dto.setBegCreditBal(add(dto.getBegCreditBal(), o.getBegCreditBal()));
            dto.setEndCreditBal(add(dto.getEndCreditBal(), o.getEndCreditBal()));
            dto.setYearCreditBal(add(dto.getYearCreditBal(), o.getYearCreditBal()));

            dto.setDebitBal(add(dto.getDebitBal(), o.getDebitBal()));
            dto.setBegDebitBal(add(dto.getBegDebitBal(), o.getBegDebitBal()));
            dto.setEndDebitBal(add(dto.getEndDebitBal(), o.getEndDebitBal()));
            dto.setYearDebitBal(add(dto.getYearDebitBal(), o.getYearDebitBal()));
        });
        return dto;
    }

    /**
     * 14/02/2019 14:57
     * 导入利润表
     * [file, periodDate]
     *
     * @return remark:1、period为 201812 201813 记录 tms_period 201812
     * 2、可以考虑加index，未来若需预览可保证顺序
     * errorList:
     * 1、存在非选择期间数据
     * 2、未映射到主体
     * @author Gary J Li
     */
    public OperationResultDto importPLExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessage.DIDNOTSELECTPERIOD);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<ProfitLossStatement> pls = Lists.newArrayList();
                String companyName = getCellStringValue(sheet.getRow(4).getCell(1));
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andNameEqualTo(companyName);
                List<Organization> organizations = organizationMapper.selectByExample(example);
                String orgId = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                    companyCode = organizations.get(0).getCode();
                }
                String periodStr = getCellStringValue(sheet.getRow(2).getCell(2)).substring(3);
                String fileType = getCellStringValue(sheet.getRow(0).getCell(2));

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.ProfitLoss.getCode(), period / 100,
                        tmsPeriod % 100, period % 100,
                        companyName, fileType);
                //添加导入类型
                dataImportLog.setImportType(importType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                // 根据机构ID和年份获取卡片
                String projId = getProjId(orgId, tmsPeriod);

                if (StringUtils.isEmpty(projId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                if (!orgIds.contains(orgId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period % 100 == 13 && tmsPeriod % 100 == 12;
                if (!tmsPeriod.equals(period) && !isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                String currencyCode = getCellStringValue(sheet.getRow(5).getCell(0)).replace(" ", "").substring(3);
                String dataDateStr = getCellStringValue(sheet.getRow(0).getCell(4)).substring(3).replace(" ", "");
                SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyyHH:mm:ss");
                Date dataDate = sdf.parse(dataDateStr);

                for (int j = 11; j <= sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    ProfitLossStatement pl = new ProfitLossStatement();
                    pl.setId(plId);
                    pl.setProjectId(projId);
                    pl.setOrganizationId(orgId);
                    pl.setDate(dataDate);
                    pl.setSource("Excel导入");
                    pl.setStatus("人工导入");
                    pl.setTmsPeriod(tmsPeriod);
                    pl.setPeriod(period);
                    pl.setEntityCode(companyCode);
                    pl.setEntityName(companyName);
                    pl.setLedgerCurrencyCode(currencyCode);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    pl.setItemName(getCellStringValue(cell1));

                    Cell cell2 = row.getCell(1);
                    if (cell2.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = getCellStringValue(cell2).replace(",", "");
                        pl.setPeriodAmt(new BigDecimal(value1));
                    } else {
                        pl.setPeriodAmt(new BigDecimal(cell2.getNumericCellValue()));
                    }

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = getCellStringValue(cell3).replace(",", "");
                        pl.setYtdAmt(new BigDecimal(value2));
                    } else {
                        pl.setYtdAmt(new BigDecimal(cell3.getNumericCellValue()));
                    }
                    pls.add(pl);
                }
                if (pls.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        ProfitLossStatementPrcExample delExample = new ProfitLossStatementPrcExample();
                        delExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        profitLossStatementPrcManualMapper.deleteByExample(delExample);
                        if (profitLossStatementPrcFinalMapper.countByExample(delExample) > 0) {
                            profitLossStatementPrcFinalMapper.deleteByExample(delExample);
                        }
                    }
                    profitLossStatementPrcManualMapper.insertBatch(pls);
                    profitLossStatementPrcFinalMapper.insertBatch(pls);
                    dataImportLog.setRecordSize(pls.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            if (!dataImportLogs.stream().filter(dil -> !dil.getImportResult()).collect(Collectors.toList()).isEmpty()) {
                return OperationResultDto.error();
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importPLExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private DataImportLog generalDataImportLog(String companyCode, String orgId, String taxPayerId, Integer type, Integer periodYear,
                                               Integer tmsPeriodMonth, Integer periodMonth, String companyName, String fileType) {
        DataImportLog dataImportLog = new DataImportLog();
        dataImportLog.setId(idService.nextId());
        dataImportLog.setTaxpayerIdNum(taxPayerId);
        dataImportLog.setType(type);
        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setCompanyCode(companyCode);
        dataImportLog.setCompanyName(companyName);
        dataImportLog.setFileType(fileType);
        dataImportLog.setPeriodYear(periodYear);
        dataImportLog.setTmsPeriodMonth(tmsPeriodMonth);
        dataImportLog.setPeriodMonth(periodMonth);
        dataImportLog.setPeriodStatus("人工导入");
        dataImportLog.setOperator(authUserHelper.getCurrentAuditor().get());
        dataImportLog.setImportResult(true);
        return dataImportLog;
    }

    /**
     * 14/02/2019 16:36
     * 导入资产负债表
     * [file, periodDate]
     *
     * @return remark:1、period为 201812 201813 记录 tms_period 201812
     * 2、可以考虑加index，未来若需预览可保证顺序
     * errorList:
     * 1、存在非选择期间数据
     * 2、未映射到主体
     * @author Gary J Li
     */
    public OperationResultDto importBSExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<BalanceSheet> bls = Lists.newArrayList();
                String companyName = getCellStringValue(sheet.getRow(5).getCell(1));
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andNameEqualTo(companyName);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = getCellStringValue(sheet.getRow(2).getCell(2)).substring(3);
                String fileType = getCellStringValue(sheet.getRow(0).getCell(2));

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.BalanceSheet.getCode(), period / 100,
                        tmsPeriod % 100, period % 100,
                        companyName, fileType);
                //添加导入类型
                dataImportLog.setImportType(importType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                // 根据机构ID和年份获取卡片
                String projId = getProjId(orgId, tmsPeriod);

                if (StringUtils.isEmpty(projId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                if (!orgIds.contains(orgId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period % 100 == 13 && tmsPeriod % 100 == 12;
                if (!tmsPeriod.equals(period) && !isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String currencyCode = getCellStringValue(sheet.getRow(5).getCell(0)).replace(" ", "").substring(3);
                String dataDateStr = getCellStringValue(sheet.getRow(0).getCell(4)).substring(3).replace(" ", "");
                SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyyHH:mm:ss");
                Date dataDate = sdf.parse(dataDateStr);
                for (int j = 11; j <= sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    BalanceSheet bl = new BalanceSheet();
                    bl.setId(plId);
                    bl.setProjectId(projId);
                    bl.setOrganizationId(orgId);
                    bl.setDate(dataDate);
                    bl.setSource("Excel导入");
                    bl.setStatus("人工导入");
                    bl.setTmsPeriod(tmsPeriod);
                    bl.setPeriod(period);
                    bl.setEntityName(companyName);
                    bl.setLedgerCurrencyCode(currencyCode);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    bl.setItemName(getCellStringValue(cell1));

                    Cell cell2 = row.getCell(1);
                    if (cell2.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = getCellStringValue(cell2).replace(",", "");
                        bl.setEndBal(new BigDecimal(value1));
                    } else {
                        bl.setEndBal(new BigDecimal(cell2.getNumericCellValue()));
                    }

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = getCellStringValue(cell3).replace(",", "");
                        bl.setBegBal(new BigDecimal(value2));
                    } else {
                        bl.setBegBal(new BigDecimal(cell3.getNumericCellValue()));
                    }
                    bls.add(bl);
                }
                if (bls.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        BalanceSheetPrcExample delExample = new BalanceSheetPrcExample();
                        delExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        balanceSheetPrcManualMapper.deleteByExample(delExample);
                        if (balanceSheetPrcFinalMapper.countByExample(delExample) > 0) {
                            balanceSheetPrcFinalMapper.deleteByExample(delExample);
                        }
                    }
                    balanceSheetPrcManualMapper.insertBatch(bls);
                    balanceSheetPrcFinalMapper.insertBatch(bls);
                    dataImportLog.setRecordSize(bls.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            if (!dataImportLogs.stream().filter(dil -> !dil.getImportResult()).collect(Collectors.toList()).isEmpty()) {
                return OperationResultDto.error();
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importBSExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    /**
     * 16/02/2019 11:57
     * 导入现金流量表
     * [file, periodDate, importType]
     *
     * @return
     * @author Gary J Li
     */
    public OperationResultDto importCFExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CashFlow> cfs = Lists.newArrayList();
                String companyName = getCellStringValue(sheet.getRow(4).getCell(1));
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andNameEqualTo(companyName);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = getCellStringValue(sheet.getRow(4).getCell(8));
                String fileType = getCellStringValue(sheet.getRow(0).getCell(0));
                Date dataDate = DateUtils.strToDate2(getCellStringValue(sheet.getRow(4).getCell(10)));

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.CashFlow.getCode(), period / 100,
                        tmsPeriod % 100, period % 100,
                        companyName, fileType);
                //添加导入类型
                dataImportLog.setImportType(importType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                // 根据机构ID和年份获取卡片
                String projId = getProjId(orgId, tmsPeriod);

                if (StringUtils.isEmpty(projId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                if (!orgIds.contains(orgId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period % 100 == 13 && tmsPeriod % 100 == 12;
                if (!tmsPeriod.equals(period) && !isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                for (int j = 6; j <= sheet.getLastRowNum(); j++) {
                    Long id = idService.nextId();
                    CashFlow cf = new CashFlow();
                    cf.setId(id);
                    cf.setOrganizationId(orgId);
                    cf.setProjectId(projId);
                    cf.setDate(dataDate);
                    cf.setEntityCode(companyCode);
                    cf.setEntityName(companyName);
                    cf.setPeriod(period);
                    cf.setTmsPeriod(tmsPeriod);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        continue;
                    }
                    cf.setItemName(getCellStringValue(cell1));

                    Cell cell2 = row.getCell(1);
                    cf.setItemName2(getCellStringValue(cell2));

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = getCellStringValue(cell3).replace(",", "");
                        cf.setPeriodAmt(new BigDecimal(value1));
                    } else {
                        cf.setPeriodAmt(new BigDecimal(cell3.getNumericCellValue()));
                    }

                    Cell cell4 = row.getCell(3);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = getCellStringValue(cell4).replace(",", "");
                        cf.setYtdAmt(new BigDecimal(value2));
                    } else {
                        cf.setYtdAmt(new BigDecimal(cell4.getNumericCellValue()));
                    }
                    cfs.add(cf);
                }
                if (cfs.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        CashFlowExample cashFlowExample = new CashFlowExample();
                        cashFlowExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        cashFlowManualMapper.deleteByExample(cashFlowExample);
                        if (cashFlowFinalMapper.countByExample(cashFlowExample) > 0) {
                            cashFlowFinalMapper.deleteByExample(cashFlowExample);
                        }
                    }
                    cashFlowManualMapper.insertBatch(cfs);
                    cashFlowFinalMapper.insertBatch(cfs);
                    dataImportLog.setRecordSize(cfs.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            if (!dataImportLogs.stream().filter(dil -> !dil.getImportResult()).collect(Collectors.toList()).isEmpty()) {
                return OperationResultDto.error();
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importCFExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }


    /**
     * 18/02/2019 12:36
     * 导入发票资料
     * [file, periodDate, importType]
     *
     * @return OperationResultDto
     * @author Gary J Li
     */
    public OperationResultDto importIDExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            String errorMsg = null;
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<InvoiceData> cfs = Lists.newArrayList();
                String[] sheetName = sheet.getSheetName().split("-");

                String taypayerNum = sheetName[0];


                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andTaxPayerNumberEqualTo(taypayerNum);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyName = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyName = organizations.get(0).getName();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                    companyCode = organizations.get(0).getCode();

                }
                String periodStr = sheetName[1].substring(0, 6);
                String fileType = "发票资料";

                Integer period = Integer.valueOf(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.InvoiceData.getCode(), period / 100,
                        tmsPeriod % 100, period % 100,
                        companyName, fileType);
                //添加导入类型
                dataImportLog.setImportType(importType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                // 根据机构ID和年份获取卡片
                String projId = getProjId(orgId, tmsPeriod);
                if (StringUtils.isEmpty(projId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
                    errorMsg = ErrorMessageCN.DoNotExistProject;
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                if (!orgIds.contains(orgId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
                    errorMsg = ErrorMessageCN.DoNotSelectCompany;
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period % 100 == 13 && tmsPeriod % 100 == 12;
                if (!tmsPeriod.equals(period) && !isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    errorMsg = ErrorMessageCN.ExistDataPeriodsError;
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                List<InvoiceData> iDatas = Lists.newArrayList();
                for (int j = 1; j <= sheet.getRow(0).getLastCellNum(); j++) {
                    InvoiceData iData = new InvoiceData();
                    Long idId = idService.nextId();
                    iData.setId(idId);
                    iData.setProjectId(projId);
                    iData.setOrganizationId(orgId);
                    iData.setCompanyCode(companyCode);
                    iData.setTmsPeriod(tmsPeriod);
                    iData.setPeriod(period);
                    iData.setCountType(j);

                    Cell cell1 = sheet.getRow(1).getCell(j);
                    if (null == cell1 || StringUtils.isEmpty(getCellStringValue(cell1))) {
                        errorMsg = ErrorMessageCN.NotOrgException;
                        continue;
                    }
                    iData.setCompanyName(getCellStringValue(cell1));

                    Cell cell2 = sheet.getRow(2).getCell(j);
                    iData.setSpecialInvoiceAmount1(getCellBigDecimalValue(cell2,evaluator).longValue());

                    Cell cell3 = sheet.getRow(3).getCell(j);
                    iData.setSpecialInvoiceAmount2(getCellBigDecimalValue(cell3,evaluator).longValue());

                    Cell cell4 = sheet.getRow(4).getCell(j);
                    iData.setSpecialInvoiceAmount3(getCellBigDecimalValue(cell4,evaluator).longValue());

                    Cell cell5 = sheet.getRow(5).getCell(j);
                    iData.setSpecialInvoiceAmount4(getCellBigDecimalValue(cell5,evaluator).longValue());

                    Cell cell6 = sheet.getRow(6).getCell(j);
                    iData.setSpecialInvoiceSalesAmount1(getCellBigDecimalValue(cell6, evaluator));

                    Cell cell7 = sheet.getRow(7).getCell(j);
                    iData.setSpecialInvoiceSalesAmount2(getCellBigDecimalValue(cell7, evaluator));

                    Cell cell8 = sheet.getRow(8).getCell(j);
                    iData.setSpecialInvoiceSalesAmount3(getCellBigDecimalValue(cell8, evaluator));

                    Cell cell9 = sheet.getRow(9).getCell(j);
                    iData.setSpecialInvoiceSalesAmount4(getCellBigDecimalValue(cell9, evaluator));

                    Cell cell10 = sheet.getRow(10).getCell(j);
                    iData.setSpecialInvoiceSalesAmount5(getCellBigDecimalValue(cell10, evaluator));

                    Cell cell11 = sheet.getRow(11).getCell(j);
                    iData.setSpecialInvoiceTaxAmount1(getCellBigDecimalValue(cell11, evaluator));

                    Cell cell12 = sheet.getRow(12).getCell(j);
                    iData.setSpecialInvoiceTaxAmount2(getCellBigDecimalValue(cell12, evaluator));

                    Cell cell13 = sheet.getRow(13).getCell(j);
                    iData.setSpecialInvoiceTaxAmount3(getCellBigDecimalValue(cell13, evaluator));

                    Cell cell14 = sheet.getRow(14).getCell(j);
                    iData.setSpecialInvoiceTaxAmount4(getCellBigDecimalValue(cell14, evaluator));

                    Cell cell15 = sheet.getRow(15).getCell(j);
                    iData.setSpecialInvoiceTaxAmount5(getCellBigDecimalValue(cell15, evaluator));

                    Cell cell16 = sheet.getRow(16).getCell(j);
                    iData.setInvoiceAmount1(getCellBigDecimalValue(cell16,evaluator).longValue());

                    Cell cell17 = sheet.getRow(17).getCell(j);
                    iData.setInvoiceAmount2(getCellBigDecimalValue(cell17,evaluator).longValue());

                    Cell cell18 = sheet.getRow(18).getCell(j);
                    iData.setInvoiceAmount3(getCellBigDecimalValue(cell18,evaluator).longValue());

                    Cell cell19 = sheet.getRow(19).getCell(j);
                    iData.setInvoiceAmount4(getCellBigDecimalValue(cell19,evaluator).longValue());

                    Cell cell20 = sheet.getRow(20).getCell(j);
                    iData.setInvoiceSalesAmount1(getCellBigDecimalValue(cell20, evaluator));

                    Cell cell21 = sheet.getRow(21).getCell(j);
                    iData.setInvoiceSalesAmount2(getCellBigDecimalValue(cell21, evaluator));

                    Cell cell22 = sheet.getRow(22).getCell(j);
                    iData.setInvoiceSalesAmount3(getCellBigDecimalValue(cell22, evaluator));

                    Cell cell23 = sheet.getRow(23).getCell(j);
                    iData.setInvoiceSalesAmount4(getCellBigDecimalValue(cell23, evaluator));

                    Cell cell24 = sheet.getRow(24).getCell(j);
                    iData.setInvoiceSalesAmount5(getCellBigDecimalValue(cell24, evaluator));

                    Cell cell25 = sheet.getRow(25).getCell(j);
                    iData.setInvoiceTaxAmount1(getCellBigDecimalValue(cell25, evaluator));

                    Cell cell26 = sheet.getRow(26).getCell(j);
                    iData.setInvoiceTaxAmount2(getCellBigDecimalValue(cell26, evaluator));

                    Cell cell27 = sheet.getRow(27).getCell(j);
                    iData.setInvoiceTaxAmount3(getCellBigDecimalValue(cell27, evaluator));

                    Cell cell28 = sheet.getRow(28).getCell(j);
                    iData.setInvoiceTaxAmount4(getCellBigDecimalValue(cell28, evaluator));

                    Cell cell29 = sheet.getRow(29).getCell(j);
                    iData.setInvoiceTaxAmount5(getCellBigDecimalValue(cell29, evaluator));

                    iDatas.add(iData);
                }
                if (iDatas.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgidata period删除记录
                        InvoiceDataExample invoiceDataExample = new InvoiceDataExample();
                        invoiceDataExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        invoiceDataMapper.deleteByExample(invoiceDataExample);
                    }
                    invoiceDataMapper.insertBatch(iDatas);
                    dataImportLog.setRecordSize(iDatas.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(errorMsg);
            }
            if (!dataImportLogs.stream().filter(dil -> !dil.getImportResult()).collect(Collectors.toList()).isEmpty()) {
                return OperationResultDto.error(errorMsg);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importIDExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }

    }

    public OperationResultDto importRLITExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            OperationResultDto operationResultDto = new OperationResultDto();
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<RedLetterInfoTable> rlits = generalRLITs(sheet, tmsPeriod, evaluator);
                if (rlits.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<RedLetterInfoTable>> atsGroupRes =
                        rlits.stream().collect(Collectors.groupingBy(RedLetterInfoTable::getSubjectNum));
                atsGroupRes.forEach((k, v) -> processRlits(k, v, orgIds, tmsPeriod, importType, dataImportLogs));
            }
            if (dataImportLogs.size() == 0)
                return OperationResultDto.error(ErrorMessage.NoData);

            addDataImportLog(dataImportLogs);
            int isSuccess = (int) dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportNotMatch);
            }
            return operationResultDto.success();
        } catch (Exception e) {
            e.printStackTrace();
            return OperationResultDto.error(e.getMessage());
        }
    }

    private void processRlits(String companyCode, List<RedLetterInfoTable> rlits, List<String> orgIds,
                              Integer tmsPeriod, Integer importType, List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);

        Integer period = rlits.get(0).getPeriod();

        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);
        DataImportLog dataImportLog = generalDataImportLog(rlits.get(0).getSubjectNum(), "", "",
                EnumImportType.RedLetterInfoTable.getCode(), period / 100,
                tmsPeriod % 100, period % 100,
                rlits.get(0).getSubjectName(), "");
        dataImportLog.setRecordSize(rlits.size());
        //添加导入类型
        dataImportLog.setImportType(importType);
        if (orgs.size() < 1) {
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        String orgId = orgs.get(0).getId();
        // 根据机构ID和年份获取卡片
        String projId = getProjId(orgId, tmsPeriod);

        if (StringUtils.isEmpty(projId)) {
            dataImportLog.setImportResult(false);
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if (!orgIds.contains(orgId)) {
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());
        boolean validateRes = true;
        for (RedLetterInfoTable rlit : rlits) {
            // 校验该主体导入所有记录区间是否一致
            if (!rlit.getPeriod().equals(period)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                new Exception(ErrorMessageCN.InconsistentPeriod);
                break;
            }
            if (!rlit.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                new Exception(ErrorMessageCN.ExistDataPeriodsError);
                break;
            }
            rlit.setOrganizationId(orgId);
            rlit.setTmsPeriod(tmsPeriod);
            rlit.setProjectId(projId);
        }
        if (validateRes && EnumTbImportType.CoverImport.getCode().equals(importType)) {
            RedLetterInfoTableExample redLetterInfoTableExample = new RedLetterInfoTableExample();
            redLetterInfoTableExample.createCriteria().andSubjectNumEqualTo(companyCode).andPeriodEqualTo(rlits.get(0).getPeriod());
            redLetterInfoTableMapper.deleteByExample(redLetterInfoTableExample);
        }
        redLetterInfoTableMapper.insertBatch(rlits);
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    public OperationResultDto importATExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AdjustmentTable> ats = generalATs(sheet, evaluator);
                if (ats.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<AdjustmentTable>> atsGroupRes =
                        ats.stream().collect(Collectors.groupingBy(AdjustmentTable::getSegment1));
                atsGroupRes.forEach((k, v) -> processAts(k, v, orgIds, tmsPeriod, importType, dataImportLogs));
            }
            addDataImportLog(dataImportLogs);
            if (dataImportLogs.size() == 0)
                return OperationResultDto.error(ErrorMessage.NoData);
            int isSuccess = (int) dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportNotMatch);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processAts(String companyCode, List<AdjustmentTable> ats, List<String> orgIds,
                            Integer tmsPeriod, Integer importType, List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);
        DataImportLog dataImportLog = generalDataImportLog(ats.get(0).getSegment1(), "", "",
                EnumImportType.AdjustmentTable.getCode(), ats.get(0).getPeriod() / 100,
                tmsPeriod % 100, ats.get(0).getPeriod() % 100,
                ats.get(0).getSegment1Name(), "");
        dataImportLog.setRecordSize(ats.size());
        //添加导入类型
        dataImportLog.setImportType(importType);
        if (orgs.size() < 1) {
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        String orgId = orgs.get(0).getId();
        // 根据机构ID和年份获取卡片
        String projId = getProjId(orgId, tmsPeriod);

        if (StringUtils.isEmpty(projId)) {
            dataImportLog.setImportResult(false);
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if (!orgIds.contains(orgId)) {
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = ats.get(0).getPeriod();
        boolean validateRes = true;
        for (AdjustmentTable at : ats) {
            // 校验该主体导入所有记录区间是否一致
            if (!at.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!at.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
            at.setOrganizationId(orgId);
            at.setTmsPeriod(tmsPeriod);
            at.setProjectId(projId);
        }
        if (validateRes && EnumTbImportType.CoverImport.getCode().equals(importType)) {
            AdjustmentTableExample adjustmentTableExample = new AdjustmentTableExample();
            adjustmentTableExample.createCriteria().andSegment1EqualTo(companyCode).andPeriodEqualTo(ats.get(0).getPeriod());
            adjustmentTableMapper.deleteByExample(adjustmentTableExample);
        }
        adjustmentTableMapper.insertBatch(ats);
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    public OperationResultDto importCPRExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CoupaPurchasingReport> ats = generalCPRs(sheet, evaluator);
                if (ats.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<CoupaPurchasingReport>> atsGroupRes =
                        ats.stream().collect(Collectors.groupingBy(CoupaPurchasingReport::getCompanyCode));
                atsGroupRes.forEach((k, v) -> processCprs(k, v, orgIds, tmsPeriod, importType, dataImportLogs));
            }
            int isSuccess = (int) dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            if (dataImportLogs.size() == 0)
                return OperationResultDto.error(ErrorMessage.NoData);
            addDataImportLog(dataImportLogs);

            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportNotMatch);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importCPRExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processCprs(String companyCode, List<CoupaPurchasingReport> cprs, List<String> orgIds,
                             Integer tmsPeriod, Integer importType, List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);

        DataImportLog dataImportLog = generalDataImportLog(cprs.get(0).getCompanyCode(), "", "",
                EnumImportType.CoupaPurchasingReport.getCode(), cprs.get(0).getPeriod() / 100,
                tmsPeriod % 100, cprs.get(0).getPeriod() % 100,
                "", "");
        dataImportLog.setRecordSize(cprs.size());
        //添加导入类型
        dataImportLog.setImportType(importType);
        if (orgs.size() < 1) {
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        String orgId = orgs.get(0).getId();
        // 根据机构ID和年份获取卡片
        String projId = getProjId(orgId, tmsPeriod);

        if (StringUtils.isEmpty(projId)) {
            dataImportLog.setImportResult(false);
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if (!orgIds.contains(orgId)) {
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setCompanyName(orgs.get(0).getName());
        dataImportLog.setFileType("Coupa采购报告");
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = cprs.get(0).getPeriod();
        boolean validateRes = true;
        for (CoupaPurchasingReport cpr : cprs) {
            // 校验该主体导入所有记录区间是否一致
            if (!cpr.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!cpr.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
            cpr.setOrganizationId(orgId);
            cpr.setTmsPeriod(tmsPeriod);
            cpr.setProjectId(projId);
        }
        if (validateRes && EnumTbImportType.CoverImport.getCode().equals(importType)) {
            CoupaPurchasingReportExample coupaPurchasingReportExample = new CoupaPurchasingReportExample();
            coupaPurchasingReportExample.createCriteria().andCompanyCodeEqualTo(companyCode).andPeriodEqualTo(cprs.get(0).getPeriod());
            coupaPurchasingReportMapper.deleteByExample(coupaPurchasingReportExample);
        }
        coupaPurchasingReportMapper.insertBatch(cprs);
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    /**
     * 22/02/2019 12:50
     * 导入发票记录表
     * [file, periodDate, importType]
     *
     * @return
     * @author Gary J Li
     */
    public OperationResultDto importCILExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();

            //先获取错误信息
            Set<String> orgsNot = new HashSet<>();
            List<DataImportErrorDto> errorDtos = getCILExcelError(workbook, orgsNot);

            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CertifiedInvoicesList> cils = Lists.newArrayList();
                String tayPayerIdNum = getCellStringValue(sheet.getRow(1).getCell(1));
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andTaxPayerNumberEqualTo(tayPayerIdNum);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";

                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                }
                String periodStr = getCellStringValue(sheet.getRow(1).getCell(4));
                String fileType = getCellStringValue(sheet.getRow(0).getCell(0));

                Integer period = DateUtils.strToPeriod2(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, tayPayerIdNum,
                        EnumImportType.CertifiedInvoicesList.getCode(), period / 100,
                        tmsPeriod % 100, period % 100,
                        "", fileType);
                //添加导入类型
                dataImportLog.setImportType(importType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                // 根据机构ID和年份获取卡片
                String projId = getProjId(orgId, tmsPeriod);
                if (StringUtils.isEmpty(projId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                if (!orgIds.contains(orgId)) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String companyName = organizations.get(0).getName();
                dataImportLog.setCompanyName(companyName);
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period % 100 == 13 && tmsPeriod % 100 == 12;
                if (!tmsPeriod.equals(period) && !isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String unit = getCellStringValue(sheet.getRow(1).getCell(11)).replace(" ", "").substring(3);

                String identifyNumber = "";
                if (sheet.getLastRowNum() > 1) {
                    Row row1 = sheet.getRow(1);
                    identifyNumber = getCellStringValue(row1.getCell(1));
                    //获取纳税人识别号,存在则整个sheet跳过
                    if (orgsNot.contains(identifyNumber)) {
                        continue;
                    }
                }

                for (int j = 3; j <= sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    CertifiedInvoicesList cil = new CertifiedInvoicesList();
                    cil.setId(plId);
                    cil.setProjectId(projId);
                    cil.setTaxpayerIdNum(tayPayerIdNum);
                    cil.setOrganizationId(orgId);
                    cil.setTmsPeriod(tmsPeriod);
                    cil.setPeriod(period);
                    cil.setUnit(unit);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    cil.setSeqNo(getCellIntegerValue(cell1));

                    Cell cell2 = row.getCell(1);
                    cil.setInvoiceCode(getCellStringValue(cell2));

                    Cell cell3 = row.getCell(2);
                    cil.setInvoiceNum(getCellStringValue(cell3));

                    Cell cell4 = row.getCell(3);
                    try {
                        String BillingDate = format.format(cell4.getDateCellValue());
                        cil.setBillingDate(DateUtils.strToDate4(BillingDate));
                    } catch (Exception e) {
                        cil.setBillingDate(DateUtils.strToDate4(getCellStringValue(cell4)));
                    }
                    Cell cell5 = row.getCell(4);
                    cil.setSalesTaxNum(getCellStringValue(cell5));

                    Cell cell6 = row.getCell(5);
                    cil.setSalespersonName(getCellStringValue(cell6));

                    Cell cell7 = row.getCell(6);
                    cil.setAmount(getCellBigDecimalValue(cell7, evaluator));

                    Cell cell8 = row.getCell(7);
                    cil.setTaxAmount(getCellBigDecimalValue(cell8, evaluator));


                    Cell cell9 = row.getCell(8);
                    cil.setVerificationMethod(getCellStringValue(cell9));
                    Cell cell10 = row.getCell(9);
                /*
                    Date dateCellValue = cell10.getDateCellValue();

                    String format1 = format.format(dateCellValue);
                    cil.setCertifiedDate(DateUtils.strToDate4(format1));*/

                    try {
                        String certDate = format.format(cell10.getDateCellValue());
                        cil.setCertifiedDate(DateUtils.strToDate4(certDate));
                    } catch (Exception e) {
                        cil.setCertifiedDate(DateUtils.strToDate4(getCellStringValue(cell10)));
                    }

                    Cell cell11 = row.getCell(10);
                    cil.setInvoiceType(getCellStringValue(cell11));

                    Cell cell12 = row.getCell(11);
                    cil.setInvoiceStatus(getCellStringValue(cell12));

                    cils.add(cil);
                }
                if (cils.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        CertifiedInvoicesListExample certifiedInvoicesListExample = new CertifiedInvoicesListExample();
                        certifiedInvoicesListExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        certifiedInvoicesListMapper.deleteByExample(certifiedInvoicesListExample);
                        if (certifiedInvoicesListMapper.countByExample(certifiedInvoicesListExample) > 0) {
                            certifiedInvoicesListMapper.deleteByExample(certifiedInvoicesListExample);
                        }
                    }
                    certifiedInvoicesListMapper.insertBatch(cils);
                    dataImportLog.setRecordSize(cils.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }

            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            if (!dataImportLogs.stream().filter(dil -> !dil.getImportResult()).collect(Collectors.toList()).isEmpty()) {
                return OperationResultDto.error();
            }
            //导入成功并将错误信息返回
            OperationResultDto a = new OperationResultDto();
            a.setResult(true);
            a.setResultMsg(reImportError(errorDtos));
            return a;
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importCILExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private List<DataImportErrorDto> getCILExcelError(Workbook workbook, Set<String> orgsNot) {
        //返回数据及错误信息
        List<DataImportErrorDto> errorDtos = Lists.newArrayList();
        //添加校验信息
        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            Sheet sheet = workbook.getSheetAt(i);
            if (isSheetEmpty(sheet)) continue;
            //获取纳税人识别号
            String identifyNumber = "";
            if (sheet.getLastRowNum() > 1) {
                Row row1 = sheet.getRow(1);
                identifyNumber = getCellStringValue(row1.getCell(1));
            }
            //先循环校验字段
            for (int j = 3; j <= sheet.getLastRowNum(); j++) {
                Row row = sheet.getRow(j);
                //校验纳税人识别号，存储不合规的
                validCILExcelCell(identifyNumber, row, orgsNot);
                DataImportErrorDto importErrorDto = new DataImportErrorDto();
                //添加错误信息
                importErrorDto.setCellLine(j);
                importErrorDto.setSheet(i + 1);
                if (validCILExcelCellError(row, importErrorDto)) {
                    errorDtos.add(importErrorDto);
                }
            }
        }
        return errorDtos;
    }

    /**
     * 导入分发-已认证发票清单，校验“金额”、“税额”、“确认/认证日期”、“发票类型”任意不为空
     * 且发票类型值只能包含“增值税专票”、“机动车发票”
     * 存在校验1或校验2不通过的（纳税人识别号）不进行导入
     *
     * @param
     * @return
     */
    private Set<String> validCILExcelCell(String identifyNumber, Row row3, Set<String> orgsNot) {
        List<String> invoiceTypes = Arrays.asList(new String[]{"增值税专票", "机动车发票"});
        //发票类型
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(10)))) {
            orgsNot.add(identifyNumber);
        } else if (!invoiceTypes.contains(getCellStringValue(row3.getCell(10)))) {
            orgsNot.add(identifyNumber);
        }
        //金额
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(6)))) {
            orgsNot.add(identifyNumber);
        }
        //税额
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(7)))) {
            orgsNot.add(identifyNumber);
        }
        //确认日期
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(9)))) {
            orgsNot.add(identifyNumber);
        }
        return orgsNot;
    }

    private Boolean validCILExcelCellError(Row row3, DataImportErrorDto importErrorDto) {
        List<String> invoiceTypes = Arrays.asList(new String[]{"增值税专票", "机动车发票"});
        Boolean fg = false;
        //发票类型
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(10)))) {
            fg = true;
            importErrorDto.setInvoiceType("为空");
        } else if (!invoiceTypes.contains(getCellStringValue(row3.getCell(10)))) {
            fg = true;
            importErrorDto.setInvoiceType("错误");
        }
        //金额
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(6)))) {
            fg = true;
            importErrorDto.setCilAmount("为空");
        }
        //税额
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(7)))) {
            fg = true;
            importErrorDto.setTaxRate("为空");
        }
        //确认日期
        if (StringUtils.isBlank(getCellStringValue(row3.getCell(9)))) {
            fg = true;
            importErrorDto.setConfirmTime("为空");
        }
        return fg;
    }

    public OperationResultDto importIRExcelFile(MultipartFile file, List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            List<DataImportErrorDto> dataImportResponseDto1 = new ArrayList<>();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                DataImportResponseDto<InvoiceRecord> dataImportResponseDto = generalIRs(sheet, evaluator);
                dataImportResponseDto1.addAll(dataImportResponseDto.getDataImportErrorDtos());
                //List<InvoiceRecord> irs = generalIRs(sheet);
                List<InvoiceRecord> irs = dataImportResponseDto.getObjectList();
                if (irs.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<InvoiceRecord>> atsGroupRes =
                        irs.stream().collect(Collectors.groupingBy(InvoiceRecord::getBillingBody));
                atsGroupRes.forEach((k, v) -> processIrs(k, v, orgIds, tmsPeriod, importType, dataImportLogs));
            }
            if (dataImportLogs.size() == 0)
                return OperationResultDto.error(ErrorMessage.NoData);
            int isSuccess = (int) dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            addDataImportLog(dataImportLogs);

            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportNotMatch);
            }
            //导入成功并将错误信息返回
            OperationResultDto a = new OperationResultDto();
            a.setResult(true);
            a.setResultMsg(reImportError(dataImportResponseDto1));
            return a;

        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importIRExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private String reImportError(List<DataImportErrorDto> errorDtos) {
        StringBuffer sb = new StringBuffer();

        for (DataImportErrorDto row : errorDtos) {
            //Sheet
            if (row.getSheet() != null) {
                sb.append("第" + row.getSheet() + "Sheet,");
            }
            //行号
            if (row.getCellLine() != null) {
                sb.append("第" + row.getCellLine() + "行,");
            }
            //发票类型
            if (!Strings.isBlank(row.getInvoiceType())) {
                sb.append("发票类型" + row.getInvoiceType() + ",");
            }
            //开票内容
            if (!Strings.isBlank(row.getInvoiceDetail())) {
                sb.append("开票内容" + row.getInvoiceDetail() + ",");
            }
            //开票主体
            if (!Strings.isBlank(row.getOrg())) {
                sb.append("开票主体" + row.getOrg() + ",");
            }
            //税率
            if (!Strings.isBlank(row.getTaxRate())) {
                sb.append("税率" + row.getTaxRate() + ",");
            }
            //税额
            if (!Strings.isBlank(row.getAmountRate())) {
                sb.append("税额" + row.getAmountRate() + ",");
            }
            //开票金额
            if (!Strings.isBlank(row.getAmount())) {
                sb.append("开票金额" + row.getAmount() + ",");
            }
            //确认/认证日期
            if (!Strings.isBlank(row.getConfirmTime())) {
                sb.append("确认/认证日期" + row.getConfirmTime() + ",");
            }
            //金额
            if (!Strings.isBlank(row.getCilAmount())) {
                sb.append("金额" + row.getCilAmount() + ",");
            }
            sb.append(";");
        }
        return sb.toString();
    }

    private void processIrs(String billingBody, List<InvoiceRecord> irs, List<String> orgIds,
                            Integer tmsPeriod, Integer importType, List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andNameEqualTo(billingBody);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);

        DataImportLog dataImportLog = generalDataImportLog("", "", "",
                EnumImportType.InvoiceRecord.getCode(), irs.get(0).getPeriod() / 100,
                tmsPeriod % 100, irs.get(0).getPeriod() % 100,
                irs.get(0).getBillingBody(), "");
        dataImportLog.setRecordSize(irs.size());
        //添加导入类型
        dataImportLog.setImportType(importType);
        if (orgs.size() < 1) {
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        String orgId = orgs.get(0).getId();
        // 根据机构ID和年份获取卡片
        String projId = getProjId(orgId, tmsPeriod);

        if (StringUtils.isEmpty(projId)) {
            dataImportLog.setImportResult(false);
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotExistProject);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if (!orgIds.contains(orgId)) {
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setCompanyName(orgs.get(0).getName());
        dataImportLog.setFileType("已开增值税发票记录");
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = irs.get(0).getPeriod();
        boolean validateRes = true;
        for (InvoiceRecord ir : irs) {
            // 校验该主体导入所有记录区间是否一致
            if (null == ir.getPeriod()) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.DoNotInputPeriod);
                validateRes = false;
                break;
            }

            if (!ir.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!ir.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
            ir.setOrganizationId(orgId);
            ir.setProjectId(projId);
            ir.setTmsPeriod(tmsPeriod);
        }
        if (validateRes && EnumTbImportType.CoverImport.getCode().equals(importType)) {
            InvoiceRecordExample invoiceRecordExample = new InvoiceRecordExample();
            invoiceRecordExample.createCriteria().andBillingBodyEqualTo(billingBody).andPeriodEqualTo(irs.get(0).getPeriod());
            invoiceRecordMapper.deleteByExample(invoiceRecordExample);
        }
        invoiceRecordMapper.insertBatch(irs);
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    private List<RedLetterInfoTable> generalRLITs(Sheet sheet, Integer periodDate, FormulaEvaluator evaluator) {
        List<RedLetterInfoTable> rlits = Lists.newArrayList();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMM");
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            RedLetterInfoTable rlit = new RedLetterInfoTable();
            Long plId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            rlit.setId(plId);
            rlit.setSubjectNum(getCellStringValue(row.getCell(0)));
            rlit.setSubjectName(getCellStringValue(row.getCell(1)));
            Object value = CitCommonUtil.getValue(row.getCell(2));
            if (!"".equals(value)) {
                rlit.setRedLetterInvoiceInfoTableNum(new BigDecimal(value.toString()).toPlainString());
            }
            rlit.setPeriod(DateUtils.dateToPeriod(row.getCell(3).getDateCellValue()));
            rlit.setFillInDate(row.getCell(3).getDateCellValue());
            rlit.setSalesTaxNumber(getCellStringValue(row.getCell(4)));
            rlit.setSalespersonName(getCellStringValue(row.getCell(5)));
            rlit.setTotalAmount(getCellBigDecimalValue(row.getCell(6), evaluator));
            rlit.setTotalTaxAmount(getCellBigDecimalValue(row.getCell(7), evaluator));
            rlit.setApplicationDescription(getCellStringValue(row.getCell(8)));
            rlit.setApplicantManager(getCellStringValue(row.getCell(9)));
            rlit.setInvoiceCode(getCellStringValue(row.getCell(10)));
            rlit.setInvoiceNumber(getCellStringValue(row.getCell(11)));
            rlits.add(rlit);
        }
        return rlits;
    }

    private DataImportResponseDto<InvoiceRecord> generalIRs(Sheet sheet, FormulaEvaluator evaluator) {
        List<InvoiceRecord> irs = Lists.newArrayList();
        List<DataImportErrorDto> errorDtos = Lists.newArrayList();
        Set<String> orgsNot = new HashSet<>();
        //先循环校验字段
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            Row row = sheet.getRow(j);
            DataImportErrorDto importErrorDto = new DataImportErrorDto();
            if (null == row.getCell(0)) {
                continue;
            }
            Boolean flag = false;
            //所属机构
            importErrorDto.setCellLine(j);
            if (StringUtils.isBlank(getCellStringValue(row.getCell(1)))) {
                flag = true;
                importErrorDto.setOrg("为空");
            }
            //如果存在有错误的机构也跳过
            if (validIRExcelCellError(row, importErrorDto)) {
                flag = true;
            }
            if (flag) {
                errorDtos.add(importErrorDto);
            }
        }

        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            InvoiceRecord ir = new InvoiceRecord();
            Long irId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            ir.setId(irId);
            ir.setSeqNo(getCellIntegerValue(row.getCell(0)));
            //所属机构
            if (StringUtils.isBlank(getCellStringValue(row.getCell(1)))) {
                continue;
            }
            //如果存在有错误的机构也跳过
            if (validIRExcelCell(row, orgsNot).contains(getCellStringValue(row.getCell(1)))) {
                continue;
            }
            ir.setBillingBody(getCellStringValue(row.getCell(1)));
            ir.setCustomerCompanyName(getCellStringValue(row.getCell(2)));
            ir.setInvoiceType(getCellStringValue(row.getCell(3)));
            ir.setBillingContent(getCellStringValue(row.getCell(4)));
            ir.setInvoiceAmount(getCellBigDecimalValue(row.getCell(5), evaluator));
            ir.setApplicant(getCellStringValue(row.getCell(6)));
            ir.setOaApplicationNum(getCellStringValue(row.getCell(7)));
            ir.setContractNo(getCellStringValue(row.getCell(8)));
            ir.setContractAmount(getCellBigDecimalValue(row.getCell(9), evaluator));
            ir.setDepartment(getCellStringValue(row.getCell(10)));
            ir.setApplicationDate(row.getCell(11).getDateCellValue());
            ir.setBillingDate(row.getCell(12).getDateCellValue());
            ir.setPeriod(DateUtils.dateToPeriod(ir.getBillingDate()));
            ir.setBillingMonth(DateUtils.strToDate5(getCellStringValue(row.getCell(13))));
            ir.setInvoiceCode(getCellStringValue(row.getCell(14)));
            ir.setInvoiceNum(getCellStringValue(row.getCell(15)));
            ir.setRemarks(getCellStringValue(row.getCell(16)));
            ir.setInvoicesAmount(getCellIntegerValue(row.getCell(17)));
            ir.setCustomerCompanyTaxNum(getCellStringValue(row.getCell(18)));
            ir.setContractSourceSystem(getCellStringValue(row.getCell(19)));
            ir.setTaxRate(getCellBigDecimalValue(row.getCell(20), evaluator));
            ir.setTaxAmount(getCellBigDecimalValue(row.getCell(21), evaluator));
            ir.setInvoiceStatus(getCellStringValue(row.getCell(22)));
            ir.setSource(getCellStringValue(row.getCell(23)));
            irs.add(ir);
        }
        //返回数据及错误信息
        DataImportResponseDto<InvoiceRecord> dataImportResponseDto = new DataImportResponseDto<InvoiceRecord>();
        dataImportResponseDto.setObjectList(irs);
        dataImportResponseDto.setDataImportErrorDtos(errorDtos);
        return dataImportResponseDto;
    }

    /**
     * 导入分发-已开增值税发票记录，校验1-发票类型、开票内容、所属机构、税率、税额、开票金额任意不为空；
     * 且校验2-校验发票类型的值只能包含“增值税专用发票”、“增值税普通发票”、“增值税电子发票”
     * 存在校验1或校验2不通过的机构不进行导入
     *
     * @param row
     * @return
     */
    private Set<String> validIRExcelCell(Row row, Set<String> orgsNot) {
        List<String> invoiceTypes = Arrays.asList(new String[]{"增值税专用发票", "增值税普通发票", "增值税电子发票"});
        String orgName = getCellStringValue(row.getCell(1));
        //发票类型
        if (StringUtils.isBlank(getCellStringValue(row.getCell(3)))) {
            orgsNot.add(orgName);
        } else if (!invoiceTypes.contains(getCellStringValue(row.getCell(3)))) {
            orgsNot.add(orgName);
        }
        //开票内容
        if (StringUtils.isBlank(getCellStringValue(row.getCell(4)))) {
            orgsNot.add(orgName);
        }
        //开票金额
        if (StringUtils.isBlank(getCellStringValue(row.getCell(5)))) {
            orgsNot.add(orgName);
        }
        //税率
        if (StringUtils.isBlank(getCellStringValue(row.getCell(20)))) {
            orgsNot.add(orgName);
        }
        //税额
        if (StringUtils.isBlank(getCellStringValue(row.getCell(21)))) {
            orgsNot.add(orgName);
        }
        return orgsNot;
    }

    private Boolean validIRExcelCellError(Row row, DataImportErrorDto importErrorDto) {
        List<String> invoiceTypes = Arrays.asList(new String[]{"增值税专用发票", "增值税普通发票", "增值税电子发票"});
        Boolean fg = false;
        //发票类型
        if (StringUtils.isBlank(getCellStringValue(row.getCell(3)))) {
            fg = true;
            importErrorDto.setInvoiceType("为空");
        } else if (!invoiceTypes.contains(getCellStringValue(row.getCell(3)))) {
            fg = true;
            importErrorDto.setInvoiceType("错误");
        }
        //开票内容
        if (StringUtils.isBlank(getCellStringValue(row.getCell(4)))) {
            fg = true;
            importErrorDto.setInvoiceDetail("为空");
        }
        //开票金额
        if (StringUtils.isBlank(getCellStringValue(row.getCell(5)))) {
            fg = true;
            importErrorDto.setAmount("为空");
        }
        //税率
        if (StringUtils.isBlank(getCellStringValue(row.getCell(20)))) {
            fg = true;
            importErrorDto.setTaxRate("为空");
        }
        //税额
        if (StringUtils.isBlank(getCellStringValue(row.getCell(21)))) {
            fg = true;
            importErrorDto.setAmountRate("为空");
        }
        return fg;
    }

    private List<AdjustmentTable> generalATs(Sheet sheet, FormulaEvaluator evaluator) {
        List<AdjustmentTable> ats = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            AdjustmentTable at = new AdjustmentTable();
            Long plId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            at.setId(plId);
            //第一版方法，因未能识别单元格格式问题，做出改进,只针对该单元格
//            String periodStr = getCellStringValue(row.getCell(0));
//            Integer period = Integer.valueOf(periodStr.substring(0,4)+periodStr.substring(5,7));
//            at.setPeriod(period);

            Object value = CitCommonUtil.getValue(row.getCell(0));
            if (!"".equals(value)) {
                String periodStr = value.toString();
                Integer period = Integer.valueOf(periodStr.substring(0, 4) + periodStr.substring(5, 7));
                at.setPeriod(period);
            }

            at.setSegment1(getCellStringValue(row.getCell(1)));
            at.setSegment2(getCellStringValue(row.getCell(2)));
            at.setSegment3(getCellStringValue(row.getCell(3)));
            at.setSegment4(getCellStringValue(row.getCell(4)));
            at.setSegment5(getCellStringValue(row.getCell(5)));
            at.setSegment6(getCellStringValue(row.getCell(6)));
            at.setSegment7(getCellStringValue(row.getCell(7)));
            at.setSegment8(getCellStringValue(row.getCell(8)));
            at.setSegment9(getCellStringValue(row.getCell(9)));
            at.setSegment10(getCellStringValue(row.getCell(10)));
            at.setSegment1Name(getCellStringValue(row.getCell(11)));
            at.setSegment2Name(getCellStringValue(row.getCell(12)));
            at.setSegment3Name(getCellStringValue(row.getCell(13)));
            at.setSegment4Name(getCellStringValue(row.getCell(14)));
            at.setSegment5Name(getCellStringValue(row.getCell(15)));
            at.setSegment6Name(getCellStringValue(row.getCell(16)));
            at.setSegment7Name(getCellStringValue(row.getCell(17)));
            at.setSegment8Name(getCellStringValue(row.getCell(18)));
            at.setSegment9Name(getCellStringValue(row.getCell(19)));
            at.setSegment10Name(getCellStringValue(row.getCell(20)));
            at.setPeriodDrBeq(getCellBigDecimalValue(row.getCell(21), evaluator));
            at.setPeriodCrBeq(getCellBigDecimalValue(row.getCell(22), evaluator));
            at.setEndBalBeq(getCellBigDecimalValue(row.getCell(23), evaluator));
            ats.add(at);
        }
        return ats;
    }

    private List<CoupaPurchasingReport> generalCPRs(Sheet sheet, FormulaEvaluator evaluator) {
        List<CoupaPurchasingReport> cprs = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            CoupaPurchasingReport cpr = new CoupaPurchasingReport();
            Long id = idService.nextId();
            cpr.setId(id);
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            cpr.setInvoiceNum(getCellStringValue(row.getCell(0)));
            cpr.setLineNum(getCellIntegerValue(row.getCell(1)));
            cpr.setHeadUpStatus(getCellStringValue(row.getCell(2)));
            cpr.setInvoiceDate(DateUtils.strToDate3(getCellStringValue(row.getCell(3))));
            cpr.setInvoiceReceiptDate(DateUtils.strToDate3(getCellStringValue(row.getCell(4))));
            cpr.setPayExpiration(getCellStringValue(row.getCell(5)));
            cpr.setPurchaseOrderNum(getCellStringValue(row.getCell(6)));
            cpr.setPurchaseOrderLineNum(getCellIntegerValue(row.getCell(7)));
            cpr.setDescription(getCellStringValue(row.getCell(8)));
            cpr.setSupplierNum(getCellStringValue(row.getCell(9)));
            cpr.setSupplier(getCellStringValue(row.getCell(10)));
            cpr.setTotalAmount(getCellBigDecimalValue(row.getCell(11), evaluator));
            cpr.setCurrency(getCellStringValue(row.getCell(12)));
            String billNum = getCellStringValue(row.getCell(13));
            cpr.setBillNum(billNum);
            cpr.setPaid(getCellBolleanValue(row.getCell(14)));
            cpr.setAmortization(getCellBolleanValue(row.getCell(15)));
            cpr.setPrepayments(getCellBolleanValue(row.getCell(16)));
            cpr.setReviewReason(getCellStringValue(row.getCell(17)));
            cpr.setCommodity(getCellStringValue(row.getCell(18)));
            cpr.setCommodityCode(getCellStringValue(row.getCell(19)));
            cpr.setFounder(getCellStringValue(row.getCell(20)));
            cpr.setCreateDate(DateUtils.strToDate3(getCellStringValue(row.getCell(21))));
            cpr.setCurrentApprover(getCellStringValue(row.getCell(22)));
            cpr.setBillingDate(DateUtils.strToDate3(getCellStringValue(row.getCell(23))));
            cpr.setPeriod(DateUtils.dateToPeriod(cpr.getBillingDate()));
            cpr.setLastUpdatedDate(DateUtils.strToDate3(getCellStringValue(row.getCell(24))));
            cpr.setLastUpdatedPerson(getCellStringValue(row.getCell(25)));
            cpr.setTotalTaxAmount(getCellBigDecimalValue(row.getCell(26), evaluator));
            cpr.setTotalTaxAmountHeader(getCellBigDecimalValue(row.getCell(27), evaluator));
            cpr.setBillingNote(getCellStringValue(row.getCell(28)));
            cprs.add(cpr);
        }
        return cprs;
    }

    private String getCellStringValue(Cell cell) {
        if (null == cell) {
            return "";
        }
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return cell.getStringCellValue().trim();
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            // 取整
            return String.valueOf((int) cell.getNumericCellValue()).trim();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Boolean getCellBolleanValue(Cell cell) {
        if (null == cell) {
            return false;
        }
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            switch (cell.getStringCellValue()) {
                case "TRUE":
                    return Boolean.TRUE;
                case "FALSE":
                    return Boolean.FALSE;
                case "是":
                    return Boolean.TRUE;
                case "否":
                    return Boolean.FALSE;
                default:
                    return null;
            }
        } else if (cell.getCellTypeEnum().equals(CellType.BOOLEAN)) {
            return cell.getBooleanCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Integer getCellIntegerValue(Cell cell) {
        if (null == cell) {
            return null;
        }
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Integer.valueOf(cell.getStringCellValue().trim());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return (int) cell.getNumericCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private BigDecimal getCellBigDecimalValue(Cell cell, FormulaEvaluator evaluator) {
        if (null == cell) {
            return null;
        }
        if (cell.getCellTypeEnum().equals(CellType.FORMULA)) {
            CellValue cellValue = evaluator.evaluate(cell);
            if (cellValue.getCellTypeEnum().equals(CellType.STRING)) {
                return new BigDecimal(cellValue.getStringValue().trim());
            } else if (cellValue.getCellTypeEnum().equals(CellType.NUMERIC)) {
                return new BigDecimal(cellValue.getNumberValue());
            }
            return null;
        }
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return new BigDecimal(cell.getStringCellValue().trim());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new BigDecimal(cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }
//    private BigDecimal getCellBigDecimalValue(Cell cell) {
//        if (null == cell) {
//            return null;
//        }
//        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
//            return new BigDecimal(cell.getStringCellValue().trim());
//        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
//            return new BigDecimal(cell.getNumericCellValue());
//        }
//        logger.warn("获取单元格数据类型未匹配");
//        return null;
//    }

    private Long getCellLongDecimalValue(Cell cell) {
        if (null == cell) {
            return null;
        }
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Long.valueOf(cell.getStringCellValue().trim());
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new Double(cell.getNumericCellValue()).longValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }


    private int addDataImportLog(List<DataImportLog> dataImportLogs) {
        if (dataImportLogs.size() == 0) return 0;
        dataImportLogMapper.displayOld(dataImportLogs.get(0).getType());
        int res = 0;
        for (DataImportLog dataImportLog : dataImportLogs) {
            res += dataImportLogMapper.insertSelective(dataImportLog);
        }
        return res;
    }

    /**
     * 25/02/2019 18:52
     * <p>
     * [type]
     *
     * @return
     * @author Gary J Li
     */
    public PageInfo<DataImportLogDto> displayImportLog(DataImportParam param) {

        DataImportLogExample example = new DataImportLogExample();

        DataImportLogExample.Criteria criteria = example.createCriteria();

        List<String> orgIds = organizationService.getMyOrgList().stream().map(OrgSelectDto::getId).collect(Collectors.toList());
        // 这里会导致任何人都可以看到未映射到主体的数据
        orgIds.add("");
        orgIds.add(Constant.ALL);
        if (!EnumImportType.Undefined.getCode().equals(param.getType())) {
            criteria.andTypeEqualTo(param.getType()).andDisplayEqualTo(true);
            criteria.andOrganizationIdIn(orgIds);
        }
        //财务数据抽取展示抽取记录除了admin能查看所有操作人员的操作，其余人员只能看到本身的
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        if (!user.getIsSuperAdmin()) {
            criteria.andOperatorEqualTo(authUserHelper.getCurrentAuditor().get());
        }
        example.setOrderByClause("update_time desc");
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        PageInfo<DataImportLogDto> pageInfo = new PageInfo<>(dataImportLogMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new DataImportLogDto())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());

        return pageInfo;
    }

    /**
     * 25/02/2019 18:52
     * <p>
     * [type]
     *
     * @return
     * @author Gary J Li
     */
    public PageInfo<DataImportLogDto> displayImportLogAll(DataImportAllParam param) {
        String startStr = param.getStartDate();
        Date strDate = DateUtils.strToDate2(startStr);
        String endStr = param.getEndDate();
        Date endDateO = DateUtils.strToDate2(endStr);
        long endDateLong = endDateO.getTime() + 86399000;
        Date endDate = new Date(endDateLong);
        /*Date strDate = DateUtils.strToDate2(param.getStartDate());
        Date endDate = DateUtils.strToDate2(param.getEndDate());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        try {
            strDate =  format.parse(format.format(strDate) + "00:00:00");
            endDate =  format.parse(format.format(endDate) + "23:59:59");
        } catch (ParseException e) {
            e.printStackTrace();
        }*/

        DataImportLogExample example = new DataImportLogExample();
        DataImportLogExample.Criteria criteria = example.createCriteria();
        List<String> orgIds = organizationService.getMyOrgList().stream().map(OrgSelectDto::getId).collect(Collectors.toList());
        //财务数据抽取展示抽取记录除了admin能查看所有操作人员的操作，其余人员只能看到本身的
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        if (user.getIsSuperAdmin()) {
            criteria.andOrganizationIdIn(orgIds).andCreateTimeBetween(strDate, endDate);
        } else {
            criteria.andOrganizationIdIn(orgIds).andCreateTimeBetween(strDate, endDate).andOperatorEqualTo(authUserHelper.getCurrentAuditor().get());
        }
        example.setOrderByClause("update_time desc");
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        PageInfo<DataImportLogDto> pageInfo = new PageInfo<>(dataImportLogMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new DataImportLogDto())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());

        return pageInfo;
    }

    /**
     * 25/02/2019 18:52
     * <p>
     * [type]
     *
     * @return
     * @author Gary J Li
     */
    public PageInfo<DataValidateLogDto> displayProcessLog(DataProcessParam param) {
/*
        Date strDate = DateUtils.strToDate2(param.getStartDate());
        Date endDate = DateUtils.strToDate2(param.getEndDate());*/

        String startStr = param.getStartDate();
        Date strDate = DateUtils.strToDate2(startStr);
        String endStr = param.getEndDate();
        Date endDateO = DateUtils.strToDate2(endStr);
        long endDateLong = endDateO.getTime() + 86399000;
        Date endDate = new Date(endDateLong);

        DataValidateLogExample example = new DataValidateLogExample();

        DataValidateLogExample.Criteria criteria = example.createCriteria();

        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());

        List<String> orgIds = organizationService.getMyOrgList().stream().map(OrgSelectDto::getId).collect(Collectors.toList());
        // 这里会导致任何人都可以看到未映射到主体的数据
        orgIds.add("");
        orgIds.add(Constant.ALL);
        criteria.andOrganizationIdIn(orgIds).andCreateTimeBetween(strDate, endDate);
        example.setOrderByClause("update_time desc");
        PageInfo<DataValidateLogDto> pageInfo = new PageInfo<>(dataValidateLogMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new DataValidateLogDto())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    /**
     * 16/03/2019 12:43
     * 1、根据类型调用EBS接口
     * 2、定长线程池超过最长时丢弃记录日志
     * 3、调用成功使用taskId作为日志id，未成功使用随机id
     * [param]
     *
     * @return
     * @author Gary J Li
     */
    public OperationResultDto callExtractFinancialData(DataExtractParam param) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andIdIn(param.getOrgIds());
        List<Organization> orgs = organizationMapper.selectByExample(example);
        Integer periodInt = DateUtils.strToPeriod(param.getPeriod());
        String period = param.getPeriod();
        // 这两个参数未来可能有用
        String effectiveDateFrom = param.getEffectiveDateFrom();
        String effectiveDateTo = param.getEffectiveDateTo();
        List<Future> resList = Lists.newArrayList();
        List<Integer> dataTypes = param.getDataTypes();
        String operator = authUserHelper.getCurrentAuditor().get();
        dataTypes.forEach(type -> {
            List<Organization> tmpOrgList = Collections.emptyList();
            if (type == EbsExtractTypeConstant.RATE) {
                tmpOrgList = orgs.stream().limit(1).collect(Collectors.toList());
            } else {
                tmpOrgList = orgs;
            }
            tmpOrgList.forEach(o -> {
                try {
                    Callable callEbs = new CallEbsThread(type, o, ebsCallUrl, dataImportLogMapper, authUserHelper,
                            idService, dtsTokenService, period, effectiveDateFrom, effectiveDateTo, operator, organizationAccountingRateMapper);
                    executorService.submit(callEbs);
//                    resList.add(future);
                } catch (RejectedExecutionException rje) {
                    // 这里可以考虑简化 没想好还
                    DataImportLog log = new DataImportLog();
                    log.setId(idService.nextId());
                    log.setTaxpayerIdNum(o.getTaxPayerNumber());
                    log.setCompanyCode(o.getCode());
                    log.setCompanyName(o.getName());
                    log.setPeriodYear(periodInt / 100);
                    log.setPeriodMonth(periodInt % 100);
                    log.setType(EnumImportType.ExtractFinancialData.getCode());
                    log.setFileType(EnumEbsExtractType.getNameByCode(type));
                    log.setImportResult(false);
                    log.setPeriodStatus("EBS抽取");
                    log.setOperator(authUserHelper.getCurrentAuditor().get());
                    log.setErrorMsg(EnumApiCodeMsg.BUSY.getMsg());
                    dataImportLogMapper.insertSelective(log);
                } catch (Exception e) {
                    logger.error(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), e.getMessage()), e);
                }
            });
            // 三天外的日志隐藏
            DataImportLog dil = new DataImportLog();
            dil.setDisplay(false);
            DataImportLogExample e = new DataImportLogExample();
            e.createCriteria().andCreateTimeLessThan(DateUtils.getThreeDayZero());
            dataImportLogMapper.updateByExampleSelective(dil, e);
        });
        // 校验是否全部调用成功 这里有点问题
        /*int res = 0;
        try{
            for(Future future : resList){
                res += future.get()!=null?(Integer)future.get():0;
            }
            System.out.println(res);
            if(res!=orgs.size()*dataTypes.size()){
                return OperationResultDto.error("抽取失败");
            }
        }catch (Exception e){
            logger.error("获取结果异常");
        }*/
        resList = null;
        return OperationResultDto.success();
    }

    /**
     * 25/02/2019 18:26
     * <p>
     * [orgIds, period, dataTypes]
     *
     * @return remark:1、这里重复写了，因为可能调用场景不同？
     * @author Gary J Li
     */
    public OperationResultDto callExtractInvoiceData(DataExtractParam dataExtractParam) {
        // todo 调用滴滴的http发票数据抽取调用接口 List<String> codes, Integer period, List<Integer> dataTypes
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andIdIn(dataExtractParam.getOrgIds());
        List<String> codes = organizationMapper.selectByExample(example).stream().map(Organization::getCode).collect(Collectors.toList());
        Integer tmsPeriod = DateUtils.strToPeriod(dataExtractParam.getPeriod());


//      data_import_log  日志记录为 EnumImportType.ExtractInvoiceData
        return OperationResultDto.success();
    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (sheet.getLastRowNum() > 0 &&
                (null == sheet.getRow(0).getCell(0) || "Version".equals(getCellStringValue(sheet.getRow(0).getCell(0)))) &&
                null == sheet.getRow(0).getCell(2)) {
            return true;
        }
        return false;
    }

    private String getProjId(String orgId, Integer tmsPeriod) {
        ProjectExample pExample = new ProjectExample();
        pExample.createCriteria().andOrganizationIdEqualTo(orgId).andYearEqualTo(tmsPeriod / 100);
        List<Project> pList = projectMapper.selectByExample(pExample);
        if (!pList.isEmpty()) {
            return pList.get(0).getId();
        }
        return "";
    }

    //         get 测试获取数据 post  触发任务请求

    /**
     * "http://172.20.201.201:8020/ebs-proxy-test/dts/glMonthlyBal?ledgerId=2021&companyCode=120200&period=2018-11"
     * "http://172.20.201.201:8020/ebs-proxy-test/dts/fsgCash/?ledgerId=2021&companyCode=120200&period=2019-02"
     * "http://172.20.201.201:8020/ebs-proxy-test/dts/fsgAsset?ledgerId=2021&period=2018-12&companyCode=120113&prcFlag=N"
     * "http://172.20.201.201:8020/ebs-proxy-test/dts//fsgProfit?ledgerId=2021&companyCode=120200&period=2018-10&prcFlag=Y"
     * "http://172.20.201.201:8020/ebs-proxy-test/dts/glJeLines/?ledgerId=2041&companyCode=110100&effectiveDateFrom=2018-02-01&effectiveDateTo=2018-02-10"
     * "http://172.20.201.201:8020/ebs-proxy-test/dts/dailyRates?period=2018-10"
     */
    private static class CallEbsThread implements Callable {

        protected final Logger logger = LoggerFactory.getLogger(this.getClass());

        private int type;

        private Organization org;

        private String period;

        private String ebsCallUrl;

        private DataImportLogMapper dataImportLogMapper;

        private AuthUserHelper authUserHelper;

        private DistributedIdService idService;

        private DtsTokenService dtsTokenService;

        private String effectiveDateFrom;

        private String effectiveDateTo;
        private String operator;

        private OrganizationAccountingRateMapper organizationAccountingRateMapper;

        CallEbsThread(int type, Organization org, String ebsCallUrl, DataImportLogMapper dataImportLogMapper,
                      AuthUserHelper authUserHelper, DistributedIdService idService, DtsTokenService dtsTokenService,
                      String period, String effectiveDateFrom, String effectiveDateTo, String operator,
                      OrganizationAccountingRateMapper organizationAccountingRateMapper) {
            this.type = type;
            this.org = org;
            this.period = period;
            this.ebsCallUrl = ebsCallUrl;
            this.authUserHelper = authUserHelper;
            this.dtsTokenService = dtsTokenService;
            this.idService = idService;
            this.dataImportLogMapper = dataImportLogMapper;
            this.organizationAccountingRateMapper = organizationAccountingRateMapper;
            this.effectiveDateFrom = effectiveDateFrom;
            this.effectiveDateTo = effectiveDateTo;
            this.operator = operator;
        }

        @Override
        public Integer call() {
            String response = null;
            String ledgerId = org.getEnterpriseAccountCode();
            String code = org.getCode();
            Integer periodInt = DateUtils.strToPeriod(period);
            int year = periodInt / 100;
            int month = periodInt % 100;
            DataImportLog log = new DataImportLog();
            if (type == EbsExtractTypeConstant.RATE) {
                log.setTaxpayerIdNum(Constant.ALL);
                log.setCompanyName(Constant.ALL);
                log.setOrganizationId(Constant.ALL);
                log.setCompanyCode(Constant.ALL);
            } else {
                log.setTaxpayerIdNum(org.getTaxPayerNumber());
                log.setCompanyName(org.getName());
                log.setOrganizationId(org.getId());
                log.setCompanyCode(code);
            }

            log.setPeriodYear(year);
            log.setPeriodMonth(month);
            log.setType(EnumImportType.ExtractFinancialData.getCode());
            log.setFileType(EnumEbsExtractType.getNameByCode(type));
            log.setPeriodStatus("EBS抽取");
            log.setOperator(operator);
            log.setId(idService.nextId());
            try {
                if (type == EbsExtractTypeConstant.RATE) {
                    //do nothing
                } else if (StringUtils.isEmpty(org.getEnterpriseAccountCode())) {
                    log.setImportResult(false);
                    log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), ErrorMessageCN.NoLedgerID));
                } else if (StringUtils.isEmpty(org.getCode())) {
                    log.setImportResult(false);
                    log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), ErrorMessageCN.NoCompanyCode));
                }
                if (StringUtils.isNotEmpty(log.getErrorMsg())) {
                    dataImportLogMapper.insertSelective(log);
                    return 0;
                }
                String secureToken = dtsTokenService.encryptInput();
                Map<String, String> headers = new HashMap<>();
                headers.put("secureToken", secureToken);
                switch (type) {
                    case EbsExtractTypeConstant.TB:
                        response = HttpUtil.post(ebsCallUrl + "/glMonthlyBal?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period,
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.JE:
                        // 这里BA反馈可按期间获取当月日记账即可
                        String effecDateFrom = DateUtils.getFirstDayOfMonth(year, month);
                        String effecDateTo = DateUtils.getLastDayOfMonth(year, month);
                        response = HttpUtil.post(ebsCallUrl + "/glJeLines" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&effectiveDateFrom=" + effecDateFrom + "&effectiveDateTo=" + effecDateTo,
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.BSPRC:
                        response = HttpUtil.post(ebsCallUrl + "/fsgAsset" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period + "&prcFlag=Y",
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.PLPRC:
                        response = HttpUtil.post(ebsCallUrl + "/fsgProfit" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period + "&prcFlag=Y",
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.BS:
                        response = HttpUtil.post(ebsCallUrl + "/fsgAsset" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period + "&prcFlag=N",
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.PL:
                        response = HttpUtil.post(ebsCallUrl + "/fsgProfit" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period + "&prcFlag=N",
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.CF:
                        response = HttpUtil.post(ebsCallUrl + "/fsgCash" + "?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period,
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.OCTB:
                        break;
                    case EbsExtractTypeConstant.RATE:
                        response = HttpUtil.post(ebsCallUrl + "/dailyRates" + "?period=" + period,
                                headers, "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    default:
                        break;
                }
                logger.info("EBS response: " + response);
                if (StringUtils.isNotEmpty(response)) {
                    EbsCallResp resp = JSON.parseObject(response, EbsCallResp.class);
                    long taskId = Long.valueOf(resp.getTaskId());
                    if (taskId == -1) {
                        log.setImportResult(false);
                        log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), resp.getTaskDesc()));
                    } else {
                        log.setId(taskId);
                        log.setImportResult(true);
                        log.setErrorMsg(String.format(EnumApiCodeMsg.CALLSUCCESS.getMsg(), resp.getTaskDesc()));
                        if (EbsExtractTypeConstant.JE == type) {
                            resp.setCompanyCode(null != resp.getSegment1() ? resp.getSegment1() : "");
                        } else if (EbsExtractTypeConstant.RATE == type) {
                            //只有汇率表是同步进行操作，所以在得到response之前已经进行数据插入及记录的更新，
                            // 在此进行记录条数的设置
                            logger.info("此时操作汇率表");
                            OrganizationAccountingRateExample example = new OrganizationAccountingRateExample();
                            //期间
                            example.createCriteria().andPeriodEqualTo(convertPeriodStr2Int(period)).andTaskIdEqualTo(String.valueOf(taskId));
                            List<OrganizationAccountingRate> itemList = organizationAccountingRateMapper.selectByExample(example);
                            logger.info("查询的汇率条数:" + itemList.size());
                            log.setRecordSize(itemList.size());
                            log.setErrorMsg(resp.getTaskDesc());
                        }
                    }
                }
                dataImportLogMapper.insertSelective(log);
            } catch (Exception e) {
                logger.error(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), e.getMessage()), e);
                log.setImportResult(false);
                log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), e.getMessage()));
                dataImportLogMapper.insertSelective(log);

            }
            return 1;
        }

        /**
         * 期间格式转换为原始的int类型
         *
         * @param periodStr
         * @return
         */
        private Integer convertPeriodStr2Int(String periodStr) {
            periodStr = StringUtils.replace(periodStr, "-", "");
            Integer period = Integer.parseInt(periodStr);
            return period;
        }

    }

}
