package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.constant.CountTypeConstant;
import pwc.taxtech.atms.constant.ExportTemplatePathConstant;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.dto.vatdto.dd.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.dd.*;
import pwc.taxtech.atms.dto.vatdto.excelheader.CashFlowHeader;
import pwc.taxtech.atms.dto.vatdto.excelheader.CertifiedInvoicesListHeader;
import pwc.taxtech.atms.dto.vatdto.excelheader.ProfitLossHeader;
import pwc.taxtech.atms.dto.vatdto.excelheader.TrialBalanceHeader;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.TrialBalanceCondition;
import pwc.taxtech.atms.vat.dpo.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Auther: Gary J Li
 * @Date: 01/02/2019 10:36
 * @Description:
 */
@Service
public class DataPreviewSerivceImpl extends BaseService {

    @Resource
    private TrialBalanceMapper trialBalanceMapper;

    @Resource
    private ProfitLossStatementPrcMapper profitLossStatementPrcMapper;

    @Resource
    private ProfitLossStatementPrcManualMapper profitLossStatementPrcManualMapper;

    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;

    @Resource
    private JournalEntryMapper journalEntryMapper;

    @Resource
    private BalanceSheetPrcMapper balanceSheetPrcMapper;

    @Resource
    private BalanceSheetPrcManualMapper balanceSheetPrcManualMapper;

    @Resource
    private BalanceSheetMapper balanceSheetMapper;

    @Resource
    private CashFlowMapper cashFlowMapper;

    @Resource
    private CashFlowManualMapper cashFlowManualMapper;

    @Resource
    private InvoiceRecordMapper invoiceRecordMapper;

    @Resource
    private RedLetterInfoTableMapper redLetterInfoTableMapper;

    @Resource
    private CoupaPurchasingReportMapper coupaPurchasingReportMapper;

    @Resource
    private CertifiedInvoicesListMapper certifiedInvoicesListMapper;

    @Resource
    private InvoiceDataMapper invoiceDataMapper;

    @Resource
    private OrganizationMapper organizationMapper;

    @Resource
    private AdjustmentTableMapper adjustmentTableMapper;

    @Resource
    private CommonDocumentHelper commonDocumentHelper;

    /**
     * 查询科目余额表
     * @param param
     * @return
     */
    public PageInfo<TrialBalanceDto> getTBDataForDisplay(TrialBalanceParam param) {

        TrialBalanceCondition condition = beanUtil.copyProperties(param, new TrialBalanceCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<TrialBalance> trialBalances = trialBalanceMapper.selectByCondition(condition);
        List<TrialBalanceDto> trialBalanceDtos = Lists.newArrayList();

        trialBalances.forEach(tb -> {
            TrialBalanceDto trialBalanceDto = new TrialBalanceDto();
            beanUtil.copyProperties(tb, trialBalanceDto);
            trialBalanceDtos.add(trialBalanceDto);
        });

        PageInfo<TrialBalanceDto> pageInfo = new PageInfo<>(trialBalanceDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    /**
     * 截至本期科目余额表
     * @param param
     * @return
     */
    public PageInfo<TrialBalanceDto> getTBEndPeriodDataForDisplay(TrialBalanceParam param) {
        TrialBalanceCondition condition = beanUtil.copyProperties(param, new TrialBalanceCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<TrialBalance> trialBalances = trialBalanceMapper.selectByConditionGroupTen(condition);

        List<TrialBalanceDto> tbEndPeriodConditionDate=getTBEndPeriodConditionDate(trialBalances,condition);

        PageInfo<TrialBalanceDto> pageInfo = new PageInfo<>(tbEndPeriodConditionDate);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public HttpServletResponse exportTBEndPeriodList(HttpServletResponse response, TrialBalanceParam param, String fileName) {
        logger.debug("start export TBEndPeriod list to excel");
        TrialBalanceCondition condition = beanUtil.copyProperties(param, new TrialBalanceCondition());
        //根据条件选择对应月份模板
        Integer month=condition.getPeriodEnd()%100;
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.TRIAL_BALANCE_PERIOD+month+".xlsx";
        List<TrialBalance> trialBalances = trialBalanceMapper.selectByConditionGroupTen(condition);
        List<TrialBalanceDto> tbEndPeriodConditionDate=getTBEndPeriodConditionDate(trialBalances,condition);
        TrialBalanceHeader header = new TrialBalanceHeader();
        if (tbEndPeriodConditionDate.size() > 0) {
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            header.setCompanyNameCn(org.getName());
            header.setPeriodEnd(param.getPeriodEnd());
            header.setLedgerName(tbEndPeriodConditionDate.get(0).getLedgerName());
            header.setLedgerCurrencyCode(tbEndPeriodConditionDate.get(0).getCurrencyCode());
            header.setPeriodYear(condition.getPeriodEnd()/100);
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(header, tbEndPeriodConditionDate, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    /**
     * 根据条件获取截止本期科目余额表，公共方法用于导出
     * @param trialBalances
     * @param condition
     * @return
     */
    private List<TrialBalanceDto> getTBEndPeriodConditionDate(List<TrialBalance> trialBalances,TrialBalanceCondition condition) {

        List<TrialBalanceDto> trialBalanceDtos = Lists.newArrayList();
        //查询单组数据封装
        for(TrialBalance item:trialBalances){
            condition.setSegment1(item.getSegment1());
            condition.setSegment2(item.getSegment2());
            condition.setSegment3(item.getSegment3());
            condition.setSegment4(item.getSegment4());
            condition.setSegment5(item.getSegment5());
            condition.setSegment6(item.getSegment6());
            condition.setSegment7(item.getSegment7());
            condition.setSegment8(item.getSegment8());
            condition.setSegment9(item.getSegment9());
            condition.setSegment10(item.getSegment10());
            //按时间倒序查询
            List<TrialBalance> trialBalances1 = trialBalanceMapper.selectByGroupCondition(condition);
            TrialBalanceDto trialBalanceDto = new TrialBalanceDto();
            beanUtil.copyProperties(trialBalances1.get(0), trialBalanceDto);
            //搜集不同月份信息
            Map<Integer,BigDecimal> map=new HashMap<>();
            for(TrialBalance a:trialBalances1){
                //判断key期间是否存在
                if(map.containsKey(a.getPeriod())){
                    //获取已存在value
                    BigDecimal bigOrg=(BigDecimal)Optional.ofNullable(map.get(a.getPeriod())).orElse(BigDecimal.ZERO);
                    map.put(a.getPeriod(),(a.getPeriodDrBeq().subtract(a.getPeriodCrBeq())).add(bigOrg));
                }else{
                    map.put(a.getPeriod(),a.getPeriodDrBeq().subtract(a.getPeriodCrBeq()));
                }
            }
            trialBalanceDto.setPeriodMap(map);
            //通过反射动态给月份赋值
            setMonthBalance(map,trialBalanceDto);
            //设置年初年末余额
            setBegEndBeq(trialBalances1,trialBalanceDto);
            trialBalanceDtos.add(trialBalanceDto);
        }
        return trialBalanceDtos;
    }


    /**
     * 设置年初年末余额
     * @param list
     * @param dto
     * @return
     */
    private TrialBalanceDto setBegEndBeq(List<TrialBalance> list,TrialBalanceDto dto){
        if(list.size()>0){
            //重新排序时间升序
            list=list.stream()
                    .sorted(Comparator.comparing(TrialBalance::getUpdateTime))
                    .collect(Collectors.toList());
            //年初余额—— 使用period等于YYYY-01的beg_bal_beq（*是本位币本期期初余额），
            // 即卡片期间的所在年的的1月的本位币本期期初余额
            dto.setBegBalBeq(list.get(0).getBegBalBeq());
            //年底余额——  使用period等于YYYY-MM（页面选择的期间）的end_bal_beq（*是本位币本期期末余额），
            // 即卡片期间的所在年的页面选择期间月的本位币本期期末余额
            dto.setEndBalBeq(list.get(list.size()-1).getEndBalBeq());
        }
        return dto;
    }

    /**
     * 通过反射动态给月份赋值
     * @param map
     * @param trialBalanceDto
     * @return
     */
    private TrialBalanceDto setMonthBalance(Map<Integer,BigDecimal> map,TrialBalanceDto trialBalanceDto){
        logger.debug("map : {}",map.toString());
        for(Map.Entry<Integer,BigDecimal> entry:map.entrySet()){
            //求余取月份
            Integer month=entry.getKey()%100;
            //根据反射动态设置属性值
            String filedName="monthBalance"+month;
            try {
                Field name=trialBalanceDto.getClass().getDeclaredField(filedName);
                name.setAccessible(true);
                name.set(trialBalanceDto,entry.getValue());
                name.setAccessible(false);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return trialBalanceDto;
    }


    /**
     * 查询利润表PRC
     * @param param
     * @return
     */
    public PageInfo<ProfitLossStatementDto> getPLDataForDisplay(ProfitLossStatementParam param) {

        ProfitLossStatementCondition condition = beanUtil.copyProperties(param, new ProfitLossStatementCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<ProfitLossStatementPrc> profitLossStatements = profitLossStatementPrcMapper.selectByCondition(condition);
        List<ProfitLossStatementDto> profitLossDtos = Lists.newArrayList();

        profitLossStatements.forEach(pl -> {
            ProfitLossStatementDto profitLossDto = new ProfitLossStatementDto();
            beanUtil.copyProperties(pl, profitLossDto);
            profitLossDtos.add(profitLossDto);
        });

        PageInfo<ProfitLossStatementDto> pageInfo = new PageInfo<>(profitLossDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<ProfitLossStatementDto> getPLStatementDataForDisplay(ProfitLossStatementParam param) {

        ProfitLossStatementCondition condition = beanUtil.copyProperties(param, new ProfitLossStatementCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<ProfitLossStatement> profitLossStatements = profitLossStatementMapper.selectByCondition(condition);
        List<ProfitLossStatementDto> profitLossDtos = Lists.newArrayList();

        profitLossStatements.forEach(pl -> {
            ProfitLossStatementDto profitLossDto = new ProfitLossStatementDto();
            beanUtil.copyProperties(pl, profitLossDto);
            profitLossDtos.add(profitLossDto);
        });

        PageInfo<ProfitLossStatementDto> pageInfo = new PageInfo<>(profitLossDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<ProfitLossStatementPrc> getPLprcManualDataForDisplay(ProfitLossStatementParam param) {

        ProfitLossStatementCondition condition = beanUtil.copyProperties(param, new ProfitLossStatementCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<ProfitLossStatementPrc> profitLossStatements = profitLossStatementPrcManualMapper.selectByCondition1(condition);
        PageInfo<ProfitLossStatementPrc> pageInfo =new PageInfo<>(profitLossStatements);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<CashFlowDto> getCFDataForDisplay(CashFlowParam param) {
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        //Integer totalCount=cashFlowMapper.selectCountByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        List<CashFlow> cashFlows = cashFlowMapper.selectByCondition(condition);
        //使用page的getTotal()
        Long total = page.getTotal();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        PageInfo<CashFlowDto> pageInfo = new PageInfo<>(cashFlowDtos);
        pageInfo.setTotal(total);
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public PageInfo<CashFlowDto> getCFManualDataForDisplay(CashFlowParam param) {
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        //Integer totalCount=cashFlowMapper.selectCountByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        List<CashFlow> cashFlows = cashFlowManualMapper.selectByCondition(condition);
        //使用page的getTotal()
        Long total = page.getTotal();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        PageInfo<CashFlowDto> pageInfo = new PageInfo<>(cashFlowDtos);
        pageInfo.setTotal(total);
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public PageInfo<AdjustmentTableDto> getAdjustmentTbDataForDisplay(AdjustmentTableParam param) {

        AdjustmentTableCondition condition = beanUtil.copyProperties(param, new AdjustmentTableCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<AdjustmentTable> adjustmentTables = adjustmentTableMapper.selectByCondition(condition);
        List<AdjustmentTableDto> adjustmentTableDtos = Lists.newArrayList();

        adjustmentTables.forEach(a -> {
            AdjustmentTableDto adjustmentTableDto = new AdjustmentTableDto();
            beanUtil.copyProperties(a, adjustmentTableDto);
            adjustmentTableDtos.add(adjustmentTableDto);
        });

        PageInfo<AdjustmentTableDto> pageInfo =new PageInfo<>(adjustmentTableDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public HttpServletResponse exportCashFlowList(HttpServletResponse response, CashFlowParam param, String fileName) {
        //Boolean isEn = StringUtils.equals(language, "en-us");
        logger.debug("start export CashFlow list to excel");
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.CASH_FLOW;
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        List<CashFlow> cashFlows = cashFlowMapper.selectByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        CashFlowHeader cashFlowHeader = new CashFlowHeader();
        if (cashFlowDtos.size() > 0) {
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            cashFlowHeader.setCompanyNameCn(org.getName());
            cashFlowHeader.setPeriodStart(param.getPeriodStart());
            cashFlowHeader.setPeriodEnd(param.getPeriodEnd());
            cashFlowHeader.setLedgerName(cashFlowDtos.get(0).getLedgerName());
            cashFlowHeader.setLedgerCurrencyCode(cashFlowDtos.get(0).getLedgerCurrencyCode());
            cashFlowHeader.setStatus(cashFlowDtos.get(0).getStatus());
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(cashFlowHeader, cashFlowDtos, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public HttpServletResponse exportCashFlowManualList(HttpServletResponse response, CashFlowParam param, String fileName) {
        //Boolean isEn = StringUtils.equals(language, "en-us");
        logger.debug("start export CashFlow list to excel");
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.CASH_FLOW_MANUAL;
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        List<CashFlow> cashFlows = cashFlowManualMapper.selectByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        CashFlowHeader cashFlowHeader = new CashFlowHeader();
        if (cashFlowDtos.size() > 0) {
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            cashFlowHeader.setCompanyNameCn(org.getName());
            cashFlowHeader.setCompanyCode(org.getCode());
            cashFlowHeader.setPeriodStart(param.getPeriodStart());
            cashFlowHeader.setLedgerName(cashFlowDtos.get(0).getLedgerName());
            cashFlowHeader.setLedgerCurrencyCode(cashFlowDtos.get(0).getLedgerCurrencyCode());
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(cashFlowHeader, cashFlowDtos, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public HttpServletResponse exportAdjustmentTableList(HttpServletResponse response, AdjustmentTableParam param, String fileName) {
        //Boolean isEn = StringUtils.equals(language, "en-us");
        logger.debug("start export AdjustmentTable list to excel");
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.ADJUSTMENT_TABLE;
        AdjustmentTableCondition condition = new AdjustmentTableCondition();
        beanUtil.copyProperties(param, condition);
        List<AdjustmentTable> adjustmentTables = adjustmentTableMapper.selectByCondition(condition);
        List<AdjustmentTableDto> adjustmentTableDtos = Lists.newArrayList();
        adjustmentTables.forEach(a -> {
            AdjustmentTableDto adjustmentTableDto = new AdjustmentTableDto();
            beanUtil.copyProperties(a, adjustmentTableDto);
            adjustmentTableDtos.add(adjustmentTableDto);
        });

        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(null, adjustmentTableDtos, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public HttpServletResponse exportBalanceSheetPrcManualList(HttpServletResponse response, BalanceSheetParam param, String fileName) {
        logger.debug("start export BalanceSheetPrcManual list to excel");
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.BALANCE_SHEET_PRC_MANUAL;
        BalanceSheetCondition condition = new BalanceSheetCondition();
        beanUtil.copyProperties(param, condition);
        List<BalanceSheetPrc> balanceSheets = balanceSheetPrcManualMapper.selectByCondition1(condition);
        List<BalanceSheetExportDto> exportDtos = Lists.newArrayList();
        balanceSheets.forEach(id -> {
            BalanceSheetExportDto exportDto = beanUtil.copyProperties(id, new BalanceSheetExportDto());
            exportDtos.add(exportDto);
        });
        ProfitLossHeader header=new ProfitLossHeader();
        if(balanceSheets.size()>0){
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            header.setCompanyNameCn(org.getName());
            header.setPeriodStart(param.getPeriodStart());
            header.setPeriodEnd(param.getPeriodEnd());
            header.setEntityCode(org.getCode());
            header.setLedgerName(balanceSheets.get(0).getLedgerName());
            header.setLedgerCurrencyCode(balanceSheets.get(0).getLedgerCurrencyCode());
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(header, exportDtos, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public HttpServletResponse exportProfitLossPrcManualList(HttpServletResponse response, ProfitLossStatementParam param, String fileName) {
        logger.debug("start export ProfitLossPRCManual list to excel");
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.PROFIT_LOSS_STATEMENT_PRC_MANUAL_REPORT;
        ProfitLossStatementCondition condition = new ProfitLossStatementCondition();
        beanUtil.copyProperties(param, condition);
        List<ProfitLossStatementPrc> profitLossStatements = profitLossStatementPrcManualMapper.selectByCondition1(condition);
        ProfitLossHeader header=new ProfitLossHeader();
        if(profitLossStatements.size()>0){
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            header.setCompanyNameCn(org.getName());
            header.setPeriodStart(param.getPeriodStart());
            header.setPeriodEnd(param.getPeriodEnd());
            header.setEntityCode(org.getCode());
            header.setLedgerName(profitLossStatements.get(0).getLedgerName());
            header.setLedgerCurrencyCode(profitLossStatements.get(0).getLedgerCurrencyCode());
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(header, profitLossStatements, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public PageInfo<JournalEntryDto> getJEDataForDisplay(JournalEntryParam param) {

        JournalEntryCondition condition = beanUtil.copyProperties(param, new JournalEntryCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<JournalEntry> journalEntrys = journalEntryMapper.selectByCondition(condition);
        List<JournalEntryDto> journalEntryDtos = Lists.newArrayList();

        journalEntrys.forEach(je -> {
            JournalEntryDto journalEntryDto = new JournalEntryDto();
            beanUtil.copyProperties(je, journalEntryDto);
            journalEntryDtos.add(journalEntryDto);
        });

        PageInfo<JournalEntryDto> pageInfo = new PageInfo<>(journalEntryDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<BalanceSheetDto> getBSDataForDisplay(BalanceSheetParam param) {

        BalanceSheetCondition condition = beanUtil.copyProperties(param, new BalanceSheetCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<BalanceSheetPrc> bsPrcList = balanceSheetPrcMapper.selectByCondition(condition);
        List<BalanceSheetDto> balanceSheetDtos = Lists.newArrayList();

        bsPrcList.forEach(bs -> {
            BalanceSheetDto balanceSheetDto = new BalanceSheetDto();
            beanUtil.copyProperties(bs, balanceSheetDto);
            balanceSheetDtos.add(balanceSheetDto);
        });

        PageInfo<BalanceSheetDto> pageInfo = new PageInfo<>(balanceSheetDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<BalanceSheetDto> getBalanceSheetDataForDisplay(BalanceSheetParam param) {

        BalanceSheetCondition condition = beanUtil.copyProperties(param, new BalanceSheetCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<BalanceSheet> bsPrcList = balanceSheetMapper.selectByCondition(condition);
        List<BalanceSheetDto> balanceSheetDtos = Lists.newArrayList();

        bsPrcList.forEach(bs -> {
            BalanceSheetDto balanceSheetDto = new BalanceSheetDto();
            beanUtil.copyProperties(bs, balanceSheetDto);
            balanceSheetDtos.add(balanceSheetDto);
        });

        PageInfo<BalanceSheetDto> pageInfo = new PageInfo<>(balanceSheetDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<BalanceSheetDto> getBSprcManualDataForDisplay(BalanceSheetParam param) {

        BalanceSheetCondition condition = beanUtil.copyProperties(param, new BalanceSheetCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<BalanceSheetPrc> bsPrcList = balanceSheetPrcManualMapper.selectByCondition1(condition);
        List<BalanceSheetDto> balanceSheetDtos = Lists.newArrayList();

        bsPrcList.forEach(bs -> {
            BalanceSheetDto balanceSheetDto = new BalanceSheetDto();
            beanUtil.copyProperties(bs, balanceSheetDto);
            balanceSheetDtos.add(balanceSheetDto);
        });

        PageInfo<BalanceSheetDto> pageInfo =new PageInfo<>(balanceSheetDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<InvoiceRecordDto> getIRDataForDisplay(InvoiceRecordParam param) {

        InvoiceRecordCondition condition = beanUtil.copyProperties(param, new InvoiceRecordCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<InvoiceRecord> invoiceRecords = invoiceRecordMapper.selectByCondition(condition);

        List<InvoiceRecordDto> invoiceRecordDtos = Lists.newArrayList();

        invoiceRecords.forEach(bs -> {
            InvoiceRecordDto balanceSheetDto = beanUtil.copyProperties(bs, new InvoiceRecordDto());
            invoiceRecordDtos.add(balanceSheetDto);
        });

        PageInfo<InvoiceRecordDto> pageInfo = new PageInfo<>(invoiceRecordDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<RedLetterInfoTableDto> getRLITDataForDisplay(RedLetterInfoTableParam param) {

        RedLetterInfoTableCondition condition = beanUtil.copyProperties(param, new RedLetterInfoTableCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<RedLetterInfoTable> redLetterInfoTables = redLetterInfoTableMapper.selectByCondition(condition);
        List<RedLetterInfoTableDto> redLetterInfoDtos = Lists.newArrayList();

        redLetterInfoTables.forEach(bs -> {
            RedLetterInfoTableDto redLetterInfoDto = beanUtil.copyProperties(bs, new RedLetterInfoTableDto());
            redLetterInfoDtos.add(redLetterInfoDto);
        });

        PageInfo<RedLetterInfoTableDto> pageInfo = new PageInfo<>(redLetterInfoDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }


    public PageInfo<CoupaPurchasingReportDto> getCPRDataForDisplay(CoupaPurchasingReportParam param) {

        CoupaPurchasingReportCondition condition = beanUtil.copyProperties(param, new CoupaPurchasingReportCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<CoupaPurchasingReport> coupaPurchasingReports = coupaPurchasingReportMapper.selectByCondition(condition);
        List<CoupaPurchasingReportDto> coupaPurchasingReportDtos = Lists.newArrayList();

        coupaPurchasingReports.forEach(cpr -> {
            CoupaPurchasingReportDto coupaPurchasingReportDto = beanUtil.copyProperties(cpr, new CoupaPurchasingReportDto());
            coupaPurchasingReportDtos.add(coupaPurchasingReportDto);
        });

        PageInfo<CoupaPurchasingReportDto> pageInfo = new PageInfo<>(coupaPurchasingReportDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }


    public PageInfo<CertifiedInvoicesListDto> getCILDataForDisplay(CertifiedInvoicesListParam param) {

        CertifiedInvoicesListCondition condition = beanUtil.copyProperties(param, new CertifiedInvoicesListCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<CertifiedInvoicesList> certifiedInvoicesLists = certifiedInvoicesListMapper.selectByCondition(condition);
        List<CertifiedInvoicesListDto> certifiedInvoicesListDtos = Lists.newArrayList();

        certifiedInvoicesLists.forEach(cil -> {
            CertifiedInvoicesListDto CertifiedInvoicesListDto = beanUtil.copyProperties(cil, new CertifiedInvoicesListDto());
            certifiedInvoicesListDtos.add(CertifiedInvoicesListDto);
        });
        PageInfo<CertifiedInvoicesListDto> pageInfo = new PageInfo<>(certifiedInvoicesListDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    /**
     * 28/02/2019 10:18
     * 导入时纵向存的，因为需求是行数一般不会增加，列数可能增加。查询时90度旋转
     * 后续增加列 1、添加CountTypeConstant(按列递增) 2、增加setIDValue case 3、js添加column
     * [param]
     *
     * @return
     * @author Gary J Li
     */
    public PageInfo<InvoiceDataDto> getIDDataForDisplay(InvoiceDataParam param) {
        InvoiceDataCondition condition = beanUtil.copyProperties(param, new InvoiceDataCondition());
        PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<InvoiceData> invoiceDatas = invoiceDataMapper.selectByCondition(condition);
        List<InvoiceDataDto> invoiceDataDtos = generalIDDtoList();
        generalIDDtoList(invoiceDatas, invoiceDataDtos);
        PageInfo<InvoiceDataDto> pageInfo = new PageInfo<>(invoiceDataDtos);
        pageInfo.setTotal(invoiceDataDtos.size());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public int getTBDownloadFilePath(TrialBalanceParam param, OutputStream os) {
        try {
            TrialBalanceCondition condition = new TrialBalanceCondition();
            beanUtil.copyProperties(param, condition);
            List<TrialBalance> trialBalances = trialBalanceMapper.selectByCondition(condition);

            Map<String, String> header = generalTBHeader();
            List<TrialBalanceExportDto> cellList = new ArrayList<>();
            trialBalances.forEach(tb -> {
                TrialBalanceExportDto d = new TrialBalanceExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("科目余额表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }


    public int getPLDownloadFilePath(ProfitLossStatementParam param, OutputStream os) {
        try {

            ProfitLossStatementCondition condition = new ProfitLossStatementCondition();
            beanUtil.copyProperties(param, condition);
            List<ProfitLossStatementPrc> profitLossStatements = profitLossStatementPrcMapper.selectByCondition(condition);
            Map<String, String> header = generalPLHeader();
            List<ProfitLossStatementExportDto> cellList = new ArrayList<>();
            profitLossStatements.forEach(pl -> {
                ProfitLossStatementExportDto d = new ProfitLossStatementExportDto();
                d = beanUtil.copyProperties(pl, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("利润表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getPLStatementDownloadFilePath(ProfitLossStatementParam param, OutputStream os) {
        try {

            ProfitLossStatementCondition condition = new ProfitLossStatementCondition();
            beanUtil.copyProperties(param, condition);
            List<ProfitLossStatement> profitLossStatements = profitLossStatementMapper.selectByCondition(condition);
            Map<String, String> header = generalPLHeader();
            List<ProfitLossStatementExportDto> cellList = new ArrayList<>();
            profitLossStatements.forEach(pl -> {
                ProfitLossStatementExportDto d = new ProfitLossStatementExportDto();
                d = beanUtil.copyProperties(pl, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("利润表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getJEDownloadFilePath(JournalEntryParam param, OutputStream os) {
        try {
            JournalEntryCondition condition = new JournalEntryCondition();
            beanUtil.copyProperties(param, condition);
            List<JournalEntry> journalEntries = journalEntryMapper.selectByCondition(condition);

            Map<String, String> header = generalJEHeader();
            List<JournalEntryExportDto> cellList = new ArrayList<>();
            journalEntries.forEach(tb -> {
                JournalEntryExportDto d = new JournalEntryExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("日记账导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getBSDownloadFilePath(BalanceSheetParam param, OutputStream os) {
        try {
            BalanceSheetCondition condition = new BalanceSheetCondition();
            beanUtil.copyProperties(param, condition);
            List<BalanceSheetPrc> balanceSheets = balanceSheetPrcMapper.selectByCondition(condition);
            Map<String, String> header = generalBSHeader();
            List<BalanceSheetExportDto> cellList = new ArrayList<>();
            balanceSheets.forEach(bs -> {
                BalanceSheetExportDto d = new BalanceSheetExportDto();
                d = beanUtil.copyProperties(bs, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("资产负债表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getBalanceSheetDownloadFilePath(BalanceSheetParam param, OutputStream os) {
        try {
            BalanceSheetCondition condition = new BalanceSheetCondition();
            beanUtil.copyProperties(param, condition);
            List<BalanceSheet> balanceSheets = balanceSheetMapper.selectByCondition(condition);
            Map<String, String> header = generalBSHeader();
            List<BalanceSheetExportDto> cellList = new ArrayList<>();
            balanceSheets.forEach(bs -> {
                BalanceSheetExportDto d = new BalanceSheetExportDto();
                d = beanUtil.copyProperties(bs, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("资产负债表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public HttpServletResponse exportInvoiceRecordList(HttpServletResponse response, InvoiceRecordParam param, String fileName) {
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.INVOICES_RECORD;
        InvoiceRecordCondition condition = beanUtil.copyProperties(param, new InvoiceRecordCondition());
        List<InvoiceRecord> invoiceDatas = invoiceRecordMapper.selectByCondition(condition);
        List<InvoiceRecordDto> invoiceDataDtos = Lists.newArrayList();
        invoiceDatas.forEach(id -> {
            InvoiceRecordDto invoiceRecordDto = beanUtil.copyProperties(id, new InvoiceRecordDto());
            invoiceDataDtos.add(invoiceRecordDto);
        });
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(invoiceDataDtos, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出发票记录异常:%s", e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse exportCILList(HttpServletResponse response, CertifiedInvoicesListParam param, String fileName) {
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.CERTIFIED_INVOICES_LIST;
        CertifiedInvoicesListCondition condition = beanUtil.copyProperties(param, new CertifiedInvoicesListCondition());
        List<CertifiedInvoicesList> datas = certifiedInvoicesListMapper.selectByCondition(condition);
        if (datas.size() < 1) {
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        CertifiedInvoicesListHeader header = new CertifiedInvoicesListHeader();
        Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
        header.setTaxPayerNumber(org.getTaxPayerNumber());
        header.setPeriod(param.getPeriodStart());
        header.setUnit(datas.get(0).getUnit());
        List<CertifiedInvoicesListDto> dtoList = Lists.newArrayList();
        datas.forEach(cil -> {
            CertifiedInvoicesListDto dto = beanUtil.copyProperties(cil, new CertifiedInvoicesListDto());
            dtoList.add(dto);
        });
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(header, dtoList, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出已认证发票清单异常:%s", e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse exportCPRList(HttpServletResponse response, CoupaPurchasingReportParam param, String fileName) {
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.COUPA_PURCHASING_REPORT;
        CoupaPurchasingReportCondition condition = beanUtil.copyProperties(param, new CoupaPurchasingReportCondition());
        List<CoupaPurchasingReport> datas = coupaPurchasingReportMapper.selectByCondition(condition);
        if (datas.size() < 1) {
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        List<CoupaPurchasingReportDto> dtos = Lists.newArrayList();
        datas.forEach(d -> {
            CoupaPurchasingReportDto dto = beanUtil.copyProperties(d, new CoupaPurchasingReportDto());
            dtos.add(dto);
        });
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(dtos, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出Coupa发票报告异常:%s", e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse exportRLITList(HttpServletResponse response, RedLetterInfoTableParam param, String fileName) {
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.RED_LETTER_INFO_TAB;
        RedLetterInfoTableCondition condition = beanUtil.copyProperties(param, new RedLetterInfoTableCondition());
        List<RedLetterInfoTable> datas = redLetterInfoTableMapper.selectByCondition(condition);
        if (datas.size() < 1) {
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        List<RedLetterInfoTableDto> dtos = Lists.newArrayList();
        datas.forEach(d -> {
            RedLetterInfoTableDto dto = beanUtil.copyProperties(d, new RedLetterInfoTableDto());
            dtos.add(dto);
        });
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(dtos, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出红字信息表异常:%s", e.getMessage()));
        }
        return null;
    }

    public HttpServletResponse exportIDList(HttpServletResponse response, InvoiceDataParam param, String fileName) {
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = ExportTemplatePathConstant.INVOICE_DATA;
        InvoiceDataCondition condition = beanUtil.copyProperties(param, new InvoiceDataCondition());
        List<InvoiceData> datas = invoiceDataMapper.selectByCondition(condition);
        if (datas.size() < 1) {
            throw new ServiceException(ErrorMessage.ExportFailed);
        }
        List<InvoiceDataDto> dtos = generalIDDtoList();
        generalIDDtoList(datas, dtos);
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(dtos, excelTemplatePathInClassPath);
        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            logger.error(String.format("导出发票资料异常:%s", e.getMessage()));
        }
        return null;
    }

    private Map<String, String> generalTBHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date", "数据日期");
        header.put("Source", "来源");
        header.put("Period", "期间");
        header.put("LedgerId", "账套ID");
        header.put("LedgerName", "账套名称");
        header.put("CurrencyCode", "账套币种");
        header.put("Status", "关账标识");
        header.put("Category", "主体性质");
        header.put("AccountCategory", "科目性质");
        header.put("AcctCode1", "一级科目代码");

        header.put("AcctName1", "一级科目说明");
        header.put("AcctName2", "二级科目说明");
        header.put("AcctName3", "三级科目说明");
        header.put("Segment1", "主体代码");
        header.put("Segment2", "成本中心代码");
        header.put("Segment3", "科目代码");
        header.put("Segment4", "辅助科目代码");
        header.put("Segment5", "利润中心代码");
        header.put("Segment6", "产品代码");
        header.put("Segment7", "项目代码");

        header.put("Segment8", "公司间代码");
        header.put("Segment9", "备用1代码");
        header.put("Segment10", "备用2代码");
        header.put("Segment1Name", "主体说明");
        header.put("Segment2Name", "成本中心说明");
        header.put("Segment3Name", "科目说明");
        header.put("Segment4Name", "辅助科目说明");
        header.put("Segment5Name", "利润中心说明");
        header.put("Segment6Name", "产品说明");
        header.put("Segment7Name", "项目说明");

        header.put("Segment8Name", "公司间说明");
        header.put("Segment9Name", "备用1说明");
        header.put("Segment10Name", "备用2说明");
        header.put("BegBal", "原币本期期初余额");
        header.put("PeriodDr", "原币本期借方发生额");
        header.put("PeriodCr", "原币本期贷方发生额");
        header.put("EndBal", "原币本期期末余额");
        header.put("QtdDr", "原币本季借方发生额");
        header.put("QtdCr", "原币本季贷方发生额");
        header.put("YtdDr", "原币本年借方发生额");

        header.put("YtdCr", "原币本年贷方发生额");
        header.put("BegBalBeq", "本位币本期期初余额");
        header.put("PeriodDrBeq", "本位币本期借方发生额");
        header.put("PeriodCrBeq", "本位币本期贷方发生额");
        header.put("EndBalBeq", "本位币本期期末余额");
        header.put("QtdDrBeq", "本位币本季借方发生额");
        header.put("QtdCrBeq", "本位币本季贷方发生额");
        header.put("YtdDrBeq", "本位币本年借方发生额");
        header.put("YtdCrBeq", "本位币本年贷方发生额");
        return header;
    }

    private Map<String, String> generalBSHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date", "数据日期");
        header.put("Source", "来源");
        header.put("Period", "期间");
        header.put("Status", "关账标识");
        header.put("LedgerId", "账套ID");
        header.put("LedgerName", "账套名称");
        header.put("LedgerCurrencyCode", "账套币种");
        header.put("EntityCode", "机构编码");
        header.put("EntityName", "机构名称");
        header.put("Category", "主体性质");
        header.put("Frequency", "频度");
        header.put("ItemName", "项目名称");
        header.put("EndBal", "期末余额");
        header.put("BegBal", "年初余额");
        header.put("PrcFlag", "是否为国外");
        return header;
    }


    private Map<String, String> generalPLHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date", "数据日期");
        header.put("Source", "来源");
        header.put("Period", "期间");
        header.put("Status", "关账标识");
        header.put("LedgerId", "账套ID");
        header.put("LedgerName", "账套名称");
        header.put("LedgerCurrencyCode", "账套币种");
        header.put("EntityCode", "机构编码");
        header.put("EntityName", "机构名称");
        header.put("Category", "主体性质");

        header.put("Frequency", "频度");
        header.put("ItemName", "项目名称");
        header.put("PeriodAmt", "本期发生额");
        header.put("YtdAmt", "本年累计");
        header.put("PrcFlag", "是否为国外");
        return header;
    }


    private Map<String, String> generalJEHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date","数据日期");
        header.put("Source","来源");
        header.put("LedgerId","账套ID");
        header.put("LedgerName","账套名称");
        header.put("CurrencyCode","账套币种");
        header.put("Status","关账标识");
        header.put("HeaderId","日记账头ID");
        header.put("LineNum","日记账行号");
        header.put("ApprovalStatus","审批状态");
        header.put("PostedStatus","过账");

        header.put("Period","会计期间");
        header.put("AccountingDate","凭证日期");
        header.put("JournalSource","日记账来源");
        header.put("Category","日记账类别");
        header.put("Name","日记账名称");
        header.put("VoucherNum","凭证编号");
        header.put("Description","摘要");
        header.put("Segment1","主体代码");
        header.put("Segment2","成本中心");
        header.put("Segment3","科目代码");

        header.put("Segment4","辅助科目");
        header.put("Segment5","利润中心");
        header.put("Segment6","产品");
        header.put("Segment7","项目");
        header.put("Segment8","公司间");
        header.put("Segment9","备用1");
        header.put("Segment10","备用2");
        header.put("Segment1Name","主体说明");
        header.put("Segment2Name","成本中心说明");
        header.put("Segment3Name","科目说明");

        header.put("Segment4Name","辅助科目说明");
        header.put("Segment5Name","利润中心说明");
        header.put("Segment6Name","产品说明");
        header.put("Segment7Name","项目说明");
        header.put("Segment8Name","公司间说明");
        header.put("Segment9Name","备用1说明");
        header.put("Segment10Name","备用2说明");
        header.put("JournalCurrencyCode","币种");
        header.put("SobCurrencyCode","本位币币种");
        header.put("AccountedDr","借方金额");

        header.put("AccountedCr","贷方金额");
        header.put("EnteredDr","本位币借方金额");
        header.put("EnteredCr","本位币贷方金额");
        header.put("CfItem","现金流量表项");
        header.put("Attribute1","城市");
        header.put("Attribute2","交易日期");
        header.put("Attribute3","对方银行账号");
        header.put("Attribute4","银行流水号");
        header.put("Attribute5","供应商编号");
        header.put("Attribute6","交易单号");

        header.put("Attribute7","供应商名称");
        header.put("Attribute8","接收编码");
        header.put("Attribute9","制单人");
        header.put("Attribute10","审核人");
        header.put("Attribute11","成本中心部门描述1");
        header.put("Attribute12","成本中心部门描述2");
        header.put("Attribute13","成本中心部门描述3");
        header.put("Attribute14","成本中心部门描述4");
        header.put("Attribute15","成本中心部门描述5");
        header.put("Attribute16","成本中心部门描述6");

        header.put("CreatedBy","创建人");
        header.put("CreatedDate","创建日期");
        header.put("LateUpdatedBy","最后更新人");
        header.put("LateUpdatedDate","最后更新日期");
        header.put("tmsPeriod","税务系统期间");
        return header;
    }

    private void generalIDDtoList(List<InvoiceData> invoiceDatas, List<InvoiceDataDto> invoiceDataDtos) {
        if (invoiceDatas.size() > 0) {
            invoiceDatas.forEach(id -> {
                setIDValue(invoiceDataDtos, "专票正数发票份数", id.getCountType(), id.getSpecialInvoiceAmount1());
                setIDValue(invoiceDataDtos, "专票负数发票份数", id.getCountType(), id.getSpecialInvoiceAmount2());
                setIDValue(invoiceDataDtos, "专票正数废票份数", id.getCountType(), id.getSpecialInvoiceAmount3());
                setIDValue(invoiceDataDtos, "专票负数废票份数", id.getCountType(), id.getSpecialInvoiceAmount4());
                setIDValue(invoiceDataDtos, "专票销项正废金额", id.getCountType(), id.getSpecialInvoiceSalesAmount1());
                setIDValue(invoiceDataDtos, "专票销项正数金额", id.getCountType(), id.getSpecialInvoiceSalesAmount2());
                setIDValue(invoiceDataDtos, "专票销项负废金额", id.getCountType(), id.getSpecialInvoiceSalesAmount3());
                setIDValue(invoiceDataDtos, "专票销项负数金额", id.getCountType(), id.getSpecialInvoiceSalesAmount4());
                setIDValue(invoiceDataDtos, "专票实际销项金额", id.getCountType(), id.getSpecialInvoiceSalesAmount5());
                setIDValue(invoiceDataDtos, "专票销项正废税额", id.getCountType(), id.getSpecialInvoiceTaxAmount1());
                setIDValue(invoiceDataDtos, "专票销项正数税额", id.getCountType(), id.getSpecialInvoiceTaxAmount2());
                setIDValue(invoiceDataDtos, "专票销项负废税额", id.getCountType(), id.getSpecialInvoiceTaxAmount3());
                setIDValue(invoiceDataDtos, "专票销项负数税额", id.getCountType(), id.getSpecialInvoiceTaxAmount4());
                setIDValue(invoiceDataDtos, "专票实际销项税额", id.getCountType(), id.getSpecialInvoiceTaxAmount5());
                setIDValue(invoiceDataDtos, "普票正数发票份数", id.getCountType(), id.getInvoiceAmount1());
                setIDValue(invoiceDataDtos, "普票负数发票份数", id.getCountType(), id.getInvoiceAmount2());
                setIDValue(invoiceDataDtos, "普票正数废票份数", id.getCountType(), id.getInvoiceAmount3());
                setIDValue(invoiceDataDtos, "普票负数废票份数", id.getCountType(), id.getInvoiceAmount4());
                setIDValue(invoiceDataDtos, "普票销项正废金额", id.getCountType(), id.getInvoiceSalesAmount1());
                setIDValue(invoiceDataDtos, "普票销项正数金额", id.getCountType(), id.getInvoiceSalesAmount2());
                setIDValue(invoiceDataDtos, "普票销项负废金额", id.getCountType(), id.getInvoiceSalesAmount3());
                setIDValue(invoiceDataDtos, "普票销项负数金额", id.getCountType(), id.getInvoiceSalesAmount4());
                setIDValue(invoiceDataDtos, "普票实际销项金额", id.getCountType(), id.getInvoiceSalesAmount5());
                setIDValue(invoiceDataDtos, "普票销项正废税额", id.getCountType(), id.getInvoiceTaxAmount1());
                setIDValue(invoiceDataDtos, "普票销项正数税额", id.getCountType(), id.getInvoiceTaxAmount2());
                setIDValue(invoiceDataDtos, "普票销项负废税额", id.getCountType(), id.getInvoiceTaxAmount3());
                setIDValue(invoiceDataDtos, "普票销项负数税额", id.getCountType(), id.getInvoiceTaxAmount4());
                setIDValue(invoiceDataDtos, "普票实际销项税额", id.getCountType(), id.getInvoiceTaxAmount5());
            });
        }
    }

    private List<InvoiceDataDto> generalIDDtoList() {
        List<InvoiceDataDto> invoiceDataDtos = Lists.newArrayList();
        invoiceDataDtos.add(new InvoiceDataDto("专票正数发票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("专票负数发票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("专票正数废票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("专票负数废票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项正废金额"));

        invoiceDataDtos.add(new InvoiceDataDto("专票销项正数金额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项负废金额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项负数金额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票实际销项金额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项正废税额"));

        invoiceDataDtos.add(new InvoiceDataDto("专票销项正数税额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项负废税额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票销项负数税额"));
        invoiceDataDtos.add(new InvoiceDataDto("专票实际销项税额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票正数发票份数"));

        invoiceDataDtos.add(new InvoiceDataDto("普票负数发票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("普票正数废票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("普票负数废票份数"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项正废金额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项正数金额"));

        invoiceDataDtos.add(new InvoiceDataDto("普票销项负废金额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项负数金额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票实际销项金额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项正废税额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项正数税额"));

        invoiceDataDtos.add(new InvoiceDataDto("普票销项负废税额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票销项负数税额"));
        invoiceDataDtos.add(new InvoiceDataDto("普票实际销项税额"));
        return invoiceDataDtos;
    }

    private void setIDValue(List<InvoiceDataDto> idDtos, String column, Integer row, Object value) {
        for (InvoiceDataDto idto : idDtos) {
            if (idto.getProjectName().equals(column)) {
                switch (row) {
                    case CountTypeConstant.TOTAL:
                        idto.setTotalAmount(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.SIXTEEN_PERCENT:
                        idto.setAmount1(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.TEN_PERCENT:
                        idto.setAmount2(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.SIX_PERCENT:
                        idto.setAmount3(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.THREE_PERCENT:
                        idto.setAmount4(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.SEVENTEEN_PERCENT:
                        idto.setAmount5(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.ELEVEN_PERCENT:
                        idto.setAmount6(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.FIVE_PERCENT:
                        idto.setAmount7(new BigDecimal(value.toString()));
                        break;
                    case CountTypeConstant.OTHER:
                        idto.setOtherAmount(new BigDecimal(value.toString()));
                        break;
                    default:
                        break;
                }
                break;
            }
        }
    }
}
