package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.constant.enums.InteractiveEnum;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dpo.*;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.*;

import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author: kevin
 * @create: 2019-05-16 21:22
 **/
@Repository
@CacheConfig(cacheNames = "InteractionInteractiveListData")
@Service
public class InteractionInteractiveServiceImpl extends BaseService {
    @Autowired
    private InteractionInteractiveSystemMapper interactionInteractiveSystemMapper;
    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private InteractionInteractivePraiseMapper interactionInteractivePraiseMapper;

    @Autowired
    private InteractionInteractiveCommentMapper interactionInteractiveCommentMapper;


    public void voteUp(String internationId) {
        InteractionInteractivePraise interactionInteractivePraise = new InteractionInteractivePraise();
        interactionInteractivePraise.setInternationId(internationId);
        interactionInteractivePraise.setId(String.valueOf(idService.nextId()));
        interactionInteractivePraise.setUserId(authUserHelper.getCurrentUserId());
        interactionInteractivePraise.setCreateTime(new Date());
        interactionInteractivePraiseMapper.insert(interactionInteractivePraise);

    }

    public void voteCancel(String internationId) {
        String userId = authUserHelper.getCurrentUserId();
        interactionInteractivePraiseMapper.deleteByInternationIdAndUserId(internationId, userId);

    }


    public void commentUp(InteractionInteractiveComment interactionInteractiveComment) {
        interactionInteractiveComment.setId(String.valueOf(idService.nextId()));
        interactionInteractiveComment.setUserId(authUserHelper.getCurrentUserId());
        interactionInteractiveComment.setCreateTime(new Date());
        interactionInteractiveCommentMapper.insert(interactionInteractiveComment);

    }

    public void commentCancel(String interactionInteractiveCommentId) {
        interactionInteractiveCommentMapper.deleteByPrimaryKey(interactionInteractiveCommentId);

    }

    @Cacheable
    public void postMessageSubmit(InteractionInteractiveSystem interactionInteractiveSystem) {
        interactionInteractiveSystem.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        interactionInteractiveSystem.setCreateTime(new Date());
        interactionInteractiveSystem.setId(String.valueOf(idService.nextId()));
        interactionInteractiveSystem.setType(InteractiveEnum.Message.getCode());
        interactionInteractiveSystemMapper.insertSelective(interactionInteractiveSystem);
    }

    public InteractionInteractiveSystemDto showComment(String id) {

        InteractionInteractiveSystemDto interactionInteractiveSystemDto = new InteractionInteractiveSystemDto();
        //所有该话题的回复
        List<InteractionInteractiveCommentDto> rootComments = interactionInteractiveCommentMapper.selectCommentsByEmailId(id);

        //根列表的回复,没有parentId的
        List<InteractionInteractiveCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionInteractiveSystemDto.setCommentRecords(comments);
        return interactionInteractiveSystemDto;
    }

    @Cacheable
    public void postQuestion(InteractionInteractiveSystem interactionInteractiveSystem) {
        interactionInteractiveSystem.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        interactionInteractiveSystem.setCreateTime(new Date());
        interactionInteractiveSystem.setId(String.valueOf(idService.nextId()));
        interactionInteractiveSystem.setType(InteractiveEnum.Question.getCode());
        interactionInteractiveSystemMapper.insertSelective(interactionInteractiveSystem);
    }

    @Autowired
    private PwcVoteTypeMapper pwcVoteTypeMapper;

    @Cacheable
    public void postVoteSubmit(InteractionInteractiveSystem interactionInteractiveSystem) {
        interactionInteractiveSystem.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        interactionInteractiveSystem.setCreateTime(new Date());
        long l = idService.nextId();
        interactionInteractiveSystem.setId(String.valueOf(l));
        interactionInteractiveSystem.setType(InteractiveEnum.Vote.getCode());
        String tag = interactionInteractiveSystem.getTag();
        String[] split = tag.substring(1, tag.length() - 1).split(",");
        for (int i = 0; i < split.length; i++) {
            PwcVoteType pwcVoteType = new PwcVoteType();
            pwcVoteType.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
            pwcVoteType.setCreateTime(new Date());
            pwcVoteType.setInteractiveId(String.valueOf(l));
            pwcVoteType.setVoteType(split[i].replaceAll("\"", ""));
            pwcVoteType.setId(String.valueOf(idService.nextId()));
            pwcVoteTypeMapper.insertSelective(pwcVoteType);
        }
        interactionInteractiveSystemMapper.insertSelective(interactionInteractiveSystem);
    }

    @Cacheable
    public PageInfo<InteractionInteractiveSystemDto> listData(InteractionInteractiveSystemDto param, int type) {
        Page page = PageHelper.startPage(param.getPageIndex(), param.getPageSize());
        String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);
        if (type != -1 && type != 0)
            param.setType(type);
        if (getUserById(userId).getIsSuperAdmin()) {
            param.setAdmin("true");
        } else {
            param.setUserId(userId);
        }
        if ("admin".equalsIgnoreCase(getUserById(userId).getUserName())) {
            param.setAdmin("true");
        }
        List<InteractionInteractiveSystemDto> interactionInteractiveSystemDtos = interactionInteractiveSystemMapper.listByDto(param);
        interactionInteractiveSystemDtos.forEach(e -> {
            e.setReadType(interactionInteractiveSystemMapper.selectReadTypeByUserIdAndEmailId(userId, e.getId()));
            e.setViewNum(Long.valueOf(interactionInteractiveSystemMapper.selectViewNum(e.getId())));
        });

        PageInfo<InteractionInteractiveSystemDto> pageInfo = new PageInfo<>(interactionInteractiveSystemDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageIndex());
        return pageInfo;
    }

    @Autowired
    private PwcTicketMapper pwcTicketMapper;

    public InteractionInteractiveSystemDto showDetail(String id) {
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = interactionInteractiveSystemMapper.getDetailById(id, authUserHelper.getCurrentUserId());
        //查询所有的投票信息
        if (interactionInteractiveSystemDto.getType() == 3) {
            List<PwcTicketDto> ticketInfo = pwcTicketMapper.getTicketInfo(id);
            if (ticketInfo.size() != 0) {
                interactionInteractiveSystemDto.setTicketInfo(ticketInfo);
                int sum = 0;
                for (int i = 0; i < ticketInfo.size(); i++) {
                    sum = sum + ticketInfo.get(i).getTotal();
                }
                interactionInteractiveSystemDto.setTotalVote(sum);
            } else {
                InteractionInteractiveSystem interactionInteractiveSystem = interactionInteractiveSystemMapper.selectByPrimaryKey(id);
                String tag = interactionInteractiveSystem.getTag();
                String[] split = tag.substring(1, tag.length() - 1).split(",");
                for (int i = 0; i < split.length; i++) {
                    PwcTicketDto pwcTicket = new PwcTicketDto();
                    pwcTicket.setAttrType(split[i].replaceAll("\"", ""));
                    ticketInfo.add(pwcTicket);
                }
                interactionInteractiveSystemDto.setTicketInfo(ticketInfo);
            }
        }
        //所有该话题的回复
        List<InteractionInteractiveCommentDto> rootComments = interactionInteractiveCommentMapper.selectCommentsByEmailId(id);

        //根列表的回复,没有parentId的
        List<InteractionInteractiveCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionInteractiveSystemDto.setCommentRecords(comments);
        return interactionInteractiveSystemDto;
    }

    private List<InteractionInteractiveCommentDto> getChildComment(String id, List<InteractionInteractiveCommentDto> comments, Map<String, String> map) {
        List<InteractionInteractiveCommentDto> childList = Lists.newArrayList();
        comments.stream()
                .filter(c -> !map.containsKey(c.getId()))
                .filter(e -> StringUtils.isNotBlank(e.getParentId()) && e.getParentId().equals(id))
                .forEach(e -> {
                    map.put(e.getId(), e.getParentId());
                    e.setReplyBody(getChildComment(e.getId(), comments, map));
                    childList.add(e);
                });
        return childList;
    }

    public PwcTicketDto subitTicket(PwcTicketDto pwcTicketDto, OperationResultDto opt) {
        PwcTicketExample example = new PwcTicketExample();
        example.createCriteria().andCreateByEqualTo(authUserHelper.getCurrentUserId());
        if (pwcTicketMapper.selectByExample(example).size() != 0) {
            opt.setResultMsg("已经投票");
            return null;
        }
        ;
        pwcTicketDto.setCreateBy(authUserHelper.getCurrentUserId());
        //判断当前登录人是否已经投票
        pwcTicketDto.setId(String.valueOf(idService.nextId()));
        pwcTicketDto.setCreateTime(new Date());
        pwcTicketMapper.insertSelective(pwcTicketDto);
        return pwcTicketDto;
    }

    public List<PwcTicketDto> pushSseVote(String id) {
        return pwcTicketMapper.getTicketInfo(id);
    }

    public InteractionInteractiveSystemDto setIntervalVote(String id) {
        InteractionInteractiveSystemDto interactionInteractiveSystemDto = new InteractionInteractiveSystemDto();
        List<PwcTicketDto> ticketInfo = pwcTicketMapper.getTicketInfo(id);
        interactionInteractiveSystemDto.setTicketInfo(ticketInfo);
        int sum = 0;
        for (int i = 0; i < ticketInfo.size(); i++) {
            sum = sum + ticketInfo.get(i).getTotal();
        }
        interactionInteractiveSystemDto.setTotalVote(sum);
        return interactionInteractiveSystemDto;
    }

    @Autowired
    private UserMapper userMapper;
    @Autowired
    JdbcTemplate jdbcTemplate;

    public HashMap preLoadTaskshowDetail(String id) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:MM:dd");
        HashMap map = new HashMap();
        //加载用户信息
        List<UserDto> users = userMapper.selectUserToUserDto();
        if (id != null) {
            PwcTaskExample example = new PwcTaskExample();
            example.createCriteria().andInteractiveIdEqualTo(id);
            map.put("lists", pwcTaskMapper.selectByExample(example));
            InteractionInteractiveSystem interactionInteractiveSystem = interactionInteractiveSystemMapper.selectByPrimaryKey(id);
            map.put("interactionInteractiveSystem", interactionInteractiveSystem);
            //查询当前任务的协同人
            List<UserDto> collaboratorUsers = interactionInteractiveSystemMapper.queryCollaboratorList(id);
            List<String> collabors = new ArrayList<>();
            for (UserDto collaboratorUser : collaboratorUsers) {
                collabors.add(collaboratorUser.getId());
            }
            map.put("collaboratorList", collabors);
            collaboratorUsers = null;
            map.put("user", interactionInteractiveSystem.getCreateBy());
            map.put("createTime", format.format(interactionInteractiveSystem.getCreateTime()));
        } else {
            map.put("lists", new ArrayList<>());
            User user = getUserById(authUserHelper.getCurrentUserId());
            map.put("user", user);
            map.put("createTime", format.format(new Date()));
        }
        map.put("userData", users);
        return map;
    }

    @Autowired
    private PwcTaskMapper pwcTaskMapper;

    @Cacheable
    public InteractionInteractiveSystemDto taskSubmit(@RequestBody InteractionInteractiveSystemDto interactionInteractiveSystem) {
        String id = null;
        List<PwcTask> pwcTasks = interactionInteractiveSystem.getPwcTasks();
        if (interactionInteractiveSystem.getId() == null) {//新增
            long l = idService.nextId();
            id = String.valueOf(l);
            if (pwcTasks != null) {
                for (PwcTask pwcTask : pwcTasks) {
                    pwcTask.setId(String.valueOf(idService.nextId()));
                    pwcTask.setInteractiveId(id);
                    pwcTask.setCreateTime(new Date());
                    pwcTask.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
                    pwcTaskMapper.insertSelective(pwcTask);
                }
            }
        } else {//修改
            id = interactionInteractiveSystem.getId();
            PwcTaskExample example = new PwcTaskExample();
            example.createCriteria().andInteractiveIdEqualTo(id);
            List<PwcTask> pwcTasks1 = pwcTaskMapper.selectByExample(example);
            if (pwcTasks != null) {
                if (pwcTasks1.size() != 0) {
                    for (PwcTask pwcTask : pwcTasks) {
                        if (pwcTask.getId() != null) {
                            pwcTask.setUpdateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
                            pwcTask.setUpdateTime(new Date());
                            pwcTaskMapper.updateByPrimaryKey(pwcTask);
                        } else {
                            pwcTask.setId(String.valueOf(idService.nextId()));
                            pwcTask.setInteractiveId(id);
                            pwcTask.setCreateTime(new Date());
                            pwcTask.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
                            pwcTaskMapper.insertSelective(pwcTask);
                        }
                    }
                } else {
                    for (PwcTask pwcTask : pwcTasks) {
                        pwcTask.setId(String.valueOf(idService.nextId()));
                        pwcTask.setInteractiveId(id);
                        pwcTask.setCreateTime(new Date());
                        pwcTask.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
                        pwcTaskMapper.insertSelective(pwcTask);
                    }
                }
            }
        }
        interactionInteractiveSystem.setCreateTime(new Date());
        interactionInteractiveSystem.setType(InteractiveEnum.Task.getCode());
        if (interactionInteractiveSystem.getId() == null) {
            interactionInteractiveSystem.setId(id);
            if (interactionInteractiveSystemMapper.insertSelective(interactionInteractiveSystem) > 0) {
                return interactionInteractiveSystem;
            }
            return null;
        }
        interactionInteractiveSystem.setId(id);
        interactionInteractiveSystem.setDuring(0);
        interactionInteractiveSystem.setUpdateTime(new Date());
        if (interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem) > 0) {
            return interactionInteractiveSystem;
        }
        return null;
    }

    public void duringUpdate(String id) {
        InteractionInteractiveSystem interactionInteractiveSystem = new InteractionInteractiveSystem();
        interactionInteractiveSystem.setId(id);
        interactionInteractiveSystem.setDuring(1);
        interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem);
    }

    @Autowired
    private PwcInteractiveCollaboratorMapper pwcInteractiveCollaboratorMapper;

    public void collaboratorSumbit(String listArr, String interactiveId) {
        PwcInteractiveCollaboratorExample example = new PwcInteractiveCollaboratorExample();
        example.createCriteria().andInteractiveIdEqualTo(interactiveId);
        pwcInteractiveCollaboratorMapper.deleteByExample(example);
        String[] arr = listArr.split(","); // 用,分割
        for (int i = 0; i < arr.length; i++) {
            PwcInteractiveCollaborator pwcTaskCollaborator = new PwcInteractiveCollaborator();
            pwcTaskCollaborator.setId(String.valueOf(idService.nextId()));
            pwcTaskCollaborator.setInteractiveId(interactiveId);
            pwcTaskCollaborator.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
            pwcTaskCollaborator.setCreateTime(new Date());
            pwcTaskCollaborator.setUserId(arr[i]);
            pwcInteractiveCollaboratorMapper.insert(pwcTaskCollaborator);
        }
    }

    public List<InteractionInteractiveSystemDto> getInteractiveAdminLists(InteractionInteractiveSystemDto interactionInteractiveSystemDto) {
        List<InteractionInteractiveSystemDto> list = interactionInteractiveSystemMapper.listByAdminDto(interactionInteractiveSystemDto);
        return list;
    }

    public void deleteInteractiveAdmin(String id) {
        InteractionInteractiveSystem interactionInteractiveSystem = new InteractionInteractiveSystem();
        interactionInteractiveSystem.setId(id);
        interactionInteractiveSystem.setDelFlag(1);
        interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem);
    }

    public void doRecommended(String id, Integer type) {
        InteractionInteractiveSystem interactionInteractiveSystem = new InteractionInteractiveSystem();
        interactionInteractiveSystem.setId(id);
        interactionInteractiveSystem.setEssenceFlag(type);//设置为精华帖
        interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem);
    }

    public void postEdit(InteractionInteractiveSystem interactionInteractiveSystem, Integer type) {
        interactionInteractiveSystemMapper.updateByPrimaryKeySelective(interactionInteractiveSystem);
    }

    public InteractionInteractiveSystem selectById(String id) {
        return interactionInteractiveSystemMapper.selectByPrimaryKey(id);
    }

    public List<InteractionInteractiveSystem> getHomeInteractiveData() {
        InteractionInteractiveSystemExample example = new InteractionInteractiveSystemExample();
        example.createCriteria().andEssenceFlagEqualTo(1);//主取精华帖
        return interactionInteractiveSystemMapper.selectByExample(example);
    }
}

