package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.StringUtil;
import pwc.taxtech.atms.dao.InteractionInternationCommentMapper;
import pwc.taxtech.atms.dao.InteractionInternationPraiseMapper;
import pwc.taxtech.atms.dao.InteractionInternationSystemMapper;
import pwc.taxtech.atms.dpo.InteractionInternationCommentDto;
import pwc.taxtech.atms.dpo.InteractionInternationSystemDto;
import pwc.taxtech.atms.dpo.InteractionShareCommentDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.entity.InteractionInternationComment;
import pwc.taxtech.atms.entity.InteractionInternationPraise;
import pwc.taxtech.atms.entity.InteractionInternationSystem;
import pwc.taxtech.atms.entity.InteractionInternationSystemExample;
import pwc.taxtech.atms.vat.entity.FileUpload;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @version 1.0
 * @program: atms
 * @description:
 * @author: Kevin
 * @create: 2019-05-07 22:26
 **/
@Service
public class InteractionInternationSystemServiceImpl extends BaseService {
    @Autowired
    private InteractionInternationSystemMapper interactionInternationSystemMapper;
    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private InteractionInternationPraiseMapper interactionInternationPraiseMapper;

    @Autowired
    private InteractionInternationCommentMapper interactionInternationCommentMapper;

    public int editTaxEmail(InteractionInternationSystem interactionInternationSystem) {
        interactionInternationSystem.setUpdateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        interactionInternationSystem.setUpdateTime(new Date());
        return interactionInternationSystemMapper.updateByPrimaryKeySelective(interactionInternationSystem);
    }

    public int addTaxEmail(InteractionInternationSystem interactionInternationSystem) {
        interactionInternationSystem.setId(String.valueOf(idService.nextId()));
        interactionInternationSystem.setCreateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        interactionInternationSystem.setCreateTime(new Date());
        return interactionInternationSystemMapper.insertSelective(interactionInternationSystem);
    }

    public InteractionInternationSystem getShareDataById(String id) {
        InteractionInternationSystemExample example = new InteractionInternationSystemExample();
        example.createCriteria().andIdEqualTo(id);
        return interactionInternationSystemMapper.selectByPrimaryKey(id);
    }

    public List<InteractionInternationSystemDto> listAdminTaxEmail(InteractionInternationSystemDto interactionInternationSystemDto) {
        String userId = authUserHelper.getCurrentUserId();
        interactionInternationSystemDto.setUserId(userId);
        List<InteractionInternationSystemDto> interactionInternationSystemDtos = interactionInternationSystemMapper.listByAdminDto(interactionInternationSystemDto);
        return interactionInternationSystemDtos;
    }

    @Autowired
    private DidiFileUploadService didiFileUploadService;

    public void pdfUpload(MultipartFile file, InteractionInternationSystem interactionInternationSystem) {
        if (null != file) {
            FileUpload fileUpload = didiFileUploadService.uploadFile(file, System.currentTimeMillis() + ".pdf", "国际税制");
            interactionInternationSystem.setAttachUrl(fileUpload.getUid());//因为上传的刷新机制，这里传id作为url
        }
        interactionInternationSystem.setCreateBy(authUserHelper.getCurrentAuditor().get());
        interactionInternationSystem.setCreateTime(new Date());
        interactionInternationSystem.setId(String.valueOf(idService.nextId()));
        interactionInternationSystem.setType(1);
        interactionInternationSystemMapper.insertSelective(interactionInternationSystem);
    }

    public void pdfUploadEdit(MultipartFile file, InteractionInternationSystem interactionInternationSystem) {
        if (null != file) {
            FileUpload fileUpload = didiFileUploadService.uploadFile(file, System.currentTimeMillis() + ".pdf", "国际税制");
            //因为上传的刷新机制，这里传id作为url
            interactionInternationSystem.setAttachUrl(fileUpload.getUid());
        }
        interactionInternationSystem.setUpdateBy(authUserHelper.getCurrentAuditor().get());
        interactionInternationSystem.setUpdateTime(new Date());
        interactionInternationSystemMapper.updateByPrimaryKeySelective(interactionInternationSystem);
    }

    @Autowired
    private HttpFileService httpFileService;

    public String pdfView(HttpServletResponse response, String uid) {
        try {
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(Arrays.asList(uid));
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            Map<String, String> urlMap = null;
            String viewUrl = null;
            if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                viewUrl = uploadDetail.getList().get(0).getViewHttpUrl();
            }
           /* inputStream = httpFileService.getUserTemplate(viewUrl);
            response.setHeader("Content-Disposition", "attachment;fileName=test.pdf");
            response.setContentType("multipart/form-data");
            OutputStream outputStream = response.getOutputStream();
            IOUtils.write(IOUtils.toByteArray(inputStream), outputStream);*/
            return viewUrl;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    public PageInfo<InteractionInternationSystemDto> listInternationSystem(InteractionInternationSystemDto param) {
        Page page = PageHelper.startPage(param.getPageIndex(), param.getPageSize());
        String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);
        List<InteractionInternationSystemDto> internationSystems = interactionInternationSystemMapper.listByDto(param);
        internationSystems.forEach(e ->
                e.setReadType(interactionInternationSystemMapper.selectReadTypeByUserIdAndEmailId(userId, e.getId()))
        );
        internationSystems.forEach(e ->
                e.setViewNum(Long.valueOf(interactionInternationSystemMapper.selectViewNum(e.getId())))
        );

        PageInfo<InteractionInternationSystemDto> pageInfo = new PageInfo<>(internationSystems);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageIndex());
        return pageInfo;

    }

    public void voteUp(String internationId) {
        InteractionInternationPraise interactionInternationPraise = new InteractionInternationPraise();
        interactionInternationPraise.setInternationId(internationId);
        interactionInternationPraise.setId(String.valueOf(idService.nextId()));
        interactionInternationPraise.setUserId(authUserHelper.getCurrentUserId());
        interactionInternationPraise.setCreateTime(new Date());
        interactionInternationPraiseMapper.insert(interactionInternationPraise);

    }

    public void voteCancel(String internationId) {
        String userId = authUserHelper.getCurrentUserId();
        interactionInternationPraiseMapper.deleteByInternationIdAndUserId(internationId, userId);

    }

    public InteractionInternationSystemDto getUserShareDataById(String id) {
        InteractionInternationSystemDto interactionInternationSystemDto = interactionInternationSystemMapper.getDetailById(id, authUserHelper.getCurrentUserId());
        //所有该话题的回复
        List<InteractionInternationCommentDto> rootComments = interactionInternationCommentMapper.selectCommentsByInternationId(id);

        //根列表的回复,没有parentId的
        List<InteractionInternationCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionInternationSystemDto.setCommentRecords(comments);

        return interactionInternationSystemDto;

    }


    public InteractionInternationSystemDto getCommendById(String id) {
        InteractionInternationSystemDto interactionInternationSystemDto = new InteractionInternationSystemDto();
        //所有该话题的回复
        List<InteractionInternationCommentDto> rootComments = interactionInternationCommentMapper.selectCommentsByInternationId(id);

        //根列表的回复,没有parentId的
        List<InteractionInternationCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionInternationSystemDto.setCommentRecords(comments);

        return interactionInternationSystemDto;

    }


    public void commentUp(InteractionInternationComment interactionInternationComment) {
        interactionInternationComment.setId(String.valueOf(idService.nextId()));
        interactionInternationComment.setUserId(authUserHelper.getCurrentUserId());
        interactionInternationComment.setCreateTime(new Date());
        interactionInternationCommentMapper.insert(interactionInternationComment);

    }

    public void commentCancel(String interactionInternationCommentId) {
        interactionInternationCommentMapper.deleteByPrimaryKey(interactionInternationCommentId);

    }

    public void delete(String interactionInternationCommentId) {
        interactionInternationSystemMapper.deleteByPrimaryKey(interactionInternationCommentId);

    }

    private List<InteractionInternationCommentDto> getChildComment(String id, List<InteractionInternationCommentDto> comments, Map<String, String> map) {

        List<InteractionInternationCommentDto> childList = Lists.newArrayList();

        comments.stream()
                .filter(c -> !map.containsKey(c.getId()))
                .filter(e -> StringUtils.isNotBlank(e.getParentId()) && e.getParentId().equals(id))
                .forEach(e -> {
                    map.put(e.getId(), e.getParentId());
                    e.setReplyBody(getChildComment(e.getId(), comments, map));
                    childList.add(e);
                });
        return childList;
    }

}

