package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.dao.InteractionTaxManageCommentMapper;
import pwc.taxtech.atms.dao.InteractionTaxManageMapper;
import pwc.taxtech.atms.dao.InteractionTaxManagePraiseMapper;
import pwc.taxtech.atms.dao.PwcTagMapper;
import pwc.taxtech.atms.dpo.InteractionTaxManageCommentDto;
import pwc.taxtech.atms.dpo.InteractionTaxManageDto;
import pwc.taxtech.atms.entity.InteractionTaxManage;
import pwc.taxtech.atms.entity.InteractionTaxManageComment;
import pwc.taxtech.atms.entity.InteractionTaxManagePraise;
import pwc.taxtech.atms.entity.PwcTag;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author: sam
 * @create: 2019-05-23 15:37
 **/
@Service
public class InteractionTaxManageServiceImpl extends BaseService {
    private Logger logger = LoggerFactory.getLogger(InteractionTaxManageServiceImpl.class);

    @Autowired
    private PwcTagMapper pwcTagMapper;

    @Autowired
    protected DistributedIdService idService;

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private InteractionTaxManageMapper interactionTaxManageMapper;

    @Autowired
    private InteractionTaxManagePraiseMapper interactionTaxManagePraiseMapper;

    @Autowired
    private InteractionTaxManageCommentMapper interactionTaxManageCommentMapper;


    public List<InteractionTaxManageDto> listAdminInteractionTaxManage(InteractionTaxManageDto param) {
        String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);
        List<InteractionTaxManageDto> interactionTaxManages = interactionTaxManageMapper.listByAdminDto(param);
        interactionTaxManages.forEach(e -> e.setCreateBy(getUserById(e.getCreateBy()).getUserName()));
        return interactionTaxManages;
    }


    public PageInfo<InteractionTaxManageDto> listInteractionTaxManage(InteractionTaxManageDto param) {
        Page page = PageHelper.startPage(param.getPageIndex(), param.getPageSize());
        String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);
        List<InteractionTaxManageDto> interactionTaxManages = interactionTaxManageMapper.listByDto(param);
        interactionTaxManages.forEach(e ->
                e.setReadType(interactionTaxManageMapper.selectReadTypeByUserIdAndInternationId(userId, e.getId()))
        );
        interactionTaxManages.forEach(e ->
                e.setViewNum(Long.valueOf(interactionTaxManageMapper.selectViewNum(e.getId())))
        );

        PageInfo<InteractionTaxManageDto> pageInfo = new PageInfo<>(interactionTaxManages);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageIndex());
        return pageInfo;
    }

    public InteractionTaxManageDto getTaxManageDetail(String id) {
        InteractionTaxManageDto interactionTaxManageDto = interactionTaxManageMapper.getDetailById(id, authUserHelper.getCurrentUserId());

        //所有该话题的回复
        List<InteractionTaxManageCommentDto> rootComments = interactionTaxManageCommentMapper.selectCommentsByInternationId(id);

        //根列表的回复,没有parentId的
        List<InteractionTaxManageCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionTaxManageDto.setCommentRecords(comments);
        return interactionTaxManageDto;
    }

    public InteractionTaxManageDto getCommend(String id) {
        InteractionTaxManageDto interactionTaxManageDto = new InteractionTaxManageDto();

        //所有该话题的回复
        List<InteractionTaxManageCommentDto> rootComments = interactionTaxManageCommentMapper.selectCommentsByInternationId(id);

        //根列表的回复,没有parentId的
        List<InteractionTaxManageCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        interactionTaxManageDto.setCommentRecords(comments);
        return interactionTaxManageDto;
    }

    public int editTaxManage(InteractionTaxManage interactionTaxManage) {
        interactionTaxManage.setUpdateBy(authUserHelper.getCurrentUserId());
        interactionTaxManage.setUpdateTime(new Date());
        return interactionTaxManageMapper.updateByPrimaryKeySelective(interactionTaxManage);
    }

    public int addTaxManage(InteractionTaxManage interactionTaxManage) {
        interactionTaxManage.setId(String.valueOf(idService.nextId()));
        interactionTaxManage.setCreateBy(authUserHelper.getCurrentUserId());
        interactionTaxManage.setCreateTime(new Date());
        return interactionTaxManageMapper.insertSelective(interactionTaxManage);
    }


    public void voteUp(String internationId) {
        InteractionTaxManagePraise interactionTaxManagePraise = new InteractionTaxManagePraise();
        interactionTaxManagePraise.setInternationId(internationId);
        interactionTaxManagePraise.setId(String.valueOf(idService.nextId()));
        interactionTaxManagePraise.setUserId(authUserHelper.getCurrentUserId());
        interactionTaxManagePraise.setCreateTime(new Date());
        interactionTaxManagePraiseMapper.insert(interactionTaxManagePraise);

    }

    public void voteCancel(String internationId) {
        String userId = authUserHelper.getCurrentUserId();
        interactionTaxManagePraiseMapper.deleteByInternationIdAndUserId(internationId, userId);

    }

    public void commentUp(InteractionTaxManageComment interactionTaxManageComment) {
        interactionTaxManageComment.setId(String.valueOf(idService.nextId()));
        interactionTaxManageComment.setUserId(authUserHelper.getCurrentUserId());
        interactionTaxManageComment.setCreateTime(new Date());
        interactionTaxManageCommentMapper.insert(interactionTaxManageComment);

    }

    public void commentCancel(String commentId) {
        interactionTaxManageCommentMapper.deleteByPrimaryKey(commentId);

    }

    private List<InteractionTaxManageCommentDto> getChildComment(String id, List<InteractionTaxManageCommentDto> comments, Map<String, String> map) {

        List<InteractionTaxManageCommentDto> childList = Lists.newArrayList();

        comments.stream()
                .filter(c -> !map.containsKey(c.getId()))
                .filter(e -> StringUtils.isNotBlank(e.getParentId()) && e.getParentId().equals(id))
                .forEach(e -> {
                    map.put(e.getId(), e.getParentId());
                    e.setReplyBody(getChildComment(e.getId(), comments, map));
                    childList.add(e);
                });
        return childList;
    }


    public PwcTag addTagSubmit(String tag, int type) {
        PwcTag pwcTag = new PwcTag();
        pwcTag.setId(String.valueOf(idService.nextId()));
        pwcTag.setTagname(tag);
        pwcTag.setType(type);//默认税享是type是1的
        pwcTag.setTagcode(UUID.randomUUID().toString());
        if (pwcTagMapper.insert(pwcTag) > 0) {
            return pwcTag;
        } else {
            return null;
        }
    }

    public void delete(String id) {
        interactionTaxManageMapper.deleteById(id);
    }
}
