package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.dao.FileTypesMapper;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.UserMapper;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.reportUpload.ReportUploadParam;
import pwc.taxtech.atms.dto.reportUpload.ReportUploadResult;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.vat.dao.ReportUploadMapper;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportUpload;
import pwc.taxtech.atms.vat.entity.ReportUploadExample;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ReportUploadService extends BaseService {

    @Resource
    private ReportUploadMapper reportUploadMapper;

    @Autowired
    protected AuthUserHelper authUserHelper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ProjectMapper projectMapper;

    @Autowired
    DidiFileUploadService didiFileUploadService;

    @Resource
    private OrganizationMapper organizationMapper;

    @Resource
    private FileTypesMapper fileTypesMapper;

    @Autowired
    TaxDocumentServiceImpl taxDocumentService;

    public PageInfo<ReportUploadResult> queryPage(ReportUploadParam param) {
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(param.getProjectId()).andPeriodEqualTo(param.getPeriod()).andServiceTypeEqualTo(EnumServiceType.VAT.getCode());
        example.setOrderByClause(" create_time DESC ");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        List<String> fileUploadUids = dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList());
        Map<String, String> urlMap = new HashMap<>();
        if(CollectionUtils.isNotEmpty(fileUploadUids)){
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(fileUploadUids);
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            if(CollectionUtils.isNotEmpty(uploadDetail.getList())){
                urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
            }
        }
        List<ReportUploadResult> resultList = new ArrayList<ReportUploadResult>();
        for(ReportUpload data:dataList){
            ReportUploadResult result = beanUtil.copyProperties(data, new ReportUploadResult());
            result.setFileUrl(urlMap.get(data.getFileUploadId()));
            resultList.add(result);
        }
        PageInfo<ReportUploadResult> pageInfo = new PageInfo<>(resultList);
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    public void saveData(MultipartFile file, ReportUpload data) {
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        String fileName = "";
        Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
        data.setOrgId(project.getOrganizationId());
        fileName += project.getName();
        fileName += "-";
        fileName += data.getPeriod();
        fileName += "-V";
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod());
        example.setOrderByClause("version desc");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(dataList)) {
            fileName += (dataList.get(0).getVersion() + 1);
            data.setVersion(dataList.get(0).getVersion() + 1);
        } else {
            fileName += 1;
            data.setVersion(1);
        }
        fileName += ".xlsx";
        data.setReportName(fileName);
        FileUpload fileUpload = didiFileUploadService.uploadFile(file, fileName, FileUploadEnum.BizSource.REPORT_UPLOAD.name());
        data.setFileUploadId(fileUpload.getUid());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        reportUploadMapper.insert(data);
    }

    public void saveData(byte[] bytes, ReportUpload data) {
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        String fileName = "";
        Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
        data.setOrgId(project.getOrganizationId());
        String period = ""+project.getYear()+(data.getPeriod()>9?data.getPeriod():("0"+data.getPeriod()));
        data.setPeriod(Integer.valueOf(period));
        fileName += project.getName();
        fileName += "_";
        fileName += "增值税及附加税费计算底稿";
        fileName += "_";
        fileName += data.getPeriod();
        fileName += "_V";
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod());
        example.setOrderByClause("version desc");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(dataList)) {
            fileName += (dataList.get(0).getVersion() + 1);
            data.setVersion(dataList.get(0).getVersion() + 1);
        } else {
            fileName += 1;
            data.setVersion(1);
        }
        fileName += ".xlsx";
        data.setReportName(fileName);
        FileUpload fileUpload = didiFileUploadService.uploadFile(bytes, fileName, FileUploadEnum.BizSource.REPORT_UPLOAD.name());
        data.setFileUploadId(fileUpload.getUid());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        data.setServiceType(EnumServiceType.VAT.getCode());
        reportUploadMapper.insert(data);

        data.setOrgId(project.getOrganizationId());
        TaxDocument taxDocument = new TaxDocument();
        taxDocument.setCompanyId(project.getOrganizationId());
        Organization organization = organizationMapper.selectByPrimaryKey(project.getOrganizationId());
        taxDocument.setCompanyName(organization.getName());
        taxDocument.setFileUploadId(data.getFileUploadId());
        taxDocument.setEnable("T");
        taxDocument.setOwnTime(data.getPeriod());
        taxDocument.setFileType("增值税及附加税费计算底稿");
        taxDocument.setTaxType("增值税");

        FileTypesExample fileTypesExample = new FileTypesExample();
        fileTypesExample.createCriteria().andFileTypeEqualTo("增值税及附加税费计算底稿");
        List<FileTypes> fileTypesList = fileTypesMapper.selectByExample(fileTypesExample);
        if(CollectionUtils.isNotEmpty(fileTypesList)){
            taxDocument.setFileAttr(fileTypesList.get(0).getFileAttr());
            taxDocument.setFileTypeId(fileTypesList.get(0).getId());
        }
        taxDocument.setFileName(fileName);
        taxDocumentService.addTaxDocumentList(null,taxDocument);

    }
}
