package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.JavaEmailUtil;
import pwc.taxtech.atms.dao.PwcEmailCommentMapper;
import pwc.taxtech.atms.dao.PwcEmailPraiseMapper;
import pwc.taxtech.atms.dao.PwcTagMapper;
import pwc.taxtech.atms.dao.TaxEmailMapper;
import pwc.taxtech.atms.dpo.InteractionInteractiveSystemDto;
import pwc.taxtech.atms.dpo.InteractionShareCommentDto;
import pwc.taxtech.atms.dpo.TaxEmailDto;
import pwc.taxtech.atms.entity.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @version 1.0
 * @program: atms
 * @description:
 * @author: Kevin
 * @create: 2019-04-26 12:28
 **/
@Service
public class TaxEamilServiceImpl extends BaseService {
    private Logger logger = LoggerFactory.getLogger(TaxEamilServiceImpl.class);
    @Autowired
    private TaxEmailMapper taxEmailMapper;
    @Autowired
    private PwcEmailPraiseMapper pwcEmailPraiseMapper;
    @Autowired
    private JavaEmailUtil javaEmailUtil;
    @Autowired
    protected DistributedIdService idService;
    @Autowired
    private AuthUserHelper authUserHelper;
    @Autowired
    private PwcEmailCommentMapper pwcEmailCommentMapper;


    public void jobEmail() {
        logger.info("start  taxemail ------------------------------------------------------" +
                "------------------------------------------------------------------------" +
                "-------------------------------------------------------------------------" +
                "-------------------------------------------------------------------------");
        try {
            List<Map<String, Object>> emails = javaEmailUtil.getEmails(true);
            emails.forEach(e -> {
                TaxEmail taxEmail = new TaxEmail();
                taxEmail.setCreateBy((String) e.get("from_person"));
                taxEmail.setCreateTime((Date) e.get("sendDate"));
                taxEmail.setId(String.valueOf(idService.nextId()));
                if (e.get("type") != null && e.get("type").equals("1")) {
                    taxEmail.setContent((String) e.get("content"));
                    taxEmail.setType(1);
                } else {
                    taxEmail.setType(0);
                    taxEmail.setUrl((String) e.get("url"));
                }
                taxEmail.setAttachUrl((String) e.get("attachUrl"));
                taxEmail.setTitle((String) e.get("subject"));
                taxEmail.setHashkey(String.valueOf((((String) e.get("attachUrl") + (String) e.get("subject")).hashCode())));
                if (judgeHashKey(taxEmail.getHashkey()))
                    taxEmailMapper.insertSelective(taxEmail);
            });

        } catch (Exception e) {
            e.printStackTrace();
            logger.warn("获取邮件失败");
        }
    }

    public boolean judgeHashKey(String hashKey) {
        TaxEmailExample example = new TaxEmailExample();
        example.createCriteria().andHashkeyEqualTo(hashKey);
        List<TaxEmail> taxEmails = taxEmailMapper.selectByExample(example);
        if (taxEmails.size() != 0)
            return false;
        return true;
    }

    public List<TaxEmailDto> listAdminTaxEmail(TaxEmailDto param) {
      /*  String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);*/
        List<TaxEmailDto> taxEmails = taxEmailMapper.listByAdminDto(param);
        return taxEmails;
    }


    public PageInfo<TaxEmailDto> listTaxEmail(TaxEmailDto param) {
        Page page = PageHelper.startPage(param.getPageIndex(), param.getPageSize());
        String userId = authUserHelper.getCurrentUserId();
        param.setUserId(userId);
        List<TaxEmailDto> taxEmails = taxEmailMapper.listByDto(param);
        taxEmails.forEach(e ->
                e.setReadType(taxEmailMapper.selectReadTypeByUserIdAndEmailId(userId, e.getId()))
        );
        taxEmails.forEach(e ->
                e.setViewNum(Long.valueOf(taxEmailMapper.selectViewNum(e.getId())))
        );

        PageInfo<TaxEmailDto> pageInfo = new PageInfo<>(taxEmails);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageIndex());
        return pageInfo;
    }

    public TaxEmailDto getTaxEailDetail(String id) {
        TaxEmailDto taxEmailDto = taxEmailMapper.getDetailById(id, authUserHelper.getCurrentUserId());

        //所有该话题的回复
        List<InteractionShareCommentDto> rootComments = pwcEmailCommentMapper.selectCommentsByEmailId(id);

        //根列表的回复,没有parentId的
        List<InteractionShareCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        taxEmailDto.setCommentRecords(comments);
        return taxEmailDto;
    }

    public TaxEmailDto loadCommends(String id) {
        TaxEmailDto taxEmailDto = new TaxEmailDto();

        //所有该话题的回复
        List<InteractionShareCommentDto> rootComments = pwcEmailCommentMapper.selectCommentsByEmailId(id);

        //根列表的回复,没有parentId的
        List<InteractionShareCommentDto> comments = rootComments.stream().
                filter(e -> StringUtils.isBlank(e.getParentId())).
                collect(Collectors.toList());

        //创建一个map,存已经加入树节点的数据,
        Map<String, String> map = Maps.newHashMapWithExpectedSize(rootComments.size());

        //从根目录遍历然后递归
        comments.stream().forEach(e ->
                e.setReplyBody(getChildComment(e.getId(), rootComments, map))
        );
        taxEmailDto.setCommentRecords(comments);
        return taxEmailDto;
    }

    public void delete(String id) {
        taxEmailMapper.deleteByPrimaryKey(id);
    }

    public int editTaxEmail(TaxEmail taxEmail) {
        taxEmail.setUpdateBy(getUserById(authUserHelper.getCurrentUserId()).getUserName());
        taxEmail.setUpdateTime(new Date());
        return taxEmailMapper.updateByPrimaryKeySelective(taxEmail);
    }

    public int addTaxEmail(TaxEmail taxEmail) {
        taxEmail.setId(String.valueOf(idService.nextId()));
        taxEmail.setCreateBy(authUserHelper.getCurrentAuditor().get());
        taxEmail.setCreateTime(new Date());
        return taxEmailMapper.insertSelective(taxEmail);
    }

    public TaxEmail getShareDataById(String id) {
        TaxEmailExample example = new TaxEmailExample();
        example.createCriteria().andIdEqualTo(id);
        return taxEmailMapper.selectByPrimaryKey(id);
    }

    public void voteUp(String emailId) {
        PwcEmailPraise pwcEmailPraise = new PwcEmailPraise();
        pwcEmailPraise.setEmailId(emailId);
        pwcEmailPraise.setId(String.valueOf(idService.nextId()));
        pwcEmailPraise.setUserId(authUserHelper.getCurrentUserId());
        pwcEmailPraise.setCreateTime(new Date());
        pwcEmailPraiseMapper.insert(pwcEmailPraise);

    }

    public void voteCancel(String emailId) {
        String userId = authUserHelper.getCurrentUserId();
        pwcEmailPraiseMapper.deleteByEmailIdAndUserId(emailId, userId);

    }

    public void commentUp(PwcEmailComment pwcEmailComment) {
        pwcEmailComment.setId(String.valueOf(idService.nextId()));
        pwcEmailComment.setUserId(authUserHelper.getCurrentUserId());
        pwcEmailComment.setCreateTime(new Date());
        pwcEmailCommentMapper.insert(pwcEmailComment);

    }

    public void commentCancel(String commentlId) {
        pwcEmailCommentMapper.deleteByPrimaryKey(commentlId);

    }

    private List<InteractionShareCommentDto> getChildComment(String id, List<InteractionShareCommentDto> comments, Map<String, String> map) {

        List<InteractionShareCommentDto> childList = Lists.newArrayList();

        comments.stream()
                .filter(c -> !map.containsKey(c.getId()))
                .filter(e -> StringUtils.isNotBlank(e.getParentId()) && e.getParentId().equals(id))
                .forEach(e -> {
                    map.put(e.getId(), e.getParentId());
                    e.setReplyBody(getChildComment(e.getId(), comments, map));
                    childList.add(e);
                });
        return childList;
    }

    @Autowired
    private PwcTagMapper pwcTagMapper;

    public PwcTag addTagSubmit(String tag, int type) {
        PwcTag pwcTag = new PwcTag();
        pwcTag.setId(String.valueOf(idService.nextId()));
        pwcTag.setTagname(tag);
        pwcTag.setType(type);//默认税享是type是1的
        pwcTag.setTagcode(UUID.randomUUID().toString());
        if (pwcTagMapper.insert(pwcTag) > 0) {
            return pwcTag;
        } else {
            return null;
        }
    }


    public Map getHomeShareData() {
        Map map = new HashMap();
        TaxEmailExample example = new TaxEmailExample();
        example.createCriteria().andTypeEqualTo(1);
        example.createCriteria().andDelFlagEqualTo(0);
        List<TaxEmail> taxEmails = taxEmailMapper.selectByExample(example);
        int len = 0;
        if (taxEmails.size() >= 5) {
            len = 5;
        } else {
            len = taxEmails.size();
        }
        List s = new ArrayList();
        for (int i = 0; i < len; i++) {
            s.add(taxEmails.get(i));
        }
        map.put("news", taxEmails);
        TaxEmailExample example1 = new TaxEmailExample();
        example1.createCriteria().andTopFlagEqualTo(1);
        example1.createCriteria().andDelFlagEqualTo(0);
        map.put("top", taxEmailMapper.selectByExample(example1));
        return map;
    }
}
