﻿analysisModule.controller('internationalDataImportController', ['$scope', '$filter', '$log', '$translate', '$timeout', '$q', '$interval'
    , 'apiInterceptor', 'Upload', 'vatImportService', 'SweetAlert', 'uiGridConstants', '$uibModal'
    , 'vatSessionService', 'enums', 'vatOperationLogService'
    , 'projectService', 'vatCommonService', 'templateService', 'orgService',
    function ($scope, $filter, $log, $translate, $timeout, $q, $interval
        , apiInterceptor, Upload, vatImportService, SweetAlert, uiGridConstants, $uibModal
        , vatSessionService, enums, vatOperationLogService
        , projectService, vatCommonService, templateService, orgService) {
        'use strict';

        var comment = vatSessionService.project.name + " " + vatSessionService.project.year + "年" + vatSessionService.month + "月";
        $scope.period = $scope.periodId;
        $scope.moduleid = enums.vatModuleEnum.Import_TrialBalance;
        $scope.chunkSize = 100000;
        $scope.success = false;
        $scope.showErrorTable = false;
        $scope.showInitTable = false;
        $scope.showImportTable = false;
        $scope.isSelectPeriod = true;
        $scope.isShowImportTotalBtn = true;
        $scope.projectID = vatSessionService.project.id;
        $scope.startRowNum = 2;
        $scope.validationType = 0;
        $scope.sheetData = {sheetNameList: [], dataList: [], selectedSheetIndex: 0};
        $scope.sheetInfo = {selectedSheetName: '', selectedSheetIndex: 0};
        $scope.balanceInputList = [];
        $scope.toInputList = [];
        $scope.importEnum = {Import: 0, CoverImport: 1, AddImport: 2}; //导入方式
        $scope.selectedPeriod = null;
        $scope.showTotalSecondRow = false;

        $scope.importExcelFileUrlList = {
            internationalBU: apiInterceptor.webApiHostUrl + '/Analysis/InternationalExcelFile',
            internationalTax: apiInterceptor.webApiHostUrl + '/Analysis/InternationalExcelFile'
        };

        $scope.maxTitleLength = constant.maxButtonTitleLength;
        $scope.companyList = [];
        $scope.checkedCompanyList = [];
        $scope.checkedCompanyCodeList = [];
        $scope.checkedCompanyTypeList = "";

        $scope.showInternationalBUDataGrid = false;
        $scope.showInternationalTaxDataGrid = false;
        $scope.importType = 100;
        $scope.selectType;

        $scope.selectCompany = '';
        $scope.selectCountry = '';

        $scope.internationalDataList = constant.internationalDataList;
        var date = new Date();
        var year = date.getFullYear();
        var month = date.getMonth() + 1;
        $scope.selectedDate = new Date(year, date.getMonth(), 1);
        $scope.startDate = new Date(year - 20, 1, 1);
        $scope.endDate = new Date(year + 20, 1, 1);
        $scope.viewMode = 1;
        $scope.dateFormat = $translate.instant('dateFormat4YearMonth');

        if (month.length < 2) {
            $scope.UploadPeriodTime = year + " - 0" + month;
        } else {
            $scope.UploadPeriodTime = year + " - " + month;
        }

        $scope.importExcelFile = null;
        $scope.country = '';
        $scope.companyName = '';

        //写日志
        var logDto = {
            ID: '',
            OperationName: '',
            ModuleID: $scope.moduleid,
            OperationObject: $translate.instant('ProfitAndLossStatement'),
            OperationType: '',
            OperationContent: '',
            OriginalState: '',
            UpdateState: '',
            CreatorID: vatSessionService.logUser.ID,
            Comment: comment,
            IP: '',
            Period: $scope.period
        };

        var param = {
            type: $scope.importType,
            period: $scope.UploadPeriodTime,
            country: $scope.selectCountry,
            companyName: $scope.selectCountry
        };

        //初始化ack-pagination
        $scope.pagingOptions = {
            pageIndex: 1, //当前页码
            totalItems: 0, //总数据
            totalPages: 10, //总页数
            maxSize: 5, //分页数字的限制。
            pageSize: constant.page.pageSizeArrary[3], //每页多少条数据,100条
            pageSizeString: constant.page.pageSizeArrary[3].toString(),
            firstPage: $translate.instant('PagingFirstPage'),
            previousPage: $translate.instant('PagingPreviousPage'),
            nextPage: $translate.instant('PagingNextPage'),
            lastPage: $translate.instant('PagingLastPage'),
        };

        //初始化查询参数
        $scope.queryParams = {
            pageInfo: {
                totalCount: 0,
                pageIndex: 0,
                pageSize: 0,
                totalPage: 0,
            },
            periodId: $scope.period,
            serviceTypeId: $scope.serviceTypeId,
            projectId: $scope.projectID
        };

        //分页获取数据
        $scope.eventService = {
            refreshInvoiceDataGrid: function () {
                $log.debug("refreshInvoiceDataGrid");
            }
        };

        var downloadTemplate = function () {
            templateService.downloadAnalysisTemplate($scope.importType).success(function (data, status, headers) {
                var fileName = '';
                constant.anlTemplateNameList.forEach(function (m) {
                    if (m.code === $scope.importType) {
                        fileName = m.name;
                        PWC.downloadCallBack(data, status, headers, fileName)
                    }
                });

            }).error(function () {
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        };

        var doExport = function () {
            var localDate = $filter('date')(new Date(), 'yyyyMMddHHmmss');
            var fileName = '';
            constant.anlDownLoadFileNameList.forEach(function (m) {
                if (m.code === $scope.importType) {
                    fileName = m.name + $scope.UploadPeriodTime + "_" + localDate;
                }
            });
            param.companyName = $scope.selectCompany;
            param.country = $scope.selectCountry;
            param.type = $scope.importType;
            param.period = $scope.UploadPeriodTime;
            vatImportService.downloadInternationalFile(param, fileName).then(function (data) {
                if (data) {
                    ackMessageBox.success(translate('FileExportSuccess'));
                }
            });

        };

        //导入事件
        var doUpload = function () {

            var impExl = $scope.importExcelFile;
            var deferred = $q.defer();
            var token = $('input[name="__RequestVerificationToken"]').val();

            if (!impExl) {
                SweetAlert.warning($translate.instant('PleaseSelectFileFirst'));
                return;
            }

            if (!$scope.selectCountry || '' === $scope.selectCountry) {
                SweetAlert.warning($translate.instant('PleaseSelectCountryFirst'));
                return;
            }

            if (!$scope.selectCompany || '' === $scope.selectCompany) {
                SweetAlert.warning($translate.instant('PleaseSelectCompanyFirst'));
                return;
            }

            var url = getUploadUrl();

            var period = $scope.UploadPeriodTime;


            Upload.upload({
                url: url,
                data: {
                    companyName: $scope.selectCompany,
                    country: $scope.selectCountry,
                    period: period,
                    type: $scope.importType
                },
                file: impExl,
                headers: {
                    'Access-Control-Allow-Origin': '*',
                    Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                    __RequestVerificationToken: token,
                    withCredentials: true
                },
                __RequestVerificationToken: token,
                withCredentials: true
            }).then(function (resp) {
                var ret = resp.data;
                $scope.fileName = '';
                $scope.importExcelFile = null;
                $('#busy-indicator-container').hide();
                deferred.resolve();
                if (ret.result) {
                    logDto.UpdateState = $translate.instant('ImportSuccess');
                    vatOperationLogService.addOperationLog(logDto);
                    SweetAlert.success($translate.instant('ImportSuccess'));
                } else {
                    if (ret.resultMsg && ret.resultMsg.length > 0) {
                        SweetAlert.warning($translate.instant(ret.resultMsg));
                    } else {
                        SweetAlert.error($translate.instant('ImportFailed'));
                    }
                }
                refreshGrid();
            }, function (resp) {
                deferred.resolve();
                if (resp.statusText === 'HttpRequestValidationException') {
                    SweetAlert.warning($translate.instant('HttpRequestValidationException'));
                } else {
                    SweetAlert.warning('SaveFail');
                }
                SweetAlert.error($translate.instant('ImportFail'));
                console.log('Error status: ' + resp.status);
            }, function (evt) {
                deferred.resolve();
                var progressPercentage = parseInt(100.0 * evt.loaded / evt.total);
                $log.debug('progress: ' + progressPercentage + '% ' + evt.config.data.file.name);
            });
        };

        var getUploadUrl = function () {
            var url = "";
            switch ($scope.importType) {
                case 100:
                    url = $scope.importExcelFileUrlList.internationalBU;
                    break;
                case 101:
                    url = $scope.importExcelFileUrlList.internationalTax;
                    break;
                default:
                    break;
            }
            return url;
        };

        var refreshGrid = function () {
            param.companyName = $scope.selectCompany;
            param.country = $scope.selectCountry;
            param.type = $scope.importType;
            param.period = $scope.UploadPeriodTime;
            switch ($scope.importType) {
                case 100:
                    getImportInternationalBUData();
                    break;
                case 101:
                    getImportInternationalTaxData();
                    break;
                default:
                    break;
            }
        };

        var getGridHeight = function () {
            if ($scope.isLoadComplete) {
                return {height: ($('.balance-ouput-grid-wrapper').height()) + "px"};
            }
            else {
                return {height: 0 + "px"};
            }
        };

        var loadinternationalBUDatagrid = function () {
            $scope.internationalBUDataGridOptions = {
                bindingOptions: {
                    dataSource: 'internationalBUDataGridSource'
                },
                showBorders: true,
                columns: [
                    {
                        dataField: "gmv",
                        width: '16%',
                        allowHeaderFiltering: false,
                        caption: 'Gmv'
                    }, {
                        dataField: "trips",
                        allowHeaderFiltering: false,
                        caption: 'Trips',
                        width: '16%'
                    }, {
                        dataField: "subsidyB",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'SubsidyB'
                    }, {
                        dataField: "subsidyC",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'SubsidyC'
                    }, {
                        dataField: "revenue",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'Revenue'
                    }, {
                        dataField: "profit",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'Profit'
                    },
                    {
                        dataField: "ref1",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref1'
                    },
                    {
                        dataField: "ref2",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref2'
                    },
                    {
                        dataField: "ref3",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref3'
                    },
                    {
                        dataField: "ref4",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref4'
                    },
                    {
                        dataField: "ref5",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref5'
                    },
                    {
                        dataField: "ref6",
                        allowHeaderFiltering: false,
                        width: '16%',
                        caption: 'ref6'
                    }],
                onContentReady: function (e) {
                    $scope.accountingRateListInstance = e.component;
                    var totalCount = e.component.totalCount();
                    if (totalCount > 0) {
                        $scope.totalCount = totalCount;
                    }
                },
                loadPanel: {
                    enabled: false
                },
                selection: {
                    mode: "single"
                },
                grouping: {
                    autoExpandAll: false
                },
                allowColumnResizing: true,
                allowColumnReordering: true,
                columnAutoWidth: true,
                showRowLines: true,
                showColumnLines: true,
                rowAlternationEnabled: true,  //单双行颜色
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                headerFilter: {
                    visible: false,
                    texts: {
                        cancel: $translate.instant('Cancel'),
                        ok: $translate.instant('Confirm'),
                        emptyValue: $translate.instant('Empty'),
                        selectAllText: $translate.instant('SelectAll')
                    }
                },
                paging: {
                    pageSize: constant.page.pageSize
                },
                pager: {
                    showPageSizeSelector: false,
                    allowedPageSizes: constant.page.pageSizeArrary,
                    showInfo: true
                }
            };
            getImportInternationalBUData();
        };

        var getImportInternationalBUData = function () {
            param.type = $scope.importType;
            vatImportService.displayAnalysisInternationalImportData(param).success(function (data) {
                if (data.data) {
                    $scope.internationalBUDataGridSource = data.data;
                } else {
                    SweetAlert.error($translate.instant('SystemError'));
                }
            });
        };

        var loadInternationalTaxDataGrid = function () {
            $scope.internationalTaxDataGridOptions = {
                bindingOptions: {
                    dataSource: 'internationalTaxDataGridSource'
                },
                showBorders: true,
                columns: [
                    {
                        dataField: "taxCategory",
                        width: '40%',
                        allowHeaderFiltering: false,
                        caption: 'Tax Category'
                    }, {
                        dataField: "taxType",
                        allowHeaderFiltering: false,
                        caption: 'Tax Type',
                        width: '30%'
                    }, {
                        dataField: "taxAmount",
                        allowHeaderFiltering: false,
                        width: '30%',
                        caption: 'Tax Amount'
                    }],
                onContentReady: function (e) {
                    $scope.internationalTaxDataInstance = e.component;
                    var totalCount = e.component.totalCount();
                    if (totalCount > 0) {
                        $scope.totalCount = totalCount;
                    }
                },
                loadPanel: {
                    enabled: false
                },
                selection: {
                    mode: "single"
                },
                grouping: {
                    autoExpandAll: false
                },
                allowColumnResizing: true,
                allowColumnReordering: true,
                columnAutoWidth: true,
                showRowLines: true,
                showColumnLines: true,
                rowAlternationEnabled: true,  //单双行颜色
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                headerFilter: {
                    visible: false,
                    texts: {
                        cancel: $translate.instant('Cancel'),
                        ok: $translate.instant('Confirm'),
                        emptyValue: $translate.instant('Empty'),
                        selectAllText: $translate.instant('SelectAll')
                    }
                },
                paging: {
                    pageSize: constant.page.pageSize
                },
                pager: {
                    showPageSizeSelector: false,
                    allowedPageSizes: constant.page.pageSizeArrary,
                    showInfo: true
                }
            };
            getImportInternationalTaxData();
        };

        var getImportInternationalTaxData = function () {
            param.type = $scope.importType;
            vatImportService.displayAnalysisInternationalImportData(param).success(function (data) {
                if (data.data) {
                    $scope.internationalTaxDataGridSource = data.data;
                } else {
                    SweetAlert.error($translate.instant('SystemError'));
                }
            });
        };

        //加载文档后操作
        var initImportFile = function (fileName) {
            var arr = fileName.name.split('.');
            if (arr[arr.length - 1] !== 'xls' && arr[arr.length - 1] !== 'xlsx') {
                SweetAlert.warning($translate.instant('ImportFileInvalidType'));
                return;
            }
            $scope.fileName = fileName.name;
            //初始化数据
            $scope.selectedMappingList = [];
            $scope.ImportErrorTab = false;
            $scope.ImportErrorTag = false;
        };


        //日志
        var showOperateLogPop = function () {
            $scope.isShowLog = true;
        };


        var initPagingControl = function (totalItemsCount) {
            $scope.pagingOptions.totalItems = totalItemsCount;
        };


        // 检查用户机构权限
        var checkUserOrganizationPermissionList = function () {

            var list = [];

            var userManageTemp = constant.vatPermission.dataImport.balanceSheet;
            list.push(userManageTemp.importCode);
            $scope.hasImportPermission = false;
            $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
                $scope.hasImportPermission = data[userManageTemp.importCode];
            });
        };


        //控制明细信息中删除权限
        var getUserPermission = function () {
            var list = [];
            var code = constant.vatPermission.dataImport.balanceSheet.importCode;
            list.push(code);

            $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
                $scope.hasEditPermission = data[code];
            });
        };


        var setButtonWrapStyle = function () {
           /* if ($scope.fileName) {
                return {width: "100%"};
            }*/
        };

        var setGridStyle = function () {
            if ($scope.showTotalSecondRow) {
                return {'margin-top': '60px'}
            }
            else {
                return {'margin-top': '55px'}
            }
        };

        var initDatePicker = function () {
            var ele1 = $("#periodDatepicker");
            ele1.datepicker({
                startDate: $scope.startDate,
                endDate: $scope.endDate,
                viewMode: $scope.viewMode,
                minViewMode: $scope.viewMode,
                autoclose: true, //选中之后自动隐藏日期选择框
                clearBtn: true, //清除按钮
                todayBtn: false, //今日按钮
                format: $scope.dateFormat //日期格式，详见 http://bootstrap-datepicker.readthedocs.org/en/release/options.html#format
            });
            ele1.datepicker("setDate", $scope.selectedDate);
        };

        var initParam = function () {
            $scope.doUpload = doUpload;
            $scope.doExport = doExport;
            $scope.downloadTemplate = downloadTemplate;
            $scope.initPagingControl = initPagingControl;
            $scope.setButtonWrapStyle = setButtonWrapStyle;
            $scope.getGridHeight = getGridHeight;
            $scope.showOperateLogPop = showOperateLogPop;
            $scope.setGridStyle = setGridStyle;
        };

        $scope.selectCompanyEvent = function (i) {
            $scope.selectCompany = i.name;
            refreshGrid();
        };

        $scope.selectCountryEvent = function () {
            ////$scope.selectCountry = country;
            refreshGrid();
        };

        $scope.selectOne = function () {
            $scope.checkedCompanyList = [];
            angular.forEach($scope.companyList, function (i) {
                var index = $scope.checkedCompanyList.indexOf(i.id);
                if (i.checked && index === -1) {
                    $scope.checkedCompanyList.push(i);
                } else if (!i.checked && index !== -1) {
                    $scope.checkedCompanyList.splice(index, 1);
                }
            });

            $scope.selectedAll = $scope.companyList.length === $scope.checkedCompanyList.length;

            $scope.checkedCompanyTypeList = "";
            $scope.checkedCompanyCodeList = [];
            angular.forEach($scope.checkedCompanyList, function (i) {
                $scope.checkedCompanyTypeList += i.name + ";";
                $scope.checkedCompanyCodeList.push(i.id);
            });
            console.log($scope.checkedCompanyList);
        };

        $scope.selectAll = function () {
            if ($scope.selectedAll) {
                $scope.selectedOne = true;
                $scope.checkedCompanyList = [];
                angular.forEach($scope.companyList, function (i, index) {
                    $scope.checkedCompanyList.push(i);
                    i.checked = true;
                })
            } else {
                $scope.selectedOne = false;
                $scope.checkedCompanyList = [];
                angular.forEach($scope.companyList, function (i, index) {
                    i.checked = false;
                })
            }
            $scope.checkedCompanyTypeList = "";
            $scope.checkedCompanyCodeList = [];
            angular.forEach($scope.checkedCompanyList, function (i) {
                $scope.checkedCompanyTypeList += i.name;
                $scope.checkedCompanyCodeList.push(i.id);
            });
            console.log($scope.checkedCompanyList);
        };

        $scope.changeTab = function (i) {
            $scope.importType = i.code;
            $scope.selectType = i.type;
            $scope.showInternationalBUDataGrid = false;
            $scope.showInternationalTaxDataGrid = false;
            $scope.fileName='';
            switch (i.code) {
                case 100:
                    $scope.showInternationalBUDataGrid = true;
                    loadinternationalBUDatagrid();
                    /*$scope.selectCountry = '';
                    $scope.selectCompany = '';*/
                    break;
                case 101:
                    $scope.showInternationalTaxDataGrid = true;
                    loadInternationalTaxDataGrid();
                    /*  $scope.selectCountry = '';
                      $scope.selectCompany = '';*/
                    break;
                default:
                    break;
            }
        };

        /*------------------------------kevin insert  ----------------------------------------------------------------*/
        var initCompanyList = function () {
            orgService.getOrgListByUserId().success(function (data) {
                var joinText = "";
                if (data) {
                    if (data) {
                        for (var a = 0; a < data.length; a++) {
                            joinText += "<option value =" + data[a].id + " style ='text-align: left;'    >" + data[a].name + "</option>";
                        }
                        $('#orgList').html(joinText);
                        $('#orgList').selectpicker('refresh');
                    }
                }
            });
        };
        $('#orgList').on('changed.bs.select', function (e, clickedIndex, isSelected, value) {
            $scope.selectCompany = e.currentTarget[clickedIndex].value;
        });
        $('#contryList').on('changed.bs.select', function (e, clickedIndex, isSelected, value) {
            $scope.selectCountry = e.currentTarget[clickedIndex].value;
        });

        var initContryList = function () {
            var joinText = "";
            if (constant.countryCNList) {
                for (var i = 0; i < constant.countryCNList.length; i++) {
                    joinText += "<option value =" + constant.countryCNList[i] + "   style ='text-align: left;' >" + constant.countryCNList[i] + "</option>";
                }
                $('#contryList').html(joinText);
                $('#contryList').selectpicker('refresh');
            }
        };
        $('#orgList').on('shown.bs.select', function () {
            $(this).find('.dropdown-menu').width("200px");
        });

        $scope.analysisInitData = function () {
            SweetAlert.swal({
                title: $translate.instant('WarningTitle'),
                text: $translate.instant('SaveAnalysisData'),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('ConfirmYes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                showCloseButton : true
            }, function (isConfirm) {
                var type = 1;//默认保留原始数据
                if (isConfirm) {
                    type = 1;
                    vatImportService.analysisInitData(type).success(function (res) {
                        if (res.resultMsg) {
                            SweetAlert.success("分析数据初始化成功！" + '消耗' + Number(res.resultMsg / 1000) + "秒");
                        }
                    }).error(function (error) {
                        SweetAlert.error(error);
                    });
                } else if (isConfirm == false) {
                    type = 0;
                    vatImportService.analysisInitData(type).success(function (res) {
                        if (res.resultMsg) {
                            SweetAlert.success("分析数据初始化成功！" + '消耗' + Number(res.resultMsg / 1000) + "秒");
                        }
                    }).error(function (error) {
                        SweetAlert.error(error);
                    });
                }
            });
        };

        $scope.handleAnalysisData = function () {
            SweetAlert.swal({
                title: $translate.instant('WarningTitle'),
                text: $translate.instant('SaveAnalysisData'),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('ConfirmYes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                showCloseButton : true
            }, function (isConfirm) {
                var type = 1;//默认保留原始数据
                if (isConfirm) {
                    type = 1;
                    vatImportService.handleAnalysisData(type).success(function (res) {
                        if (res.resultMsg) {
                            SweetAlert.success("手动加载分析模块数据成功！" + '消耗' + Number(res.resultMsg) + "秒");
                        }
                    }).error(function (error) {
                        SweetAlert.error(error);
                    });
                } else if (isConfirm == false) {
                    type = 0;
                    vatImportService.handleAnalysisData(type).success(function (res) {
                        if (res.resultMsg) {
                            SweetAlert.success("手动加载分析模块数据成功！" + '消耗' + Number(res.resultMsg) + "秒");
                        }
                    }).error(function (error) {
                        SweetAlert.error(error);
                    });
                }
            });
        };

        /*------------------------------------------------------------------------------------------------------------*/

        //开始
        (function initialize() {
            $log.debug('internationalDataImportController.ctor()...');
            initParam();
            checkUserOrganizationPermissionList();
            getUserPermission();
            initDatePicker();
            initCompanyList();
            initContryList();

            $scope.$watch('importExcelFile', function (newValue, oldValue) {
                if (newValue !== null && newValue !== oldValue) {
                    initImportFile(newValue);
                }
            });

            $scope.$watch('UploadPeriodTime', function (newValue, oldValue) {
                refreshGrid();
            });

            $timeout(function () {
                $scope.isLoadComplete = true;
            }, 500);
        })();
    }
]);