﻿backlogManageModule.controller('backlogCalendarController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService', 'apiInterceptor', 'jqFetch', '$filter', 'Upload',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService, apiInterceptor, jqFetch, $filter, Upload) {
        'use strict';
        $log.debug('backlogCalendarController.ctor()...');

        $scope.scheduleList = [];

        var SCHEDULE_CATEGORY = [
            'milestone',
            'task'
        ];

        function ScheduleInfo() {
            this.id = null;
            this.calendarId = null;

            this.title = null;
            this.body = null;
            this.isAllday = false;
            this.start = null;
            this.end = null;
            this.category = '';
            this.dueDateClass = '';

            this.color = null;
            this.bgColor = null;
            this.dragBgColor = null;
            this.borderColor = null;
            this.customStyle = '';

            this.isFocused = false;
            this.isPending = false;
            this.isVisible = true;
            this.isReadOnly = false;
            this.goingDuration = 0;
            this.comingDuration = 0;
            this.recurrenceRule = '';

            this.raw = {
                memo: '',
                hasToOrCc: false,
                hasRecurrenceRule: false,
                location: null,
                class: 'public', // or 'private'
                creator: {
                    name: '',
                    avatar: '',
                    company: '',
                    email: '',
                    phone: ''
                }
            };
        }


        $scope.calendarList = [];

        function CalendarInfo() {
            this.id = null;
            this.name = null;
            this.checked = true;
            this.color = null;
            this.bgColor = null;
            this.borderColor = null;
        }

        function addCalendar(calendar) {
            $scope.calendarList.push(calendar);
        }

        function findCalendar(id) {
            var found;

            $scope.calendarList.forEach(function (calendar) {
                if (calendar.id === id) {
                    found = calendar;
                }
            });

            return found || $scope.calendarList[0];
        }


        var cal, resizeThrottled;
        var useCreationPopup = true;
        var datePicker, selectedCalendar;
        var COMMON_CUSTOM_THEME = {
            'common.border': '1px solid #ffbb3b',
            'common.backgroundColor': '#ffbb3b0f',
            'common.holiday.color': '#f54f3d',
            'common.saturday.color': '#3162ea',
            'common.dayname.color': '#333'
        };
        cal = new tui.Calendar('#calendar', {
            defaultView: 'month',
            theme: COMMON_CUSTOM_THEME, // set theme
            useCreationPopup: false,
            useDetailPopup: true,
            calendars: $scope.calendarList,
            month: {
                isAlways6Week: false
            },
            disableClick: true,
            disableDblClick: true,
            template: {
                milestone: function (model) {
                    return '<span class="calendar-font-icon ic-milestone-b"></span> <span style="background-color: ' + model.bgColor + '">' + model.title + '</span>';
                },
                allday: function (schedule) {
                    return getTimeTemplate(schedule, true);
                },
                time: function (schedule) {
                    return getTimeTemplate(schedule, false);
                },
                popupDetailDate: function (isAllDay, start, end) {
                    return start.getFullYear() + '-' + (start.getMonth() + 1) + '-' + start.getDate();
                }
            }
        });
        $scope.commentListOptions = {
            noDataText: null,
            editEnabled: true,
            allowItemDeleting: false,
            bindingOptions: {
                dataSource: 'detailEdit.comments'
            }
        };
        $scope.unEditFileListOptions = {
            itemTemplate: function (item, index, container) {
                var div = $('<div></div>');
                $('<span style="float: left">' + item.fileName + '</span>').appendTo(div);
                $('<i style="float: right;color: #dc6900"class="fa fa-download"></i>').on('click', function () {
                    $scope.downFile(item.uid, item.fileName, false);
                }).appendTo(div);
                div.appendTo(container);
            },
            noDataText: null,
            editEnabled: true,
            allowItemDeleting: false,
            bindingOptions: {
                dataSource: 'detailEdit.unEditFileUploads'
            }
        };
        $scope.editFileListOptions = {
            itemTemplate: function (item, index, container) {
                var div = $('<div></div>');
                $('<span style="float: left">' + item.fileName + '</span>').appendTo(div);
                $('<i style="float: right;color: #dc6900"class="fa fa-download"></i>').on('click', function () {
                    $scope.downFile(item.uid, item.fileName, false);
                }).appendTo(div);
                div.appendTo(container);
            },
            noDataText: null,
            editEnabled: true,
            allowItemDeleting: true,
            bindingOptions: {
                dataSource: 'detailEdit.editFileUploads'
            },
            onItemDeleting: function (e) {
                var d = $.Deferred();
                DevExpress.ui.dialog.confirm($translate.instant('BDLEditDelMsg'))
                    .done(function (value) {
                        d.resolve(!value);
                    })
                    .fail(d.reject);
                e.cancel = d.promise();
            }
        };
        $scope.downFile = function (fileUoloadId, fileName, zipFlag) {
            var fileUoloadIds = fileUoloadId.split(',');
            var ids = [];
            angular.forEach(fileUoloadIds, function (item) {
                if (item && item != '') {
                    ids.push(item);
                }
            });
            if (ids.length == 0) {
                SweetAlert.error($translate.instant('UnFileDownload'));
                return;
            }
            DevExpress.ui.dialog.confirm($translate.instant('DownloadMsg'))
                .done(function (value) {
                    if (value) {
                        $('#busy-indicator-container').show();
                        var xhr = new XMLHttpRequest();
                        xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/detail/download", true);
                        xhr.responseType = 'arraybuffer';
                        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                        xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
                        xhr.onload = function () {
                            // var headers = xhr.getAllResponseHeaders();
                            // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                            if (this.status === 200) {
                                var blob = new Blob([this.response], {type: "arraybuffer"});
                                if (typeof window.navigator.msSaveBlob !== 'undefined') {
                                    window.navigator.msSaveBlob(blob, fileName)
                                } else {
                                    var URL = window.URL || window.webkitURL;
                                    var objectUrl = URL.createObjectURL(blob);
                                    if (fileName) {
                                        var a = document.createElement('a');
                                        if (typeof a.download === 'undefined') {
                                            window.location = objectUrl
                                        } else {
                                            a.href = objectUrl;
                                            a.download = fileName;
                                            document.body.appendChild(a);
                                            a.click();
                                            a.remove();
                                        }
                                    } else {
                                        window.location = objectUrl;
                                    }
                                    $('#busy-indicator-container').hide();
                                }
                            }
                        };
                        xhr.send(JSON.stringify({"fileUploadIds": ids, "zipFlag": zipFlag}));
                    }
                });

        };
        $scope.backlogTypeData = null;
        $scope.getBacklogType = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/calendar/type')
                .then(function (res) {
                    if (res) {
                        $scope.backlogTypeData = res;
                    } else {
                        SweetAlert.error($translate.instant('GetRevenueTypeError'));
                    }
                })
        };
        //编辑
        $scope.editDetail = function (data) {
            if (data.handleFlag <= 0) {
                $scope.detailEditReadOnly = true;
            } else {
                if ('UN_BEGIN' == data.backlogStatus) {
                    $scope.detailEditReadOnly = true;
                } else {
                    $scope.detailEditReadOnly = false;
                }
            }
            $scope.detailEdit = null;
            $scope.detailEdit = data;
            if ($scope.detailEdit.amount && (typeof $scope.detailEdit.amount == "number")) {
                $scope.detailEdit.amount = Number($scope.detailEdit.amount).formatAmount(2);
            }
            $scope.detailEdit.editFileUploads = [];
            $scope.detailEdit.unEditFileUploads = [];
            $scope.detailEdit.comments = [];
            var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
            $scope.detailEdit.backlogTypeStr = backlogType.name;
            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
            $scope.detailEdit.backlogTypeParentStr = parentBacklogType.name;
            $http.post('/backlogManage/backlog/log', {"detailId": data.id}, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        angular.forEach(res, function (item) {
                            if (item.currUserFlag) {
                                $scope.detailEdit.comment = item.comment;
                            } else {
                                if (item.comment) {
                                    $scope.detailEdit.comments.push(item.dealRoleStr + "：" + item.comment);
                                }
                            }
                            if (item.fileUploads) {
                                angular.forEach(item.fileUploads, function (item1) {
                                    if (item.currUserFlag) {
                                        $scope.detailEdit.editFileUploads.push(item1)
                                    } else {

                                        $scope.detailEdit.unEditFileUploads.push(item1);
                                    }

                                });
                            }
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
            $('#detailEditDiv').modal('show');
        };

        $scope.editBacklogStatus = function (status) {
            $('#busy-indicator-container').show();
            $http.post('/backlogManage/backlog/detail/status', {
                "ids": [$scope.detailEdit.id],
                "status": status,
            }, apiConfig.createVat())
                .success(function (res) {
                    $('#busy-indicator-container').hide();
                    if (res && 0 === res.code) {
                        if (status == 'BEGIN') {
                            SweetAlert.success($translate.instant('BDLHandleBeginBacklogSuccess'));
                            $scope.detailEdit.backlogStatus = status;
                            $scope.detailEdit.backlogStatusStr = '进行中';
                            $scope.detailEditReadOnly = false;
                        } else if (status == 'VOIDED') {
                            SweetAlert.success($translate.instant('BDLHandleVoidedBacklogSuccess'));
                        }
                        setSchedules().then(function () {
                            refreshScheduleVisibility();
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }

                });
        };
        //上传附件
        $('#configUpload').on('change', function () {
            $('#busy-indicator-container').show();
            Upload.upload({
                url: apiInterceptor.webApiHostUrl + '/backlogManage/backlog/upload',
                data: {},
                file: $('#configUpload')[0].files[0],
                headers: {
                    'Access-Control-Allow-Origin': '*',
                    Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                    withCredentials: true
                },
                withCredentials: true
            }).then(function (resp) {
                if (resp && resp.data && resp.data.result) {
                    $scope.detailEdit.editFileUploads.push(resp.data.data);
                }
                $('#busy-indicator-container').hide();
            }, function (resp) {
                SweetAlert.error($translate.instant('SystemError'));
                $('#busy-indicator-container').hide();
            }, function (evt) {

            });
        });

        $scope.saveBacklogLog = function (status) {
            $('#busy-indicator-container').show();
            var fileUploadIds = '';
            angular.forEach($scope.detailEdit.editFileUploads, function (item) {
                fileUploadIds += (',' + item.uid);
            });
            $http.post('/backlogManage/backlog/log/edit', {
                "detailId": $scope.detailEdit.id,
                "fileUploadIds": fileUploadIds,
                "comment": $scope.detailEdit.comment,
                "status": status
            }, apiConfig.createVat())
                .success(function (res) {
                    $('#busy-indicator-container').hide();
                    if (res && 0 === res.code) {
                        if (status == 'COMPLETED') {
                            SweetAlert.success($translate.instant('BDLEditEditHandleCompleteSuccess'));
                        } else if (status == 'REJECT') {
                            SweetAlert.success($translate.instant('BDLEditEditHandleRejectSuccess'));
                        } else {
                            SweetAlert.success($translate.instant('BDLEditSuccess'));
                        }
                        $('#detailEditDiv').modal('hide');
                        setSchedules().then(function () {
                            refreshScheduleVisibility();
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
        };


        // event handlers
        cal.on({
            'clickMore': function (e) {
            },
            'clickSchedule': function (e) {
                $scope.comments = [];

                var calendarData = $filter('filter')($scope.calendarData, function (value) {
                    var date1 = new Date(value.day);
                    var dateStr1 = date1.getFullYear() + '-' + date1.getMonth() + '-' + date1.getDate();
                    var date2 = e.schedule.start;
                    var dateStr2 = date2.getFullYear() + '-' + date2.getMonth() + '-' + date2.getDate();
                    if (dateStr1 == dateStr2) {
                        return true;
                    } else {
                        return false;
                    }
                })[0].detailMap;

                $scope.dataList = calendarData[e.schedule.id];
                var str = '';
                angular.forEach($scope.dataList, function (item) {
                    if (item.orgName) {
                        str += ('<a style="cursor:pointer" href="javacript:void(0);" id="' + item.id + '">' + item.orgName + '-' + item.backlogStatusStr + '</a></br>');
                    } else {
                        str += ('<a style="cursor:pointer" href="javacript:void(0);" id="' + item.id + '">' + item.businessLine + '-' + item.backlogStatusStr + '</a></br>');
                    }
                });
                $(".tui-full-calendar-popup-detail-item.tui-full-calendar-popup-detail-item-separate").html(str);
                angular.forEach($scope.dataList, function (item) {
                    $("#" + item.id).on("click", function () {
                        $scope.editDetail(item);
                    });
                });
                // var param = $scope.searchParam;
                // param.backlogTypeId = e.schedule.calendarId;
                // $http.post('/backlogManage/backlog/detail', param, apiConfig.createVat())
                //     .success(function (res) {
                //         if (res && res.list) {
                //             $scope.dataList = res.list;
                //             var str = '';
                //             angular.forEach(res.list, function (item) {
                //                 if (item.orgName) {
                //                     str += ('<a style="cursor:pointer" href="javacript:void(0);" id="' + item.id + '">' + item.orgName + '-' + item.backlogStatusStr + '</a></br>');
                //                 } else {
                //                     str += ('<a style="cursor:pointer" href="javacript:void(0);" id="' + item.id + '">' + item.businessLine + '-' + item.backlogStatusStr + '</a></br>');
                //                 }
                //             });
                //             $(".tui-full-calendar-popup-detail-item.tui-full-calendar-popup-detail-item-separate").html(str);
                //         } else {
                //             SweetAlert.error($translate.instant('SystemError'));
                //         }
                //         angular.forEach($scope.dataList, function (item) {
                //             $("#" + item.id).on("click", function () {
                //                 $scope.editDetail(item);
                //             });
                //         });
                //
                //     })
            },
            'clickDayname': function (date) {
            },
            'beforeCreateSchedule': function (e) {
                // saveNewSchedule(e);
            },
            'beforeUpdateSchedule': function (e) {

            },
            'beforeDeleteSchedule': function (e) {

            },
            'afterRenderSchedule': function (e) {
            },
            'clickTimezonesCollapseBtn': function (timezonesCollapsed) {
                return true;
            }
        });


        /**
         * Get time template for time and all-day
         * @param {Schedule} schedule - schedule
         * @param {boolean} isAllDay - isAllDay or hasMultiDates
         * @returns {string}
         */
        function getTimeTemplate(schedule, isAllDay) {
            var html = [];
            var start = moment(schedule.start.toUTCString());
            if (!isAllDay) {
                html.push('<strong>' + start.format('HH:mm') + '</strong> ');
            }
            if (schedule.isPrivate) {
                html.push('<span class="calendar-font-icon ic-lock-b"></span>');
                html.push(' Private');
            } else {
                // if (schedule.isReadOnly) {
                //     html.push('<span class="calendar-font-icon ic-readonly-b"></span>');
                // } else if (schedule.recurrenceRule) {
                //     html.push('<span class="calendar-font-icon ic-repeat-b"></span>');
                // } else if (schedule.attendees.length) {
                //     html.push('<span class="calendar-font-icon ic-user-b"></span>');
                // } else if (schedule.location) {
                //     html.push('<span class="calendar-font-icon ic-location-b"></span>');
                // }
                html.push(' ' + schedule.title);
            }

            return html.join('');
        }

        /**
         * A listener for click the menu
         * @param {Event} e - click event
         */
        function onClickMenu(e) {
            var target = $(e.target).closest('a[role="menuitem"]')[0];
            var action = getDataAction(target);
            var options = cal.getOptions();
            var viewName = '';
            switch (action) {
                case 'toggle-daily':
                    viewName = 'day';
                    break;
                case 'toggle-weekly':
                    viewName = 'week';
                    break;
                case 'toggle-monthly':
                    options.month.visibleWeeksCount = 0;
                    viewName = 'month';
                    break;
                case 'toggle-weeks2':
                    options.month.visibleWeeksCount = 2;
                    viewName = 'month';
                    break;
                case 'toggle-weeks3':
                    options.month.visibleWeeksCount = 3;
                    viewName = 'month';
                    break;
                case 'toggle-narrow-weekend':
                    options.month.narrowWeekend = !options.month.narrowWeekend;
                    options.week.narrowWeekend = !options.week.narrowWeekend;
                    viewName = cal.getViewName();

                    target.querySelector('input').checked = options.month.narrowWeekend;
                    break;
                case 'toggle-start-day-1':
                    options.month.startDayOfWeek = options.month.startDayOfWeek ? 0 : 1;
                    options.week.startDayOfWeek = options.week.startDayOfWeek ? 0 : 1;
                    viewName = cal.getViewName();

                    target.querySelector('input').checked = options.month.startDayOfWeek;
                    break;
                case 'toggle-workweek':
                    options.month.workweek = !options.month.workweek;
                    options.week.workweek = !options.week.workweek;
                    viewName = cal.getViewName();

                    target.querySelector('input').checked = !options.month.workweek;
                    break;
                default:
                    break;
            }

            cal.setOptions(options, true);
            cal.changeView(viewName, true);

            // setDropdownCalendarType();
            setRenderRangeText();
            // setSchedules();
        }

        function onClickNavi(e) {
            var action = getDataAction(e.target);

            switch (action) {
                case 'move-prev':
                    cal.prev();
                    break;
                case 'move-next':
                    cal.next();
                    break;
                case 'move-today':
                    cal.today();
                    break;
                default:
                    return;
            }

            setRenderRangeText();
            // setSchedules();
        }


        function onChangeCalendars(e) {
            $scope.searchParam.orgIdList = [];
            var orgElements = Array.prototype.slice.call(document.querySelectorAll('#orgSearch input'));
            orgElements.forEach(function (input) {
                if (input.checked == true) {
                    $scope.searchParam.orgIdList.push(input.value);
                }
            });
            $scope.searchParam.businessIdList = [];
            var businessElements = Array.prototype.slice.call(document.querySelectorAll('#businessSearch input'));
            businessElements.forEach(function (input) {
                if (input.checked == true) {
                    $scope.searchParam.businessIdList.push(input.value);
                }
            });
            $scope.searchParam.backlogStatusList = [];
            var statusElements = Array.prototype.slice.call(document.querySelectorAll('#statusSearch input'));
            statusElements.forEach(function (input) {
                if (input.checked == true) {
                    $scope.searchParam.backlogStatusList.push(input.value);
                }
            });
            $scope.searchParam.currDeal = null;
            var currElements = Array.prototype.slice.call(document.querySelectorAll('#currSearch input'));
            var currCount = 0;
            var currFlag = false;
            currElements.forEach(function (input) {
                if (input.checked == true) {
                    if (input.value == 'isCurr') {
                        currFlag = true;
                    }
                    currCount += 1;
                }
            });
            if (currCount < 2 && currCount > 0) {
                $scope.searchParam.currDeal = currFlag;
            }
            var calendarId = e.target.value;
            var checked = e.target.checked;
            var viewAll = document.querySelector('.lnb-calendars-item input');
            var calendarElements = Array.prototype.slice.call(document.querySelectorAll('#calendarList input'));
            var allCheckedCalendars = true;

            if (calendarId === 'all') {
                calendarElements.forEach(function (input) {
                    var span = input.parentNode;
                    input.checked = checked;
                    span.style.backgroundColor = checked ? span.style.borderColor : 'transparent';
                });

                $scope.calendarList.forEach(function (calendar) {
                    calendar.checked = checked;
                });
            } else {
                if (e.target.name == 'typeSearch') {

                    findCalendar(calendarId).checked = checked;

                    allCheckedCalendars = calendarElements.every(function (input) {
                        return input.checked;
                    });

                    if (allCheckedCalendars) {
                        viewAll.checked = true;
                    } else {
                        viewAll.checked = false;
                    }
                    viewAll.nextElementSibling.style.color = viewAll.checked ? 'grey' : 'white';
                }
            }
            if (e.target.name == 'typeSearch') {
                refreshScheduleVisibility();
            } else {
                var iDocument = e.target.nextElementSibling;
                iDocument.style.color = checked ? 'grey' : 'white';
                setSchedules().then(function () {
                    refreshScheduleVisibility();
                });
            }

        }

        function refreshScheduleVisibility() {
            var calendarElements = Array.prototype.slice.call(document.querySelectorAll('#calendarList input'));

            $scope.calendarList.forEach(function (calendar) {
                cal.toggleSchedules(calendar.id, !calendar.checked, false);
            });

            cal.render(true);

            calendarElements.forEach(function (input) {
                var span = input.nextElementSibling;
                span.style.backgroundColor = input.checked ? span.style.borderColor : 'transparent';
            });
        }

        function setDropdownCalendarType() {
            var calendarTypeName = document.getElementById('calendarTypeName');
            var calendarTypeIcon = document.getElementById('calendarTypeIcon');
            var options = cal.getOptions();
            var type = cal.getViewName();
            var iconClassName;

            if (type === 'day') {
                type = 'Daily';
                iconClassName = 'calendar-icon ic_view_day';
            } else if (type === 'week') {
                type = 'Weekly';
                iconClassName = 'calendar-icon ic_view_week';
            } else if (options.month.visibleWeeksCount === 2) {
                type = '2 weeks';
                iconClassName = 'calendar-icon ic_view_week';
            } else if (options.month.visibleWeeksCount === 3) {
                type = '3 weeks';
                iconClassName = 'calendar-icon ic_view_week';
            } else {
                type = 'Monthly';
                iconClassName = 'calendar-icon ic_view_month';
            }

            calendarTypeName.innerHTML = type;
            calendarTypeIcon.className = iconClassName;
        }

        function setRenderRangeText() {
            var renderRange = document.getElementById('renderRange');
            var options = cal.getOptions();
            var viewName = cal.getViewName();
            var html = [];
            if (viewName === 'day') {
                html.push(moment(cal.getDate().getTime()).format('YYYY.MM.DD'));
            } else if (viewName === 'month' &&
                (!options.month.visibleWeeksCount || options.month.visibleWeeksCount > 4)) {
                html.push(moment(cal.getDate().getTime()).format('YYYY.MM'));
            } else {
                html.push(moment(cal.getDateRangeStart().getTime()).format('YYYY.MM.DD'));
                html.push(' ~ ');
                html.push(moment(cal.getDateRangeEnd().getTime()).format(' MM.DD'));
            }
            renderRange.innerHTML = html.join('');
        }

        function setSchedules() {
            var defer = $q.defer();
            cal.clear();
            $scope.scheduleList = [];
            $scope.calendarData = null;
            $http.post('/backlogManage/backlog/detail/calendar', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        var schedule;
                        $scope.calendarData = res;
                        angular.forEach(res, function (item) {
                            for (var key in item.detailMap) {
                                var dataDetail = item.detailMap[key][0];
                                schedule = new ScheduleInfo();
                                schedule.id = dataDetail.commonId;
                                schedule.calendarId = String(dataDetail.backlogTypeId);
                                schedule.title = dataDetail.commonName;
                                schedule.isReadOnly = true;
                                var startDate = moment(new Date(item.day).getTime())
                                var endDate = moment(new Date(item.day).getTime());
                                schedule.start = startDate.toDate();
                                schedule.end = endDate.toDate();
                                schedule.category = 'time';
                                schedule.color = '#ffffff';
                                var backlogType = $filter('filter')($scope.backlogTypeData, {id: dataDetail.backlogTypeId})[0];
                                schedule.bgColor = backlogType.color;
                                schedule.dragBgColor = backlogType.color;
                                schedule.borderColor = backlogType.color;
                                schedule.body = dataDetail.commonName;
                                $scope.scheduleList.push(schedule);
                            }
                        });
                        cal.createSchedules($scope.scheduleList);
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                    // if (res) {
                    //     var schedule;
                    //     angular.forEach(res, function (item) {
                    //         schedule = new ScheduleInfo();
                    //         schedule.id = item.commonId;
                    //         schedule.calendarId = String(item.backlogTypeId);
                    //         schedule.title = item.commonName;
                    //         schedule.isReadOnly = true;
                    //         var startDate = moment(new Date(item.startDate).getTime())
                    //         var endDate = moment(new Date(item.endDate).getTime());
                    //         schedule.start = startDate.toDate();
                    //         schedule.end = endDate.toDate();
                    //         schedule.category = 'time';
                    //         schedule.color = '#ffffff';
                    //         var backlogType = $filter('filter')($scope.backlogTypeData, {id: item.backlogTypeId})[0]
                    //         schedule.bgColor = backlogType.color;
                    //         schedule.dragBgColor = backlogType.color;
                    //         schedule.borderColor = backlogType.color;
                    //         schedule.body = item.commonName;
                    //         $scope.scheduleList.push(schedule);
                    //     });
                    //     cal.createSchedules($scope.scheduleList);
                    // } else {
                    //     SweetAlert.error($translate.instant('SystemError'));
                    // }
                    defer.resolve(res);
                });
            return defer.promise;
        }

        function setEventListener() {
            $('#menu-navi').on('click', onClickNavi);
            $('.dropdown-menu a[role="menuitem"]').on('click', onClickMenu);
            $('#lnb-calendars').on('change', onChangeCalendars);
            window.addEventListener('resize', resizeThrottled);
        }

        function getDataAction(target) {
            return target.dataset ? target.dataset.action : target.getAttribute('data-action');
        }

        resizeThrottled = tui.util.throttle(function () {
            cal.render();
        }, 50);


        $scope.searchParam = {};
        $scope.backlogTypeData = null;
        (function () {
            $scope.getBacklogType();
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/calendar/type').then(function (res) {
                if (res) {
                    var calendar;
                    $scope.backlogTypeData = res;
                    angular.forEach(res, function (item) {
                        if (item.level == 2) {
                            calendar = new CalendarInfo();
                            calendar.id = item.id;
                            calendar.name = item.fullName;
                            calendar.color = item.color;
                            calendar.bgColor = item.color;
                            calendar.dragBgColor = item.color;
                            calendar.borderColor = item.color;
                            addCalendar(calendar);
                        }
                    });
                    var calendarList = document.getElementById('calendarList');
                    var html = [];
                    $scope.calendarList.forEach(function (calendar) {
                        html.push('<div class="lnb-calendars-item" ><label>' +
                            '<input type="checkbox" name="typeSearch" class="tui-full-calendar-checkbox-round" value="' + calendar.id + '" checked>' +
                            '<i class="fa fa-check" style="color:white;text-align:center;border-color: ' + calendar.borderColor + '; background-color: ' + calendar.borderColor + ';"></i>' +
                            '<span>' + calendar.name + '</span>' +
                            '</label></div>'
                        );
                    });
                    calendarList.innerHTML = html.join('\n');

                    //添加部门筛选
                    jqFetch.get(apiInterceptor.webApiHostUrl + '/org/getMyOrgList')
                        .then(function (res) {
                            if (res && 0 === res.code) {
                                var orgLeft = '<div class="lnb-calendars-d1" id="orgSearch" style="height: 210px;overflow: auto">';
                                var dataMap = [];
                                angular.forEach(res.data, function (item) {
                                    orgLeft += '<div class="lnb-calendars-item"><label>' +
                                        '<input class="tui-full-calendar-checkbox-round" type="checkbox" value="' + item.id + '" checked>' +
                                        '<i class="fa fa-check" style="color:grey;text-align:center;border-color:grey;background-color:white;"></i>' +
                                        '<span>' + item.name + '</span> </label></div>';
                                    dataMap[item.businessUnitId] = item.businessUnitName;
                                })
                                orgLeft += '</div>';
                                $("#lnb-calendars").append(orgLeft);
                                var businessLeft = '<div class="lnb-calendars-d1" id="businessSearch" style="height: 120px;overflow: auto">';

                                for (var key in dataMap) {
                                    if (key != 'null' && key.length > 18) {
                                        businessLeft += '<div class="lnb-calendars-item"><label>' +
                                            '<input class="tui-full-calendar-checkbox-round" type="checkbox" value="' + key + '" checked>' +
                                            '<i class="fa fa-check" style="color:grey;text-align:center;border-color:grey;background-color:white;"></i>' +
                                            '<span>' + dataMap[key] + '</span> </label></div>';
                                    }
                                }
                                businessLeft += '</div>';
                                $("#lnb-calendars").append(businessLeft);

                                //添加状态筛选
                                var statusList = [
                                    {'key': '未开始', 'val': 'UN_BEGIN'},
                                    {'key': '进行中', 'val': 'BEGIN'},
                                    {'key': '待审核', 'val': 'UN_AUDIT'},
                                    {'key': '退回修正', 'val': 'REJECT'},
                                    {'key': '已完成', 'val': 'COMPLETED'},
                                    {'key': '已失效', 'val': 'VOIDED'}
                                ];
                                var statusLeft = '<div class="lnb-calendars-d1" id="statusSearch" style="height: 120px;overflow: auto">';
                                angular.forEach(statusList, function (item) {
                                    statusLeft += '<div class="lnb-calendars-item"><label>' +
                                        '<input class="tui-full-calendar-checkbox-round" type="checkbox"   value="' + item.val + '" checked>' +
                                        '<i class="fa fa-check" style="color:grey;text-align:center;border-color:grey;background-color:white;"></i>' +
                                        '<span>' + item.key + '</span> </label></div>';
                                });
                                statusLeft += '</div>';
                                $("#lnb-calendars").append(statusLeft);

                                //添加个人/他人事项筛选
                                var currLeft = '<div class="lnb-calendars-d1" id="currSearch">';
                                currLeft += '<div class="lnb-calendars-item"><label>' +
                                    '<input class="tui-full-calendar-checkbox-round" type="checkbox"   value="isCurr" checked>' +
                                    '<i class="fa fa-check" style="color:grey;text-align:center;border-color:grey;background-color:white;"></i>' +
                                    '<span>我的事项</span> </label></div>';
                                currLeft += '<div class="lnb-calendars-item"><label>' +
                                    '<input class="tui-full-calendar-checkbox-round" type="checkbox"   value="isNotCurr" checked>' +
                                    '<i class="fa fa-check" style="color:grey;text-align:center;border-color:grey;background-color:white;"></i>' +
                                    '<span>他人事项</span> </label></div>';
                                currLeft += '</div>';
                                $("#lnb-calendars").append(currLeft);
                            } else {
                                SweetAlert.error($translate.instant('GetRevenueTypeError'));
                            }
                        })

                    window.cal = cal;
                    // setDropdownCalendarType();
                    setRenderRangeText();
                    setSchedules().then(function () {
                        refreshScheduleVisibility();
                        setEventListener();
                    });

                }
            });

        })();


    }
]);