﻿vatModule.controller('backlogCaseController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService', 'apiInterceptor', 'jqFetch', '$filter', 'Upload',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService, apiInterceptor, jqFetch, $filter, Upload) {
        'use strict';
        $log.debug('backlogCaseController.ctor()...');

        // var dataGrid = $("#detailGridContainer").dxDataGrid("instance");
        // var selectedKeys = dataGrid.getSelectedRowKeys();
        // var selectedData = dataGrid.getSelectedRowsData();

        //表格配置
        $scope.revenueGridOptions = $.extend(true, {}, dxDataGridService.BASIC_GRID_OPTIONS, {
            selection: {
                mode: "multiple",
                showCheckBoxesMode: "always",
                allowSelectAll: true,
                selectAllMode: "page"
            },
            showBorders: true,
            sorting: {//排序
                ascendingText: "Sort Ascending",
                clearText: "Clear Sorting",
                descendingText: "Sort Descending",
                mode: "single"
            },
            showRowLines: true,
            columnAutoWidth: true,
            allowColumnReordering: true,
            onSelectionChanged: function (selectedItems) {
                $scope.selectedBacklog = selectedItems.selectedRowsData;

            },
            columns: [
                {
                    dataField: 'backlogCode',
                    caption: $translate.instant('BDLColumnCode'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
                        return parentBacklogType.name;
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnSubType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        return backlogType.name;
                    }
                },
                {
                    dataField: 'backlogName',
                    caption: $translate.instant('BDLColumnName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'businessLine',
                    caption: $translate.instant('BDLColumnBusinessLine'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'orgName',
                    caption: $translate.instant('BDLColumnOrgName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnStartDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.chinaBacklogStartDate, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnEndDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.chinaBacklogEndDate, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: 'backlogStatusStr',
                    caption: $translate.instant('BDLColumnStatus'),
                    fixed: true
                },
                {
                    dataField: 'dealUser',
                    caption: $translate.instant('BDLColumnDealUser'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'auditUser',
                    caption: $translate.instant('BDLColumnAuditUser'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'createUser',
                    caption: $translate.instant('BDLColumnCreate'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'backlogGenerateTypeStr',
                    caption: $translate.instant('BDLColumnGenerateType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnFileFlag'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var hasFile = false;
                        if (data.fileUploadIds) {
                            angular.forEach(data.fileUploadIds.split(','), function (item) {
                                if (!hasFile && item && item.length > 1) {
                                    hasFile = true;
                                }
                            });
                        }
                        return hasFile ? '是' : '否';
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BDLColumnHandle'),
                    fixed: true,
                    width: '80px',
                    alignment: 'center',
                    cellTemplate: function (container, options) {
                        try {
                            $('<a>' + $translate.instant('BDLColumnHandleView') + '|</a>&nbsp;&nbsp;')
                                .on('click', function () {
                                    $scope.detailView(options.data);
                                }).appendTo(container);
                            if (options.data.fileUploadIds && options.data.fileUploadIds.split(',').length > 1) {
                                $('<a>' + $translate.instant('BDLColumnHandleDown') + '</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.downFile(options.data.fileUploadIds, options.data.backlogName + '.zip', true)
                                    }).appendTo(container);
                            }
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
            ],
            bindingOptions: {
                dataSource: 'pageConfDataSource'
            }
        });


        //下载选择附件
        $scope.downSelectFile = function () {
            var fileUoloadIds = ',';
            angular.forEach($scope.selectedBacklog, function (item) {
                if (item && item.fileUploadIds) {
                    fileUoloadIds += ',' + item.fileUploadIds;
                }
            });
            $scope.downFile(fileUoloadIds, 'files.zip', true);
        }

        //下载附件
        $scope.downFile = function (fileUoloadId, fileName, zipFlag) {
            var fileUoloadIds = fileUoloadId.split(',');
            var ids = [];
            angular.forEach(fileUoloadIds, function (item) {
                if (item && item != '') {
                    ids.push(item);
                }
            });
            if (ids.length == 0) {
                SweetAlert.error($translate.instant('UnFileDownload'));
                return;
            }
            DevExpress.ui.dialog.confirm($translate.instant('DownloadMsg'))
                .done(function (value) {
                    if (value) {
                        $('#busy-indicator-container').show();
                        var xhr = new XMLHttpRequest();
                        xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/detail/download", true);
                        xhr.responseType = 'arraybuffer';
                        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                        xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
                        xhr.onload = function () {
                            // var headers = xhr.getAllResponseHeaders();
                            // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                            if (this.status === 200) {
                                var blob = new Blob([this.response], {type: "arraybuffer"});
                                if (typeof window.navigator.msSaveBlob !== 'undefined') {
                                    window.navigator.msSaveBlob(blob, fileName)
                                } else {
                                    var URL = window.URL || window.webkitURL;
                                    var objectUrl = URL.createObjectURL(blob);
                                    if (fileName) {
                                        var a = document.createElement('a');
                                        if (typeof a.download === 'undefined') {
                                            window.location = objectUrl
                                        } else {
                                            a.href = objectUrl;
                                            a.download = fileName;
                                            document.body.appendChild(a);
                                            a.click();
                                            a.remove();
                                        }
                                    } else {
                                        window.location = objectUrl;
                                    }
                                    $('#busy-indicator-container').hide();
                                }
                            }
                        };
                        xhr.send(JSON.stringify({"fileUploadIds": ids, "zipFlag": zipFlag}));
                    }
                });

        };

        $scope.exportBacklogDetail = function () {
            $('#busy-indicator-container').show();
            var xhr = new XMLHttpRequest();
            xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/detail/export", true);
            xhr.responseType = 'arraybuffer';
            xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.onload = function () {
                // var headers = xhr.getAllResponseHeaders();
                // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                var fileName = '事项详情.xlsx';
                if (this.status === 200) {
                    var blob = new Blob([this.response], {type: "arraybuffer"});
                    if (typeof window.navigator.msSaveBlob !== 'undefined') {
                        window.navigator.msSaveBlob(blob, fileName)
                    } else {
                        var URL = window.URL || window.webkitURL;
                        var objectUrl = URL.createObjectURL(blob);
                        if (fileName) {
                            var a = document.createElement('a');
                            if (typeof a.download === 'undefined') {
                                window.location = objectUrl
                            } else {
                                a.href = objectUrl;
                                a.download = fileName;
                                document.body.appendChild(a);
                                a.click();
                                a.remove();
                            }
                        } else {
                            window.location = objectUrl;
                        }
                        $('#busy-indicator-container').hide();
                    }
                }
            };
            xhr.send(JSON.stringify($scope.searchParam));
        };


        $scope.detailView = function (data) {
            $scope.detailEdit = null;
            $scope.detailEdit = data;
            if($scope.detailEdit.amount&&(typeof $scope.detailEdit.amount == "number" )){
                $scope.detailEdit.amount = Number($scope.detailEdit.amount).formatAmount(2);
            }
            $scope.detailEdit.unEditFileUploads = [];
            $scope.detailEdit.comments = [];
            var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
            $scope.detailEdit.backlogTypeStr = backlogType.name;
            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
            $scope.detailEdit.backlogTypeParentStr = parentBacklogType.name;
            $http.post('/backlogManage/backlog/log', {"detailId": data.id}, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        angular.forEach(res, function (item) {
                            if (item.comment) {

                                $scope.detailEdit.comments.push(item.dealRoleStr + "[" + item.dealUserName + "]：" + item.comment);
                            }

                            if (item.fileUploads) {
                                angular.forEach(item.fileUploads, function (item1) {
                                    $scope.detailEdit.unEditFileUploads.push(item1);
                                });
                            }
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
            $($scope.detailViewDiv).modal('show');
        };


        //关闭配置框
        $scope.cancelModal = function () {
            $('#configForm')[0].reset();
        };


        $scope.selectedBacklog = [];
        $scope.backlogTypeLevelOneArray = [];
        $scope.backlogTypeLevelTwoArray = [];
        $scope.backlogTypeData = null;
        $scope.getBacklogType = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/calendar/type')
                .then(function (res) {
                    if (res) {
                        $scope.backlogTypeData = res;
                        $scope.backlogTypeLevelOneArray = $filter('filter')($scope.backlogTypeData, {id: "6"});
                        $scope.backlogTypeLevelTwoArray = $filter('filter')($scope.backlogTypeData, {parentId: "6"});
                    } else {
                        SweetAlert.error($translate.instant('GetRevenueTypeError'));
                    }
                })
        };
        $scope.businessListData = [];
        $scope.orgListData = null;
        $scope.getOrglist = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/org/getMyOrgList')
                .then(function (res) {
                    if (res && 0 === res.code) {
                        $scope.orgListData = res.data;
                        var dataMap = [];
                        angular.forEach(res.data, function (item) {
                            dataMap[item.businessUnitId] = item.businessUnitName;
                        });
                        for (var key in dataMap) {
                            if (key != 'null' && key.length > 18) {
                                $scope.businessListData.push({"ID": key, "name": dataMap[key]});
                            }

                        }
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        //刷新页面
        $scope.refreshGrid = function () {
            $scope.searchParam.backlogTypeParentId = 6;
            $scope.searchParam.pageInfo = {
                pageIndex: $scope.pagingOptions.pageIndex,
                totalItems: $scope.pagingOptions.totalItems,
                pageSize: $scope.pagingOptions.pageSize
            };
            $http.post('/backlogManage/backlog/detail', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.pageConfDataSource = res.list;
                        $scope.pagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        //搜索框的处理
        $scope.searchboxService = {

            showOrHideSearchBox: function () {
                $scope.hasShowMoreSearchBox = !$scope.hasShowMoreSearchBox;
            },
            search: function () {
                $scope.pagingOptions.pageIndex = 1;
                $scope.refreshGrid();
            },
            resetBox: function () {
                $scope.searchParam = {};
                $scope.searchParam.backlogStartDate = null;
                $scope.searchParam.backlogEndDate = null;
            }
        };
        $scope.getCurrentMonthFirst = function () {
            var date = new Date();
            date.setDate(1);
            return date;
        };
        $scope.getCurrentMonthLast = function () {
            var date = new Date();
            var currentMonth = date.getMonth();
            var nextMonth = currentMonth >= 11 ? 0 : ++currentMonth;
            var nextYear = currentMonth >= 11 ? date.getFullYear() + 1 : date.getFullYear();
            var nextMonthFirstDay = new Date(nextYear, nextMonth, 1);
            var oneDay = 1000 * 60 * 60 * 24;
            var lastTime = new Date(nextMonthFirstDay.getTime() - oneDay);
            return lastTime;
        };
        (function initialize() {
            $scope.detailViewDiv = "#detailViewDiv";
            $scope.formParam = {};
            //日期
            $scope.dateFormat = $translate.instant('dateFormat4YearMonthDay');

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 20  //每页多少条数据
            };

            $scope.hasShowMoreSearchBox = true;
            $scope.detailEditReadOnly = false;
            $scope.detailEdit = {"unEditFileUploads": [], "editFileUploads": [], "comments": []};
            $scope.searchParam = {"backlogStartDate":$scope.getCurrentMonthFirst(),"backlogEndDate":$scope.getCurrentMonthLast()};
            $scope.searchOptions = {
                statusOptions: {
                    showClearButton: true,
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.backlogStatus'
                    },
                    dataSource: [
                        {'key': '未开始', 'val': 'UN_BEGIN'},
                        {'key': '进行中', 'val': 'BEGIN'},
                        {'key': '待审核', 'val': 'UN_AUDIT'},
                        {'key': '退回修正', 'val': 'REJECT'},
                        {'key': '已完成', 'val': 'COMPLETED'},
                        {'key': '已失效', 'val': 'VOIDED'}
                    ]
                },
                generalTypeOptions: {
                    showClearButton: true,
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.backlogGenerateType'
                    },
                    dataSource: [
                        {'key': '单次', 'val': 1},
                        {'key': '周期', 'val': 2}
                    ]
                },
                fileOptions: {
                    showClearButton: true,
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.fileFlag'
                    },
                    dataSource: [
                        {'key': '是', 'val': true},
                        {'key': '否', 'val': false}
                    ]
                },
                outTimeOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.outTimeFlag',
                        selectedItem: 'searchParam.outTime',
                    },
                    showClearButton: true,
                    items: [
                        {'key': 'green', 'val': 1},
                        {'key': 'yellow', 'val': 2},
                        {'key': 'red', 'val': 3}]
                },
                orgOptions: {

                    displayExpr: 'key',
                    valueExpr: 'val',
                    deferRendering: false,
                    bindingOptions: {
                        value: 'searchParam.orgId',
                        dataSource: 'orgList'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                businessOptions: {
                    displayExpr: 'name',
                    valueExpr: 'ID',
                    // width: '95%',
                    bindingOptions: {
                        value: 'searchParam.businessId',
                        dataSource: 'businessListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                typeLevelOneOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    value: 'searchParam.backlogTypeParentId',
                    bindingOptions: {
                        dataSource: 'backlogTypeLevelOneArray'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true

                },
                typeLevelTwoOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'searchParam.backlogTypeId',
                        dataSource: 'backlogTypeLevelTwoArray'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                startDateBoxStart: {
                    type: "date",
                    showClearButton: true,
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'searchParam.backlogStartDate',
                        max: 'searchParam.backlogEndDate'
                    },
                },
                endDateBoxStart: {
                    showClearButton: true,
                    type: "date",
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'searchParam.backlogEndDate',
                        min: 'searchParam.backlogStartDate'
                    }
                },
            };

            $scope.unEditFileListOptions = {
                itemTemplate: function (item, index, container) {
                    var div = $('<div></div>');
                    $('<span style="float: left">' + item.fileName + '</span>').appendTo(div);
                    $('<i style="float: right;color: #dc6900"class="fa fa-download"></i>').on('click', function () {
                        $scope.downFile(item.uid, item.fileName, false);
                    }).appendTo(div);
                    div.appendTo(container);
                },
                noDataText: null,
                editEnabled: true,
                allowItemDeleting: false,
                bindingOptions: {
                    dataSource: 'detailEdit.unEditFileUploads'
                }
            };
            $scope.commentListOptions = {
                noDataText: null,
                editEnabled: true,
                allowItemDeleting: false,
                bindingOptions: {
                    dataSource: 'detailEdit.comments'
                }
            };

            function init() {
                $scope.refreshGrid();
                $scope.getBacklogType();

                setTimeout(function () {
                    $scope.getOrglist();

                }, 1000);
            }

            init()

        })();
    }
]);