﻿vatModule.controller('backlogConfigController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService', 'apiInterceptor', 'jqFetch', '$filter', 'Upload',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService, apiInterceptor, jqFetch, $filter, Upload) {
        'use strict';
        $log.debug('backlogConfigController.ctor()...');

        // var dataGrid = $("#detailGridContainer").dxDataGrid("instance");
        // var selectedKeys = dataGrid.getSelectedRowKeys();
        // var selectedData = dataGrid.getSelectedRowsData();

        //表格配置
        $scope.configCommonGridOptions = $.extend(true, {}, dxDataGridService.BASIC_GRID_OPTIONS, {
            showBorders: true,
            sorting: {//排序
                ascendingText: "Sort Ascending",
                clearText: "Clear Sorting",
                descendingText: "Sort Descending",
                mode: "single"
            },
            showRowLines: true,
            columnAutoWidth: true,
            allowColumnReordering: true,
            columns: [
                {
                    dataField: '',
                    caption: $translate.instant('BCCLColumnType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
                        return parentBacklogType.name;
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCCLColumnSubType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        return backlogType.name;
                    }
                },
                {
                    dataField: 'name',
                    caption: $translate.instant('BCCLColumnName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCCLColumnAutoFlag'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return data.autoFlag ? '是' : '否';
                    }
                },
                {
                    dataField: 'generateTypeStr',
                    caption: $translate.instant('BCCLColumnGenerateType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'statusStr',
                    caption: $translate.instant('BCCLColumnStatus'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCCLColumnHandle'),
                    fixed: true,
                    width: '80px',
                    alignment: 'center',
                    cellTemplate: function (container, options) {
                        try {
                            $('<a>' + $translate.instant('BCCLColumnHandleView') + '|</a>&nbsp;&nbsp;')
                                .on('click', function () {
                                    $scope.updateConfigCommon(options.data, true);
                                }).appendTo(container);
                            if (options.data.status == 'UN_COMPLETED') {
                                $('<a >' + $translate.instant('BCCLColumnHandleEdit') + '|</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.updateConfigCommon(options.data, false);
                                    }).appendTo(container);
                            }
                            if (options.data.status == 'COMPLETED') {
                                $('<a >' + $translate.instant('BCCLColumnHandleAllot') + '|</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.openConfigList(options.data);
                                    }).appendTo(container);
                            }
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
            ],
            bindingOptions: {
                dataSource: 'pageConfDataSource'
            },
            masterDetail: {
                enabled: true,
                template: "detail"
            },
            loadPanel: {
                enabled: false
            },
            grouping: {
                autoExpandAll: false
            },
            noDataText: $translate.instant('NoDataText'),
        });

        $scope.getSubDetailGrid = function (key) {
            return {
                //todo 绑定数据
                dataSource: new DevExpress.data.DataSource({
                    load: function (loadOptions) {
                        $scope.selectedConfigCommon = key.data;
                        var queryParam = {
                            "commonId": key.data.id,
                            "orgId": $scope.searchParam.orgId,
                            "status": $scope.searchParam.configStatus,
                            "dealUser": $scope.searchParam.dealUser,
                            "auditUser": $scope.searchParam.auditUser,
                            "createUser": $scope.searchParam.createUser,
                            "code": $scope.searchParam.code,
                            "groupCode": $scope.searchParam.groupCode,
                            "businessLineId": $scope.searchParam.businessLineId
                        };
                        var deferred = $q.defer();
                        $http.post('/backlogManage/backlog/config', queryParam, apiConfig.createVat())
                            .success(function (res) {
                                if (res && res.list) {
                                    deferred.resolve({data: res.list, totalCount: res.list.length});
                                } else {
                                    SweetAlert.error($translate.instant('SystemError'));
                                }
                            }).error(function () {
                            deferred.reject("Data Loading Error");
                        });
                        return deferred.promise;
                    }
                }),
                columnAutoWidth: true,
                showBorders: true,
                paging: {
                    pageSize: constant.page.pageSize
                },
                pager: {
                    showPageSizeSelector: true,
                    allowedPageSizes: constant.page.pageSizeArrary,
                    showInfo: true
                },
                columns: [
                    {
                        dataField: 'code',
                        caption: $translate.instant('BCLColumnCode'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'businessLineName',
                        caption: $translate.instant('BCLColumnBusinessLine'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'orgName',
                        caption: $translate.instant('BCLColumnOrgName'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BCLColumnStartDate'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            return $filter('date')(data.validityStartDate, 'yyyy-MM-dd HH:mm:ss');
                        }
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BCLColumnEndDate'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            return $filter('date')(data.validityEndDate, 'yyyy-MM-dd HH:mm:ss');
                        }
                    },
                    {
                        dataField: 'timeZone',
                        caption: $translate.instant('BCLColumnTimeZone'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'statusStr',
                        caption: $translate.instant('BCLColumnStatus'),
                        fixed: true
                    },
                    {
                        dataField: 'dealUserName',
                        caption: $translate.instant('BCLColumnDealUser'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'auditUserName',
                        caption: $translate.instant('BCLColumnAuditUser'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'createUserName',
                        caption: $translate.instant('BCLColumnCreate'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'groupCode',
                        caption: $translate.instant('BCLColumnGroupId'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BCLColumnHandle'),
                        fixed: true,
                        width: '80px',
                        alignment: 'center',
                        cellTemplate: function (container, options) {
                            try {
                                $('<a>' + $translate.instant('BCLColumnHandleView') + '</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.updateAllotConfig(options.data, true);
                                    }).appendTo(container);
                            } catch (e) {
                                $log.error(e);
                            }
                        }
                    },
                ]
            };
        }
        $scope.exportBacklogConfig = function () {
            $('#busy-indicator-container').show();
            var xhr = new XMLHttpRequest();
            xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/config/export", true);
            xhr.responseType = 'arraybuffer';
            xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.onload = function () {
                // var headers = xhr.getAllResponseHeaders();
                // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                var fileName = '事项配置分配详情.xlsx';
                if (this.status === 200) {
                    var blob = new Blob([this.response], {type: "arraybuffer"});
                    if (typeof window.navigator.msSaveBlob !== 'undefined') {
                        window.navigator.msSaveBlob(blob, fileName)
                    } else {
                        var URL = window.URL || window.webkitURL;
                        var objectUrl = URL.createObjectURL(blob);
                        if (fileName) {
                            var a = document.createElement('a');
                            if (typeof a.download === 'undefined') {
                                window.location = objectUrl
                            } else {
                                a.href = objectUrl;
                                a.download = fileName;
                                document.body.appendChild(a);
                                a.click();
                                a.remove();
                            }
                        } else {
                            window.location = objectUrl;
                        }
                        $('#busy-indicator-container').hide();
                    }
                }
            };
            xhr.send(JSON.stringify($scope.searchParam));
        };
        $scope.assembleConfigAllotEdit = function (configCommon) {
            $scope.configAllotEdit.commonId = configCommon.id;
            $scope.configAllotEdit.generateType = configCommon.generateType;
            $scope.configAllotEdit.generateTypeStr = configCommon.generateTypeStr;
            $scope.configAllotEdit.autoFlagStr = configCommon.autoFlag ? '是' : '否';
            var backlogType = $filter('filter')($scope.backlogTypeData, {id: configCommon.backlogTypeId})[0];
            $scope.configAllotEdit.backlogTypeStr = backlogType.name;
            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
            $scope.configAllotEdit.backlogTypeParentStr = parentBacklogType.name;
            $scope.configAllotEdit.name = configCommon.name;
            $scope.configAllotEdit.describe = configCommon.describe;
        }
        //事项分配列表dx配置项
        $scope.configGridOptions = $.extend(true, {}, dxDataGridService.BASIC_GRID_OPTIONS, {
            selection: {
                mode: "multiple",
                showCheckBoxesMode: "always",
                allowSelectAll: true,
                selectAllMode: "page"
            },
            showBorders: true,
            sorting: {//排序
                ascendingText: "Sort Ascending",
                clearText: "Clear Sorting",
                descendingText: "Sort Descending",
                mode: "single"
            },
            showRowLines: true,
            columnAutoWidth: true,
            allowColumnReordering: true,
            onSelectionChanged: function (selectedItems) {
                $scope.selectedCofing = selectedItems.selectedRowsData;
            },
            columns: [
                {
                    dataField: 'code',
                    caption: $translate.instant('BCLColumnCode'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
                        return parentBacklogType.name;
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnSubType'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                        return backlogType.name;
                    }
                },
                {
                    dataField: 'name',
                    caption: $translate.instant('BCLColumnName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'businessLineName',
                    caption: $translate.instant('BCLColumnBusinessLine'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'orgName',
                    caption: $translate.instant('BCLColumnOrgName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnStartDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.validityStartDate, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnEndDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.validityEndDate, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: 'timeZone',
                    caption: $translate.instant('BCLColumnTimeZone'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'statusStr',
                    caption: $translate.instant('BCLColumnStatus'),
                    fixed: true
                },
                {
                    dataField: 'dealUserName',
                    caption: $translate.instant('BCLColumnDealUser'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'auditUserName',
                    caption: $translate.instant('BCLColumnAuditUser'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'createUserName',
                    caption: $translate.instant('BCLColumnCreate'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'generateTypeStr',
                    caption: $translate.instant('BCLColumnGenerateType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'groupCode',
                    caption: $translate.instant('BCLColumnGroupId'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnFileFlag'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        if (data.fileUploadIds && data.fileUploadIds != ',') {
                            return '是';
                        } else {
                            return '否';
                        }
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BCLColumnHandle'),
                    fixed: true,
                    width: '80px',
                    alignment: 'center',
                    cellTemplate: function (container, options) {
                        try {

                            if ("COMPLETED" == options.data.status || "VOIDED" == options.data.status) {
                                $('<a>' + $translate.instant('BCLColumnHandleView') + '|</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.updateAllotConfig(options.data, true);
                                    }).appendTo(container);
                            }
                            if ("UN_COMPLETED" == options.data.status) {
                                $('<a >' + $translate.instant('BCLColumnHandleEdit') + '|</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.updateAllotConfig(options.data, false);
                                    }).appendTo(container);
                            }

                            if (options.data.fileUploadIds && options.data.fileUploadIds.split(',').length > 1) {
                                $('<a>' + $translate.instant('BCLColumnHandleDownload') + '</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.downFile(options.data.fileUploadIds, options.data.name + '.zip', true)
                                    }).appendTo(container);
                            }
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
            ],
            bindingOptions: {
                dataSource: 'configListDataSource'
            }
        });
        //弹出事项分配列表
        $scope.openConfigList = function (configCommon) {
            $scope.selectedConfigCommon = configCommon;
            $($scope.configListDiv).modal('show');
            //$scope.getConfigCommonList();
            $scope.refreshConfigListGrid();
        };
        $scope.avoidConfig = function () {
            if ($scope.selectedBacklog.length < 1) {
                SweetAlert.error($translate.instant('UnSelectedRow'));
                return;
            }
            DevExpress.ui.dialog.confirm($translate.instant('DisableMsg'))
                .done(function (value) {
                    if (value) {
                        var configIds = [];
                        angular.forEach($scope.selectedCofing, function (item) {
                            if (item) {
                                configIds.push(item.id);
                            }
                        });
                        $http.post('/backlogManage/backlog/disable', {"configIds": configIds}, apiConfig.createVat())
                            .success(function (res) {
                                if (res && 0 === res.code) {
                                    SweetAlert.success($translate.instant('BCLAllotDisableSuccess'));
                                    $scope.refreshConfigListGrid();
                                } else {
                                    SweetAlert.error($translate.instant('SystemError'));
                                }
                            });
                    }
                });

        };
        //弹出分配事项页面
        $scope.allotConfig = function () {
            $scope.configEditReadOnly = false;
            $scope.configAllotEdit = {};
            $scope.configAllotEdit.createUser = vatSessionService.userPermission.ID;
            $scope.configAllotEdit.createUserName = vatSessionService.userPermission.userName;
            $scope.assembleConfigAllotEdit($scope.selectedConfigCommon);
            $scope.configAllotEdit.warnStartTimeDate = null;
            $scope.configAllotEdit.warnEndTimeDate = null;
            $scope.configAllotEdit.validityStartDate = null;
            $scope.configAllotEdit.validityEndDate = null;
            $scope.configAllotEdit.orgId = null;
            $scope.configAllotEdit.businessLineId = null;

            $($scope.configAllotDiv).modal('show');
        };
        //弹出编辑分配事项配置
        $scope.updateAllotConfig = function (config, readOnly) {
            $scope.configEditReadOnly = readOnly;
            $scope.configAllotEdit = {};
            $scope.configAllotEdit = config;
            $scope.assembleConfigAllotEdit($scope.selectedConfigCommon);
            $scope.configAllotEdit.choose = !config.orgId ? '2' : '1';
            if (!($scope.configAllotEdit.warnMonth instanceof Array)) {
                $scope.configAllotEdit.warnMonth = JSON.parse(config.warnMonth);
            }
            if ($scope.configAllotEdit.auditUser && !($scope.configAllotEdit.auditUser instanceof Array) && $scope.configAllotEdit.auditUser != '') {
                var auditArry = $scope.configAllotEdit.auditUser.split(',');
                $scope.configAllotEdit.auditUser = auditArry;
            }
            if ($scope.configAllotEdit.warnStartTime) {
                var startTimeArry = $scope.configAllotEdit.warnStartTime.split(':');
                $scope.configAllotEdit.warnStartTimeDate = new Date();
                $scope.configAllotEdit.warnStartTimeDate.setHours(startTimeArry[0], startTimeArry[1]);
            }
            if ($scope.configAllotEdit.warnEndTime) {
                var endTimeArry = $scope.configAllotEdit.warnEndTime.split(':');
                $scope.configAllotEdit.warnEndTimeDate = new Date();
                $scope.configAllotEdit.warnEndTimeDate.setHours(endTimeArry[0], endTimeArry[1]);
            }
            if ($scope.configAllotEdit.amount && (typeof $scope.configAllotEdit.amount == "number")) {
                $scope.configAllotEdit.amount = Number($scope.configAllotEdit.amount).formatAmount(2);
            }
            $scope.getUserListData($scope.configAllotEdit.orgId, $scope.configAllotEdit.businessLineId);
            $($scope.configAllotDiv).modal('show');
        };
        //弹出新增事项配置
        $scope.addConfigCommon = function () {
            $scope.configCommonReadOnly = false;
            $scope.backlogTypeLevelOneEditArray = $filter('filter')($scope.backlogTypeData, {level: "1"});
            $scope.backlogTypeLevelTwoEditArray = $filter('filter')($scope.backlogTypeData, {level: "2"});
            $scope.configCommonEdit = {};
            $($scope.configCommonAddDiv).modal('show');
        };
        //弹出编辑事项配置
        $scope.updateConfigCommon = function (configCommon, readOnly) {
            $scope.backlogTypeLevelOneEditArray = $filter('filter')($scope.backlogTypeData, {level: "1"});
            $scope.backlogTypeLevelTwoEditArray = $filter('filter')($scope.backlogTypeData, {level: "2"});
            var backlogType = $filter('filter')($scope.backlogTypeData, {id: configCommon.backlogTypeId})[0];
            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
            $scope.configCommonReadOnly = readOnly;
            $scope.configCommonEdit = {};
            $scope.configCommonEdit.describe = configCommon.describe;
            $scope.configCommonEdit.generateType = configCommon.generateType;
            $scope.configCommonEdit.id = configCommon.id;
            $scope.configCommonEdit.name = configCommon.name;
            $scope.configCommonEdit.status = configCommon.status;
            $scope.configCommonEdit.backlogTypeId = configCommon.backlogTypeId + "";
            $scope.configCommonEdit.backlogTypeParentId = parentBacklogType.id;
            $scope.configCommonEdit.autoFlag = configCommon.autoFlag;
            $($scope.configCommonAddDiv).modal('show');
        };
        //编辑事项配置
        $scope.conformEditConfigCommon = function (status) {
            $scope.configCommonEdit.status = status;

            $http.post('/backlogManage/backlog/config/common/edit', $scope.configCommonEdit, apiConfig.createVat())
                .success(function (res) {
                    if (res && 0 === res.code) {
                        if ('UN_COMPLETED' == status) {
                            SweetAlert.success($translate.instant('BCCLEditSuccess'));
                        } else if ('COMPLETED' == status) {
                            SweetAlert.success($translate.instant('BCCLEditComplete'));
                        }

                        $scope.refreshGrid();
                        $($scope.configCommonAddDiv).modal('hide');
                    } else if (res && -1 === res.code) {
                        var msg = '';
                        angular.forEach(res.message.split(','), function (errorMsg) {
                            msg += $translate.instant(errorMsg) + "\r\n";
                        });
                        SweetAlert.error(msg);
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
        };
        //分配事项
        $scope.conformEditConfig = function (status) {
            if (status) {
                $scope.configAllotEdit.status = status;
            }
            // $scope.configAllotEdit.validityStartDate = new Date($scope.configAllotEdit.validityStartDate);
            // $scope.configAllotEdit.validityEndDate = new Date($scope.configAllotEdit.validityEndDate);
            if ($scope.configAllotEdit.warnStartTimeDate) {
                $scope.configAllotEdit.warnStartTime = $scope.configAllotEdit.warnStartTimeDate.getHours() + ':' + $scope.configAllotEdit.warnStartTimeDate.getMinutes();
            }
            if ($scope.configAllotEdit.warnEndTimeDate) {
                $scope.configAllotEdit.warnEndTime = $scope.configAllotEdit.warnEndTimeDate.getHours() + ':' + $scope.configAllotEdit.warnEndTimeDate.getMinutes();
            }
            $scope.configAllotEdit.warnMonth = JSON.stringify($scope.configAllotEdit.warnMonth);
            var auditUserTmp = null;
            if ($scope.configAllotEdit.auditUser) {
                auditUserTmp = $scope.configAllotEdit.auditUser;
                var auditUser = '';
                angular.forEach($scope.configAllotEdit.auditUser, function (item) {
                    auditUser += (',' + item);
                });
                $scope.configAllotEdit.auditUser = auditUser;
            }
            var fileUids = '';
            angular.forEach($scope.configAllotEdit.fileUploads, function (item) {
                fileUids += (',' + item.uid);
                ;
            });
            $scope.configAllotEdit.fileUploadIds = fileUids;
            if ($scope.configAllotEdit.amount) {
                $scope.configAllotEdit.amount = $scope.configAllotEdit.amount.replace(/,/g, '');
            }

            $http.post('/backlogManage/backlog/config/edit', $scope.configAllotEdit, apiConfig.createVat())
                .success(function (res) {
                    if (res && 0 === res.code) {
                        if (status) {
                            SweetAlert.success($translate.instant('BCLAllotComplete'));
                        } else {
                            SweetAlert.success($translate.instant('BCLAllotSuccess'));
                        }
                        $scope.refreshConfigListGrid();
                        $($scope.configAllotDiv).modal('hide');
                    } else if (res && -1 === res.code) {
                        var msg = '';
                        angular.forEach(res.message.split(','), function (errorMsg) {
                            msg += $translate.instant(errorMsg) + "\r\n";
                        });

                        if ($scope.configAllotEdit.warnMonth) {
                            $scope.configAllotEdit.warnMonth = JSON.parse($scope.configAllotEdit.warnMonth);
                        }
                        $scope.configAllotEdit.auditUser = auditUserTmp;
                        SweetAlert.error(msg);
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
        };
        //上传附件
        $('#configUpload').on('change', function () {
            $('#busy-indicator-container').show();
            Upload.upload({
                url: apiInterceptor.webApiHostUrl + '/backlogManage/backlog/upload',
                data: {},
                file: $('#configUpload')[0].files[0],
                headers: {
                    'Access-Control-Allow-Origin': '*',
                    Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                    withCredentials: true
                },
                withCredentials: true
            }).then(function (resp) {
                if (resp && resp.data && resp.data.result) {
                    $scope.configAllotEdit.fileUploads = $scope.configAllotEdit.fileUploads ? $scope.configAllotEdit.fileUploads : [];
                    $scope.configAllotEdit.fileUploads.push(resp.data.data);
                }
                $('#busy-indicator-container').hide();
            }, function (resp) {
                SweetAlert.error($translate.instant('SystemError'));
                $('#busy-indicator-container').hide();
            }, function (evt) {

            });
        });

        //下载选择附件
        $scope.downSelectFile = function () {
            var fileUoloadIds = ',';
            angular.forEach($scope.selectedCofing, function (item) {
                if (item && item.fileUploadIds) {
                    fileUoloadIds += ',' + item.fileUploadIds;
                }
            });
            $scope.downFile(fileUoloadIds, 'files.zip', true);
        }

        //下载附件
        $scope.downFile = function (fileUoloadId, fileName, zipFlag) {
            var fileUoloadIds = fileUoloadId.split(',');
            var ids = [];
            angular.forEach(fileUoloadIds, function (item) {
                if (item && item != '') {
                    ids.push(item);
                }
            });
            if (ids.length == 0) {
                SweetAlert.error($translate.instant('UnFileDownload'));
                return;
            }
            DevExpress.ui.dialog.confirm($translate.instant('DownloadMsg'))
                .done(function (value) {
                    if (value) {
                        $('#busy-indicator-container').show();
                        var xhr = new XMLHttpRequest();
                        xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/detail/download", true);
                        xhr.responseType = 'arraybuffer';
                        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                        xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
                        xhr.onload = function () {
                            // var headers = xhr.getAllResponseHeaders();
                            // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                            if (this.status === 200) {
                                var blob = new Blob([this.response], {type: "arraybuffer"});
                                if (typeof window.navigator.msSaveBlob !== 'undefined') {
                                    window.navigator.msSaveBlob(blob, fileName)
                                } else {
                                    var URL = window.URL || window.webkitURL;
                                    var objectUrl = URL.createObjectURL(blob);
                                    if (fileName) {
                                        var a = document.createElement('a');
                                        if (typeof a.download === 'undefined') {
                                            window.location = objectUrl
                                        } else {
                                            a.href = objectUrl;
                                            a.download = fileName;
                                            document.body.appendChild(a);
                                            a.click();
                                            a.remove();
                                        }
                                    } else {
                                        window.location = objectUrl;
                                    }
                                    $('#busy-indicator-container').hide();
                                }
                            }
                        };
                        xhr.send(JSON.stringify({"fileUploadIds": ids, "zipFlag": zipFlag}));
                    }
                });

        };

        //关闭事项配置框
        $scope.cancelModal = function () {

        };

        //获取事项类型选项列表
        $scope.selectedCofingCommon = [];
        $scope.selectedCofing = [];
        $scope.backlogTypeData = null;
        $scope.getBacklogType = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/calendar/type')
                .then(function (res) {
                    if (res) {
                        $scope.backlogTypeData = res;
                        $scope.backlogTypeLevelOneArray = $filter('filter')($scope.backlogTypeData, {level: "1"});
                        $scope.backlogTypeLevelTwoArray = $filter('filter')($scope.backlogTypeData, {level: "2"});
                        $scope.backlogTypeLevelOneEditArray = $filter('filter')($scope.backlogTypeData, {level: "1"});
                        $scope.backlogTypeLevelTwoEditArray = $filter('filter')($scope.backlogTypeData, {level: "2"});
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        //刷新事项配置页面
        $scope.refreshGrid = function () {
            $scope.searchParam.pageInfo = {
                pageIndex: $scope.pagingOptions.pageIndex,
                totalItems: $scope.pagingOptions.totalItems,
                pageSize: $scope.pagingOptions.pageSize
            };
            $http.post('/backlogManage/backlog/config/common', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.pageConfDataSource = res.list;
                        $scope.pagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        //刷新事项配置分配页面
        $scope.refreshConfigListGrid = function () {
            $scope.configSearchParam.pageInfo = {
                pageIndex: $scope.configListPagingOptions.pageIndex,
                totalItems: $scope.configListPagingOptions.totalItems,
                pageSize: $scope.configListPagingOptions.pageSize,
            };
            $scope.configSearchParam.commonId = $scope.selectedConfigCommon.id;
            $http.post('/backlogManage/backlog/config', $scope.configSearchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.configListDataSource = res.list;
                        $scope.configListPagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        //搜索框的处理
        $scope.searchboxService = {

            showOrHideSearchBox: function () {
                $scope.hasShowMoreSearchBox = !$scope.hasShowMoreSearchBox;
            },
            search: function () {
                $scope.pagingOptions.pageIndex = 1;
                $scope.refreshGrid();
            },
            resetBox: function () {
                $scope.searchParam = {};
                $scope.backlogTypeLevelOneArray = $filter('filter')($scope.backlogTypeData, {level: "1"});
                $scope.backlogTypeLevelTwoArray = $filter('filter')($scope.backlogTypeData, {level: "2"});

            }
        };
        //搜索框的处理
        $scope.configearchboxService = {

            search: function () {
                $scope.configListPagingOptions.pageIndex = 1;
                $scope.refreshConfigListGrid();
            },
            resetBox: function () {
                $scope.configSearchParam = {};
            }
        };

        //分配事项管理模块功能

        //获取公司选项列表
        $scope.businessListData = [];
        $scope.orgListData = null;
        $scope.getOrglist = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/org/getMyOrgList')
                .then(function (res) {
                    if (res && 0 === res.code) {
                        $scope.orgListData = res.data;
                        var dataMap = [];
                        angular.forEach(res.data, function (item) {
                            dataMap[item.businessUnitId] = item.businessUnitName;
                        });
                        for (var key in dataMap) {
                            if (key != 'null' && key.length > 18) {
                                $scope.businessListData.push({"ID": key, "name": dataMap[key]});
                            }

                        }
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        //获取事项配置选项列表
        $scope.configCommonListData = null;
        $scope.getConfigCommonList = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/config/common/selected')
                .then(function (res) {
                    if (res) {
                        $scope.configCommonListData = res;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        $scope.dayListData = [];
        $scope.getDayList = function () {
            for (var i = 1; i <= 31; i++) {
                $scope.dayListData.push({'key': i + '日', 'val': i < 10 ? ('0' + i) : ('' + i)});
            }
        };
        $scope.timeZoneData = [];
        $scope.getTimeZone = function () {
            $.get('app-resources/timezones/timezones.json', function (timezone) {
                angular.forEach(timezone, function (item) {
                    $scope.timeZoneData.push({
                        'key': item.timezone + '(' + item.zonename + ')', 'val': item.timezone
                    });
                });
            });
        };
        //获取事项配置选项列表
        $scope.userListData = null;
        $scope.getUserListData = function (orgId, businessId) {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/user/' + orgId + '/' + businessId)
                .then(function (res) {
                    if (res) {
                        $scope.userListData = res;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        (function initialize() {
            //事项分配编辑对象
            $scope.configAllotEdit = {fileUploads: []};
            $scope.orgAllotReadOnly = true;
            $scope.businessAllotReadOnly = true;
            //事项配置编辑对象
            $scope.configCommonEdit = {};
            //弹出模态框定义
            $scope.configCommonAddDiv = "#configCommonAddDiv";
            $scope.configListDiv = "#configListDiv";
            $scope.configAllotDiv = "#configAllotDiv";

            //事项配置列表分页设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 20  //每页多少条数据
            };
            //事项分配列表分页设置
            $scope.configListPagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 20  //每页多少条数据
            };
            //事项配置列表是否更多查询
            $scope.hasShowMoreSearchBox = true;
            //事项分配列表分页设置
            $scope.configListHasShowMoreSearchBox = true;
            //事项配置列表查询对象
            $scope.searchParam = {};
            //事项分配列表查询对象
            $scope.configSearchParam = {};
            //事项分配列表查询select框配置
            $scope.configSearchOptions = {
                businessOptions: {
                    displayExpr: 'name',
                    valueExpr: 'ID',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configSearchParam.businessLineId',
                        dataSource: 'businessListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                statusOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configSearchParam.status'
                    },
                    dataSource: [
                        {'key': '创建中', 'val': 'UN_COMPLETED'},
                        {'key': '创建完成', 'val': 'COMPLETED'},
                        {'key': '已失效', 'val': 'VOIDED'}
                    ]
                },
                fileFlagOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configSearchParam.fileFlag'
                    },
                    dataSource: [
                        {'key': '是', 'val': true},
                        {'key': '否', 'val': false}
                    ]
                },
                validityStartDateBoxStart: {
                    type: "date",
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'configSearchParam.startDate',
                        max: 'configSearchParam.endDate'
                    },
                },
                validityEndDateBoxStart: {
                    type: "date",
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'configSearchParam.endDate',
                        min: 'configSearchParam.startDate'
                    }
                },
                orgOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configSearchParam.orgId',
                        dataSource: 'orgListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                chooseAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configAllotEdit.choose',
                        readOnly: 'configEditReadOnly'
                    },
                    dataSource: [
                        {'key': '公司', 'val': '1'},
                        {'key': '业务线', 'val': '2'}
                    ],
                    onSelectionChanged: function (obj) {
                        if (obj && obj.selectedItem) {
                            if ($scope.configEditReadOnly) {
                                $scope.orgAllotReadOnly = $scope.configEditReadOnly;
                                $scope.businessAllotReadOnly = $scope.configEditReadOnly;
                            } else {
                                $scope.configAllotEdit.orgId = null;
                                $scope.configAllotEdit.businessLineId = null;
                                $scope.configAllotEdit.dealUser = null;
                                $scope.configAllotEdit.auditUser = null;
                                $scope.userListData = null;
                                $scope.orgAllotReadOnly = obj.selectedItem.val == '2' ? true : false;
                                $scope.businessAllotReadOnly = obj.selectedItem.val == '1' ? true : false;
                            }

                        }
                    }
                },
                monthAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configAllotEdit.warnMonth',
                        readOnly: 'configEditReadOnly'
                    },
                    dataSource: [
                        {'key': '1月', 'val': '01'},
                        {'key': '2月', 'val': '02'},
                        {'key': '3月', 'val': '03'},
                        {'key': '4月', 'val': '04'},
                        {'key': '5月', 'val': '05'},
                        {'key': '6月', 'val': '06'},
                        {'key': '7月', 'val': '07'},
                        {'key': '8月', 'val': '08'},
                        {'key': '9月', 'val': '09'},
                        {'key': '10月', 'val': '10'},
                        {'key': '11月', 'val': '11'},
                        {'key': '12月', 'val': '12'}
                    ]
                },
                startDayAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configAllotEdit.warnStartDay',
                        dataSource: 'dayListData',
                        readOnly: 'configEditReadOnly'
                    }
                },
                endDayAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configAllotEdit.warnEndDay',
                        dataSource: 'dayListData',
                        readOnly: 'configEditReadOnly'
                    },

                },
                orgAllotOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.orgId',
                        dataSource: 'orgListData',
                        readOnly: 'orgAllotReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true,
                    onSelectionChanged: function (obj) {
                        if (obj && obj.selectedItem) {
                            $scope.configAllotEdit.businessLineId = obj.selectedItem.businessUnitId;
                            $scope.getUserListData(obj.selectedItem.id, null);
                        }
                    }
                },
                dealUserAllotOptions: {
                    displayExpr: 'userName',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.dealUser',
                        dataSource: 'userListData',
                        readOnly: 'configEditReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                auditUserAllotOptions: {
                    displayExpr: 'userName',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.auditUser',
                        dataSource: 'userListData',
                        readOnly: 'configEditReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                createUserAllotOptions: {
                    displayExpr: 'userName',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.createUser',
                        dataSource: 'userListData',
                        readOnly: 'configEditReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                businessAllotOptions: {
                    displayExpr: 'name',
                    valueExpr: 'ID',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.businessLineId',
                        dataSource: 'businessListData',
                        readOnly: 'businessAllotReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true,
                    onSelectionChanged: function (obj) {
                        if (obj && obj.selectedItem) {
                            if ($scope.configAllotEdit.choose == '2') {
                                $scope.getUserListData(null, obj.selectedItem.ID);
                            }
                        }
                    }
                },
                validityStartDateAllotBoxStart: {
                    type: "datetime",
                    displayFormat: 'yyyy-MM-dd HH:mm',
                    bindingOptions: {
                        value: 'configAllotEdit.validityStartDate',
                        max: 'configAllotEdit.validityEndDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                validityEndDateAllotBoxStart: {
                    type: "datetime",
                    displayFormat: 'yyyy-MM-dd HH:mm',
                    bindingOptions: {
                        value: 'configAllotEdit.validityEndDate',
                        min: 'configAllotEdit.validityStartDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                startDateAllotBoxStart: {
                    type: "datetime",
                    displayFormat: 'yyyy-MM-dd HH:mm:ss',
                    bindingOptions: {
                        value: 'configAllotEdit.validityStartDate',
                        max: 'configAllotEdit.validityEndDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                endDateAllotBoxStart: {
                    type: "datetime",
                    displayFormat: 'yyyy-MM-dd HH:mm:ss',
                    bindingOptions: {
                        value: 'configAllotEdit.validityEndDate',
                        min: 'configAllotEdit.validityStartDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                startTimeAllotBoxStart: {
                    displayFormat: 'HH:mm',
                    type: "time",
                    bindingOptions: {
                        value: 'configAllotEdit.warnStartTimeDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                endTimeAllotBoxStart: {
                    displayFormat: 'HH:mm',
                    type: "time",
                    bindingOptions: {
                        value: 'configAllotEdit.warnEndTimeDate',
                        readOnly: 'configEditReadOnly'
                    }
                },
                timeZoneAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configAllotEdit.timeZone',
                        dataSource: 'timeZoneData',
                        readOnly: 'configEditReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                fileListAllotOptions: {
                    itemTemplate: function (item, index, container) {
                        var div = $('<div></div>');
                        $('<span style="float: left">' + item.fileName + '</span>').appendTo(div);
                        $('<i style="float: right;color: #dc6900"class="fa fa-download"></i>').on('click', function () {
                            $scope.downFile(item.uid, item.fileName, false);
                        }).appendTo(div);
                        div.appendTo(container);
                    },
                    noDataText: '',
                    editEnabled: true,
                    allowItemDeleting: true,
                    itemDeleteMode: 'static',
                    bindingOptions: {
                        dataSource: 'configAllotEdit.fileUploads',
                        readOnly: 'configEditReadOnly'
                    },
                    onItemDeleting: function (e) {
                        var d = $.Deferred();
                        DevExpress.ui.dialog.confirm($translate.instant('BDLEditDelMsg'))
                            .done(function (value) {
                                d.resolve(!value);
                            })
                            .fail(d.reject);
                        e.cancel = d.promise();
                    }
                }
            };

            //事项配置列表查询select框配置
            $scope.searchOptions = {
                businessOptions: {
                    displayExpr: 'name',
                    valueExpr: 'ID',
                    // width: '95%',
                    bindingOptions: {
                        value: 'searchParam.businessLineId',
                        dataSource: 'businessListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                orgOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'searchParam.orgId',
                        dataSource: 'orgListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                statusOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.status'
                    },
                    dataSource: [
                        {'key': '创建中', 'val': 'UN_COMPLETED'},
                        {'key': '创建完成', 'val': 'COMPLETED'},
                        {'key': '已失效', 'val': 'VOIDED'}
                    ]
                },
                configStatusOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.configStatus'
                    },
                    dataSource: [
                        {'key': '创建中', 'val': 'UN_COMPLETED'},
                        {'key': '创建完成', 'val': 'COMPLETED'},
                        {'key': '已失效', 'val': 'VOIDED'}
                    ]
                },
                generalTypeOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.generateType'
                    },
                    dataSource: [
                        {'key': '单次', 'val': 1},
                        {'key': '周期', 'val': 2}
                    ]
                },
                autoFlagOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.autoFLag'
                    },
                    dataSource: [
                        {'key': '是', 'val': true},
                        {'key': '否', 'val': false}
                    ]
                },
                typeLevelOneOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'queryFieldModel.backlogTypeParentId',
                        dataSource: 'backlogTypeLevelOneArray'
                    },
                    // height: '30px',
                    placeholder: '',
                    onSelectionChanged: function (obj) {
                        if (obj && obj.selectedItem) {
                            $scope.backlogTypeLevelTwoArray = $filter('filter')($scope.backlogTypeData, {parentId: obj.selectedItem.id});
                        }

                    }
                },
                typeLevelTwoOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'queryFieldModel.backlogTypeId',
                        dataSource: 'backlogTypeLevelTwoArray'
                    },
                    // height: '30px',
                    placeholder: '',
                },
                typeLevelOneEditOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configCommonEdit.backlogTypeParentId',
                        dataSource: 'backlogTypeLevelOneEditArray',
                        readOnly: 'configCommonReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    onSelectionChanged: function (obj) {
                        if (obj && obj.selectedItem) {
                            $scope.backlogTypeLevelTwoEditArray = $filter('filter')($scope.backlogTypeData, {parentId: obj.selectedItem.id});
                        }

                    }
                },
                typeLevelTwoEditOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    // width: '95%',
                    bindingOptions: {
                        value: 'configCommonEdit.backlogTypeId',
                        dataSource: 'backlogTypeLevelTwoEditArray',
                        readOnly: 'configCommonReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                },
                generalTypeEditOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configCommonEdit.generateType',
                        readOnly: 'configCommonReadOnly'
                    },
                    dataSource: [
                        {'key': '单次', 'val': 1},
                        {'key': '周期', 'val': 2}
                    ]
                },
                autoFlagEditOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'configCommonEdit.autoFlag',
                        readOnly: 'configCommonReadOnly'
                    },
                    dataSource: [
                        {'key': '是', 'val': true},
                        {'key': '否', 'val': false}
                    ]
                },
            };

            function init() {
                $scope.refreshGrid();
                $scope.getBacklogType();
                setTimeout(function () {
                    $scope.getDayList();
                    $scope.getOrglist();
                    $scope.getTimeZone();
                }, 1000);
            }

            init()

        })();
    }
]);