﻿vatModule.controller('backlogGroupDetailController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService', 'apiInterceptor', 'jqFetch', '$filter', 'Upload',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService, apiInterceptor, jqFetch, $filter, Upload) {
        'use strict';
        $log.debug('backlogGroupDetailController.ctor()...');

        // var dataGrid = $("#detailGridContainer").dxDataGrid("instance");
        // var selectedKeys = dataGrid.getSelectedRowKeys();
        // var selectedData = dataGrid.getSelectedRowsData();

        //表格配置
        $scope.groupGridOptions = $.extend(true, {}, dxDataGridService.BASIC_GRID_OPTIONS, {
            selection: {
                mode: "multiple",
                showCheckBoxesMode: "always",
                allowSelectAll: true,
                selectAllMode: "page"
            },
            showBorders: true,
            keyExpr: ["id", "name", "month"],
            sorting: {//排序
                ascendingText: "Sort Ascending",
                clearText: "Clear Sorting",
                descendingText: "Sort Descending",
                mode: "single"
            },
            showRowLines: true,
            columnAutoWidth: true,
            allowColumnReordering: true,
            onSelectionChanged: function (selectedItems) {
                $scope.selectedGroupData = selectedItems.selectedRowsData;
            },
            columns: [
                {
                    dataField: 'groupCode',
                    caption: $translate.instant('BGLColumnGroupCode'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BGLColumnName'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        if (data.month) {
                            return data.name + '(' + data.month + ')';
                        } else {
                            return data.name;
                        }
                    }
                },
                {
                    dataField: 'orgName',
                    caption: $translate.instant('BDLColumnOrgName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'businessLine',
                    caption: $translate.instant('BDLColumnBusinessLine'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'timeZone',
                    caption: $translate.instant('BGLColumnTimeZone'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BGLColumnStartDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.chinaWarnStartTime, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BGLColumnEndDate'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        return $filter('date')(data.chinaWarnEndTime, 'yyyy-MM-dd HH:mm:ss');
                    }
                },
                {
                    dataField: 'generateTypeStr',
                    caption: $translate.instant('BGLColumnGenerateType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'createUserName',
                    caption: $translate.instant('BGLColumnCreator'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: '',
                    caption: $translate.instant('BGLColumnStatus'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    calculateCellValue: function (data) {
                        var status = data.status;
                        if (status) {
                            var statusArray = status.split(',');
                            var unBeginCount = 0;
                            var complectedCount = 0;
                            var voidedCount = 0;
                            angular.forEach(statusArray, function (item) {
                                if (item == 'UN_BEGIN') {
                                    unBeginCount++;
                                }
                                if (item == 'COMPLETED') {
                                    complectedCount++;
                                }
                                if (item == 'VOIDED') {
                                    voidedCount++;
                                }
                            });
                            if (unBeginCount == statusArray.length) {
                                return '未开始';
                            } else if (complectedCount == statusArray.length) {
                                return '已完成';
                            } else if (voidedCount == statusArray.length || (complectedCount + voidedCount) == statusArray.length) {
                                return '已失效';
                            } else {
                                return '进行中';
                            }
                        } else {
                            return '创建完成';
                        }

                    }
                },
                {
                    dataField: '',
                    caption: $translate.instant('BGLColumnHandle'),
                    fixed: true,
                    width: '80px',
                    alignment: 'center',
                    cellTemplate: function (container, options) {
                        try {
                            $('<a >' + $translate.instant('BGLColumnHandleView') + '</a>&nbsp;&nbsp;')
                                .on('click', function () {
                                    $scope.updateGroup(options.data);
                                }).appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
            ],
            bindingOptions: {
                dataSource: 'pageConfDataSource'
            },
            masterDetail: {
                enabled: true,
                template: "detail"
            },
            loadPanel: {
                enabled: false
            },
            grouping: {
                autoExpandAll: false
            },
            noDataText: $translate.instant('NoDataText'),
        });

        $scope.avoidGroupDetail = function () {
            DevExpress.ui.dialog.confirm($translate.instant('DisableMsg'))
                .done(function (value) {
                    if (value) {
                        var groupDetails = [];
                        angular.forEach($scope.selectedGroupData, function (item) {
                            if (item) {
                                groupDetails.push({"groupId": item.id, "month": item.month});
                            }
                        });
                        $http.post('/backlogManage/backlog/disable', {"groupDetails": groupDetails}, apiConfig.createVat())
                            .success(function (res) {
                                if (res && 0 === res.code) {
                                    SweetAlert.success($translate.instant('BCLAllotDisableSuccess'));
                                    $scope.refreshGrid();
                                } else {
                                    SweetAlert.error($translate.instant('SystemError'));
                                }
                            });
                    }
                });

        };
        $scope.getSubDetailGrid = function (key) {
            return {
                //todo 绑定数据
                dataSource: new DevExpress.data.DataSource({
                    load: function (loadOptions) {
                        var queryParam = {
                            "backlogGroupId": key.data.id,
                            "month": key.data.month,
                        };
                        var deferred = $q.defer();
                        $http.post('/backlogManage/backlog/detail', queryParam, apiConfig.createVat())
                            .success(function (res) {
                                if (res && res.list) {
                                    deferred.resolve({data: res.list, totalCount: res.list.length});
                                } else {
                                    SweetAlert.error($translate.instant('SystemError'));
                                }
                            }).error(function () {
                            deferred.reject("Data Loading Error");
                        });
                        return deferred.promise;
                    }
                }),
                columnAutoWidth: true,
                showBorders: true,
                paging: {
                    pageSize: constant.page.pageSize
                },
                pager: {
                    showPageSizeSelector: true,
                    allowedPageSizes: constant.page.pageSizeArrary,
                    showInfo: true
                },
                columns: [
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnOutTimeFlag'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        alignment: 'center',
                        cellTemplate: function (container, options) {
                            try {
                                switch (options.data.outTimeFlag) {
                                    case 0:
                                        break;
                                    case  1:
                                        $('<i class="fa fa-circle" style="color:green" aria-hidden="true"></i>')
                                            .appendTo(container);
                                        break;
                                    case  2:
                                        $('<i class="fa fa-circle" style="color:yellow" aria-hidden="true"></i>')
                                            .appendTo(container);
                                        break;
                                    case  3:
                                        $('<i class="fa fa-circle" style="color:red" aria-hidden="true"></i>')
                                            .appendTo(container);
                                        break;
                                }
                            } catch (e) {
                                $log.error(e);
                            }
                        }
                    },
                    {
                        dataField: 'backlogCode',
                        caption: $translate.instant('BDLColumnCode'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnType'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
                            return parentBacklogType.name;
                        }
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnSubType'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
                            return backlogType.name;
                        }
                    },
                    {
                        dataField: 'backlogName',
                        caption: $translate.instant('BDLColumnName'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnStartDate'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            return $filter('date')(data.chinaBacklogStartDate, 'yyyy-MM-dd HH:mm:ss');
                        }
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnEndDate'),
                        fixed: true,
                        allowHeaderFiltering: true,
                        calculateCellValue: function (data) {
                            return $filter('date')(data.chinaBacklogEndDate, 'yyyy-MM-dd HH:mm:ss');
                        }
                    },
                    {
                        dataField: 'currentUser',
                        caption: $translate.instant('BDLColumnCurrentUser'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'backlogStatusStr',
                        caption: $translate.instant('BDLColumnStatus'),
                        fixed: true
                    },
                    {
                        dataField: 'dealUser',
                        caption: $translate.instant('BDLColumnDealUser'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'auditUser',
                        caption: $translate.instant('BDLColumnAuditUser'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: 'createUser',
                        caption: $translate.instant('BDLColumnCreate'),
                        fixed: true,
                        allowHeaderFiltering: true
                    },
                    {
                        dataField: '',
                        caption: $translate.instant('BDLColumnHandle'),
                        fixed: true,
                        width: '80px',
                        alignment: 'center',
                        cellTemplate: function (container, options) {
                            try {
                                $('<a >' + $translate.instant('BDLColumnHandleView') + '</a>&nbsp;&nbsp;')
                                    .on('click', function () {
                                        $scope.detailView(options.data);
                                    }).appendTo(container);
                            } catch (e) {
                                $log.error(e);
                            }
                        }
                    },
                ],
            };
        }

        $scope.exportBacklogGroup = function () {
            $('#busy-indicator-container').show();
            var xhr = new XMLHttpRequest();
            xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/group/export", true);
            xhr.responseType = 'arraybuffer';
            xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.onload = function () {
                // var headers = xhr.getAllResponseHeaders();
                // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                var fileName = '事项组详情.xlsx';
                if (this.status === 200) {
                    var blob = new Blob([this.response], {type: "arraybuffer"});
                    if (typeof window.navigator.msSaveBlob !== 'undefined') {
                        window.navigator.msSaveBlob(blob, fileName)
                    } else {
                        var URL = window.URL || window.webkitURL;
                        var objectUrl = URL.createObjectURL(blob);
                        if (fileName) {
                            var a = document.createElement('a');
                            if (typeof a.download === 'undefined') {
                                window.location = objectUrl
                            } else {
                                a.href = objectUrl;
                                a.download = fileName;
                                document.body.appendChild(a);
                                a.click();
                                a.remove();
                            }
                        } else {
                            window.location = objectUrl;
                        }
                        $('#busy-indicator-container').hide();
                    }
                }
            };
            xhr.send(JSON.stringify($scope.searchParam));
        };
        $scope.detailView = function (data) {
            $scope.detailEdit = null;
            $scope.detailEdit = data;
            if($scope.detailEdit.amount&&(typeof $scope.detailEdit.amount == "number" )){
                $scope.detailEdit.amount = Number($scope.detailEdit.amount).formatAmount(2);
            }
            $scope.detailEdit.unEditFileUploads = [];
            $scope.detailEdit.comments = [];
            var backlogType = $filter('filter')($scope.backlogTypeData, {id: data.backlogTypeId})[0];
            $scope.detailEdit.backlogTypeStr = backlogType.name;
            var parentBacklogType = $filter('filter')($scope.backlogTypeData, {id: backlogType.parentId})[0];
            $scope.detailEdit.backlogTypeParentStr = parentBacklogType.name;
            $http.post('/backlogManage/backlog/log', {"detailId": data.id}, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        angular.forEach(res, function (item) {
                            if (item.comment) {

                                $scope.detailEdit.comments.push(item.dealRoleStr + "[" + item.dealUserName + "]：" + item.comment);
                            }

                            if (item.fileUploads) {
                                angular.forEach(item.fileUploads, function (item1) {
                                    $scope.detailEdit.unEditFileUploads.push(item1);
                                });
                            }
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                });
            $($scope.detailViewDiv).modal('show');
        };

        $scope.updateGroup = function (group) {
            $scope.groupReadOnly = true;
            $scope.groupEdit = group;
            $scope.groupEdit.choose = !group.orgId ? '2' : '1';
            $($scope.groupEditDiv).modal('show');
        };
        //关闭配置框
        $scope.cancelModal = function () {
            $('#configForm')[0].reset();
        };

        //获取事项类型选项列表
        $scope.backlogTypeData = null;
        $scope.getBacklogType = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/backlogManage/backlog/calendar/type')
                .then(function (res) {
                    if (res) {
                        $scope.backlogTypeData = res;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        $scope.businessListData = [];
        $scope.orgListData = null;
        $scope.getOrglist = function () {
            jqFetch.get(apiInterceptor.webApiHostUrl + '/org/getMyOrgList')
                .then(function (res) {
                    if (res && 0 === res.code) {
                        $scope.orgListData = res.data;
                        var dataMap = [];
                        angular.forEach(res.data, function (item) {
                            dataMap[item.businessUnitId] = item.businessUnitName;
                        });
                        for (var key in dataMap) {
                            if (key != 'null' && key.length > 18) {
                                $scope.businessListData.push({"ID": key, "name": dataMap[key]});
                            }

                        }
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        $scope.timeZoneData = [];
        $scope.getTimeZone = function () {
            $.get('app-resources/timezones/timezones.json', function (timezone) {
                angular.forEach(timezone, function (item) {
                    $scope.timeZoneData.push({
                        'key': item.timezone + '(' + item.zonename + ')', 'val': item.timezone
                    });
                });
            });
        };

        //刷新页面
        $scope.refreshGrid = function () {
            $scope.searchParam.pageInfo = {
                pageIndex: $scope.pagingOptions.pageIndex,
                totalItems: $scope.pagingOptions.totalItems,
                pageSize: $scope.pagingOptions.pageSize
            };
            $http.post('/backlogManage/backlog/group/detail', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.pageConfDataSource = res.list;
                        $scope.pagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        //搜索框的处理
        $scope.searchboxService = {

            showOrHideSearchBox: function () {
                $scope.hasShowMoreSearchBox = !$scope.hasShowMoreSearchBox;
            },
            search: function () {
                $scope.pagingOptions.pageIndex = 1;
                $scope.refreshGrid();
            },
            resetBox: function () {
                $scope.searchParam = {};
            }
        };
        $scope.downFile = function (fileUoloadId, fileName, zipFlag) {
            var fileUoloadIds = fileUoloadId.split(',');
            var ids = [];
            angular.forEach(fileUoloadIds, function (item) {
                if (item && item != '') {
                    ids.push(item);
                }
            });
            if (ids.length == 0) {
                SweetAlert.error($translate.instant('UnFileDownload'));
                return;
            }
            DevExpress.ui.dialog.confirm($translate.instant('DownloadMsg'))
                .done(function (value) {
                    if (value) {
                        $('#busy-indicator-container').show();
                        var xhr = new XMLHttpRequest();
                        xhr.open('POST', apiInterceptor.webApiHostUrl + "/backlogManage/backlog/detail/download", true);
                        xhr.responseType = 'arraybuffer';
                        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
                        xhr.setRequestHeader("Authorization", apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken());
                        xhr.onload = function () {
                            // var headers = xhr.getAllResponseHeaders();
                            // fileName = headers['x-file-name'] ? headers['x-file-name'] : fileName;
                            if (this.status === 200) {
                                var blob = new Blob([this.response], {type: "arraybuffer"});
                                if (typeof window.navigator.msSaveBlob !== 'undefined') {
                                    window.navigator.msSaveBlob(blob, fileName)
                                } else {
                                    var URL = window.URL || window.webkitURL;
                                    var objectUrl = URL.createObjectURL(blob);
                                    if (fileName) {
                                        var a = document.createElement('a');
                                        if (typeof a.download === 'undefined') {
                                            window.location = objectUrl
                                        } else {
                                            a.href = objectUrl;
                                            a.download = fileName;
                                            document.body.appendChild(a);
                                            a.click();
                                            a.remove();
                                        }
                                    } else {
                                        window.location = objectUrl;
                                    }
                                    $('#busy-indicator-container').hide();
                                }
                            }
                        };
                        xhr.send(JSON.stringify({"fileUploadIds": ids, "zipFlag": zipFlag}));
                    }
                });

        };
        (function initialize() {
            //弹出模态框定义
            $scope.groupEditDiv = "#groupEditDiv";

            $scope.detailViewDiv = "#detailViewDiv";
            $scope.selectedGroupData = null;
            $scope.allottedConfigIds = [];
            $scope.groupEdit = {};
            $scope.formParam = {};
            //日期
            $scope.dateFormat = $translate.instant('dateFormat4YearMonthDay');

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 20  //每页多少条数据
            };

            $scope.detailEditReadOnly = false;
            $scope.detailEdit = {};
            $scope.searchParam = {};
            $scope.detailSearchParam = {};
            $scope.searchOptions = {
                startDateBoxStart: {
                    type: "date",
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'searchParam.startDate',
                        max: 'searchParam.endDate'
                    },
                },
                endDateBoxStart: {
                    type: "date",
                    displayFormat: 'yyyy-MM-dd',
                    bindingOptions: {
                        value: 'searchParam.endDate',
                        min: 'searchParam.startDate'
                    }
                },
                generalTypeOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.generateType'
                    },
                    dataSource: [
                        {'key': '单次', 'val': 1},
                        {'key': '周期', 'val': 2}
                    ]
                },
                orgOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    deferRendering: false,
                    bindingOptions: {
                        value: 'searchParam.orgId',
                        dataSource: 'orgListData'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                }
            };

            $scope.groupEditOptions = {
                autoFlagEditOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'groupEdit.autoFlag',
                        readOnly: 'groupReadOnly'
                    },
                    dataSource: [
                        {'key': '是', 'val': true},
                        {'key': '否', 'val': false}
                    ]
                },
                generalTypeEditOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'groupEdit.generateType',
                        readOnly: 'groupReadOnly'
                    },
                    dataSource: [
                        {'key': '单次', 'val': 1},
                        {'key': '周期', 'val': 2}
                    ]
                },
                chooseAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'groupEdit.choose',
                        readOnly: 'groupReadOnly'
                    },
                    dataSource: [
                        {'key': '公司', 'val': '1'},
                        {'key': '业务线', 'val': '2'}
                    ]
                },
                orgAllotOptions: {
                    displayExpr: 'name',
                    valueExpr: 'id',
                    deferRendering: false,
                    bindingOptions: {
                        value: 'groupEdit.orgId',
                        dataSource: 'orgListData',
                        readOnly: 'groupReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true,
                    onSelectionChanged: function (obj) {
                        if (obj&&obj.selectedItem) {
                            $scope.groupEdit.businessLineId = obj.selectedItem.businessUnitId;
                        }
                    }
                },
                businessAllotOptions: {
                    displayExpr: 'name',
                    valueExpr: 'ID',
                    // width: '95%',
                    bindingOptions: {
                        value: 'groupEdit.businessLineId',
                        dataSource: 'businessListData',
                        readOnly: 'groupReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },
                timeZoneAllotOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    // width: '95%',
                    bindingOptions: {
                        value: 'groupEdit.timeZone',
                        dataSource: 'timeZoneData',
                        readOnly: 'groupReadOnly'
                    },
                    // height: '30px',
                    placeholder: '',
                    showClearButton: true,
                    searchEnabled: true,
                    showSelectionControls: true
                },

            };
            $scope.unEditFileListOptions = {
                itemTemplate: function (item, index, container) {
                    var div = $('<div></div>');
                    $('<span style="float: left">' + item.fileName + '</span>').appendTo(div);
                    $('<i style="float: right;color: #dc6900"class="fa fa-download"></i>').on('click', function () {
                        $scope.downFile(item.uid, item.fileName, false);
                    }).appendTo(div);
                    div.appendTo(container);
                },
                noDataText: null,
                editEnabled: true,
                allowItemDeleting: false,
                bindingOptions: {
                    dataSource: 'detailEdit.unEditFileUploads'
                }
            };
            $scope.commentListOptions = {
                noDataText: null,
                editEnabled: true,
                allowItemDeleting: false,
                bindingOptions: {
                    dataSource: 'detailEdit.comments'
                }
            };

            function init() {
                $scope.refreshGrid();
                $scope.getBacklogType();
                setTimeout(function () {
                    $scope.getOrglist();
                    $scope.getTimeZone();
                }, 1000);
            }

            init()

        })();
    }
]);