﻿commonModule.directive('textEdit', ['$log', 'region', 'enums',
    function ($log, region, enums) {
        'use strict';
        $log.debug('tax-nav-template.ctor()...');
        return {
            restrict: 'AECM',
            templateUrl: '/app/common/pwc/text-edit/text-edit.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                editType: "=",
                readType: "=",
                writeType: '=',
                uploadObj: '=',
                base64Upload: '=',
                richText: '=',
                onChange: '&',
                onChangeHandle: '=',
                editSetting: '=',
                editorInit: '='
            },
            controller: 'textEditController',
            link: function ($scope, $element, $attr) {
                var $editor = null;
                $scope.editorInit = function (editorDomId, index) {
                    $('div[data-class  = "text-edit"]').each(function (_index, ele) {
                        if (_index == index) {
                            $(this).children('div').attr('id', editorDomId)
                        }
                    });
                    var uploadServer = function (editor) {
                        editor.customConfig.uploadImgHooks = {
                            before: function (xhr, editor, files) {
                                if ($scope.upload != null && $scope.upload != undefined && $scope.upload.before != undefined) {
                                    $scope.upload.before();
                                }
                                // 图片上传之前触发
                                // xhr 是 XMLHttpRequst 对象，editor 是编辑器对象，files 是选择的图片文件

                                // 如果返回的结果是 {prevent: true, msg: 'xxxx'} 则表示用户放弃上传
                                // return {
                                //     prevent: true,
                                //     msg: '放弃上传'
                                // }
                            },
                            success: function (xhr, editor, result) {
                                // 图片上传并返回结果，图片插入成功之后触发
                                // xhr 是 XMLHttpRequst 对象，editor 是编辑器对象，result 是服务器端返回的结果
                                if ($scope.upload != null && $scope.upload != undefined && $scope.upload.success != undefined) {
                                    $scope.upload.success();
                                }
                            },
                            fail: function (xhr, editor, result) {
                                // 图片上传并返回结果，但图片插入错误时触发
                                // xhr 是 XMLHttpRequst 对象，editor 是编辑器对象，result 是服务器端返回的结果
                                if ($scope.upload != null && $scope.upload != undefined && $scope.upload.fail != undefined) {
                                    $scope.upload.fail();
                                }
                            },
                            error: function (xhr, editor) {
                                // 图片上传出错时触发
                                // xhr 是 XMLHttpRequst 对象，editor 是编辑器对象
                                if ($scope.upload != null && $scope.upload != undefined && $scope.upload.error != undefined) {
                                    $scope.upload.error();
                                }
                            },
                            timeout: function (xhr, editor) {
                                // 图片上传超时时触发
                                // xhr 是 XMLHttpRequst 对象，editor 是编辑器对象
                                if ($scope.upload != null && $scope.upload != undefined && $scope.upload.timeout != undefined) {
                                    $scope.upload.timeout();
                                }
                            },

                            // 如果服务器端返回的不是 {errno:0, data: [...]} 这种格式，可使用该配置
                            // （但是，服务器端返回的必须是一个 JSON 格式字符串！！！否则会报错）
                            customInsert: function (insertImg, result, editor) {
                                // 图片上传并返回结果，自定义插入图片的事件（而不是编辑器自动插入图片！！！）
                                // insertImg 是插入图片的函数，editor 是编辑器对象，result 是服务器端返回的结果

                                // 举例：假如上传图片成功后，服务器端返回的是 {url:'....'} 这种格式，即可这样插入图片：
                                var url = result.url
                                insertImg(url)
                                // result 必须是一个 JSON 格式字符串！！！否则报错
                            }
                        }
                    }
                    var editor = null;
                    var initEdit = function () {
                        var base64Upload = true;
                        if ($scope.base64Upload != null && $scope.base64Upload != undefined) {
                            base64Upload = $scope.base64Upload;
                        }
                        var E = window.wangEditor
                        editor = new E('#' + editorDomId);
                        // 通过 url 参数配置 debug 模式。url 中带有 wangeditor_debug_mode=1 才会开启 debug 模式
                        editor.customConfig.debug = location.href.indexOf('wangeditor_debug_mode=1') > 0
                        editor.customConfig.pasteFilterStyle = false;//关闭掉粘贴样式的过滤
                        // 或者 var editor = new E( document.getElementById('editor') )

                        // 自定义配置颜色（字体颜色、背景色）
                        editor.customConfig.colors = [
                            '#000000',
                            '#eeece0',
                            '#1c487f',
                            '#4d80bf',
                            '#c24f4a',
                            '#8baa4a',
                            '#7b5ba1',
                            '#46acc8',
                            '#f9963b',
                            '#ffffff'
                        ];
                        // 表情面板可以有多个 tab ，因此要配置成一个数组。数组每个元素代表一个 tab 的配置
                        editor.customConfig.emotions = [
                            {
                                // tab 的标题
                                title: '默认',
                                // type -> 'emoji' / 'image'
                                type: 'image',
                                // content -> 数组
                                content: emotion.data
                            },
                            {
                                // tab 的标题
                                title: 'emoji',
                                // type -> 'emoji' / 'image'
                                type: 'emoji',
                                // content -> 数组
                                content: ['😀', '😃', '😄', '😁', '😆']
                            }
                        ];
                        // 自定义字体
                        editor.customConfig.fontNames = [
                            '宋体',
                            '微软雅黑',
                            'Arial',
                            'Tahoma',
                            'Verdana'
                        ];
                        //editor.customConfig.uploadImgServer = '/upload'  // 上传图片到服务器

                        // 将图片大小限制为 5M
                        editor.customConfig.uploadImgMaxSize = 5 * 1024 * 1024;
                        editor.customConfig.uploadImgHeaders = {
                            'Accept': 'text/x-json'
                        }
                        // 将 timeout 时间改为 3s
                        editor.customConfig.uploadImgTimeout = 3000;
                        if (base64Upload) {
                            editor.customConfig.uploadImgShowBase64 = true
                        } else {
                            uploadServer(editor);
                        }
                        editor.customConfig.customAlert = function (info) {
                            // info 是需要提示的内容
                            SweetAlert.error("上传失败");
                            console.log("上传失败原因" + info);
                        }
                        editor.customConfig.onchange = function (html) {
                            $scope.onChange(html);
                        }
                        $scope.$watch('onChangeHandle', function (newVal) {
                            editor.change && editor.change()
                        });
                        editor.create();
                        $editor = editor;
                        //读取内容时候支持三种格式的读取
                    }
                    var freshStatus = function () {
                        $scope.richText = {};//初始化
                        $scope.richText.read = function () {
                            var content = null;
                            if ($scope.readType != undefined && $scope.readType != null) {
                                if ($scope.readType == 0) {////html
                                    content = editor.txt.html();
                                } else if ($scope.readType == 1) {
                                    content = editor.txt.text();
                                } else if ($scope.readType == "json") {
                                    var json = editor.txt.getJSON()  // 获取 JSON 格式的内容
                                    content = JSON.stringify(json);
                                } else {
                                    content = editor.txt.html();
                                }
                            } else {//默认html
                                content = editor.txt.html();
                            }
                            return content;
                        };

                        $scope.richText.write = function (content) {
                            if ($scope.writeType != undefined && $scope.writeType != null) {
                                if ($scope.writeType == 0) {////html
                                    editor.txt.html(content);
                                } else if ($scope.writeType == 1) {
                                    editor.txt.text();
                                } else {
                                    editor.txt.html(content);
                                }
                            } else {//默认html
                                editor.txt.html(content);
                            }
                        }

                        $scope.richText.clear = function () {
                            editor.txt.clear();
                        }
                        $scope.richText.append = function (content) {
                            editor.txt.append(content)
                        }
                    }
                    initEdit();
                    freshStatus();
                    return $editor;
                }
            }
        };
    }
]);