(function (window, undefined) {
    PWC = window.PWC || {};
    //string extention
    if (!String.prototype.format) {
        String.prototype.format = function () {
            var args = arguments;
            return this.replace(/{(\d+)}/g, function (match, number) {
                return typeof args[number] !== 'undefined' ? args[number] : match;
            });
        };
    }

    if (!String.prototype.formatObj) {
        String.prototype.formatObj = function () {
            var str = this.toString();
            if (!arguments.length) {
                return str;
            }
            var args = typeof arguments[0];
            args = (("string" === args || "number" === args) ? arguments : arguments[0]);
            for (var arg in args) {
                if (args.hasOwnProperty(arg)) {
                    str = str.replace(RegExp("\\{" + arg + "\\}", "gi"), args[arg]);
                }
            }
            return str;
        }
    }

    if (!String.prototype.replaceToEnd) {
        String.prototype.replaceToEnd = function (search, replacement) {
            var target = this;
            return target.replace(new RegExp(search, 'g'), replacement);
        }
    }


    if (!String.prototype.endsWith) {
        String.prototype.endsWith = function (pattern) {
            var d = this.length - pattern.length;
            return d >= 0 && this.lastIndexOf(pattern) === d;
        };
    }

    if (!String.prototype.startWith) {
        String.prototype.startWith = function (pattern) {
            var reg = new RegExp("^" + pattern);
            return reg.test(this);
        };
    }

    if (!Number.prototype.formatAmount) {
        Number.prototype.formatAmount = function (decPlaces, type) {
            if (type) {
                return Number(decPlaces.toFixed(2));
            }
            decPlaces = isNaN(decPlaces = Math.abs(decPlaces)) ? 2 : decPlaces;
            var n = this.toFixed(decPlaces);
            if (decPlaces) {
                var i = n.substr(0, n.length - (decPlaces + 1));
                var j = '.' + n.substr(-decPlaces);
            } else {
                i = n;
                j = '';
            }

            function reverse(str) {
                var sr = '';
                for (var l = str.length - 1; l >= 0; l--) {
                    sr += str.charAt(l);
                }
                return sr;
            }

            if (parseInt(i)) {
                i = reverse(reverse(i).replace(/(\d{3})(?=\d)/g, "$1" + ','));
            }
            return i + j;
        };
    }


    PWC.AccountToFloat = function (data) {
        if (data == null || data == undefined) return data;
        var result = {
            value: '',
            isNumber: true
        };
        var formatExp = /^\d+$/;
        //根据.拆分
        var array1 = data.split('.');
        if (array1.length > 0) {
            //根据,拆分
            var array2 = array1[0].split(',');
            for (var i = 0; i < array2.length; i++) {
                var item = array2[i];
                if (formatExp.test(item)) {
                    result.value += item;
                } else {
                    result.isNumber = false;
                    break;
                }
            }
        }
        if (result.isNumber && array1.length === 2) {
            if (formatExp.test(array1[1])) {
                result.value += '.' + array1[1];
            } else {
                result.isNumber = false;
            }
        }
        return result;
    };
    //解决toFixed的小数精度问题 ：http://blog.csdn.net/langqiao123/article/details/42173721
    Number.prototype.toFixed = function (d) {
        var s = this + "";
        if (!d) d = 0;
        if (s.indexOf(".") == -1) s += ".";
        s += new Array(d + 1).join("0");
        if (new RegExp("^(-|\\+)?(\\d+(\\.\\d{0," + (d + 1) + "})?)\\d*$").test(s)) {
            var s = "0" + RegExp.$2, pm = RegExp.$1, a = RegExp.$3.length, b = true;
            if (a == d + 2) {
                a = s.match(/\d/g);
                if (parseInt(a[a.length - 1]) > 4) {
                    for (var i = a.length - 2; i >= 0; i--) {
                        a[i] = parseInt(a[i]) + 1;
                        if (a[i] == 10) {
                            a[i] = 0;
                            b = i != 1;
                        } else break;
                    }
                }
                s = a.join("").replace(new RegExp("(\\d+)(\\d{" + d + "})\\d$"), "$1.$2");

            }
            if (b) s = s.substr(1);
            return (pm + s).replace(/\.$/, "");
        }
        return this + "";

    };

    if (!Array.prototype.sortBy) {
        Array.prototype.sortBy = function (f) {
            return this.sort(function (a, b) {
                if (a[f] === b[f])
                    return 0;
                if (a[f] && b[f]) {
                    return a[f].toLowerCase() < b[f].toLowerCase() ? -1 : 1;
                }
                return a[f] < b[f] ? -1 : 1;
            });
        };
    }

    //Get duplicate object in array
    if (!Array.prototype.objectDuplicate) {
        Array.prototype.duplicate = function () {
            var tmp = [];
            this.concat().sort().sort(function (a, b) {
                if (isObjectValueEqual(a, b) && tmp.indexOf(a) === -1) {
                    tmp.push(a);
                }
            });
            return tmp;
        };
    }

    //check two object Is congruent
    function isObjectValueEqual(a, b) {
        if (typeof a == 'number' && typeof b == 'number') {
            return a == b
        }


        var aProps = Object.getOwnPropertyNames(a);
        var bProps = Object.getOwnPropertyNames(b);

        if (aProps.length != bProps.length) {
            return false;
        }

        for (var i = 0; i < aProps.length; i++) {
            var propName = aProps[i];
            if (Object.prototype.toString.call(a[propName]) == '[object Object]' || Object.prototype.toString.call(b[propName]) == '[object Object]') {
                var flag = isObjectValueEqual(a[propName], b[propName]);
                if (!flag) {
                    return flag;
                }
            } else if (a[propName] !== b[propName]) {
                return false;
            }
        }
        return true;
    }

    if (!Object.keys) {
        Object.keys = function (obj) {
            if (obj !== Object(obj)) {
                throw new TypeError('Object.keys called on non-object');
            }
            var ret = [],
                p;
            for (p in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, p)) {
                    ret.push(p);
                }
            }
            return ret;
        }
    }

    PWC.newGuid = function () {
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
        };
        return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
    }

    // https://github.com/BlueHuskyStudios/Micro-JS-Enum/wiki
    PWC.enum = function (enumArray) {
        var self = {
            all: [],
            keys: enumArray,
        };
        for (var i = enumArray.length; i--;) {
            self[enumArray[i]] = self.all[i] = i;
        }
        return self;
    };

    // get callee arguments
    // export injected args to global
    // only work in non use strict mode
    // usage PWC.debugger(); // 'use strict';
    PWC.debugger = function () {
        var fn = arguments.callee.caller;
        if (fn) {
            var tmp = fn.toString().match(/\(.*?\)/)[0];
            var argumentNames = tmp.replace(/[()\s]/g, '').split(',');

            var result = {};
            [].splice.call(fn.arguments, 0).forEach(function (arg, i) {
                result[argumentNames[i]] = arg;
            });
            angular.extend(window, result);
        } else {
            console.error('debugger not work, check and disable strict mode');
        }
    }

    // http://anentropic.wordpress.com/2009/06/25/javascript-iso8601-parser-and-pretty-dates/
    // get the milliseconds from 1970/1/1 until now.
    // input date format: ISO8601 format date string.
    PWC.parseISO8601 = function (str) {
        // we assume str is a UTC date ending in ‘Z’
        var parts = str.split('T');
        var dateParts = parts[0].split('-');
        var timeParts = parts[1].split('Z');
        timeParts = timeParts[0].split('-');
        timeParts = timeParts[0].split('+');
        var timeSubParts = timeParts[0].split(':');
        var timeSecParts = timeSubParts[2].split('.');
        var timeHours = Number(timeSubParts[0]);
        var date = new Date;

        date.setUTCFullYear(Number(dateParts[0]));
        date.setUTCMonth(Number(dateParts[1]) - 1);
        date.setUTCDate(Number(dateParts[2]));
        date.setUTCHours(Number(timeHours));
        date.setUTCMinutes(Number(timeSubParts[1]));
        date.setUTCSeconds(Number(timeSecParts[0]));
        if (timeSecParts[1])
            date.setUTCMilliseconds(Number(timeSecParts[1]));
        return date;
    };

    // Parse date object from a local time string.
    PWC.parseFromLocalTimeString = function (dateString) {

        var parts = dateString.split('T');
        var dateParts = parts[0].split('-');
        var timeParts = parts[1].split('Z');
        timeParts = timeParts[0].split('-');
        timeParts = timeParts[0].split('+');
        var timeSubParts = timeParts[0].split(':');
        var timeSecParts = timeSubParts[2].split('.');
        var timeHours = Number(timeSubParts[0]);
        var date = new Date;

        date.setFullYear(Number(dateParts[0]));
        date.setMonth(Number(dateParts[1]) - 1);
        date.setDate(Number(dateParts[2]));
        date.setHours(Number(timeHours));
        date.setMinutes(Number(timeSubParts[1]));
        date.setSeconds(Number(timeSecParts[0]));
        if (timeSecParts[1])
            date.setMilliseconds(Number(timeSecParts[1]));
        return date;
    };

    PWC.getDateFromTimeSpan = function (dateString, timeSpan, isLocalTime) {
        var datePart = dateString.split('T')[0];
        var timeString = datePart + 'T' + timeSpan;

        if (isLocalTime) {
            return PWC.parseFromLocalTimeString(timeString);
        }
        else {
            return PWC.parseISO8601(timeString);
        }
    }

    // convert the database utc time to local time.
    PWC.toLocalTime = function (dateString) {
        return (new Date(PWC.parseISO8601(dateString)));
    };

    // date extention
    if (!Date.prototype.formatDateTime) {
        Date.prototype.formatDateTime = function (formatStr) {
            var date = this;
            var zeroize = function (value, length) {
                if (!length) {
                    length = 2;
                }

                value = new String(value);
                for (var i = 0, zeros = ''; i < (length - value.length); i++) {
                    zeros += '0';
                }
                return zeros + value;
            };

            return formatStr.replace(/"[^"]*"|'[^']*'|\b(?:d{1,4}|M{1,4}|yy(?:yy)?|([hHmstT])\1?|[lLZ])\b/g, function ($0) {
                switch ($0) {
                    case 'yyyyMMdd':
                        return 'AAAAA';
                    case 'd':
                        return date.getDate();
                    case 'dd':
                        return zeroize(date.getDate());
                    case 'ddd':
                        return ['Sun', 'Mon', 'Tue', 'Wed', 'Thr', 'Fri', 'Sat'][date.getDay()];
                    case 'dddd':
                        return ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'][date.getDay()];
                    case 'M':
                        return date.getMonth() + 1;
                    case 'MM':
                        return zeroize(date.getMonth() + 1);
                    case 'MMM':
                        return ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][date.getMonth()];
                    case 'MMMM':
                        return ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'][date.getMonth()];
                    case 'yy':
                        return new String(date.getFullYear()).substr(2);
                    case 'yyyy':
                        return date.getFullYear();
                    case 'h':
                        return date.getHours() % 12 || 12;
                    case 'hh':
                        return zeroize(date.getHours() % 12 || 12);
                    case 'H':
                        return date.getHours();
                    case 'HH':
                        return zeroize(date.getHours());
                    case 'm':
                        return date.getMinutes();
                    case 'mm':
                        return zeroize(date.getMinutes());
                    case 's':
                        return date.getSeconds();
                    case 'ss':
                        return zeroize(date.getSeconds());
                    case 'l':
                        return date.getMilliseconds();
                    case 'll':
                        return zeroize(date.getMilliseconds());
                    case 'tt':
                        return date.getHours() < 12 ? 'am' : 'pm';
                    case 'TT':
                        return date.getHours() < 12 ? 'AM' : 'PM';
                }
            });
        };
    }

    if (!Date.prototype.dateTimeToString) {
        Date.prototype.dateTimeToString = function (formatStr) {
            switch (formatStr) {
                case 'yyyyMMdd':
                    return this.formatDateTime('yyyy') + '-' + this.formatDateTime('MM') + '-' + this.formatDateTime('dd');
                case 'yyyyMMdd hh:mm':
                    return this.formatDateTime('yyyy') + '-' + this.formatDateTime('MM') + '-' + this.formatDateTime('dd') + " " + this.formatDateTime('HH') + ":" + this.formatDateTime('mm');
                case 'yyyyMMddHHmmss':
                    return this.formatDateTime('yyyy') + this.formatDateTime('MM') + this.formatDateTime('dd') + this.formatDateTime('HH') + this.formatDateTime('mm') + this.formatDateTime('ss');
                case 'dd/MM/yyyy':
                    return this.formatDateTime('dd') + '/' + this.formatDateTime('MM') + '/' + this.formatDateTime('yyyy');
            }
        }
    }

    /* 得到日期年月日等加数字后的日期 */
    if (!Date.prototype.dateAdd) {
        Date.prototype.dateAdd = function (interval, number) {
            var d = this;
            var k = {
                'y': 'FullYear',
                'q': 'Month',
                'm': 'Month',
                'w': 'Date',
                'd': 'Date',
                'h': 'Hours',
                'n': 'Minutes',
                's': 'Seconds',
                'ms': 'MilliSeconds'
            };
            var n = {'q': 3, 'w': 7};
            eval('d.set' + k[interval] + '(d.get' + k[interval] + '()+' + ((n[interval] || 1) * number) + ')');
            return d;
        };
    }

    PWC.clearArray = function (array) {
        if (_.isArray(array)) {
            while (array.length > 0) {
                array.pop();
            }
        }
    };

    PWC.today = function () {
        var now = new Date();
        var today = new Date(now.getFullYear(), now.getMonth(), now.getDate());
        return today;
    };

    PWC.toDate = function (value) {
        if (_.isString(value)) {
            value = new Date(value);
        }

        if (_.isDate(value)) {
            var date = new Date(value.getFullYear(), value.getMonth(), value.getDate());
            return date;
        }

        return null;
    };

    //考虑传入字符的中文和英文的区别
    PWC.getLength = function strlen(str) {
        var len = 0;
        if (PWC.isNullOrEmpty(str)) {
            return 0;
        }
        for (var i = 0; i < str.length; i++) {
            var c = str.charCodeAt(i);
            //单字节加1
            if ((c >= 0x0001 && c <= 0x007e) || (0xff60 <= c && c <= 0xff9f)) {
                len++;
            } else {
                len += 2;
            }
        }
        return len;
    };

    //不考虑中英文区别，直接获取length
    PWC.getStringLength = function (str) {
        if (PWC.isNullOrEmpty(str)) {
            return 0;
        }
        else {
            return str.length;
        }
    };

    PWC.nIndexOf = function (str, pat, n) {
        var len = str.length, i = -1;
        while (n-- && i++ < len) {
            i = str.indexOf(pat, i);
            if (i < 0) break;
        }
        return i;
    };

    PWC.isNullOrWhiteSpace = function (obj) {
        if (_.isUndefined(obj) || _.isNull(obj) || _.isEmpty(obj)) {
            return true;
        }

        return false;
    };

    PWC.isNullOrEmpty = function (obj) {
        if (_.isUndefined(obj) || _.isNull(obj) || $.trim(obj) === '') {
            return true;
        }
        return false;
    };

    //验证是否为正整数
    PWC.isPositiveInteger = function (obj) {
        var r = /^\+?[1-9][0-9]*$/;　　//正整数
        return r.test(obj);
    };

    //验证是否为整数
    PWC.isInteger = function (obj) {
        var r = /^\d+$/;　　//正整数
        return r.test(obj);
    };

    PWC.limitString = function (str, len) {
        if (str.length <= len) {
            return str;
        }
        else {
            return str.substring(0, len) + "...";
        }
    }

    PWC.round = function (num, decimals) {
        if (PWC.isNullOrEmpty(num) || isNaN(parseFloat(num))) {
            return num;
        }
        return parseFloat(num).formatAmount(decimals);
    };


    PWC.parseFloatRound = function (number, decimals) {
        if (_.isNumber(number)) {

            return parseFloat(parseFloat(number).toFixed(decimals));
        }
        else {
            var numberNoCommas = number.replace(/,/g, '');
            if (PWC.isNullOrEmpty(numberNoCommas) || isNaN(parseFloat(numberNoCommas))) {
                return num;
            }
            return parseFloat(parseFloat(numberNoCommas).toFixed(decimals));
        }
    }

    PWC.checkDate = function (dateval) {
        var arr = new Array();
        //yyyy.mm.dd
        if (dateval.indexOf(".") != -1) {
            dateval = dateval.replace(/\./g, '-');
        }

        if (dateval.indexOf("-") != -1) {
            arr = dateval.toString().split("-");
        } else if (dateval.indexOf("/") != -1) {
            arr = dateval.toString().split("/");
        } else {
            return false;
        }

        //yyyy-mm-dd || yyyy/mm/dd
        if (arr[0].length == 4) {
            var date = new Date(arr[0], arr[1] - 1, arr[2]);
            if (date.getFullYear() == arr[0] && date.getMonth() == arr[1] - 1 && date.getDate() == arr[2]) {
                return true;
            }
        }
        //dd-mm-yyyy || dd/mm/yyyy
        if (arr[2] && arr[2].length == 4) {
            var date = new Date(arr[2], arr[1] - 1, arr[0]);
            if (date.getFullYear() == arr[2] && date.getMonth() == arr[1] - 1 && date.getDate() == arr[0]) {
                return true;
            }
        }
        //mm-dd-yyyy || mm/dd/yyyy
        if (arr[2] && arr[2].length == 4) {
            var date = new Date(arr[2], arr[0] - 1, arr[1]);
            if (date.getFullYear() == arr[2] && date.getMonth() == arr[0] - 1 && date.getDate() == arr[1]) {
                return true;
            }
        }

        return false;
    }

    PWC.checkMonth = function (dateval) {
        var arr = new Array();
        //yyyy.mm
        if (dateval.indexOf(".") != -1) {
            dateval = dateval.replace(/\./g, '-');
        }

        if (dateval.indexOf("-") != -1) {
            arr = dateval.toString().split("-");
        } else if (dateval.indexOf("/") != -1) {
            arr = dateval.toString().split("/");
        } else {
            return false;
        }

        //yyyy-mm || yyyy/mm
        if (arr[0].length == 4) {
            var date = new Date(arr[0], arr[1] - 1, 1);
            if (date.getFullYear() == arr[0] && date.getMonth() == arr[1] - 1 && date.getDate() == 1) {
                return true;
            }
        }
        //dd-mm-yyyy || dd/mm/yyyy
        if (arr[1] && arr[1].length == 4) {
            var date = new Date(arr[1], arr[0] - 1, 1);
            if (date.getFullYear() == arr[2] && date.getMonth() == arr[1] - 1 && date.getDate() == 1) {
                return true;
            }
        }
        //mm-yyyy || mm/yyyy
        if (arr[1] && arr[1].length == 4) {
            var date = new Date(arr[1], arr[0] - 1, 1);
            if (date.getFullYear() == arr[2] && date.getMonth() == arr[0] - 1 && date.getDate() == 1) {
                return true;
            }
        }

        return false;
    }

    //控制在input中输入金额时，必须为数字，且只能保留2位小数
    PWC.inputNumberFormat = function (obj) {
        obj.value = obj.value.replace(/[^\d.]/g, "");  //清除“数字”和“.”以外的字符
        obj.value = obj.value.replace(/\.{2,}/g, "."); //只保留第一个. 清除多余的
        obj.value = obj.value.replace(".", "$#$").replace(/\./g, "").replace("$#$", ".");
        obj.value = obj.value.replace(/^(\-)*(\d+)\.(\d\d).*$/, '$1$2.$3');//只能输入两个小数
        if (obj.value.indexOf(".") < 0 && obj.value != "") {//以上已经过滤，此处控制的是如果没有小数点，首位不能为类似于 01、02的金额
            obj.value = parseFloat(obj.value);
        }
    }

    PWC.stringNumberFormat = function (string) {
        string = string + "";
        string = string.replace(/[^\d.]/g, "");  //清除“数字”和“.”以外的字符
        string = string.replace(/\.{2,}/g, "."); //只保留第一个. 清除多余的
        string = string.replace(".", "$#$").replace(/\./g, "").replace("$#$", ".");
        string = string.replace(/^(\-)*(\d+)\.(\d\d).*$/, '$1$2.$3');//只能输入两个小数
        if (string.indexOf(".") < 0 && string != "") {//以上已经过滤，此处控制的是如果没有小数点，首位不能为类似于 01、02的金额
            string = parseFloat(string);
        }
        return string;
    }

    //UI Grid to trigger the click for the tick icon
    PWC.triggerRowSelectOnClick = function (gridID) {
        $(gridID + ' .ui-grid-contents-wrapper > [role=grid] .ui-grid-canvas')
            .on('click', '.ui-grid-row', function (ev) {
                var jqRow = $(this); // '.ui-grid-row'
                var index = jqRow.index();
                var commonAncestor = jqRow.closest(gridID + ' .ui-grid-contents-wrapper');

                var selectButtonQuery = [
                    '.ui-grid-pinned-container', // left side class
                    '[role=grid] .ui-grid-canvas', // redundant, but doesn't hurt
                    '.ui-grid-row .ui-grid-selection-row-header-buttons' // select button class
                ].join(' ');
                var checkboxDiv = commonAncestor.find(selectButtonQuery);
                checkboxDiv.get(index).click();
            });
    };

    PWC.uiTree = {};
    //ui tree 折叠所有
    PWC.uiTree.collapsedAll = function (childNodes) {
        if (!childNodes || childNodes.length == 0)
            return;

        var collapseNode = function (c) {
            if (!c) return;

            c.collapse();

            var childs = c.childNodes();
            for (var i = 0, len = childs.length; i < len; i++) {
                collapseNode(childs[i]);
            }
        }

        for (var i = 0, len = childNodes.length; i < len; i++) {
            collapseNode(childNodes[i]);
        }
    };

    //ui tree 展开所有
    PWC.uiTree.expandAll = function (childNodes) {

        if (!childNodes || childNodes.length == 0)
            return;

        var expandNode = function (c) {
            if (!c) return;

            c.expand();

            var childs = c.childNodes();
            for (var i = 0, len = childs.length; i < len; i++) {
                expandNode(childs[i]);
            }
        }

        for (var i = 0, len = childNodes.length; i < len; i++) {
            expandNode(childNodes[i]);
        }
    };
    PWC.numToLetter = function (num) {
        num++;
        var remainder = 0;
        var colHandler = function (num, result) {
            if (num > 0) {
                if (num >= 1 && num <= 26) {
                    result = String.fromCharCode(64 + parseInt(num));
                } else {
                    while (num > 26) {
                        var count = parseInt(num / 26);
                        var remainder = num % 26;
                        if (remainder == 0) {
                            remainder = 26;
                            count--;
                        }

                        result = String.fromCharCode(64 + parseInt(remainder)) + result;
                        num = count;
                    }
                    result = String.fromCharCode(64 + parseInt(num)) + result;
                }
                return result;
            }
        }

        return colHandler(num, '');
    };
    PWC.letterToNum = function (str) {
        str = str.toUpperCase();
        var num = 0;
        for (var i = str.length - 1; i >= 0; i--) {
            num += (str.charCodeAt(str.length - i - 1) - 64) * Math.pow(26, i);
        }

        num--;
        return num;
    };
    PWC.numToExcelChar = function (row, col) {
        if (!_.isNumber(col) || isNaN(col) || col < 0
            || !_.isNumber(row) || isNaN(row) || row < 0) {
            return '';
        }

        var rowHandler = function (num, n) {
            num = (++num).toString();
            var len = num.length;
            while (num.length < n) {
                num = "0" + num;
                len++;
            }

            return num;
        }

        return PWC.numToLetter(col) + rowHandler(row, 2);
    };
    PWC.excelCharToNum = function (str) {
        if (!/^[A-Z]+[0-9]+$/.test(str)) {
            return -1; // Invalid Excel string
        }

        var colStr = /^[A-Z]+/.exec(str)[0];
        var rowStr = /[0-9]+$/.exec(str)[0];

        return {
            row: Number(rowStr) - 1,
            col: PWC.letterToNum(colStr)
        };
    };

    var toString = Object.prototype.toString;
    var isObject = function (o) {
        return toString.call(o) == '[object Object]'
    }
    var isArray = function (o) {
        return toString.call(o) == '[object Array]'
    }

    PWC.assignModel = function (source, dest) {
        if (isObject(source) && isObject(dest)) {
            for (var dProp in dest) {
                if (dest.hasOwnProperty(dProp) === true && source.hasOwnProperty(dProp) == true) {
                    dest[dProp] = source[dProp];
                }
            }
        }
    };

    PWC.toDict = function (array, keyProp) {
        var dic = {};
        if (isArray(array)) {
            array.forEach(function (item) {
                dic[item[keyProp]] = item;
            });
        }
        return dic;
    };

    PWC.isHavePermissionForOrg = function (orgID, permission) {
        var isShow = false;
        if (permission) {
            if (permission.isSuperAdmin || permission.isADUser) {
                isShow = true;
            }
            else {
                // 查找机构
                var findOrg = _.find(permission.organizationPermissionList, function (num) {
                    return num.id === orgID;
                });
                if (findOrg && findOrg.permissionList && findOrg.permissionList.length > 0) {
                    // 查看具体权限
                    isShow = true;
                }
            }
        }
        return isShow;
    }

    PWC.isHavePermission = function (permissionCode, permission) {
        var isShow = false;
        if (permission) {

            if (permission.isSuperAdmin) {
                isShow = true;
            }
            else if (permission.isAdmin && permissionCode.startWith('01')) {
                isShow = true;
            }
            else if (permission.isADUser && !permissionCode.startWith('01')) {
                isShow = true;
            }
            else {
                var find = _.find(permission.permissionList, function (num) {
                    return num.code.startWith(permissionCode);
                });

                if (find) {
                    isShow = true;
                }
            }
        }

        return isShow;
    };

    PWC.isHaveOrganizationPermission = function (orgID, permissionCode, permission) {
        var isShow = false;
        if (permission) {

            if (permission.isSuperAdmin) {
                isShow = true;
            }
            else if (permission.isAdmin && permissionCode.startWith('01')) {
                isShow = true;
            }
            else if (permission.isADUser && !permissionCode.startWith('01')) {
                isShow = true;
            }
            else {

                // 查找机构
                var findOrg = _.find(permission.organizationPermissionList, function (num) {

                    return num.id === orgID;
                });

                if (findOrg) {

                    // 查看具体权限
                    var find = _.find(findOrg.permissionList, function (num) {
                        return num.code.startWith(permissionCode);
                    });

                    if (find) {
                        isShow = true;
                    }
                }

            }
        }

        return isShow;
    };

    PWC.hasSpecialChar = function (value) {
        var reg = /^[\(\)（）\w\u4e00-\u9fa5]+$/;
        if (reg.test(value)) {
            return false;
        } else {
            return true;
        }
    };

    PWC.simulateProgress = function () {

        $('#busy-indicator-container').show();
        setTimeout(function () {

            $('#busy-indicator-container').hide();
        }, 2000);
    };
    PWC.convertDataForMap = function (data) {
        var res = [];
        for (var i = 0; i < data.length; i++) {
            var geoCoord = constant.geoCoordMap[data[i].name];
            if (geoCoord) {
                res.push({
                    name: data[i].name,
                    value: geoCoord.concat(data[i].value)
                });
            }
        }
        return res;
    };
    PWC.sortJSON = function (data) {
        if (_.isString(data)) {
            data = JSON.parse(data);
        }

        // Sort object keys
        data = _.chain(data).pairs().sortBy(function (x) {
            return x[0];
        }).value();

        // Build new object
        data = _.object.apply(_, _.zip.apply(_, data));
        data = JSON.stringify(data, null, 4);

        return data;
    };

    PWC.convertChineseCurrency = function (money) {
        //汉字的数字
        var cnNums = new Array('零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖');
        //基本单位
        var cnIntRadice = new Array('', '拾', '佰', '仟');
        //对应整数部分扩展单位
        var cnIntUnits = new Array('', '万', '亿', '兆');
        //对应小数部分单位
        var cnDecUnits = new Array('角', '分', '毫', '厘');
        //整数金额时后面跟的字符
        var cnInteger = '整';
        //整型完以后的单位
        var cnIntLast = '元';
        //最大处理的数字
        var maxNum = 999999999999999.9999;
        //金额整数部分
        var integerNum;
        //金额小数部分
        var decimalNum;
        //输出的中文金额字符串
        var chineseStr = '';
        //分离金额后用的数组，预定义
        var parts;
        if (money == '') {
            return '';
        }
        money = parseFloat(money);
        if (money >= maxNum) {
            //超出最大处理数字
            return '';
        }
        if (money == 0) {
            chineseStr = cnNums[0] + cnIntLast + cnInteger;
            return chineseStr;
        }
        //转换为字符串
        money = money.toString();
        if (money.indexOf('.') == -1) {
            integerNum = money;
            decimalNum = '';
        } else {
            parts = money.split('.');
            integerNum = parts[0];
            decimalNum = parts[1].substr(0, 4);
        }
        //获取整型部分转换
        if (parseInt(integerNum, 10) > 0) {
            var zeroCount = 0;
            var IntLen = integerNum.length;
            for (var i = 0; i < IntLen; i++) {
                var n = integerNum.substr(i, 1);
                var p = IntLen - i - 1;
                var q = p / 4;
                var m = p % 4;
                if (n == '0') {
                    zeroCount++;
                } else {
                    if (zeroCount > 0) {
                        chineseStr += cnNums[0];
                    }
                    //归零
                    zeroCount = 0;
                    chineseStr += cnNums[parseInt(n)] + cnIntRadice[m];
                }
                if (m == 0 && zeroCount < 4) {
                    chineseStr += cnIntUnits[q];
                }
            }
            chineseStr += cnIntLast;
        }
        //小数部分
        if (decimalNum != '') {
            var decLen = decimalNum.length;
            for (var i = 0; i < decLen; i++) {
                var n = decimalNum.substr(i, 1);
                if (n != '0') {
                    chineseStr += cnNums[Number(n)] + cnDecUnits[i];
                }
            }
        }
        if (chineseStr == '') {
            chineseStr += cnNums[0] + cnIntLast + cnInteger;
        } else if (decimalNum == '') {
            chineseStr += cnInteger;
        }
        return chineseStr;
    };


    PWC.convertChineseSimple = function (money) {
        //汉字的数字
        var cnNums = new Array('零', '一', '二', '三', '四', '五', '六', '七', '八', '九');
        //基本单位
        var cnIntRadice = new Array('', '十', '百', '千');
        //对应整数部分扩展单位
        var cnIntUnits = new Array('', '万', '亿', '兆');
        //对应小数部分单位
        var cnDecUnits = new Array('角', '分', '毫', '厘');
        //整数金额时后面跟的字符 '整'
        var cnInteger = '';
        //整型完以后的单位 '元'
        var cnIntLast = '';
        //最大处理的数字
        var maxNum = 999999999999999.9999;
        //金额整数部分
        var integerNum;
        //金额小数部分
        var decimalNum;
        //输出的中文金额字符串
        var chineseStr = '';
        //分离金额后用的数组，预定义
        var parts;
        if (money == '') {
            return '';
        }
        money = parseFloat(money);
        if (money >= maxNum) {
            //超出最大处理数字
            return '';
        }
        if (money == 0) {
            chineseStr = cnNums[0] + cnIntLast + cnInteger;
            return chineseStr;
        }
        //转换为字符串
        money = money.toString();
        if (money.indexOf('.') == -1) {
            integerNum = money;
            decimalNum = '';
        } else {
            parts = money.split('.');
            integerNum = parts[0];
            decimalNum = parts[1].substr(0, 4);
        }
        //获取整型部分转换
        if (parseInt(integerNum, 10) > 0) {
            var zeroCount = 0;
            var IntLen = integerNum.length;
            for (var i = 0; i < IntLen; i++) {
                var n = integerNum.substr(i, 1);
                var p = IntLen - i - 1;
                var q = p / 4;
                var m = p % 4;
                if (n == '0') {
                    zeroCount++;
                } else {
                    if (zeroCount > 0) {
                        chineseStr += cnNums[0];
                    }
                    //归零
                    zeroCount = 0;
                    chineseStr += cnNums[parseInt(n)] + cnIntRadice[m];
                }
                if (m == 0 && zeroCount < 4) {
                    chineseStr += cnIntUnits[q];
                }
            }
            chineseStr += cnIntLast;
        }
        //小数部分
        if (decimalNum != '') {
            var decLen = decimalNum.length;
            for (var i = 0; i < decLen; i++) {
                var n = decimalNum.substr(i, 1);
                if (n != '0') {
                    chineseStr += cnNums[Number(n)] + cnDecUnits[i];
                }
            }
        }
        if (chineseStr == '') {
            chineseStr += cnNums[0] + cnIntLast + cnInteger;
        } else if (decimalNum == '') {
            chineseStr += cnInteger;
        }
        return chineseStr;
    };

    PWC.getEnumKeyByValue = function (enumObj, value) {
        for (var key in enumObj) {
            if (enumObj[key] === value) {
                return key;
            }
        }
        return "";
    };

    PWC.prefixInteger = function (num, length) {
        return (Array(length).join('0') + num).slice(-length);
    };


    PWC.tryParseStringToNum = function (data) {
        // 加data.indexOf(' ') < 0防止" "被转换为0
        if (!_.isEmpty(data) && !isNaN(Number(data)) && data.indexOf(' ') < 0) {
            data = Number(data);
        }

        return data;
    };

    if (this._) {
        _.mixin({
            merge: function () {
                return _.reduce(arguments, function (list, obj) {
                    return _.extend(list, obj);
                }, {});
            }
        });
    }
    ;

    PWC.warning = function (title, text, callback) {
        swal({
                title: title,
                text: text,
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('Yes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                closeOnCancel: true
            },
            callback);
    };
    PWC.alert = function (title, text, callback) {
        swal({
                title: title,
                text: text,
                type: "info",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('Yes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                closeOnCancel: true
            },
            callback);
    };

    /------------------------------------------------kevin insert -----------------------------------/
    PWC.downloadCallBack = function (data, status, headers, fileName) {
        var octetStreamMime = 'application/octet-stream';
        var contentType = octetStreamMime;
        if (headers)
            contentType = headers('content-type') || octetStreamMime;
        if (window.navigator.msSaveBlob) {
            var blob = new Blob([data], {
                type: contentType
            });
            navigator.msSaveBlob(blob, fileName);
        } else {
            var urlCreator = window.URL || window.webkitURL || window.mozURL || window.msURL;
            if (urlCreator) {
                var a = document.createElement('a');
                var blob = new Blob([data], {
                    type: contentType
                });
                var url = urlCreator.createObjectURL(blob);
                a.href = url;
                a.target = '_blank';
                a.download = fileName + ".xlsx";
                document.body.appendChild(a);
                a.click();
            }
        }
    };
    PWC.addIndex = function(data){
        for(var i = 0, j= data.length; i<j; i++){
            data[i].xh = i + 1;
        }
        return data;
    }
    /**
     * @param scope   angular 作用域
     * @param translate  翻译器
     *这里指定 dataSource  为 companyList
     */
    PWC.orgManySelect = function (scope, translate, exp) {
        var dataSource = "companyList";
        var bindValue = "formParam.orgIds";
        scope.selectOrgOptions = {
            displayExpr: 'name',
            valueExpr: 'id',
            width: function () {
                return window.innerWidth / 4.5;
            },
            bindingOptions: {
                value: bindValue,
                dataSource: dataSource
            },
            height: '35px',
            placeholder: translate.instant('PleaseSelected'),
            showClearButton: true,
            searchEnabled: true,
            noDataText: translate.instant('RevenueNoOrgData'),
            showSelectionControls: true,
            visible: true,
            deferRendering: false
        };
        if (exp)
            _.extend(scope.selectOrgOptions, exp);
    };
    PWC.objIsEmpty = function (params) {
        var flag = true;

        for (var key in params) {
            if (params[key]) {
                flag = false;
                return flag
            }
        }
        return flag;
    };

    //给String 对象添加replaceAll方法
    String.prototype.replaceAll = function (s1, s2) {
        return this.replace(new RegExp(s1, "gm"), s2);
    }

    /*-----------------------------------------------------------------------------------------*/
})(window)